app.controller('AppointmentBookEditorController', ['$scope', '$http', 'restServer','$rootScope', 'Enums', function($scope, $http, restServer, $rootScope, enums) {
    var genderPreferenceNoPreference = 105;
    var appointHeaderId;
    var contextUsageHintAppointmentBook = "AppointmentBook", contextUsageHintRegister = "Register";
    $scope.value = "";
    // some bs prop to save noteType to since it wasn't saving directly on scope
    $scope.local = {};
    $scope.currentAppointment = { details: [] };
    $scope.showDatePicker = false;

    $scope.$on("newAppointment", function (event, args) {
        $scope.local.noteType = _.where($scope.noteTypes,{id:-1})[0];
        createNewAppointmentHeader(moment(args.date, "M-D-YYYY"), moment(args.startTime, "h:m"));
        $scope.currentDetail.employee = args.employee;
    });

    function setEmployee(appointDetail, employeeGlobalId, employeeDisplayName, raiseChanged) {
        appointDetail.employee = {
            id: employeeGlobalId,
            displayName: employeeDisplayName,
        }

        if (raiseChanged) {
            $scope.employeeChanged();
        }
    }

    function setService(appointDetail, serviceItemId, serviceShortcut, serviceDisplayName, raiseChanged) {
        appointDetail.service = {
            id: serviceItemId,
            shortcut: serviceShortcut,
            displayName: serviceDisplayName
        }

        if (raiseChanged) {
            $scope.serviceChanged();
        }
    }

    $scope.$on("editAppointment", function (event, args) {
        restServer.getAppointHeaderForAppointDetail(args.appointDetailId).then(function (result) {
            appointHeaderId = result.data.appointHeaderId;
                    for (var i = 0; i < result.data.details.length; i++) {
                        var appointDetail = result.data.details[i];

                        appointDetail.stateEnum = enums.StateEnum.Read;
                        appointDetail.client = {
                            id: appointDetail.clientGlobalId,
                            firstName: appointDetail.clientFirstName,
                            lastName: appointDetail.clientLastName
                        }
                        setService(appointDetail, appointDetail.serviceItemId, appointDetail.serviceShortcut, appointDetail.serviceDisplayName);

                        setEmployee(appointDetail, appointDetail.steps[0].employeeGlobalId, appointDetail.steps[0].employeeDisplayName);

                        appointDetail.originalEmployeeId = appointDetail.steps[0].employeeGlobalId;

                        if (appointDetail.appointTypeItemId != undefined) {
                            appointDetail.appointType = {
                                id: appointDetail.appointTypeItemId,
                                displayName: appointDetail.appointTypeDisplayName,
                                shortcut: appointDetail.appointTypeShortcut
                            }
                        }

                        if ($scope.genderPreferences != undefined) {
                            appointDetail.genderPreference = _.where($scope.genderPreferences, { id: appointDetail.genderPreferenceEnum })[0];
                        }

                        appointDetail.steps.forEach(function(appointDetailStep) {
                            appointDetailStep.stateEnum = enums.StateEnum.Read;
                            calculateStepDuration(appointDetailStep);
                        });
                    }

                    result.data.payingClient = {
                        firstName: result.data.bookedForClientFirstName,
                        lastName: result.data.bookedForClientLastName,
                        id: result.data.bookedForClientGlobalId
                    };
                    if (result.data.appointCatItemCatId != undefined) {
                        result.data.appointCat = {
                            displayName: result.data.appointCatDisplayName,
                            shortcut: result.data.appointCatShortcut
                        };
                    }
                    result.data.appointmentDate = moment(result.data.appointmentDate).format();

                    if (result.data.note != undefined && result.data.note.contextUsagesHint != '') {
                        var noteTypeId;
                        if (result.data.note.contextUsagesHint == contextUsageHintAppointmentBook) {
                            noteTypeId = enums.AppointNoteTypeEnum.Checkin;
                        } else if (result.data.note.contextUsagesHint == contextUsageHintRegister) {
                            noteTypeId = enums.AppointNoteTypeEnum.Register;
                        } else {
                            noteTypeId = enums.AppointNoteTypeEnum.CheckinRegister;
                        }
                        $scope.local.noteType = _.where($scope.noteTypes, { id: noteTypeId })[0];
                    } else {
                        $scope.local.noteType = _.where($scope.noteTypes, { id: -1 })[0];
                    }

                    $scope.currentAppointment = result.data;
                    $scope.selectAppointDetail(args.appointDetailId);
                });
    });

    function calculateStepDuration(appointDetailStep) {
        var hours = Math.floor(appointDetailStep.stepMinutes / 60);
        var minutes = appointDetailStep.stepMinutes % 60;
        appointDetailStep.stepDuration = moment(hours + ':' + minutes, 'H:m').format('HH:mm');
    }

    function createNewAppointmentHeader(date, startTime) {
        $scope.currentAppointment = {
            appointCat: undefined,
            payingClient: undefined,
            details: [],
            note: {
                value:''
            },
        };
        $scope.currentAppointment.appointmentDate = date.format();
        createNewAppointDetail(startTime);
    }

    function createNewAppointDetail(startTime, employee, appointType, client) {
        $scope.currentDetail = {
            stateEnum: enums.StateEnum.Add,
            steps: [],
            appointDetailId: NewGuid(),
            genderPreferenceEnum: genderPreferenceNoPreference,
            isNew: true,
            client: client,
            employee: employee,
            appointType: appointType,
            startTimeSegmented: {
                
            }
        }

        if ($scope.genderPreferences != undefined) {
            $scope.currentDetail.genderPreference = _.where($scope.genderPreferences, { id: $scope.currentDetail.genderPreferenceEnum })[0];
        }

        if (startTime != undefined) {
            $scope.currentDetail.startTimeSegmented = startTime.getTempTime($rootScope.isAmPmTime);
            $scope.currentDetail.startTime = startTime.format();
        }
    }

    $scope.saveAppointDetail = function () {
        if (!isObject($scope.currentAppointment.payingClient)) {
            $rootScope.$broadcast("showMessage", { message: "You must select a paying client in order to proceed" });
            return;
        }
        if (!isObject($scope.currentDetail.service)) {
            $rootScope.$broadcast("showMessage", { message: "You must select a service in order to proceed" });
            return;
        }

        if ($scope.appointTypeRequired() && !isObject($scope.currentDetail.appointType)) {
            $rootScope.$broadcast("showMessage", { message: "You must supply an appointment type in order to proceed" });
            return;
        }

        if ($scope.currentDetail.isNew != undefined && $scope.currentDetail.isNew) {            
            $scope.currentDetail.isNew = false;
            $scope.currentAppointment.details.push($scope.currentDetail);
            var currentClient = $scope.currentDetail.client;
            createNewAppointDetail(moment($scope.currentDetail.startTimeSegmented.hours + ":" + $scope.currentDetail.startTimeSegmented.minutes + " " + $scope.currentDetail.startTimeSegmented.amPmDesignator, "h:m a").add($scope.currentDetail.totalMinutes, "minutes"),
$scope.currentDetail.employee, $scope.currentDetail.appointType, $scope.currentDetail.client);
            if (currentClient != undefined) {
                $scope.currentDetail.client = { id: currentClient.id, firstName: currentClient.firstName, lastName: currentClient.lastName };
            }
        } else {
            createNewAppointDetail(moment($scope.currentDetail.startTimeSegmented.hours + ":" + $scope.currentDetail.startTimeSegmented.minutes + " " + $scope.currentDetail.startTimeSegmented.amPmDesignator, "h:m a").add($scope.currentDetail.totalMinutes, "minutes"),
$scope.currentDetail.employee, $scope.currentDetail.appointType, $scope.currentDetail.client);
        }
    }

    $scope.appointTypeRequired = function() {
        var setting = $scope.$parent.getSetting(enums.SettingEnum.AppointDetailTypeRequired);
        return setting != null && setting.toBoolean();
    }

    var appointDetailBeingDeleted;
    $scope.removeAppointDetail = function (appointDetail) {
        if (appointDetail.stateEnum == enums.StateEnum.Add) {
            appointDetailBeingDeleted = appointDetail;
            $rootScope.$broadcast("showMessage", {
                message: "Are you sure you want to delete this appointment?",
                buttonsType: enums.ButtonTypeEnum.YesNo,
                callback: deleteAppointDetailConfirmation
            });
        } else {
            $scope.$parent.deleteAppointmentFromHeader($scope.currentAppointment, [appointDetail.appointDetailId], false);
        }
    }

    $scope.delete = function() {
        var appointDetailIdsToDelete = [];
        $scope.currentAppointment.details.forEach(function(detail) {
            if (detail.stateEnum != enums.StateEnum.Add) {
                appointDetailIdsToDelete.push(detail.appointDetailId);
            }
        });
        $scope.$parent.deleteAppointmentFromHeader($scope.currentAppointment, appointDetailIdsToDelete, true);
    }
function deleteAppointDetailConfirmation(response) {
    if (response == enums.MessageBoxButtonResult.Yes) {
        for (var i = $scope.currentAppointment.details.length - 1; i >= 0; i--) {
            if ($scope.currentAppointment.details[i].appointDetailId == appointDetailBeingDeleted.appointDetailId) {
                $scope.currentAppointment.details.splice(i, 1);
                break;
            }
        }
        createNewAppointDetail(moment(appointDetailBeingDeleted.startTime), appointDetailBeingDeleted.employee,
            appointDetailBeingDeleted.appointType, appointDetailBeingDeleted.client);
    }
}

    $scope.getTotalPrice = function() {
        var totalPrice = 0;
        $scope.currentAppointment.details.forEach(function (detail) {
            if (detail.stateEnum != enums.StateEnum.Delete) {
                totalPrice += detail.price;
            }
        });
        return totalPrice;
    }

    function rebuildSteps() {
        
        var employeeId = $scope.currentDetail.employee != undefined && $scope.currentDetail.employee.id != undefined ? $scope.currentDetail.employee.id : NewGuid();
        restServer.createDefaultAppointDetail(employeeId, $scope.currentDetail.service.id, function (response) {
            $scope.currentDetail.price = response.price;

            for (var j = $scope.currentDetail.steps.length - 1; j >= 0; j--) {
                var step = $scope.currentDetail.steps[j];
                // if this step was added during this edit sesssion, it can just be removed since it doesn't need to be cleaned up in db
                if (step.stateEnum == enums.StateEnum.Add) {
                    $scope.currentDetail.steps.splice(j, 1);
                } else {
                    step.stateEnum = enums.StateEnum.Delete;
                }
            }
            $scope.currentDetail.totalMinutes = response.totalMinutes;
            response.steps.forEach(function (step) {
                step.stateEnum = enums.StateEnum.Add;
                calculateStepDuration(step);
                $scope.currentDetail.steps.push(step);
            });
        });
    }

    var appointHeader;
    var affectedColumns;
    function appointHeaderInPastWarningCallback(response) {
        if (response == enums.MessageBoxButtonResult.Yes) {
            $scope.saveAppointment(_saveAppointmentData);
        }
    }

    $scope.saveAppointment = function(workflowData) {
        _saveAppointmentData = workflowData;
        
        if (_saveAppointmentData.pastChecked == undefined && moment($scope.currentAppointment.appointmentDate).isInPast()) {
            _saveAppointmentData.pastChecked = true;
            $rootScope.$broadcast("showMessage", {
                message: "You are attempting to booking an appointment in the past.  Do you want to continue?",
                buttonsType: enums.ButtonTypeEnum.YesNo,
                callback: appointHeaderInPastWarningCallback
            });
            return;
        }

        if (!isObject($scope.currentAppointment.payingClient)) {
            $rootScope.$broadcast("showMessage", { message: "You haven't chosen a paying client yet.  Please select one before saving." });
            return;
        }

        if ($scope.currentAppointment.details.length == 0) {
            $rootScope.$broadcast("showMessage", { message: "You haven't chosen any services yet.  Please select one or more before saving." });
            return;
        }

        affectedColumns = [];
        appointHeader = $scope.currentAppointment;
        appointHeader.bookedForClientGlobalId = appointHeader.payingClient.id;
        appointHeader.appointCatItemCatId = appointHeader.appointCat != undefined ? appointHeader.appointCat.id : null;

        var isValid = true;
        appointHeader.appointmentDate = moment(appointHeader.appointmentDate).format();
        $scope.currentAppointment.details.forEach(function (appointDetail) {
            appointDetail.startDate = moment(appointDetail.startDate).format();
            var originalAppointmentDate;
            if ($scope.appointTypeRequired() && !isObject(appointDetail.appointType)) {
                $rootScope.$broadcast("showMessage", {
                    message: "You must supply an appointment type for each service before saving"
                });
                isValid = false;
                return;
            }
            appointDetail.clientGlobalId = appointDetail.client.id,
                appointDetail.serviceItemId = appointDetail.service.id,
                appointDetail.appointTypeItemId = appointDetail.appointType != undefined ? appointDetail.appointType.id : null,
            appointDetail.genderPreferenceEnum = appointDetail.genderPreference.id;
            if (appointDetail.startDate != $scope.currentAppointment.appointmentDate) {
                originalAppointmentDate = moment(appointDetail.startDate);
                appointDetail.startDate = $scope.currentAppointment.appointmentDate;
                setState(appointDetail, enums.StateEnum.Edit);
            }

            // employee for appointdetail has changed so need to refresh that column as well
            if (appointDetail.originalEmployeeId != undefined && appointDetail.originalEmployeeId.toLowerCase() != appointDetail.employee.id.toLowerCase()) {
                affectedColumns.push({ date: moment(appointHeader.appointmentDate), employeeGlobalId: appointDetail.originalEmployeeId });
                if (originalAppointmentDate != undefined) {
                    affectedColumns.push({ date: originalAppointmentDate, employeeGlobalId: appointDetail.originalEmployeeId });
                }
            }
            affectedColumns.push({ date: moment(appointHeader.appointmentDate), employeeGlobalId: appointDetail.employee.id });
            if (originalAppointmentDate != undefined) {
                affectedColumns.push({ date: originalAppointmentDate, employeeGlobalId: appointDetail.employee.id });
            }
        });
        setNoteContextUsagesHint();
        if (isValid) {
            $scope.$parent.showProcessingOverlay();
            restServer.saveAppointHeader(appointHeader, saveAppointHeaderCallback);
        } 
    }
    
    function saveAppointHeaderCallback(response) {
        
        if (response.warnings == undefined) {
            $rootScope.$broadcast("appointmentSaved", {
                affectedColumns: affectedColumns
            });
        } else {
            $rootScope.$broadcast("showMessage", { message: response.warnings, buttonsType: enums.ButtonTypeEnum.YesNo, callback: checkWarningCallback });
        }
        $scope.$parent.hideProcessingOverlay();
    }

    function checkWarningCallback(result) {
        if (result == enums.MessageBoxButtonResult.Yes) {
            appointHeader.warningsAlreadyChecked = true;
            restServer.saveAppointHeader(appointHeader, saveAppointHeaderCallback);
        }
    }
    function setNoteContextUsagesHint() {
        if ($scope.currentAppointment.note != undefined) {
            if ($scope.local.noteType != undefined && $scope.local.noteType.id != undefined && $scope.local.noteType.id != -1) {
                if ($scope.local.noteType.id == enums.AppointNoteTypeEnum.Checkin) {
                    $scope.currentAppointment.note.contextUsagesHint = contextUsageHintAppointmentBook;
                } else if ($scope.local.noteType.id == enums.AppointNoteTypeEnum.Register) {
                    $scope.currentAppointment.note.contextUsagesHint = contextUsageHintRegister;
                } else if ($scope.local.noteType.id == enums.AppointNoteTypeEnum.CheckinRegister) {
                    $scope.currentAppointment.note.contextUsagesHint = contextUsageHintAppointmentBook + ',' + contextUsageHintRegister;
                }
            } else {
                $scope.currentAppointment.note.contextUsagesHint = '';
            }
        }
    }

    $scope.cancel = function () {
        if (appointHeaderId != undefined) {
            restServer.deleteLock(appointHeaderId, "AppointHeader");
        }
        $rootScope.$broadcast("cancelAppointmentSave");
    }

    $scope.selectAppointDetail = function(appointDetailId) {
        $scope.currentDetail = _.where($scope.currentAppointment.details, { appointDetailId: appointDetailId })[0];
        var startTime = moment($scope.currentDetail.startTime);
        $scope.currentDetail.startTimeSegmented = startTime.getTempTime($rootScope.isAmPmTime);
    }

    var indexedClients = [];
    $scope.appointDetailsToFilter = function () {
        indexedClients = [];
        return $scope.currentAppointment.details;
    }

    $scope.appointDetailClients = function() {
        var uniqueClients = [];
        var uniqueClientIds = [];
        if ($scope.currentAppointment != undefined) {
            $scope.currentAppointment.details.forEach(function (appointDetail) {
                if (uniqueClientIds.indexOf(appointDetail.client.id) == -1) {
                    uniqueClientIds.push(appointDetail.client.id);
                    uniqueClients.push(appointDetail.client);
                }
            });
        }
        return uniqueClients;
    }

    $scope.filterAppointDetailByClient = function(appointDetail) {
        var clientIsNew = indexedClients.indexOf(appointDetail.client.id);
        if (clientIsNew) {
            indexedClients.push(appointDetail.client.id);
        }
        return clientIsNew;
    }
    
    $scope.filterDisplayStep = function(step) {
        return step.stateEnum == enums.StateEnum.Add || step.stateEnum == enums.StateEnum.Edit || step.stateEnum == enums.StateEnum.Read;
    }

    $scope.currentDetailChanged = function () {
        setState($scope.currentDetail, enums.StateEnum.Edit);
    }

    $scope.serviceChanged = function () {
        if (isObject($scope.currentDetail.service)) {
            setState($scope.currentDetail, enums.StateEnum.Edit);
            rebuildSteps();
        }
        checkImplicitAdd();
    }
    
    $scope.clientChanged = function() {
        if (isObject($scope.currentDetail.client)) {
            if ($scope.currentAppointment.payingClient == undefined) {
                $scope.currentAppointment.payingClient = $scope.currentDetail.client;
                $scope.$apply();
            }
            checkClientRestrictedWarning($scope.currentDetail.client);
        }
        setState($scope.currentDetail, enums.StateEnum.Edit);
        checkImplicitAdd();
    }

    $scope.payingClientChanged = function () {
        if (isObject($scope.currentAppointment.payingClient)){
            if ($scope.currentDetail.client == undefined) {
                $scope.currentDetail.client = $scope.currentAppointment.payingClient;
                $scope.$apply();
            }
            checkClientRestrictedWarning($scope.currentAppointment.payingClient);
        }
    }

    function checkClientRestrictedWarning(client) {
        if (client.isRestricted) {
            $rootScope.$broadcast("showMessage", { message: 'Client Restricted', messageClass: 'high' });
        }
    }
    function checkImplicitAdd() {
        if ($scope.currentDetail.isNew && isObject($scope.currentDetail.service) && isObject($scope.currentDetail.client)) {
            $scope.currentDetail.isNew = false;
            $scope.currentAppointment.details.push($scope.currentDetail);
        }
    }

    $scope.employeeChanged = function() {
        if ($scope.currentDetail.service != undefined && $scope.currentDetail.service.id != undefined) {
            setState($scope.currentDetail, enums.StateEnum.Edit);
            rebuildSteps();
        }
        $scope.$apply();
    }

    $scope.currentDetailTimeChanged = function () {
        $scope.$apply();
        $scope.currentDetail.startTime = moment().getTimeFromTempTime($scope.currentDetail.startTimeSegmented, $rootScope.isAmPmTime).format();
        setState($scope.currentDetail, enums.StateEnum.Edit);
    }

    function setState(obj, newState) {
        if (newState == enums.StateEnum.Edit && obj.stateEnum == enums.StateEnum.Read) {
            obj.stateEnum = newState;
        }
    }
    

    $scope.getClients = function (filter)
    {
        return restServer.LoadFilteredClients(filter).then(function(response) {
            return response.data;
        });
            }

    $scope.getServices = function(filter) {
        return restServer.LoadFilteredServices(filter).then(function (response) {
            return response.data;
        });
    }

    $scope.getAppointTypes = function (filter) {
        return restServer.LoadFilteredAppointTypes(filter).then(function (response) {
            return response.data;
        });
    }

    $scope.getEmployees = function (filter) {
        return restServer.LoadFilteredEmployees(filter).then(function (response) {
            return response.data;
        });
    }

    $scope.getAppointCats = function(filter) {
        return restServer.LoadFilteredAppointCats(filter).then(function (response) {
            return response.data;
        });
    }

    $scope.noteTypes = [
    {
        displayName: '',
        id:-1
    },
        {
            displayName: "Checkin",
            id: enums.AppointNoteTypeEnum.Checkin
        },
        {
            displayName: "Register",
            id: enums.AppointNoteTypeEnum.Register
        },
        {
            displayName: "Chk/Reg",
            id: enums.AppointNoteTypeEnum.CheckinRegister
        }
    ];
    $scope.getGenderPreferences = function () {
        if ($scope.genderPreferences == undefined) {
            $scope.genderPreferences = [];
            restServer.getEnumList(23).then(function (response) {
                $scope.genderPreferences = response.data;
                
                if ($scope.currentAppointment != undefined) {
                    $scope.currentAppointment.details.forEach(function(detail) {
                        detail.genderPreference = _.where($scope.genderPreferences, { id: detail.genderPreferenceEnum })[0];
                    });
                }

                if ($scope.currentDetail != undefined) {
                    $scope.currentDetail.genderPreference = _.where($scope.genderPreferences, { id: $scope.currentDetail.genderPreferenceEnum })[0];
                }
            });
        }
        return $scope.genderPreferences;

    }

    $scope.editClient = function () {
        if (isObject($scope.currentDetail.client)) {
            $rootScope.$broadcast("addEditClient", {
                clientId: $scope.currentDetail.client.id,
                mode: 1,
                target: 2
            });
        }
    }

    $scope.addClient = function () {
        $rootScope.$broadcast("addEditClient", {
            mode: 2,
            target: 2
        });
    }

    $scope.editPayingClient = function () {
        if (isObject($scope.currentDetail.client)) {
            $rootScope.$broadcast("addEditClient", {
                clientId: $scope.currentAppointment.payingClient.id,
                mode: 1,
                target: 1
            });
        }
    }

    $scope.addPayingClient = function () {
        $rootScope.$broadcast("addEditClient", {
            mode: 2,
            target: 1
        });
    }

    $scope.toggleDatePicker = function () {
        $('#dateInput').focus();
    };

    $scope.$on("clientSaved", function (event, args) {
        if (args.mode == 2) {
            if (args.target == 1) {
                $scope.currentAppointment.payingClient = {
                    lastName: args.client.lastName,
                    firstName: args.client.firstName,
                    id: args.client.clientGlobalId
                }
                $scope.payingClientChanged();
            }
            else if (args.target == 2) {
                $scope.currentDetail.client = {
                    lastName: args.client.lastName,
                    firstName: args.client.firstName,
                    id: args.client.clientGlobalId
                }
                $scope.clientChanged();
            }
        }
    });

    $scope.showSteps = function() {
        $scope.stepsVisible = true;
    }

    $scope.hideSteps = function() {
        $scope.stepsVisible = false;
    }

    $scope.stepDurationChanged = function (step) {
        var stepDuration = moment(step.stepDuration, "HH:mm");
        step.stepMinutes = stepDuration.hours() * 60 + stepDuration.minutes();
        $scope.currentDetail.totalMinutes = 0;
        $scope.currentDetail.steps.forEach(function (step) {
            if (step.stateEnum != enums.StateEnum.Delete) {
                $scope.currentDetail.totalMinutes += step.stepMinutes;
            }
        });
        setState($scope.currentDetail, enums.StateEnum.Edit);
        
        //Make all appointments later than appointment being edited contiguous to appointment being edited       
        var nextStartTime = moment( $scope.currentDetail.startTime);
        nextStartTime.add($scope.currentDetail.totalMinutes, 'm');
        var orderedDetails = _.sortBy($scope.currentAppointment.details, 'startTime');
        orderedDetails.forEach(function(detail) {
            if (detail.startTime > $scope.currentDetail.startTime) {
                detail.startTime = nextStartTime.format();
                setState(detail, enums.StateEnum.Edit);
                nextStartTime.add(detail.totalMinutes, 'm');
            }
        });

        setState(step, enums.StateEnum.Edit);
    }
    var potentialServiceStepDurations;
    $scope.getPotentialServiceStepDurations = function() {
        if (potentialServiceStepDurations == undefined) {
            var current = moment("00:00", "HH:mm");
            potentialServiceStepDurations = [];
            for (var i = 0; i < 20; i++) {
                potentialServiceStepDurations.push(current.format("HH:mm"));
                current.add(15, 'm');
            }
        }
        return potentialServiceStepDurations;
    }

    $scope.historyLineActions = [
        {
            displayName: "Emp & Service",
            id: enums.ClientHistoryActionEnum.AddEmployeeAndService
        },
        {
            displayName: "Employee",
            id: enums.ClientHistoryActionEnum.AddEmployee
        },
        {
            displayName: "Service",
            id: enums.ClientHistoryActionEnum.AddService
        }
    ];

    $scope.showClientHistory = function () {
        if ($scope.currentDetail.client.id != undefined) {
            $scope.historyVisible = true;
            restServer.getClientAppoimentHistory($scope.currentDetail.client.id, function(response) {
                $scope.appointmentHistory = response;
            });
        }
    }

    $scope.hideHistory = function() {
        $scope.historyVisible = false;
    }

    $scope.historyActionChosen = function(historyLine) {
        switch (historyLine.historyLineAction.id) {
            case enums.ClientHistoryActionEnum.AddEmployeeAndService:
                setEmployee($scope.currentDetail, historyLine.employeeGlobalId, historyLine.employeeDisplayName, true);
                setService($scope.currentDetail, historyLine.serviceItemId, historyLine.serviceShortcut, historyLine.serviceDisplayName, true);
                break;
            case enums.ClientHistoryActionEnum.AddEmployee:
                setEmployee($scope.currentDetail, historyLine.employeeGlobalId, historyLine.employeeDisplayName, true);
                break;
            case enums.ClientHistoryActionEnum.AddService:
                setService($scope.currentDetail, historyLine.serviceItemId, historyLine.serviceShortcut, historyLine.serviceDisplayName, true);
                break;
        }
        $scope.historyVisible = false;
    }

    $scope.showChangeLog = function () {
        if ($scope.currentAppointment.appointHeaderId != undefined) {
            $scope.changeLogVisible = true;
            restServer.getAppointHeaderAuditHistory($scope.currentAppointment.appointHeaderId, function(response) {
                $scope.changes = response;
            });
        }
        
    }

    $scope.hideChangeLog = function() {
        $scope.changeLogVisible = false;
    }
}]);