﻿app.controller('BlockDefinitionController', [
    '$scope', '$http', 'restServer', '$rootScope', 'Enums', function ($scope, $http, restServer, $rootScope, enums) {
        var appointBlockDefinitionId;
        $scope.selections = {};
        $scope.$on("addBlockDefinition", function () {
            var startTime = moment();
            
            var intervalMinutes = $scope.$parent.minutes;
            if (startTime.minutes() > intervalMinutes[intervalMinutes.length - 1]) {
                startTime.minutes(0);
                startTime.add(1, 'h');
            } else {
                for (var i = 0; i < intervalMinutes.length; i++) {
                    if (startTime.minutes() <= intervalMinutes[i]) {
                        startTime.minutes(intervalMinutes[i]);
                        break;
                    }
                }
            }
            
            $scope.appointBlockDefinition = {
                blockType: undefined,
                alternateBlockReason: '',
                schedule: {
                    frequencyTypeEnum: enums.BlockDefinitionFrequenceyTypeEnum.OneTime,
                    startDate: moment(),
                    endDate: moment(),
                    weeklyIsSunday: false,
                    weeklyIsMonday: false,
                    weeklyIsTuesday: false,
                    weeklyIsWednesday: false,
                    weeklyIsThursday: false,
                    weeklyIsFriday: false,
                    weeklyIsSaturday: false,
                    frequency: 1,
                    numberOfOccurrences: 1
                },
                employees: []
            };

            $scope.tempStartTime = startTime.getTempTime($rootScope.isAmPmTime);
            var endTime = startTime.add(1, 'h');
            $scope.tempEndTime = endTime.getTempTime($rootScope.isAmPmTime);
        });


        $scope.$on("editBlockDefinition", function (event, args) {
            appointBlockDefinitionId = args.AppointBlockDefinitionId;
            restServer.getAppointBlockDefinition(args.appointBlockDefitionId, function (response) {
                response.blockType = {
                    id: response.blockTypeItemId,
                    displayName: response.blockTypeDisplayName,
                    shortcut: response.blockTypeShortcut
                };
                if (response.schedule.startDate != undefined) {
                    response.schedule.startDate = moment(response.schedule.startDate);
                }
                if (response.schedule.endDate != undefined) {
                    response.schedule.endDate = moment(response.schedule.endDate);
                }
                if (response.schedule.frequencyTypeEnum == enums.BlockDefinitionFrequenceyTypeEnum.Weekly) {
                    response.schedule.frequency = response.schedule.weeklyRecurringWeekFrequency;
                } else {
                    response.schedule.frequency = response.schedule.dailyRecurringFrequency;
                }
                var startTime = moment(response.schedule.startTime);
                $scope.tempStartTime = startTime.getTempTime($rootScope.isAmPmTime);
                var endTime = moment(response.schedule.endTime);
                $scope.tempEndTime = endTime.getTempTime($rootScope.isAmPmTime);
                response.employees = [];
                response.appointBlockDefinitionEmployeeGlobals.forEach(function (detail) {
                    response.employees.push({
                        id: detail.employeeGlobalId,
                        displayName: detail.employeeGlobalDisplayName
                    });
                });
                $scope.appointBlockDefinition = response;
                updateNumberOfOccurrences();
                updateFrequencyLabel();
            });
        });

        $scope.save = function () {
            var schedule = $scope.appointBlockDefinition.schedule;
            schedule.startTime = moment().getTimeFromTempTime($scope.tempStartTime, $rootScope.isAmPmTime);
            schedule.endTime = moment().getTimeFromTempTime($scope.tempEndTime, $rootScope.isAmPmTime);
            if (schedule.frequencyTypeEnum == enums.BlockDefinitionFrequenceyTypeEnum.Weekly) {
                schedule.weeklyRecurringWeekFrequency = schedule.frequency;
                schedule.dailyRecurringFrequency = 1;
            } else {
                schedule.weeklyRecurringWeekFrequency = 1;
                schedule.dailyRecurringFrequency = schedule.frequency;
            }
            if (isValid()) {
                $scope.appointBlockDefinition.blockTypeItemId = $scope.appointBlockDefinition.blockType.id;

                $scope.appointBlockDefinition.appointBlockDefinitionEmployeeGlobals = [];

                
                $scope.appointBlockDefinition.employees.forEach(function (employee) {
                    var employeeFound = false;
                    for (var i = 0; i < $scope.appointBlockDefinition.appointBlockDefinitionEmployeeGlobals.length; i++) {
                        if (employee.id == $scope.appointBlockDefinition.appointBlockDefinitionEmployeeGlobals[i].employeeGlobalId) {
                            employeeFound = true;
                            break;
                        }
                    }
                    if (!employeeFound) {
                        $scope.appointBlockDefinition.appointBlockDefinitionEmployeeGlobals.push({ employeeGlobalId: employee.id });
                    }
                });
                
                schedule.startDate = moment(schedule.startDate).format();
                schedule.endDate = moment(schedule.endDate).format();
                schedule.startTime = schedule.startTime.format();
                schedule.endTime = schedule.endTime.format();
                $scope.$parent.showProcessingOverlay();
                restServer.saveAppointBlockDefinition($scope.appointBlockDefinition, function () {
                    $scope.$parent.hideProcessingOverlay();
                    $scope.$emit("blockDefinitionSaved");
                });
            }

        }

        function isValid() {
            var block = $scope.appointBlockDefinition;

            var message = '';
            
            if (block.blockType == undefined || block.blockType.id == undefined) {
                message += '<li>Block type is required</li>';
            }

            if (block.employees.length == 0) {
                message += '<li>At least one employee is required</li>';
            }

            if (block.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Weekly && !block.schedule.weeklyIsSunday &&
                !block.schedule.weeklyIsMonday && !block.schedule.weeklyIsTuesday && !block.schedule.weeklyIsWednesday && !block.schedule.weeklyIsThursday
                && !block.schedule.weeklyIsFriday && !block.schedule.weeklyIsSaturday) {
                message += '<li>You must select at least one day when using a weekly schedule</li>';
            }

            if (moment(block.schedule.endDate) < moment(block.schedule.startDate)) {
                message += "<li>End date can't be less than start date</li>";
            }

            if (block.schedule.endTime <= block.schedule.startTime) {
                message += "<li>End time must be greater than start time</li>";
            }

            if (message != '') {
                message = "<ul>" + message + "</ul>";
                $rootScope.$broadcast("showMessage", { message: message });
                return false;
            }
            return true;
        }

        $scope.selectedEmployeeChanged = function() {
            if (!isObject($scope.selections.selectedEmployee)) {
                return;
            }

            $scope.appointBlockDefinition.employees.push($scope.selections.selectedEmployee);
            $('#selectedEmployeeInput').trigger("blur");
            $scope.selections.selectedEmployee = {};
            $scope.$apply();
        }

        $scope.selectedBlockTypeChanged = function() {
            if (!isObject($scope.appointBlockDefinition.blockType) || $scope.appointBlockDefinition.blockType.id == undefined) {
                return;
            }

            $scope.appointBlockDefinition.alternateBlockReason = $scope.appointBlockDefinition.blockType.displayName;
            $scope.$apply();
        }

        $scope.deleteEmployee = function (index) {
            $scope.appointBlockDefinition.employees.splice(index, 1);
        };

        $scope.getEmployees = function (filter) {
            return restServer.LoadFilteredEmployees(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.getBlockTypes = function (filter) {
            return restServer.LoadFilteredBlockTypes(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.numberOccurrencesChanged = function () {
            var val = $scope.appointBlockDefinition.schedule.numberOfOccurrences;
            if (!isNaN(parseInt(val)) && val >= 1) {
                if ($scope.appointBlockDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                    $scope.appointBlockDefinition.schedule.endDate = moment($scope.appointBlockDefinition.schedule.startDate).add(val - 1, 'd');
                } else {
                    $scope.appointBlockDefinition.schedule.endDate = moment($scope.appointBlockDefinition.schedule.startDate).add(val * 7 * $scope.appointBlockDefinition.schedule.frequency - 1, 'd');
                }
            }
        }

        $scope.frequencyChanged = function () {
            updateNumberOfOccurrences();
            updateFrequencyLabel();
        }

        $scope.endDateChanged = function () {
            if (moment($scope.appointBlockDefinition.schedule.endDate) < moment($scope.appointBlockDefinition.schedule.startDate)) {
                $rootScope.$broadcast("showMessage", { message: "End date can't be less than start date" });
                return;
            }
            updateNumberOfOccurrences();
        }

        function updateNumberOfOccurrences() {
            if ($scope.appointBlockDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                $scope.appointBlockDefinition.schedule.numberOfOccurrences = Math.floor((moment($scope.appointBlockDefinition.schedule.endDate).diff(moment($scope.appointBlockDefinition.schedule.startDate), 'days') + 1) / $scope.appointBlockDefinition.schedule.frequency);
            } else {
                $scope.appointBlockDefinition.schedule.numberOfOccurrences = Math.ceil((moment($scope.appointBlockDefinition.schedule.endDate).diff(moment($scope.appointBlockDefinition.schedule.startDate), 'days') + 1) / (7 * $scope.appointBlockDefinition.schedule.frequency));
            }
            if ($scope.appointBlockDefinition.schedule.numberOfOccurrences < 1) {
                $scope.appointBlockDefinition.schedule.numberOfOccurrences = 1;
            }
        }

        function updateFrequencyLabel() {
            if ($scope.appointBlockDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                $scope.frequencyLabel = "days";
            } else {
                $scope.frequencyLabel = "weeks";
            }
        }

        $scope.cancel = function () {
            if (appointBlockDefinitionId != undefined) {
                restServer.deleteLock(appointBlockDefinitionId, "AppointBlockDefinition");
            }
            $scope.$parent.hideBlockDefinitionEditor();
        }

        $scope.showStartDatePicker = function () {
            $('#blockDefStartDate').focus();
        };

        $scope.showEndDatePicker = function () {
            $('#blockDefEndDate').focus();
        };
    }
]);