﻿app.controller('BlockEditorController', ['$scope', '$http', 'restServer','$rootScope', function($scope, $http, restServer, $rootScope) {
    $scope.$on("newBlock", function (event, args) {
        createNewBlock(moment(args.date, "M-D-YYYY"), moment(args.startTime, "h:m"));
        $scope.block.employee = args.employee;
    });

    var blockId;
    $scope.$on("editBlock", function (event, args) {
        blockId = args.blockId;
        restServer.getAppointBlock(args.blockId, function(response) {
            response.employee = {
                id: response.employeeGlobalId,
                displayName: response.employeeDisplayName
            }
            response.originalEmployeeGlobalId = response.employeeGlobalId;
            response.blockType = {
                id: response.blockTypeItemId,
                displayName: response.blockTypeDisplayName,
                shortcut: response.blockTypeShortcut
            }
            
            response.startDate = moment(response.startDate);
            response.originalStartDate = response.startDate;

            var startTime = moment(response.startTime);
            response.startTimeTemp = startTime.getTempTime($rootScope.isAmPmTime); 
            var endTime = moment(response.startTime).add(response.totalMinutes, 'minutes');
            response.endTime = endTime.getTempTime($rootScope.isAmPmTime); 
           $scope.block = response;
        });
    });

    $scope.selectedBlockTypeChanged = function() {
        if (!isObject($scope.block.blockType) || $scope.block.blockTypeItemId == $scope.block.blockType.id) {
            return;
        }
        
        $scope.block.alternateBlockReason = $scope.block.blockType.displayName;
        $scope.$apply();
    }

    function createNewBlock(date, startTime) {
        $scope.block = {
            startDate: date
        };
        
        if (startTime != undefined) {
            $scope.block.startTimeTemp = startTime.getTempTime($rootScope.isAmPmTime);
            var endTime = moment(startTime).add(15, 'minutes');
            $scope.block.endTime = endTime.getTempTime($rootScope.isAmPmTime); 
        }
    }

    $scope.saveBlock = function () {
        if (!isObject($scope.block.blockType)) {
            $rootScope.$broadcast("showMessage", { message: "Select a block type" });
            return;
        }

        var startTime = moment().getTimeFromTempTime($scope.block.startTimeTemp, $rootScope.isAmPmTime);
        var endTime = moment().getTimeFromTempTime($scope.block.endTime, $rootScope.isAmPmTime);

        if (endTime.diff(startTime) <= 0) {
            $rootScope.$broadcast("showMessage", { message: "End time must be greater than start time" });
            return;
        }

        var affectedColumns = [];

        $scope.block.employeeGlobalId = $scope.block.employee.id;
        $scope.block.blockTypeItemId = $scope.block.blockType.id;
        $scope.block.endDate = moment($scope.block.startDate).format();

        
        $scope.block.totalMinutes = (endTime.hours() - startTime.hours()) * 60 + (endTime.minutes() - startTime.minutes());
        $scope.block.startTime = startTime.format();
        if ($scope.block.appointBlockId != undefined && ($scope.block.employeeGlobalId != $scope.block.originalEmployeeGlobalId ||
            $scope.block.originalStartDate != $scope.block.startDate)) {
            affectedColumns.push({ date: $scope.block.originalStartDate, employeeGlobalId: $scope.block.originalEmployeeGlobalId });
        }
        affectedColumns.push({ date: $scope.block.startDate, employeeGlobalId: $scope.block.employeeGlobalId });
        $scope.block.startDate = moment($scope.block.startDate).format();

        $scope.$parent.showProcessingOverlay();
        restServer.saveAppointBlock($scope.block, function (response) {
            $scope.$parent.hideProcessingOverlay();
            $rootScope.$broadcast("blockSaved", { affectedColumns: affectedColumns });
        });
    }

    $scope.cancel = function () {
        if (blockId != undefined) {
            restServer.deleteLock(blockId, "AppointBlock");
        }
        $rootScope.$broadcast("cancelBlockSave");
    }


    $scope.getEmployees = function(filter) {
        return restServer.LoadFilteredEmployees(filter).then(function(response) {
            return response.data;
        });
    };

    $scope.getBlockTypes = function (filter) {
        return restServer.LoadFilteredBlockTypes(filter).then(function (response) {
            return response.data;
        });
    }
    $scope.showStartDatePicker = function () {
        $('#startDate').focus();
    };
}]);