app.controller('ClientEditorController', ['$scope', '$http', 'Enums', 'restServer', '$rootScope', function ($scope, $http, enums, restServer, $rootScope) {

    if ($rootScope.culture === 'en-gb' || $rootScope.culture === "en-au") {
        $scope.stateLabel = "County";
        $scope.postalCodeLabel = "Post Code";
    } else if ($rootScope.culture === "en-ca") {
        $scope.stateLabel = "Province";
        $scope.postalCodeLabel = "Postal Code";
    } else {
        $scope.stateLabel = "State";
        $scope.postalCodeLabel = "Zip Code";
    }
    var unselectedYear = 1804;
    $scope.copyOfClient = {};
    
    $scope.emailTypes = enums.GetEnumList(enums.EmailTypeEnum);
    $scope.phoneTypes = enums.GetEnumList(enums.PhoneTypeEnum);
    $scope.addressTypes = enums.GetEnumList(enums.AddressTypeEnum);
    $scope.genders = enums.GetEnumList(enums.GenderEnum);
    $scope.visitNoteTypes = [];

    $scope.months = [ { value: 1, displayName: "January"}, { value: 2, displayName: "February"}, { value: 3, displayName: "March"}, { value: 4, displayName: "April"}, { value: 5, displayName: "May"}, { value: 6, displayName: "June"}, { value: 7, displayName: "July"}, { value: 8, displayName: "August"}, { value: 9, displayName: "September"}, { value: 10, displayName: "October"}, { value: 11, displayName: "November"}, { value: 12, displayName: "December"}];
    
    var GetYearsList = function() {
        var currentYear = new Date().getFullYear();
        var year = currentYear - 100;
        var years = [];

        while (year <= currentYear) {
            years.push({value: year, displayName: "" + year});
            year++;
        };

        return years;
    };

    $scope.years = GetYearsList();

    function getDaysInMonth(month, year) {
        var date = new Date(year, month, 1);
        var days = [];
        while (date.getMonth() === month) {
            var dateNumber = new Date(date).getDate();
            days.push({ value: dateNumber, displayName: "" + dateNumber });
            date.setDate(date.getDate() + 1);
        }
        return days;
    }

    $scope.dates = [];


    $scope.selectedSection = "General";
    $scope.sections = ["General", "Visit Notes"];

    $scope.editMode = true;

    
    function createNewClient() {
        $scope.selections = {};
        $scope.client = {};
        $scope.$apply();
    }

    var addEditClientArgs;
    $scope.$on("addEditClient", function (event, args) {
        addEditClientArgs = args;
        if (args.clientId != undefined) {
            $scope.selections = {};
            restServer.getClient(args.clientId, onGetClientSuccess);
        } else {
            createNewClient();
        }
    });

    $scope.FixPhoneNumberField = function() {
        angular.element(document).ready(function () {

            $('#phoneNumberField').unbind('keyup.characterFilter');

            $('#phoneNumberField').on('keyup.characterFilter', (function(e) {
                if (/\D/g.test(this.value)) {
                    // Filter non-digits from input value.
                    this.value = this.value.replace(/\D/g, '');
                }
            }));
        });
    }

    $scope.SetGender = function (genderEnum) {
        $scope.client.genderEnum = genderEnum;
    };

    $scope.SetPhoneType = function(phoneTypeEnum) {
        $scope.client.phoneTypeEnum = phoneTypeEnum;
    };

    $scope.SetEmailType = function(emailTypeEnum) {
        $scope.client.emailTypeEnum = emailTypeEnum;
    };

    $scope.SetAddressType = function(addressTypeEnum) {
        $scope.client.addressTypeEnum = addressTypeEnum;
    };

    $scope.ClearBirthday = function() {
        $scope.selections.selectedMonth = undefined;
        $scope.selections.selectedDate = undefined;
        $scope.selections.selectedYear = undefined;
        $scope.UpdateDates();
    }

    $scope.ClearPhoneNumber = function() {
        $scope.selections.selectedPhoneType = undefined;
        $scope.client.phoneTypeEnum = 0;
        $scope.client.phoneNumber = "";
    };

    $scope.ClearAddress = function () {
        $scope.selections.selectedAddressType = undefined;
        $scope.client.address1 = '';
        $scope.client.address2 = '';
        $scope.client.city = '';
        $scope.client.state = '';
        $scope.client.zipCode = '';
        $scope.client.country = '';
    };

    $scope.ClearEmail = function() {
        $scope.selections.selectedEmailType = undefined;
        $scope.client.emailTypeEnum = 0;
        $scope.client.emailAddress = "";
    };

    $scope.UpdateDates = function () {
        var dateList;
        if ($scope.selections.selectedYear != undefined) {
            dateList = getDaysInMonth($scope.selections.selectedMonth.value - 1, $scope.selections.selectedYear.value);
        } else {
            dateList = getDaysInMonth($scope.selections.selectedMonth.value - 1, unselectedYear);
        }

        if ($scope.dates.length !== dateList.length) {
            $scope.dates = dateList;
            $scope.selectedDate = undefined;
        };
    };


    $scope.SetMonth = function (month) {
        if (isObject(month)) {
            $scope.selections.selectedMonth = month;
            
            $scope.UpdateDates();
        } else {
            $scope.selections.selectedMonth = undefined;
            
        }
    };

    $scope.SetYear = function (year) {
        if (isObject(year)) {
            $scope.selections.selectedYear = year;
           
            $scope.UpdateDates();
        } else {
            $scope.selections.selectedYear = undefined;
        }
    };

    $scope.SetDate = function (date) {
        if (isObject(date)) {
            $scope.selections.selectedDate = date;
        } else {
            $scope.selections.selectedDate = undefined;

        }
    };


    var setTypeSelects = function() {
        // Set Selected Gender to Current Gender
        for (var g = 0; g < $scope.genders.length; g++) {
            if ($scope.client.genderEnum === $scope.genders[g].value) {
                $scope.selections.selectedGender = $scope.genders[g];
                break;
            }
        };

        // Set Selected Email Type to Current Type
        for (var e = 0; e < $scope.emailTypes.length; e++) {
            if ($scope.client.emailTypeEnum === $scope.emailTypes[e].value) {
                $scope.selections.selectedEmailType = $scope.emailTypes[e];
                break;
            }
        };

        // Set Selected Phone Type to Current Type
        for (var p = 0; p < $scope.phoneTypes.length; p++) {
            if ($scope.client.phoneTypeEnum === $scope.phoneTypes[p].value) {
                $scope.selections.selectedPhoneType = $scope.phoneTypes[p];
                break;
            }
        };

        // Set Selected Address Type to Current Type
        for (var a = 0; a < $scope.addressTypes.length; a++) {
            if ($scope.client.addressTypeEnum === $scope.addressTypes[a].value) {
                $scope.selections.selectedAddressType = $scope.addressTypes[a];
                break;
            }
        };
    }

    var setBirthdaySelects = function () {
        var birthDate = moment($scope.client.birthDate);
        // Set Selected Birthday Month to Current Birthday Month
        for (var m = 0; m < $scope.months.length; m++) {
            if (birthDate.month()+1 === $scope.months[m].value) {
                $scope.selections.selectedMonth = $scope.months[m];
                break;
            }  
        };


        // Set Selected Birthday Year to Current Birthday Year
        for (var y = 0; y < $scope.years.length; y++) {
            if (birthDate.year() === $scope.years[y].value) {
                $scope.selections.selectedYear = $scope.years[y];
                break;
            }
        };
        
        $scope.UpdateDates();

        // Set Selected Birthday Date to Current Birthday Date
        for (var d = 0; d < $scope.dates.length; d++) {
            if (birthDate.date() === $scope.dates[d].value) {
                $scope.selections.selectedDate = $scope.dates[d];
                break;
            }
        };

    };

    var clearForm = function() {
        $scope.client = {};
        $scope.copyOfClient = {};
        $scope.selections = {};

        $scope.emailTypes = enums.GetEnumList(enums.EmailTypeEnum);
        $scope.phoneTypes = enums.GetEnumList(enums.PhoneTypeEnum);
        $scope.addressTypes = enums.GetEnumList(enums.AddressTypeEnum);
        $scope.genders = enums.GetEnumList(enums.GenderEnum);
        //$scope.visitNoteTypes = [];

        $scope.months = [{ value: 1, displayName: "January" }, { value: 2, displayName: "February" }, { value: 3, displayName: "March" }, { value: 4, displayName: "April" }, { value: 5, displayName: "May" }, { value: 6, displayName: "June" }, { value: 7, displayName: "July" }, { value: 8, displayName: "August" }, { value: 9, displayName: "September" }, { value: 10, displayName: "October" }, { value: 11, displayName: "November" }, { value: 12, displayName: "December" }];
        $scope.years = GetYearsList();
        $scope.dates = [];

        $scope.selectedSection = "General";
        $scope.sections = ["General", "Visit Notes"];

        $scope.editMode = true;
    };

    var onSaveClientSuccess = function (response) {
        $scope.client.clientGlobalId = response.clientGlobalId;
        addEditClientArgs.client = $scope.client;
        $rootScope.$broadcast("clientSaved", addEditClientArgs);
        clearForm();
    };

    var onGetClientSuccess = function(response) {
        $scope.client = response;
        
        setTypeSelects();
        if (response.birthDate != undefined) {
            setBirthdaySelects();
        }
    };

    var onGetVisitNoteTypesSuccess = function(response) {
        $scope.visitNoteTypes = response;
    };


    $scope.RequireBirthday = function() {
        return ($scope.selections.selectedMonth != undefined || $scope.selections.selectedDate != undefined || $scope.selections.selectedYear != undefined)
            && ($scope.selections.selectedMonth == undefined || $scope.selections.selectedDate == undefined);
    }

    $scope.RequireEmailFields = function() {
        if ($scope.client !== undefined && $scope.client !== null) {
            if (

                ($scope.client.emailTypeEnum === 0 || $scope.client.emailTypeEnum === undefined || $scope.client.emailTypeEnum === null) &&
                ($scope.client.emailAddress === '' || $scope.client.emailAddress === undefined || $scope.client.emailAddress === null)
            ) {
                return false;
            } else {
                return true;
            }

        } else {
            return false;
        }
    };

    $scope.RequirePhoneFields = function () {
        
        if ($scope.client !== undefined && $scope.client !== null) {
            if (isPhoneRequired() && $scope.client.phoneTypeEnum == undefined) {
                return true;
            }

            if (
                ($scope.client.phoneTypeEnum === 0 || $scope.client.phoneTypeEnum === undefined || $scope.client.phoneTypeEnum === null) &&
                ($scope.client.phoneNumber === '' || $scope.client.phoneNumber === undefined || $scope.client.phoneNumber === null)
            ) {
                return false;
            } else {
                return true;
            }

        } else {
            return false;
        }
    };

    function isPhoneRequired() {
        var setting = $scope.getSetting(enums.SettingEnum.RequireClientPhoneNumber);
        if (setting != null) {
            return setting.toBoolean();
        } else {
            return false;
        }
    }

    $scope.RequireAddressFields = function() {
        if ($scope.client !== undefined && $scope.client !== null) {

            // If they selected an address type require address fields
            if (($scope.client.addressType !== '' && $scope.client.addressType !== undefined && $scope.client.addressType !== null)) {
                return true;
            }

            // If any of the address fields are filled in require the mandatory address fields
            else if
            (
                ($scope.client.address1 !== '' && $scope.client.address1 !== undefined && $scope.client.address1 !== null) ||
                ($scope.client.address2 !== '' && $scope.client.address2 !== undefined && $scope.client.address2 !== null) ||
                ($scope.client.city !== '' && $scope.client.city !== undefined && $scope.client.city !== null) ||
                ($scope.client.state !== '' && $scope.client.state !== undefined && $scope.client.state !== null) ||
                ($scope.client.zipCode !== '' && $scope.client.zipCode !== undefined && $scope.client.zipCode !== null) ||
                ($scope.client.country !== '' && $scope.client.country !== undefined && $scope.client.country !== null)
            ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    $scope.Edit = function () {
        $scope.copyOfClient = _.clone($scope.client);
        $scope.editMode = true;
    };

    $scope.Cancel = function() {
        $rootScope.$broadcast("editClientCanceled");
    };

    $scope.SaveClient = function (clientForm) {
        if (clientForm.$valid) {

            if ($scope.selections.selectedMonth != undefined) {
                var birthDate = moment();
                birthDate.month($scope.selections.selectedMonth.value - 1);
                birthDate.date($scope.selections.selectedDate.value);
                birthDate.year($scope.selections.selectedYear != undefined ? $scope.selections.selectedYear.value : unselectedYear);
                $scope.client.birthDate = birthDate;
            } else {
                $scope.client.birthDate = undefined;
            }
            restServer.saveClient($scope.client, onSaveClientSuccess);
        } else {
            if (clientForm.firstName.$invalid) {
                    $rootScope.$broadcast("showMessage", { message: "First Name is required to save a client." });
                    return;
            }

            if (clientForm.lastName.$invalid) {
                    $rootScope.$broadcast("showMessage", { message: "Last Name is required to save a client." });
                    return;   
            }

            if (clientForm.gender.$invalid) {
                    $rootScope.$broadcast("showMessage", { message: "Gender is required to save a client." });
                    return;
            }

            if (clientForm.emailType.$invalid || clientForm.emailAddress.$invalid) {
                $rootScope.$broadcast("showMessage", { message: "An email type, and email address is required to save an email." });
                return;
            }

            if (clientForm.phoneType.$invalid || clientForm.phoneNumber.$invalid) {
                if (isPhoneRequired()) {
                    $rootScope.$broadcast("showMessage", { message: "You must enter a phone number to save this client." });
                } else {
                    $rootScope.$broadcast("showMessage", { message: "A phone type, and phone number is required to save a phone number." });
                }

                return;
            }

            if (clientForm.address1.$invalid || clientForm.city.$invalid || clientForm.state.$invalid || clientForm.zipCode.$invalid || clientForm.addressType.$invalid) {
                $rootScope.$broadcast("showMessage", { message: "An address line, city, state, and postal code is required to save an address." });
                return;
            }
        }
    };

    $scope.Refresh = function() {
        $scope.getClient($scope.clientGlobalId, onGetClientSuccess);

    };

    restServer.getVisitNoteTypes(onGetVisitNoteTypesSuccess);
}]);