app.controller('ClientEditorv2Controller', ['$scope', '$http', 'Enums', 'restServer', function ($scope, $http, enums, restServer) {
    this.client = {};
    this.copyOfClient = {};

    this.emailTypes = enums.GetEnumList(enums.EmailTypeEnum);
    this.phoneTypes = enums.GetEnumList(enums.PhoneTypeEnum);
    this.addressTypes = enums.GetEnumList(enums.AddressTypeEnum);
    this.genders = enums.GetEnumList(enums.GenderEnum);
    this.visitNoteTypes = [];

    this.months = [ { value: 1, displayName: "January"}, { value: 2, displayName: "February"}, { value: 3, displayName: "March"}, { value: 4, displayName: "April"}, { value: 5, displayName: "May"}, { value: 6, displayName: "June"}, { value: 7, displayName: "July"}, { value: 8, displayName: "August"}, { value: 9, displayName: "September"}, { value: 10, displayName: "October"}, { value: 11, displayName: "November"}, { value: 12, displayName: "December"}];
    
    var GetYearsList = function() {
        var currentYear = new Date().getFullYear();
        var year = currentYear - 100;
        var years = [];

        while (year <= currentYear) {
            years.push({value: year, displayName: "" + year});
            year++;
        };

        return years;
    };

    this.years = GetYearsList();

    function getDaysInMonth(month, year) {
        var date = new Date(year, month, 1);
        var days = [];
        while (date.getMonth() === month) {
            var dateNumber = new Date(date).getDate();
            days.push({ value: dateNumber, displayName: "" + dateNumber });
            date.setDate(date.getDate() + 1);
        }
        return days;
    }

    this.dates = [];


    this.selectedSection = "General";
    this.sections = ["General", "Visit Notes"];

    this.selectedVisitNoteType = {};

    this.editMode = true;

    this.selectedGender = {};
    this.selectedPhoneType = {};
    this.selectedEmailType = {};
    this.selectedAddressType = {};
    this.selectedMonth = {};
    this.selectedDate = {};
    this.selectedYear = {};


    var addEditClientArgs;
    this.$on("addEditClient", function (event, args) {
        addEditClientArgs = args;
        if (args.clientId != undefined) {
            restServer.getClient(args.clientId, onGetClientSuccess);
        }
    });

    this.SetGender = function(genderEnum) {
        this.client.genderEnum = genderEnum;
    };

    this.SetPhoneType = function(phoneTypeEnum) {
        this.client.phoneTypeEnum = phoneTypeEnum;
    };

    this.SetEmailType = function(emailTypeEnum) {
        this.client.emailTypeEnum = emailTypeEnum;
    };

    this.SetAddressType = function(addressTypeEnum) {
        this.client.addressTypeEnum = addressTypeEnum;
    };

    this.ClearPhoneNumber = function() {
        this.selectedPhoneType = undefined;
        this.client.phoneTypeEnum = 0;
        this.client.phoneNumber = "";
    };

    this.ClearAddress = function() {
    };

    this.ClearEmail = function() {
        this.selectedEmailType = undefined;
        this.client.emailTypeEnum = 0;
        this.client.emailAddress = "";
    };

    this.UpdateDates = function () {

        var dateList;
        if (this.client.birthdayYear !== undefined) {
            dateList = getDaysInMonth(this.client.birthdayMonth-1, this.client.birthdayYear);
        } else {
            dateList = getDaysInMonth(this.client.birthdayMonth-1, 1804);
        }

        if (this.dates.length !== dateList.length) {
            this.dates = dateList;
            this.selectedDate = undefined;
            this.client.birthdayDate = 0;
        };
    };


    this.SetMonth = function (month) {
        if (isObject(month)) {
            this.selectedMonth = month;
            this.client.birthdayMonth = month.value;
            this.UpdateDates();
        } else {
            this.selectedMonth = undefined;
            this.client.birthdayMonth = 0;
        }
    };

    this.SetYear = function (year) {
        if (isObject(year)) {
            this.selectedYear = year;
            this.client.birthdayYear = year.value;
            this.UpdateDates();
        } else {
            this.selectedYear = undefined;
            this.client.birthdayYear = 1;
        }
    };

    this.SetDate = function (date) {
        if (isObject(date)) {
            this.selectedDate = date;
            this.client.birthdayDate = date.value;
        } else {
            this.selectedDate = undefined;
            this.client.birthdayDate = 0;
        }
    };


    var setTypeSelects = function() {
        // Set Selected Gender to Current Gender
        for (var g = 0; g < this.genders.length; g++) {
            if (this.client.genderEnum === this.genders[g].value) {
                this.selectedGender = this.genders[g];
                break;
            }
        };

        // Set Selected Email Type to Current Type
        for (var e = 0; e < this.emailTypes.length; e++) {
            if (this.client.emailTypeEnum === this.emailTypes[e].value) {
                this.selectedEmailType = this.emailTypes[e];
                break;
            }
        };

        // Set Selected Phone Type to Current Type
        for (var p = 0; p < this.phoneTypes.length; p++) {
            if (this.client.phoneTypeEnum === this.phoneTypes[p].value) {
                this.selectedPhoneType = this.phoneTypes[p];
                break;
            }
        };

        // Set Selected Address Type to Current Type
        for (var a = 0; a < this.addressTypes.length; a++) {
            if (this.client.addressTypeEnum === this.addressTypes[a].value) {
                this.selectedAddressType = this.addressTypes[a];
                break;
            }
        };
    }

    var setBirthdaySelects = function() {
        // Set Selected Birthday Month to Current Birthday Month
        for (var m = 0; m < this.months.length; m++) {
            if (this.client.birthdayMonth === this.months[m].value) {
                this.selectedMonth = this.months[m];
                break;
            }  
        };

        // Set Selected Birthday Date to Current Birthday Date
        for (var d = 0; d < this.dates.length; d++) {
            if (this.client.birthdayDate === this.dates[d].value) {
                this.selectedDate = this.dates[d];
                break;
            }
        };

        // Set Selected Birthday Year to Current Birthday Year
        for (var y = 0; y < this.years.length; y++) {
            if (this.client.birthdayYear === this.years[y].value) {
                this.selectedYear = this.years[y];
                break;
            }
        };
    };

    var clearForm = function() {
        this.client = {};
        this.copyOfClient = {};
        this.selectedGender = {};
        this.selectedPhoneType = {};
        this.selectedEmailType = {};
        this.selectedAddressType = {};
        this.selectedMonth = {};
        this.selectedDate = {};
        this.selectedYear = {};

        this.emailTypes = enums.GetEnumList(enums.EmailTypeEnum);
        this.phoneTypes = enums.GetEnumList(enums.PhoneTypeEnum);
        this.addressTypes = enums.GetEnumList(enums.AddressTypeEnum);
        this.genders = enums.GetEnumList(enums.GenderEnum);
        this.visitNoteTypes = [];

        this.months = [{ value: 1, displayName: "January" }, { value: 2, displayName: "February" }, { value: 3, displayName: "March" }, { value: 4, displayName: "April" }, { value: 5, displayName: "May" }, { value: 6, displayName: "June" }, { value: 7, displayName: "July" }, { value: 8, displayName: "August" }, { value: 9, displayName: "September" }, { value: 10, displayName: "October" }, { value: 11, displayName: "November" }, { value: 12, displayName: "December" }];
        this.years = GetYearsList();
        this.dates = [];

        this.selectedSection = "General";
        this.sections = ["General", "Visit Notes"];

        this.selectedVisitNoteType = {};

        this.editMode = true;
    };

    var onSaveClientSuccess = function (response) {
        this.client.clientGlobalId = response.clientGlobalId;
        addEditClientArgs.client = this.client;
        $rootScope.$broadcast("clientSaved", addEditClientArgs);
        clearForm();
    };

    var onGetClientSuccess = function(response) {
        this.client = response;
        setTypeSelects();
        setBirthdaySelects();
    };

    var onGetVisitNoteTypesSuccess = function(response) {
        this.visitNoteTypes = response;
    };


    this.RequireEmailFields = function() {
        if (this.client !== undefined && this.client !== null) {
            if (
                (this.client.emailTypeEnum === 0) &&
                (this.client.emailAddress === '' || this.client.emailAddress === undefined || this.client.emailAddress === null)
            ) {
                return false;
            } else {
                return true;
            }

        } else {
            return false;
        }
    };

    this.RequirePhoneFields = function () {
        if (this.client !== undefined && this.client !== null) {
            if (
                (this.client.phoneNumberType === 0 || this.client.phoneNumberType === undefined || this.client.phoneNumberType === null) &&
                (this.client.phoneNumber === '' || this.client.phoneNumber === undefined || this.client.phoneNumber === null)
            ) {
                return false;
            } else {
                return true;
            }

        } else {
            return false;
        }
    };

    this.RequireAddressFields = function() {
        if (this.client !== undefined && this.client !== null) {

            // If they selected an address type require address fields
            if ((this.client.addressType !== '' && this.client.addressType !== undefined && this.client.addressType !== null)) {
                return true;
            }

            // If any of the address fields are filled in require the mandatory address fields
            else if
            (
                (this.client.address1 !== '' && this.client.address1 !== undefined && this.client.address1 !== null) ||
                (this.client.address2 !== '' && this.client.address2 !== undefined && this.client.address2 !== null) ||
                (this.client.city !== '' && this.client.city !== undefined && this.client.city !== null) ||
                (this.client.state !== '' && this.client.state !== undefined && this.client.state !== null) ||
                (this.client.zipCode !== '' && this.client.zipCode !== undefined && this.client.zipCode !== null) ||
                (this.client.country !== '' && this.client.country !== undefined && this.client.country !== null)
            ) {
                return true;
            } else {
                return false;
            }
        } else {
            return false;
        }
    };

    this.Edit = function () {
        this.copyOfClient = _.clone(this.client);
        this.editMode = true;
    };

    this.Cancel = function() {
        $rootScope.$broadcast("editClientCanceled");
    };

    this.SaveClient = function () {
        restServer.saveClient(this.client, onSaveClientSuccess);
    };

    this.Refresh = function() {
        this.getClient(this.clientGlobalId, onGetClientSuccess);
    };

    restServer.getVisitNoteTypes(onGetVisitNoteTypesSuccess);
}]);