﻿app.controller('StandingController', [
    '$scope', '$http', 'restServer', '$rootScope', 'Enums', function ($scope, $http, restServer, $rootScope, enums) {
        var appointStandingDefinitionId;
        var defaultAppType;
        $scope.addDetail = function () {
            $scope.appointStandingDefinition.appointStandingDefinitionDetails.push({
                stateEnum: enums.StateEnum.Add,
                appointType: defaultAppType
            });
        }

        $scope.$on("addStanding", function(event, args) {
            $scope.appointStandingDefinition = {
                client: undefined,
                appointCat: undefined,
                schedule: {
                    frequencyTypeEnum: enums.StandingFrequenceyTypeEnum.Weekly,
                    startDate: moment(),
                    endDate: moment().add(6, 'd'),
                    weeklyIsSunday: false,
                    weeklyIsMonday: false,
                    weeklyIsTuesday: false,
                    weeklyIsWednesday: false,
                    weeklyIsThursday: false,
                    weeklyIsFriday: false,
                    weeklyIsSaturday: false,
                    frequency: 1,
                    numberOfOccurrences: 1
                },
                appointStandingDefinitionDetails: []
            };

            defaultAppType = args.repeatRequestAppointType;
            $scope.addDetail();
            $scope.tempStartTime = moment("9:00 AM", "h:m a").getTempTime($rootScope.isAmPmTime);
            updateFrequencyLabel();
        });

        $scope.$on("editStanding", function (event, args) {
            appointStandingDefinitionId = args.appointStandingDefinitionId;
            restServer.getAppointStandingDefinition(args.appointStandingDefinitionId, function (response) {
                response.client = {
                    id: response.clientGlobalId,
                    firstName: response.clientGlobalFirstName,
                    lastName: response.clientGlobalLastName
                };
                if (response.appointCatItemCatId != undefined) {
                    response.appointCat = {
                        id: response.appointCatItemCatId,
                        displayName: response.appointCatItemCatDisplayName,
                        shortcut: response.appointCatItemCatShortcut
                    }
                };
                if (response.schedule.startDate != undefined) {
                    response.schedule.startDate = moment(response.schedule.startDate);
                }
                if (response.schedule.endDate != undefined) {
                    response.schedule.endDate = moment(response.schedule.endDate);
                }
                if (response.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Weekly) {
                    response.schedule.frequency = response.schedule.weeklyRecurringWeekFrequency;
                } else {
                    response.schedule.frequency = response.schedule.dailyRecurringFrequency;
                }
                var startTime = moment(response.schedule.startTime);
                $scope.tempStartTime = startTime.getTempTime($rootScope.isAmPmTime);
                response.appointStandingDefinitionDetails.forEach(function(detail) {
                    detail.stateEnum = enums.StateEnum.Edit;
                    detail.employee = {
                        id: detail.employeeGlobalId,
                        displayName: detail.employeeGlobalDisplayName
                    };
                    detail.service = {
                        id: detail.serviceItemId,
                        displayName: detail.serviceItemDisplayName,
                        shortcut: detail.serviceItemShortcut
                    };
                    if (detail.appointTypeItemId != undefined) {
                        detail.appointType = {
                            id: detail.appointTypeItemId,
                            displayName: detail.appointTypeDisplayName,
                            shortcut: detail.appointTypeShortcut
                        };
                    }
                });
                $scope.appointStandingDefinition = response;
                updateNumberOfOccurrences();
                updateFrequencyLabel();
            });
        });

        
        
        

        $scope.editClient = function () {
            if (isObject($scope.appointStandingDefinition.client)) {
                $rootScope.$broadcast("addEditClient", {
                    clientId: $scope.appointStandingDefinition.client.id,
                    mode: 1,
                    source: 2
                });
            }
        }

        $scope.addClient = function () {
            $rootScope.$broadcast("addEditClient", {
                mode: 2,
                source: 2
            });
        }

        $scope.$on("clientSaved", function (event, args) {
            if (args.source==2 && args.mode == 2) {
                $scope.appointStandingDefinition.client = {
                    lastName: args.client.lastName,
                    firstName: args.client.firstName,
                    id: args.client.clientGlobalId
                }
            }
        });

        $scope.removeDetail = function (index) {
            var detail = $scope.appointStandingDefinition.appointStandingDefinitionDetails[index];
            if (detail.stateEnum == enums.StateEnum.Add) {
                $scope.appointStandingDefinition.appointStandingDefinitionDetails.splice(index, 1);
            } else {
                detail.stateEnum = enums.StateEnum.Delete;
            }
        }

        $scope.filterDetails = function (detail) {
            return detail.stateEnum == enums.StateEnum.Add || detail.stateEnum == enums.StateEnum.Edit;
        }

        $scope.getClients = function (filter) {
            return restServer.LoadFilteredClients(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.getServices = function (filter) {
            return restServer.LoadFilteredServices(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.getAppointTypes = function (filter) {
            return restServer.LoadFilteredAppointTypes(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.getEmployees = function (filter) {
            return restServer.LoadFilteredEmployees(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.getAppointCats = function (filter) {
            return restServer.LoadFilteredAppointCats(filter).then(function (response) {
                return response.data;
            });
        }

        $scope.save = function () {
            if (isValid()) {
                $scope.appointStandingDefinition.clientGlobalId = $scope.appointStandingDefinition.client.id;
                if ($scope.appointStandingDefinition.appointCat != undefined && $scope.appointStandingDefinition.appointCat.id != undefined) {
                    $scope.appointStandingDefinition.appointCatItemCatId = $scope.appointStandingDefinition.appointCat.id;
                }
                var appointOrder = 1;
                $scope.appointStandingDefinition.appointStandingDefinitionDetails.forEach(function(detail) {
                    detail.serviceItemId = detail.service.id;
                    detail.employeeGlobalId = detail.employee.id;
                    if (detail.appointType != undefined && detail.appointType.id != undefined) {
                        detail.appointTypeItemId = detail.appointType.id;
                    }
                    detail.appointOrder = appointOrder++;
                });
                var schedule = $scope.appointStandingDefinition.schedule;
                schedule.startTime = moment().getTimeFromTempTime($scope.tempStartTime, $rootScope.isAmPmTime).format();
                if (schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Weekly) {
                    schedule.weeklyRecurringWeekFrequency = schedule.frequency;
                    schedule.dailyRecurringFrequency = 1;
                } else {
                    schedule.weeklyRecurringWeekFrequency = 1;
                    schedule.dailyRecurringFrequency = schedule.frequency;
                }
                schedule.startDate = moment(schedule.startDate).format();
                schedule.endDate = moment(schedule.endDate).format();

                $scope.$parent.showProcessingOverlay();
                restServer.saveAppointStandingDefinition($scope.appointStandingDefinition, function () {
                    $scope.$parent.hideProcessingOverlay();
                    $scope.$emit("standingSaved");
                });
            }
            
        }

        function isValid() {
            var standing = $scope.appointStandingDefinition;
           
            var message = '';
            var isEmployeeError = false;
            var isServiceError = false;

            if (standing.client == undefined || standing.client.id == undefined) {
                message += '<li>Client is required</li>';
            }

            standing.appointStandingDefinitionDetails.forEach(function (detail) {
                if (!isEmployeeError && (detail.employee == undefined || detail.employee.id == undefined)) {
                    message += '<li>Employee is required for all details</li>';
                    isEmployeeError = true;
                }
                if (!isServiceError && (detail.service == undefined || detail.service.id == undefined)) {
                    message += '<li>Service is required for all details</li>';
                    isServiceError = true;
                }
            });

            if (standing.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Weekly && !standing.schedule.weeklyIsSunday &&
                !standing.schedule.weeklyIsMonday && !standing.schedule.weeklyIsTuesday && !standing.schedule.weeklyIsWednesday && !standing.schedule.weeklyIsThursday
                && !standing.schedule.weeklyIsFriday && !standing.schedule.weeklyIsSaturday) {
                message += '<li>You must select at least one day when using a weekly schedule</li>';
            }
            
            if (moment(standing.schedule.endDate) < moment(standing.schedule.startDate)) {
                message += "<li>End date cannot precede the start date</li>";
            }

            if (message != '') {
                message = "<ul>" + message + "</ul>";
                $rootScope.$broadcast("showMessage", { message: message });
                return false;
            }
            return true;
        }

        $scope.numberOccurrencesChanged = function() {
            var val = $scope.appointStandingDefinition.schedule.numberOfOccurrences;
            if (!isNaN(parseInt(val)) && val >= 1){
                if ($scope.appointStandingDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                    $scope.appointStandingDefinition.schedule.endDate = moment($scope.appointStandingDefinition.schedule.startDate).add(val - 1, 'd');
                } else {
                    $scope.appointStandingDefinition.schedule.endDate = moment($scope.appointStandingDefinition.schedule.startDate).add(val * 7 * $scope.appointStandingDefinition.schedule.frequency - 1, 'd');
                }
            }
        }

        $scope.frequencyChanged = function() {
            updateNumberOfOccurrences();
            updateFrequencyLabel();
        }

        $scope.endDateChanged = function() {
            if (moment($scope.appointStandingDefinition.schedule.endDate) < moment($scope.appointStandingDefinition.schedule.startDate)) {
                $rootScope.$broadcast("showMessage", { message: "End date can't be less than start date" });
                return;
            }
            updateNumberOfOccurrences();
        }

        function updateNumberOfOccurrences() {
            if ($scope.appointStandingDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                $scope.appointStandingDefinition.schedule.numberOfOccurrences = Math.floor((moment($scope.appointStandingDefinition.schedule.endDate).diff(moment($scope.appointStandingDefinition.schedule.startDate), 'days') + 1) / $scope.appointStandingDefinition.schedule.frequency);
            } else {
                $scope.appointStandingDefinition.schedule.numberOfOccurrences = Math.ceil((moment($scope.appointStandingDefinition.schedule.endDate).diff(moment($scope.appointStandingDefinition.schedule.startDate), 'days') + 1) / (7 * $scope.appointStandingDefinition.schedule.frequency));
            }
            if ($scope.appointStandingDefinition.schedule.numberOfOccurrences < 1) {
                $scope.appointStandingDefinition.schedule.numberOfOccurrences = 1;
            }
        }

        function updateFrequencyLabel() {
            if ($scope.appointStandingDefinition.schedule.frequencyTypeEnum == enums.StandingFrequenceyTypeEnum.Daily) {
                $scope.frequencyLabel = "days";
            } else {
                $scope.frequencyLabel = "weeks";
            }
        }

        $scope.cancel = function () {
            if (appointStandingDefinitionId != undefined) {
                restServer.deleteLock(appointStandingDefinitionId, "AppointStandingDefinition");
            }
            $scope.$parent.hideStandingEditor();
        }

        $scope.showStartDatePicker = function () {
            $('#standingStartDate').focus();
        };

        $scope.showEndDatePicker = function () {
            $('#standingEndDate').focus();
        };
    }
]);