﻿app.directive('hideCurrentPopup', function() {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            var mouseUpHandler = function(e) {
                if (scope.currentPopupId != undefined) {
                    var container = $("#" + scope.currentPopupId);

                    if ((!container.is(e.target) // if the target of the click isn't the container...
                            && container.has(e.target).length === 0) // ... nor a descendant of the container
                        || $(e.target).is("button")) // or is a button within container
                    {
                        container.hide();
                        scope.currentPopupId = undefined;
                    }
                }
            }
            $(document).on("mouseup", mouseUpHandler);
        }
    }
});

app.directive('toggleSelected', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            element.bind('click', function () {
                element.toggleClass(attrs.toggleSelected);
            });
        }
    };
});

app.directive('singleSelect', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            if (attrs.defaultSelected.toBoolean()) {
                element.addClass(attrs.singleSelect);
            }
            element.bind('click', function () {
                $("." + attrs.singleSelect).removeClass(attrs.singleSelect);
                element.addClass(attrs.singleSelect);
            });
        }
    };
});

app.directive('autoSelect', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            if (scope.$parent.appointDetailIdsToDelete.indexOf(attrs.detailId) != -1) {
                element.addClass("divSelected");
            }
        }
    };
});

app.directive('typeaheadFocus', function() {
    return {
        require: 'ngModel',
        link: function(scope, element, attr, ngModel) {

            //trigger the popup on 'click' because 'focus'
            //is also triggered after the item selection
            element.bind('click', function() {

                var viewValue = ngModel.$viewValue;
                var modelValue = ngModel.$modelValue;

                //restore to null value so that the typeahead can detect a change
                if (ngModel.$viewValue == ' ') {
                    ngModel.$setViewValue(null);
                }

                //force trigger the popup
                ngModel.$setViewValue(' ');

                //set the actual value in case there was already a value in the input
                ngModel.$setViewValue(modelValue || ' ');
                
            });

            //compare function that treats the empty space as a match
            scope.emptyOrMatch = function(actual, expected) {
                if (expected == ' ') {
                    return true;
                }
                return actual.indexOf(expected) > -1;
            };
        }
    };
});

app.directive('typeaheadRequired', ['$parse', function($parse) {
    return {
        require: 'ngModel',
        link: function (scope, element, attr, ngModel) {
            var val = attr.typeaheadRequired;
            var isRequired = true;
            if (val != '') {
                isRequired = scope.$eval(val);
            }

            var modelValue = $parse(attr.ngModel);
            var lastLegitValue;
            scope.$watch(modelValue, function (value) {
                if (value == undefined) {
                    lastLegitValue = undefined;
                }
                else if (isObject(value)) {
                    lastLegitValue = value;
                }
            });
            
            element.bind('blur', function () {
                // If its required and there was an option selected but now its either been partially or completely cleared out, set it back to that option
                if (isRequired && !isObject(ngModel.$modelValue) && lastLegitValue != undefined) {
                    ngModel.$setViewValue(lastLegitValue);
                    ngModel.$render();
                    scope.$apply();
                }

                // If its not required and a previously option was selected and it was partially clearly out, set it back to that option
                // In order to clear out the selected option, the field nears to be emptied
                if (!isRequired && !isObject(ngModel.$modelValue) && ngModel.$viewValue != '' && lastLegitValue != undefined) {
                    ngModel.$setViewValue(lastLegitValue);
                    ngModel.$render();
                    scope.$apply();
                }
            });
        }
    }
}]);

app.directive('toggleButton', ['$parse', function ($parse) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            var setClass = function(isSelected) {
                if (isSelected) {
                    element.removeClass(attrs.unselectedClass);
                    element.addClass(attrs.selectedClass);
                } else {
                    element.removeClass(attrs.selectedClass);
                    element.addClass(attrs.unselectedClass);
                }
            }
            var model = $parse(attrs.toggleButton);

            scope.$watch(model, function (value) {
                setClass(value);
            });
            setClass(model(scope));
            element.bind('click', function () {
                model.assign(scope, !model(scope));
                setClass(model(scope));
            });
           
        }
    }
}]);

app.directive('scrollIf', function () {
    return function (scope, element, attrs) {
        scope.$watch(attrs.scrollIf, function (value) {
            if (value) {
                element.parent().scrollTop(0);
                var childOffsetInParent = element.offset().top - element.parent().offset().top;
                if (childOffsetInParent + element.height() > element.parent().height()) {
                    var scrollTop = element.height() - (element.parent().height() - childOffsetInParent);
                    element.parent().scrollTop(scrollTop);
                }
            }
        });
    }
});

app.directive('minInt', function () {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            element.bind('keydown', function (e) {
                var key = e.which || e.keyCode;

                if (!(key >= 48 && key <= 57) && // Interval (0-9)
                     (key !== 8) && // backspace
                    (key !== 46))              //delete 
                {
                    e.preventDefault();
                    return false;
                }
            });

            element.bind('blur', function(e) {
                if (element.val() < attrs.minInt) {
                    element.val(attrs.minInt);
                }
            });
        }
    }
});

app.directive('modalTypeAhead', ['$parse', 'restServer',function($parse, restServer) {
    return {
        restrict: 'A',
        link: function (scope, element, attr) {
            var template;
            var loadMethod;

            // Wrap input so that X button can be positioned relative to input and so that x button stays positioned with input when scrolling
            element.wrap("<div style='position: relative;width:100%'></div>");
            var input = $(element);
            var inputParent = input.parent();
            
            // Add X Button
            inputParent.append(
                "<button class='editor-client-btn clearFilter' tabindex='-1'></button>");

           // Add List and selectedItem containers
            $("<div class='listContainer'>" +
                "<div class='list'></div>" +
                "<div class='selectedItem typeaheadItem' style='displayNone'></div></div>").insertAfter(inputParent);

            var root = $(inputParent.parent());
            scope.restServer = restServer;

            var container = $(root.find(".listContainer"));
            var list = $(root.find(".list"));
            
            var selectedItemDisplay = $(root.find(".selectedItem"));
            var selectedIndex = 0;
            var previousIndex = 0;
            var model = $parse(attr.model);
            var clickHandler;
            var resizeHandler;
            var selectedItem;
            var data = [];
            var selectionChanged = false;
            var page = 0;
            var pageSize = 20;
            var hasMoreRecords = true;
            var isRequired = true;
            var placement;

            if (attr.modaltypeaheadRequired != '') {
                isRequired = scope.$eval(attr.modaltypeaheadRequired);
            }
            configure();

            scope.$watch(model, function (value) {

                var val = !jQuery.isEmptyObject(value) ? template(value): '';
                setInputVal(val);
                    selectedItemDisplay.html(jQuery.isEmptyObject(scope.$eval(attr.model)) ? '': template(scope.$eval(attr.model)));
            });
                
            function setInputVal(val) {
                input.val($("<textarea/>").html(val).val());
            }
            function select() {
                selectionChanged = true;
                setSelectedItem(data[selectedIndex]);
                hide();
            }

            function hide() {
                container.hide();
                $(document).off("click", clickHandler);
                $(window).off("resize", resizeHandler);
                
                // Clear the list to minimize dom impact especially in case where the list is long
                list.empty();
//                input.blur();

                if (selectionChanged) {
                    if (selectedItem != undefined) {
                        // Assign selecte item to model
                        model.assign(scope, selectedItem);
                    } else if (!isRequired) {
                        // Set model to undefined doesn't seem to work but this does
                        model.assign(scope, undefined);
                    } else {
                        setInputVal(jQuery.isEmptyObject(scope.$eval(attr.model)) ? '': template(scope.$eval(attr.model)));
                    }
                    scope.$apply();

                    if (attr.changed) {
                        scope.$eval(attr.changed);
                    }
                } else if (!isRequired && input.val() == "") {
                    model.assign(scope, undefined);
                }
                else // Just in case input text was changed without making selection, revert back to previous selected
                {
                    setInputVal(jQuery.isEmptyObject(scope.$eval(attr.model)) ? '': template(scope.$eval(attr.model)));
                }
                
            }

            function selectNext() {
                //console.log("Length:" + list.children().length + " Previous:" + previousIndex + " Current:" + selectedIndex)
                list.children().eq(previousIndex).toggleClass("typeaheadSelected");
                var current = list.children().eq(selectedIndex);
                current.toggleClass("typeaheadSelected");
                current.parent().scrollTop(0);
                var childOffsetInParent = current.offset().top - current.parent().offset().top;
                if (childOffsetInParent + current.outerHeight() > current.parent().outerHeight()) {
                    var scrollTop = current.outerHeight() - (current.parent().outerHeight() - childOffsetInParent);
                    current.parent().scrollTop(scrollTop);
                }
            }

            function doFilter(reset) {
                if (reset) {
                    container.width(input.width());
                    hasMoreRecords = true;
                    selectedIndex = 0;
                    page = 0;
                    repositionTop();
                } else {
                    page++;
                }

                if (hasMoreRecords) {
                    scope.$eval("restServer." + loadMethod + "('" + input.val().replace(/'/g, "\\'") + "'," + page + "," + pageSize + ")").then(function (response) {
                        if (response.data.length < pageSize) {
                            hasMoreRecords = false;
                        }
                        
                        if (reset)
                        {
                            data = [];
                            list.empty();
                        }

                        response.data.forEach(function (item) {
                            
                            var classString = 'typeaheadItem';
                            if (data.length == 0) {
                                classString += " typeaheadSelected";
                            }
                            var indexString = " data-index=" + data.length;
                            list.append("<div class='" +classString + "' " +indexString + " style=''>" + template(item) + "</div>");
                            
                            
                            
                            data.push(item);
                        });
                        repositionTop();
                    });
                    
                }
            }

            function setSelectedItem(val) {
                selectedItem = val;
                if (!jQuery.isEmptyObject(selectedItem)) {
                    selectedItemDisplay.show();
                    list.hide();
                } else {
                    selectedItemDisplay.hide();
                    list.show();
                }
            }

            function repositionTop() {
                var top;
                if (placement == "top") {
                    top = input.offset().top - container.outerHeight();
                } else {
                    top = input.offset().top + input.outerHeight();
                }

                container.css("top", top + "px");
            }

            function configure() {
                var addMethod;
                var editMethod;
                placement = attr.placement != undefined ? attr.placement.toLowerCase() : "bottom";
                var defaultTemplate = "{{shortcut}}: {{displayName}}";
                if (attr.modalTypeAhead == undefined) {
                    throw "Modal type not specificied";
                }
                switch (attr.modalTypeAhead.toLowerCase()) {
                case "client":
                    template = Handlebars.compile("{{lastName}}, {{firstName}}");
                    loadMethod = "LoadFilteredClients";
                    addMethod = "addClient";
                    editMethod = "editClient";
                    break;
                case "service":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadFilteredServices";
                    break;
                case "employee":
                    template = Handlebars.compile("{{displayName}}");
                    loadMethod = "LoadFilteredEmployees";
                    break;
                case "jobfunction":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadFilteredJobFunctions";
                    break;
                case "appointcat":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadFilteredAppointCats";
                    break;
                case "appointtype":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadFilteredAppointTypes";
                    break;
                case "blocktype":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadFilteredBlockTypes";
                    break;
                case "appointreason":
                    template = Handlebars.compile(defaultTemplate);
                    loadMethod = "LoadAppointReasons";
                    break;
                default:
                    throw "Modal typea ahead configuration exception: Modal type could not be found";
                }

                var hideActionButtons = attr.hideActionButtons != undefined && attr.hideActionButtons;

                if (!hideActionButtons) {
                    if (editMethod != undefined) {
                        container.append("<button style='margin-right:2px;' class='editTypeAhead' tabindex='-1'><span class='glyphicon glyphicon-pencil'></span></button>");
                        $(container.find(".editTypeAhead")).on("click", function () {
                            scope.$eval(editMethod + "()");
                        });
                    }
                    if (addMethod != undefined) {
                        container.append("<button class='addTypeAhead' tabindex='-1'><span class='glyphicon glyphicon-plus'></span></button>");
                        $(container.find(".addTypeAhead")).on("click", function () {
                            scope.$eval(addMethod + "()");
                        });
                    }
                }
            }

            input.on("input", function() {
                setSelectedItem(undefined);
                doFilter(true);
            });

            list.on("click", "div", function () {
                selectedIndex = $(this).data("index");
                selectNext();
                select();
            });

            input.on("focus", function () {
                if (container.is(":hidden")) {
                    repositionTop();
                    selectionChanged = false;
                    setSelectedItem(scope.$eval(attr.model));
                    
                    container.show();
                    clickHandler = function (e) {
                        if (!inputParent.is(e.target) // if the target of the click isn't the container...
                            && inputParent.has(e.target).length === 0
                            && !container.is(e.target) && container.has(e.target).length == 0) // ... nor a descendant of the container
                        {
                            hide();
                        }
                    }
                    resizeHandler = function() {
                        var top = input.offset().top + input.outerHeight();
                        container.css("top", top + "px");
                    }
                    $(document).on("click", clickHandler);
                    $(window).on("resize", resizeHandler);
                }
                doFilter(true);
            });

            input.on("keydown", function (e) {
                previousIndex = selectedIndex;
                switch (e.keyCode) {
                    case 40: // down arrow
                        if (list.children().length > 0 && previousIndex != list.children().length - 1) {


                            selectedIndex++;
                                // Get next page
                            if (selectedIndex == list.children().length - 1) {
                                doFilter(false);
                            }
                            selectNext();
                        }
                        break;
                    case 38: // up arrow
                        if (previousIndex != 0) {
                            selectedIndex--;
                            selectNext();
                        }

                        break;
                    case 13: // return
                        select(selectedIndex);
                        break;
                    case 9: // tab
                        hide();
                        break;
                }

            });

            

            $(inputParent.find(".clearFilter")).on("click", function () {
                input.val("");
                input.focus();
                selectionChanged = true;
                setSelectedItem(undefined);
            });

            list.on('scroll', function() {
                // Get next page
                if ($(this).scrollTop() + $(this).innerHeight() >= this.scrollHeight) {
                    doFilter(false);
                }
            });

           
        }
    }
}]);

app.directive('security', ['$rootScope','Enums', function ($rootScope, enums) {
    return {
        restrict: 'A',
        link: function (scope, element, attrs) {
            if ($rootScope.userSecurityAttribs == undefined) {
                scope.$on("userSecurityAttribsRetrieved", function () {
                    process();
                });
            } else {
                process();
            }

            function process() {
                for (var i = 0; i < $rootScope.userSecurityAttribs.length; i++) {
                    var secAttr = $rootScope.userSecurityAttribs[i];
                    if (secAttr.securityKey == attrs.security) {
                        if (secAttr.securityAllowEnum == enums.SecurityAllowEnum.Hidden) {
                            element.remove();
                        }
                        
                        return;
                    }
                }
            }
        }
    }
}]);

app.directive('alert', [
    'restServer', 'Enums', function(restServer, enums) {
        return {
            restrict: 'A',
            scope: {
                entity1Id: "@entity1Id",
                entity2Id: "@entity2Id"
            },
            template: "<div class='alertIcon' ng-class='getAlertIconClass(alerts)' ng-show='alerts.length>0'>{{alerts.length}}</div>" +
                "<div class='alertItems'><div ng-repeat='alert in alerts' class='alertItem' ng-class='getAlertItemClass(alert)'><p>{{alert.alertMessage}}</p></div></div>",
            link: function (scope, element, attrs) {
                var alertIcon = element.find(".alertIcon");
                var alertItems = element.find(".alertItems");

                alertIcon.on("click", function () {
                    if (alertItems.is(":hidden")) {
                        show();
                    } else {
                        hide();
                    }
                });

                var clickHandler = function (e) {
                    if (!alertIcon.is(e.target)) // if the target of the click isn't the icon
                    {
                        hide();
                    }
                }

                function hide() {
                    alertItems.hide();
                    $(document).off("click", clickHandler);
                }

                function show() {
                    alertItems.show();
                    $(document).on("click", clickHandler);
                }

                switch (attrs.alert.toLowerCase()) {
                    case "client":
                    case "clientnote":
                        scope.$watch("entity1Id", handleEntityChange);
                    break;
                    case "service":
                        scope.$watch("entity1Id", handleEntityChange);
                        scope.$watch("entity2Id", handleEntityChange);
                        break;
                    default:
                        throw "Alert context unknown";
                }
                function handleEntityChange() {
                    scope.alerts = undefined;
                    switch (attrs.alert.toLowerCase()) {
                        case "client":
                            if (scope.entity1Id != undefined && scope.entity1Id != "") {
                                restServer.getClientAlerts(scope.entity1Id, handleRestResponse);
                            }
                            break
                        case "clientnote":
                            if (scope.entity1Id != undefined && scope.entity1Id != "") {
                                restServer.getClientNoteAlerts(scope.entity1Id, handleRestResponse);
                            }
                            break
                        case "service":
                            if (scope.entity1Id != undefined && scope.entity1Id != "" && scope.entity2Id != undefined && scope.entity2Id != "") {
                                restServer.getServiceAlerts(scope.entity1Id, scope.entity2Id, handleRestResponse);
                            }
                            break;
                    }
                }

                function handleRestResponse(response) {
                    scope.alerts = response;
                }

                scope.getAlertItemClass = function (alertItem) {
                    var className = '';
                    if (attrs.alert.toLowerCase() == "clientnote") {
                        className = 'clientnote';
                    } else {
                        className = 'normal';
                        switch (alertItem.alertPriorityLevel) {
                        case enums.AlertPriorityLevelEnum.High:
                            className = 'high';
                            break;
                        case enums.AlertPriorityLevelEnum.Good:
                            className = 'good';
                            break;
                        }
                    }
                    return className;
                }

                scope.getAlertIconClass = function(alerts) {
                    var className = '';
                    if (alerts != undefined && alerts.length > 0) {
                        if (attrs.alert.toLowerCase() == "clientnote") {
                            className = 'clientnote';
                        } else {
                            var nonNormalAlerts = _.filter(alerts, function(alert) {
                                return alert.alertPriorityLevel == enums.AlertPriorityLevelEnum.High || alert.alertPriorityLevel == enums.AlertPriorityLevelEnum.Good;
                            });
                            var priorityLevel;
                            // Color of icon should be for high or good first and fallback to normal
                            if (nonNormalAlerts.length > 0) {
                                priorityLevel = nonNormalAlerts[0].alertPriorityLevel;
                            } else {
                                priorityLevel = enums.AlertPriorityLevelEnum.Normal;
                            }
                            switch (priorityLevel) {
                            case enums.AlertPriorityLevelEnum.High:
                                className = 'high';
                                break;
                            case enums.AlertPriorityLevelEnum.Normal:
                                className = 'normal';
                                break;
                            case enums.AlertPriorityLevelEnum.Good:
                                className = 'good';
                                break;
                            }
                        }
                    }
                    return className;
                }
            }
        }
    }
]);

app.directive("upDownInput", ['$timeout', function ($timeout) {
    return {
        restrict: 'E',
        template: '<button class="downarrow-btn" ng-click="decrease()">' +
            '</button><input class="form-control" ng-model="model" min-int="{{min}}" ng-change="changedLocal()" />' +
            '<button class="uparrow-btn" ng-click="increase()"></button>' +
                            '</div>',
        scope: {
            model: '=',
            min: '@',
            changed: '&'
        },
        link: function(scope, element, attrs) {
            scope.increase = function() {
                scope.model++;
                scope.changedLocal();
            }

            scope.decrease = function () {
                if (scope.min != undefined) {
                    if (scope.model == scope.min) {
                        return;
                    }
                }
                scope.model--;
                scope.changedLocal();
            }

            scope.changedLocal = function () {
                $timeout(function () {
                    scope.changed();
                });
            }
        }
    }
}]);

app.directive("testDirective", function() {
    return {
        restrict: 'E',
        template: '<div class="popupContainer"><div class="popupTable"><div class="popupCell" ng-transclude>' +
            '</div></div></div>',
        transclude: true
    }
});

app.directive("timePicker", ['$rootScope', function(root) {
    return {
        restrict: 'E',
        scope: {
            hoursModel: "=",
            minutesModel: "=",
            amPmModel: "=",
            timeChanged:"&"
        },
        template: '<select ng-model="hoursModel" ng-change="timeChanged()" ng-options="hour for hour in root.hours" class="form-control small" style="margin-right:5px"></select><select ng-model="minutesModel" ng-change="timeChanged()" ng-options="minute for minute in root.minutes" class="form-control small" style="margin-right:5px"></select><select ng-model="amPmModel" ng-change="timeChanged()" ng-hide="!root.isAmPmTime" ng-options="designator for designator in root.amPmDesignator" class="form-control small"></select>',
        link: function(scope, element, attrs) {
            scope.root = root;
        }
    }
}]);

