app.factory('AuthInterceptor', ['$location', function ($location) {
    return {
        request: function (config) {
            config.headers = config.headers || {};
            var token = sessionStorage.getItem('sessionId');
            if (token) {
                config.headers.Authorization = 'Basic ' + token;
            }
            return config;
        },
        responseError: function (rejection) {
            if (rejection.status === 401) {
                $location.path('#login');
            }
            return rejection;
        }
    };
}]);

app.factory('restServer', ['$rootScope', '$http', '$filter', '$location', '$routeParams', '$rootScope', function ($rootScope, $http, $filter, $location, $routeParams, $rootScope) {
	var restServer = {};
	var subdomain = localStorage.getItem('subdomain');
    var employeeGlobalId = localStorage.getItem('userEmployee');
   
    var baseUrl;
    if (sessionStorage.getItem('BASE_URL') == null) {
        baseUrl = window.location.protocol + "//" + window.location.host + '/Mobile/api/';
    } else {
        baseUrl = sessionStorage.getItem('BASE_URL') + '/Mobile/api/';
    }
 
    function getUrl(subUrl) {
        return baseUrl + subUrl + '?subdomain=' + subdomain;
    }

	restServer.SetOnSuccessAndError = function(promise, onSuccess, extraData)
	{
		if(onSuccess)
		{
			promise.success(function(data, status, headers, config) {
			    if (data.hasOwnProperty("responseStatus")) {
			        $rootScope.$broadcast("serverError", data.responseStatus);
			    } else {
			        onSuccess(data, extraData);
			    }
			});
		}
		
		promise.error(function(data, status, headers, config) {
		    if (data.hasOwnProperty("responseStatus")) {
		        $rootScope.$broadcast("serverError", data.responseStatus);
		    } else {
		        $rootScope.$broadcast("serverError",
		            {
		                "errorCode": status,
		                "message": data,
		                "stackTrace": ""
		            }
		        );
		    }
		});
	}

	restServer.LoadAppointmentBookStructure = function (dates, jobFunctionIds, employeeIds, serviceIds) {
	    var url = getUrl('AppointmentBook/Dimensions');
	    
	    for (var i = 0; i < dates.length; i++) {
	        // For some reason, servicestack didn't like certain dates like 12/31/2015 in M/D/YYYY format
	        dates[i] = moment(dates[i]).format("M/D/YYYY");
	    }
	    var promise = $http.post(url,
	    {
            
                'EmployeeIds': employeeIds,
                'ServiceIds': serviceIds,
                'JobFunctionIds': jobFunctionIds,
	            'Dates': dates,
                'SortOrder': ['DATE','EMPLOYEE']
	        
	    });
	    return promise;
	}

	restServer.LoadAppointmentBookData = function (dates, employeeIds, onSuccess) {
	    var url = getUrl('AppointmentBook/DimensionData');
	    var promise = $http.post(url,
	    {
	        
	            'EmployeeIds': employeeIds,
	            'Dates': dates
	        
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.LoadAppointmentBookDetailPopup = function (appointDetailId, onSuccess) {
	    var url = getUrl('AppointmentBook/AppointmentPopupDetails');
	    
	    var promise = $http.get(url,
	    {
            params: {
	            AppointDetailId: appointDetailId
	        }
	    });

        restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.LoadAppointmentBookBlockPopup = function (appointBlockId, onSuccess) {
	    var url = getUrl('AppointmentBook/BlockPopupDetails');

	    var promise = $http.get(url,
	    {
            params: {
                AppointBlockId: appointBlockId
            }
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.MoveAppointDetail = function (appointDetailId, date, time, employeeGlobalId, warningsAlreadyChecked, onSuccess) {
	    var url = getUrl('AppointmentBook/AppointDetailMove');

	    var promise = $http.post(url,
	    {
	        AppointDetailId: appointDetailId,
	        Date: date,
	        Time: time,
	        EmployeeGlobalId: employeeGlobalId,
	        WarningsAlreadyChecked: warningsAlreadyChecked
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.CopyAppointDetail = function (newAppointDetailId, currentAppointDetailId, date, time, employeeGlobalId,warningsAlreadyChecked, onSuccess) {
	    var url = getUrl('AppointmentBook/AppointDetailCopy');

	    var promise = $http.post(url,
	    {
	        NewAppointDetailId: newAppointDetailId,
            CurrentAppointDetailId: currentAppointDetailId,
	        Date: date,
	        Time: time,
	        EmployeeGlobalId: employeeGlobalId,
	        WarningsAlreadyChecked: warningsAlreadyChecked
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.MoveAppointBlock = function (appointBlockId, date, time, employeeGlobalId, onSuccess) {
	    var url = getUrl('AppointmentBook/AppointBlockMove');

	    var promise = $http.post(url,
	    {
	        AppointBlockId: appointBlockId,
	        Date: date,
	        Time: time,
	        EmployeeGlobalId: employeeGlobalId
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.LoadBusinessInfo = function () {
	    var url = getUrl('AppointmentBook/BusinessInfo');
	    
	    var promise = $http.get(url,
	    {
	    });

	    return promise;
	}

	restServer.LoadServices = function (onSuccess) {
	    var url = getUrl('AppointmentBook/Services');
	    var promise = $http.get(url,
	    {
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.LoadFilteredServices = function(filter, page, pageSize) {
        var url = getUrl('AppointmentBook/Services');
        return $http.get(url,
        {
            params: {
                filter: filter,
                page: page,
                pageSize: pageSize
            }
        });
    }

    restServer.LoadFilteredBlockTypes = function (filter) {
        var url = getUrl('AppointmentBook/BlockTypes');
        return $http.get(url,
	    {
	        params: {
	            filter: filter
	        }
	    });
    }

	restServer.LoadAppointTypes = function (onSuccess) {
	    var url = getUrl('AppointmentBook/AppointTypes');
	    var promise = $http.get(url,
	    {
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}
	restServer.LoadFilteredAppointTypes = function(filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/AppointTypes');
	    return $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
	            pageSize: pageSize
	        }
	    });
	}

	restServer.LoadAppointCats = function (onSuccess) {
	    var url = getUrl('AppointmentBook/AppointCats');
	    var promise = $http.get(url,
	    {
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess, onError);
	}

	restServer.LoadFilteredAppointCats = function (filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/AppointCats');
	    return $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
	            pageSize: pageSize
	        }
	    });
	}

	restServer.LoadAppointReasons = function (filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/AppointReasons');
	   return  $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
	            pageSize: pageSize
	        }
	    });

	}

	restServer.LoadEmployees = function (onSuccess) {
	    var url = getUrl('AppointmentBook/Employees');
	    var promise = $http.get(url,
	    {
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}
    
	restServer.LoadFilteredEmployees = function (filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/Employees');
	    return $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
	            pageSize: pageSize
	        }
	    });
	}

	restServer.LoadFilteredJobFunctions = function (filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/JobFunctions');
	    return $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
	            pageSize: pageSize
	        }
	    });
	}

	restServer.LoadGenders = function () {
	    var url = getUrl('AppointmentBook/Genders');
	    return $http.get(url,
	    {
	    });
	}

	restServer.LoadClients = function (onSuccess) {
	    var url = getUrl('AppointmentBook/Clients');
	    var promise = $http.get(url,
	    {
	    });

	    restServer.SetOnSuccessAndError(promise, onSuccess);
	}

	restServer.LoadFilteredClients = function (filter, page, pageSize) {
	    var url = getUrl('AppointmentBook/Clients');
	    return $http.get(url,
	    {
	        params: {
	            filter: filter,
	            page: page,
                pageSize: pageSize
	}
	    });
	}

    restServer.deleteAppointDetails = function (appointDeatailIds, appointReasonItemId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointDetails');
        var promise = $http.delete(url,
       {
           params: {
               appointDetailIds: appointDeatailIds,
               AppointReasonItemId: appointReasonItemId
           }
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.deleteAppointBlock = function (appointBlockId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlock');
        var promise = $http.delete(url,
       {
           params: {
               appointBlockId: appointBlockId
           }
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.deleteAppointBlockDefinition = function(appointBlockDefinitionId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlockDefinition');
        var promise = $http.delete(url,
       {
           params: {
               appointBlockDefinitionId: appointBlockDefinitionId
           }
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.deleteAppointStandingDefinition = function (appointStandingDefinitionId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointStandingDefinition');
        var promise = $http.delete(url,
       {
           params: {
               appointStandingDefinitionId: appointStandingDefinitionId
           }
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.confirmAppointDetail= function (appointDetailId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointDetailConfirm');
        var promise = $http.post(url,
       {
           AppointDetailId: appointDetailId
           
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.changeAppointDetailState = function (appointDetailId, newState, onSuccess) {
        var url = getUrl('AppointmentBook/AppointDetailChangeState');
        var promise = $http.post(url,
       {
           AppointDetailId: appointDetailId,
           NewState: newState
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.changeAppointDetailsState = function (appointDetailIds, newState, onSuccess) {
        var url = getUrl('AppointmentBook/AppointDetailsChangeState');
        var promise = $http.post(url,
       {
           AppointDetailIds: appointDetailIds,
           NewState: newState
       });
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.saveAppointStandingDefinition = function(data, onSuccess) {
        var url = getUrl('AppointmentBook/AppointStandingDefinition');
        var promise = $http.post(url,data);
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.saveAppointBlockDefinition = function (data, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlockDefinition');
        var promise = $http.post(url, data);
        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.createDefaultAppointDetail = function (employeeGlobalId, serviceItemId, onSuccess) {
        var url = getUrl('AppointmentBook/CreateDefault');

        var promise = $http.get(url,
        {
            params: {
                ServiceId: serviceItemId,
                EmployeeId: employeeGlobalId
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.saveAppointHeader = function (appointHeader, onSuccess) {
        var url = getUrl('AppointmentBook/AppointHeader');

        var promise = $http.post(url, appointHeader);

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.saveAppointBlock = function(appointBlock, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlock');

        var promise = $http.post(url, appointBlock);

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getAppointBlock = function(appointBlockId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlock');

        var promise = $http.get(url, {
            params: {
                appointBlockId: appointBlockId
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getEnumList = function (enumTypeId) {
        var url = getUrl('AppointmentBook/EnumList');

        return $http.get(url,
        {
            params: {
                EnumTypeId: enumTypeId
            }
        });
    }

    restServer.getAppointStandingDefinition = function(appointStandingDefinitionId , onSuccess) {
        var url = getUrl('AppointmentBook/AppointStandingDefinition');

        var promise = $http.get(url,
        {
            params: {
                AppointStandingDefinitionId: appointStandingDefinitionId
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getAppointBlockDefinition = function(appointBlockDefinitionId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointBlockDefinition');

        var promise = $http.get(url,
            {
                params: {
                    AppointBlockDefinitionId: appointBlockDefinitionId
    }
    });

    restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getAppointHeaderForAppointDetail = function(appointDetailId) {
        var url = getUrl('AppointmentBook/AppointHeaderForAppointDetail');

        return $http.get(url,
        {
            params: {
                AppointDetailId: appointDetailId
            }
        });
    }

    restServer.getFutureAppointments = function (clientId) {
        var url = getUrl('AppointmentBook/FutureAppointments');

        return $http.get(url,
        {
            params: {
                ClientGlobalId: clientId
            }
        });
    }

    restServer.getVisitNoteTypes = function (onSuccess) {
        var url = getUrl('AppointmentBook/VisitNoteTypes');

        var promise = $http.get(url, {
            params: {

            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getClient = function(clientGlobalId, onSuccess) {
        var url = getUrl('AppointmentBook/FullClient');

        var promise = $http.get(url,
        {
            params: {
                ClientGlobalId: clientGlobalId
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.saveClient = function(client, onSuccess) {
        var url = getUrl('AppointmentBook/FullClient');

        var promise = $http.post(url,
        {
            Client: client
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getClientAppoimentHistory = function (clientGlobalId, onSuccess) {
        var url = getUrl('AppointmentBook/ClientAppointmentHistory');

        var promise = $http.get(url,
        {
            params: {
                ClientGlobalId: clientGlobalId
            }   
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getAppointHeaderAuditHistory = function (appointHeaderId, onSuccess) {
        var url = getUrl('AppointmentBook/AppointHeaderAuditHistory');

        var promise = $http.get(url,
        {
            params: {
                AppointHeaderId: appointHeaderId
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getUserSecurityAttribs = function () {
        var url = getUrl('AppointmentBook/UserSecurityAttribs');

        return $http.get(url,
        {
        });
    }

    restServer.addLock = function (entityId, noun, onSuccess) {
        var url = getUrl('AppointmentBook/Lock');

        var promise = $http.post(url,
        {
            entityId: entityId,
            noun: noun
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.deleteLock = function (entityId, noun, onSuccess) {
        var url = getUrl('AppointmentBook/Lock');

        var promise = $http.delete(url,
        {
            params: {
                entityId: entityId,
                noun: noun
            }
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.userElevateSecurity = function(userName, password, securityKey, onSuccess) {
        var url = getUrl('AppointmentBook/UserElevateSecurity');

        var promise = $http.post(url,
        {
            userName: userName,
            password: password,
            securityKey: securityKey
        });

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getClientAlerts = function (clientId, onSuccess) {
        var url = getUrl('Alerts/Client/' + clientId);

        var promise = $http.get(url, {});

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getClientNoteAlerts = function (clientId, onSuccess) {
        var url = getUrl('Alerts/ClientNote/' + clientId);

        var promise = $http.get(url, {});

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getServiceAlerts = function (serviceId, employeeId, onSuccess) {
        var url = getUrl('Alerts/Service/' + serviceId + '/' + employeeId);

        var promise = $http.get(url, {});

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }

    restServer.getEmployee = function(employeeId, onSuccess) {
        var url = getUrl('/Employee/' + employeeId);

        var promise = $http.get(url, {});

        restServer.SetOnSuccessAndError(promise, onSuccess);
    }
    return restServer;

}]);