﻿app.service('Security', [
    '$rootScope', 'securityKeys', 'Enums', 'restServer', function($rootScope, securityKeys, enums, restServer) {
        this.setSecurityAttribs = function(securityAttribs) {
            $rootScope.userSecurityAttribs = securityAttribs;
            $rootScope.$broadcast("userSecurityAttribsRetrieved");
        }

        this.getSecurityAttr = function(securityKey) {
            var securityAttrib;
            for (var i = 0; i < $rootScope.userSecurityAttribs.length; i++) {
                var secAttrTemp = $rootScope.userSecurityAttribs[i];
                if (secAttrTemp.securityKey == securityKey) {
                    securityAttrib = secAttrTemp;
                    break;
                }
            }
            if (securityAttrib == undefined) {
                securityAttrib = {
                    securityAllowEnum: enums.SecurityAllowEnum.Write,
                    elevatableEnum: enums.ElevatableEnum.Restrict
                }
            }

            securityAttrib.canWrite = function() {
                return this.securityAllowEnum == enums.SecurityAllowEnum.Write;
            }

            securityAttrib.canExecute = function() {
                return this.securityAllowEnum == enums.SecurityAllowEnum.Executable;
            }

            securityAttrib.canElevate = function() {
                return this.elevatableEnum == enums.ElevatableEnum.Allow;
            }

            return securityAttrib;
        }

        var parent = this;

        this.checkSecurityElevatable = function(securityKey, callback, params) {
        // Disable this for now until override prompt complete
        var secAttr = this.getSecurityAttr(securityKey);
        if (secAttr.canWrite()) {
            callback(params);
        } else if (secAttr.canExecute() && secAttr.canElevate()) {
            parent.elevateUserName = '';
            parent.elevatePassword = '';
            parent.message = '';
            $rootScope.$broadcast("showMessage", {
                message: "",
                buttonsType: enums.ButtonTypeEnum.OkCancel,
                customTemplateId: "divSecurityOverridePrompt",
                hidePopupOnButtonClick: false,
                callback: function(result) {
                    if (result == enums.MessageBoxButtonResult.Ok) {
                        if (parent.elevateUserName.trim() == '' || parent.elevatePassword.trim() == '') {
                            parent.message = 'Username and password are required';
                            return;
                        }
                        restServer.userElevateSecurity(parent.elevateUserName, parent.elevatePassword, securityKey, function(response) {
                            if (!response.canAuthenticate) {
                                parent.message = "Invalid login credentials";
                            } else if (!response.canElevate) {
                                parent.message = "Unable to authenticate user for security elevation";
                            } else
                            {
                                callback(params);
                                $rootScope.$broadcast("hideMessage");
                            };
                        });

                    }
                }
            });


        }
        }

    this.keys = securityKeys;
        }]);