define([
    'jquery',
    'moment',
    'common/base-router',
    //data
    'data/consts',
    'data/desktops',
    'data/game-messages',
    'data/drawer',
    'data/smart-receipt',
    'data/sale-lines',
    //views
    'views/login',
    'views/menu',
    'views/desktops',
    'views/widget-details-modal',
    'views/sales',
    'views/game-messages',
    'views/game-message-modal',
    'views/yes-no-modal',
    'views/register',
    'views/workstation-modal',
    'views/drawer/drawer',
    'views/drawer/drawer-history-modal',
    'views/smart-receipt/smart-receipt',
    'bridge'
], function ($, moment, BaseRouter,
             //data
             Сonsts, desktopsList, gameMessagesList,
             drawerModel, SmartReceiptModel, saleLinesCollection,
             // views
             LoginView, MenuView, DesktopsView, WidgetDetailsModal,
             SalesView, GameMessagesView, GameMessageModalView,
             YesNoModal,
             RegisterView, WorkstationModal,
             DrawerView, DrawerHistoryModal, SmartReceiptView
) {
    'use strict';

    var oldUpdateHash = Backbone.History.prototype._updateHash;

    var nua = navigator.userAgent.toLowerCase();
    var isAndroid = ((nua.indexOf('mozilla/5.0') > -1 && nua.indexOf('android ') > -1 && nua.indexOf('applewebkit') > -1) && !(nua.indexOf('chrome') > -1));
    // Replace default _updateHash function
    Backbone.History.prototype._updateHash = function (location, fragment, replace) {
        // Check if we are android
        if (isAndroid) {
            // For android browsers, just make sure replace is false
            replace = false;
        }
        // Call the old _updateHashFunction
        oldUpdateHash(location, fragment, replace);
    };

    if (!window.IS_DEVICE && location.hash != '#menu') {
        // for debugging in browser
        sessionStorage.clear();
    }

    var loginView = new LoginView();
    var menuView = new MenuView();
    var desktopsView = new DesktopsView();
    var salesView = new SalesView();
    var gameMessagesView = new GameMessagesView();
    var registerView = new RegisterView();
    var drawerView = new DrawerView({model: drawerModel});
    var smartReceiptView = new SmartReceiptView({
        model: new SmartReceiptModel(),
        collection: saleLinesCollection
    });

    var EstimatedSalesWidgetLabels = {
        'EstimatedSales': 'Estimated Sales:',
        'GiftLiabilitiesBalance': 'Gift Liabilities Balance:',
        'EstimatedCashFlow': 'Estimated Cash Flow:'
    };

    function updateWidgetsValues(widget) {
        var dotIndex = '',
            values = $.extend(true, {}, widget.values); // deep copy of values object

        // find float numbers and round to two decimal signs
        _.each(values, function (value, key, list) {
            if (value) {
                dotIndex = value.indexOf('.');
                if (dotIndex != -1 && !isNaN(+value)) {
                    value = $.global.format(+value, 'n');
                }
                list[key] = value;
            }
        });

        if (widget.requestNoun == 'EstimatedSalesWidget') {
            // for Estimated Sales widget
            var _groupList = [], temp = [];
            _.each(values, function (value, key) {

                if (!EstimatedSalesWidgetLabels[key])
                    return;

                temp = [];
                temp.push({
                    "n": "SalesTypeName",
                    "vg": EstimatedSalesWidgetLabels[key]
                });
                temp.push({
                    "n": "SalesTypeValue",
                    "vg": $.global.format(+value.replace(',', ''), 'c')
                });
                _groupList.push({"p": temp});
            });
            widget.groupList = _groupList;
        }
    }

    function updateWidgetGroupList(widget) {
        var groupList = widget.groupList,
            groupListUpdated = [],
            groupListRender = [];

        if (groupList.length) {
            var widgetDef = _.findWhere(Сonsts.WidgetDefinitions, {noun: widget.requestNoun});
            var displayData = widgetDef && widgetDef.displayData || [];

            _.each(groupList, function (value, key, list) {
                if (typeof(value) == 'string') {
                    return;
                }

                var groupListElement = _.object(_.pluck(value['p'], 'n'), _.map(value['p'], function (item) {
                    delete item['n'];
                    var key = _.keys(item)[0];
                    var value = _.values(item)[0];

                    // crutch for empty object
                    if (typeof value === 'object' && value.__type === 'System.DBNull, mscorlib') {
                        value = ''
                    }
                    if (widget.requestNoun != 'RegisterSalesServiceHoursWidget' && key == 'vd' && value !== undefined) {
                        value = $.global.format(value, 'c');
                    }
                    return value;
                }));
                var m;

                if (groupListElement["AppointmentDateTime"]) {
                    m = moment(groupListElement["AppointmentDateTime"]);
                    if (m.isValid()) {
                        groupListElement["AppointmentDateTime"] = m.format('L hh:mm A');
                        groupListElement["AppointmentDate"] = m.format('L');
                        groupListElement["AppointmentTime"] = m.format('hh:mm A');
                        groupListElement["AppointmentHours"] = m.format('hh A');
                    }
                }
                if (groupListElement["BreakTime"]) {
                    m = moment(groupListElement["BreakTime"]);
                    if (m.isValid()) {
                        groupListElement["BreakTimeHHMM"] = m.format('hh:mm A');
                    }
                }
                if (groupListElement["StartTime"]) {
                    m = moment(groupListElement["StartTime"]);
                    if (m.isValid()) {
                        groupListElement["StartTimeHHMM"] = m.format('hh:mm A');
                    }
                }
                if (groupListElement["DateTransaction"]) {
                    m = moment(groupListElement["DateTransaction"]);
                    if (m.isValid()) {
                        groupListElement["DateTransaction"] = m.format('L hh:mm A');
                    }
                }

                groupListUpdated.push(groupListElement);
                if (displayData.length) {
                    groupListRender.push(_.pick(groupListElement, displayData));
                }
            });
            widget.groupListUpdated = groupListUpdated;
            widget.groupListRender = groupListRender;
        }
    }

    var Application = BaseRouter.extend({

        routes: {
            '': 'startup',
            'login': 'showLogin',
            'menu': 'showMainMenu',
            'desktops': 'showDesktops',
            'desktops/:desktopId': 'getDesktop',
            'desktops/:desktopId/:widgetId': 'showWidgetDetails',
            'sales': 'showSales',
            'game-messages': 'showGameMessages',
            'game-messages/:messageId/:returnToList(/:refreshList)': 'showMessageDetails',
            'register': 'showRegister',
            'workstation': 'selectWorkstation',
            'drawer': 'showDrawer',
            'drawer-history/:workstationId': 'showDrawerHistory',
            'smart-receipt': 'showSmartReceipt',
            'smart-receipt/:showHold': 'showSmartReceipt',
            'appointment-book': 'navigateToAppointmentBook',
            'help': 'help'
        },

        initialize: function () {
            $(document).on('touchstart', '#home', function () {
                if (NativeQueue.empty()) {
                    Backbone.history.navigate('menu', {trigger: true, replace: true});
                }
            });

            //this.history = [];

            Backbone.history.on('route', function (obj, name) {
                //this.history.push(name);
                $('#menu').toggleClass('hidden', !sessionStorage.getItem('sessionId'));
                $('#main-wrapper').toggleClass('hidden', !sessionStorage.getItem('sessionId'));
                $('body').toggleClass('authenticated', !!sessionStorage.getItem('sessionId'));
                $(':focus').blur();
            }, this);

            Backbone.history.start();
        },

        startup: function () {
        },

        showMainMenu: function () {
            lockScreenToPortraitMode(false);
            this.changeView(menuView);
        },

        showDesktops: function () {
            getDesktops().done(function (data) {
                // fix: not triggered 'remove' event on set
                var forRemove = _.reject(desktopsList.map(function(desktop) {
                    return _.some(data, function(item) {
                        return desktop.get('id') == item.id
                    }) ? null : desktop;
                }), function(item) {
                    return !item;
                });
                desktopsList.set(data);
                // fix: not triggered 'remove' event on set
                _.each(forRemove, function(item) {
                    desktopsList.trigger('remove', item, item, desktopsList);
                });
                var desktop = _.first(data);
                if (desktop) {
                    Backbone.history.navigate('desktops/' + desktop.id, {trigger: true, replace: true});
                }
            }.bind(this));
            this.changeView(desktopsView);
        },

        getDesktop: function (desktopId) {
            var queue = new Queue();
            var desktop = desktopsList.get(desktopId);
            var widgets = null;
            // show loading
            //$('#' + desktop.id).addClass('loading');
            $('#' + desktop.id).closest('.desktop-scroll').addClass('loading');

            // if came from history.back, show needed desktop
            var selectedIndex = $('#desktops-select option[value=' + desktopId + ']').index();
            if (desktopsView.activeIndex ==0 || desktopsView.activeIndex != selectedIndex) {
                desktopsView.swipeview && desktopsView.swipeview.goToPage(selectedIndex);
            }

            // Getting desktop widgets
            queue.append(function () {
                getWidgets(desktop.id).done(function (data) {
                    widgets = data;
                    this.resolve();
                }.bind(this)).fail(this.reject.bind(this));
            });
            // Getting desktop widget details
            return queue.append(function () {
                getDesktopData(desktop.id).done(function (data) {

                    getSettingCall().done(function () {

                        desktop.widgets.set(_.map(widgets, function (widget) {
                            var item = _.extend({
                                groupList: []
                            }, widget, _.find(data.widgets, function (item) {
                                return item.id == widget.id;
                            }));
                            updateWidgetsValues(item);
                            updateWidgetGroupList(item);
                            desktop.widgets.trigger('change', widget, data.widgets);
                            return item;
                        }));
                        this.resolve();

                    }.bind(this));

                }.bind(this)).fail(this.reject.bind(this));
            }).always(function () {
                $('#' + desktop.id).closest('.desktop-scroll').removeClass('loading');
            });
        },

        showWidgetDetails: function (desktopId, widgetId) {
            setTimeout(function () {
                new WidgetDetailsModal({desktopId: desktopId, widgetId: widgetId}).render();
            }, 0);
        },

        showSales: function () {
            this.changeView(salesView);
        },

        showLogin: function () {
            this.changeView(loginView);
        },

        showGameMessages: function () {
            lockScreenToPortraitMode(true);
            setTimeout(function () {
                this.changeView(gameMessagesView);
            }.bind(this), 0);
        },

        showMessageDetails: function (messageId, returnToList, refreshList) {
            // returnToList - just return to list (return from message details to history
            // refreshList - open message history after closing push message details (on login)
            if (gameMessagesList.findWhere({"globalMessageId": messageId}) === undefined) {
                window.history.back();
            } else {
                setTimeout(function () {
                    new GameMessageModalView({
                        messageId: messageId,
                        returnToList: returnToList,
                        refreshList: refreshList
                    }).render();
                }, 0);
            }
        },

        showRegister: function () {

            if (localStorage.getItem("domain") === ".meevocloud.co.uk") {
                alert("Sorry! This feature is currently not available.");
                return;
            }

            lockScreenToPortraitMode(false);
            if (localStorage.getItem('workstation')) {
                this.changeView(registerView);
            } else {
                Backbone.history.navigate('workstation', {trigger: true, replace: true});
            }
        },

        selectWorkstation: function () {
            setTimeout(function(){
                new WorkstationModal().render();
            },0);
        },

        showDrawer: function () {
            this.changeView(drawerView);
        },

        showDrawerHistory: function (workstationId) {
            setTimeout(function(){
                new DrawerHistoryModal({workstationId: workstationId}).render();
            },0);
        },

        showSmartReceipt: function (showHold) {
            if ($('html').hasClass('mobile')) {
                lockScreenToPortraitMode(true);
            }
            smartReceiptView.noMenuTap = showHold ? true : false;
            this.changeView(smartReceiptView);
            setTimeout(function () {
                if (showHold) {
                    if ($('html').hasClass('mobile')) {
                        smartReceiptView.navigateToOnHoldModal();
                    } else {
                        smartReceiptView.createSubmenuDropDown();
                        $('.sub-menu-item[data-url="onhold"]').trigger('touchend');
                    }
                }
            }, 100);
        },

        navigateToAppointmentBook: function () {
            Backbone.history.navigate('menu', { trigger: true, replace: true });
            var employeeGlobalId = window.SETTINGS['userEmployee'] != null ? window.SETTINGS['userEmployee'].id : null;
            var url = "AppointmentBook/index.html";

            redirectToUrl(url);
        },

        help: function () {
            openUrlInBrowser(window.HELP_URL);
            Backbone.history.navigate('menu', {trigger: true, replace: true});
        }

    });

    return new Application();
});
