'use strict';

define([
    'backbone',
    'data/game-messages'
], function (Backbone, gameMessagesList) {

    /**
     * Called when App going to background
     * */
    window.onAppPause = function () {
    };

    /**
     * Called when App going to foreground
     * */
    window.onAppResume = function () {
        $(':focus').blur();
        window.handleOrientation(true);
        if (window.location.hash != '#desktops') {
            window.app.currentView && window.app.currentView.refresh();
        }
        if (window.location.hash == '#game-messages') {
            window.app.currentView.getData();
        }
        if (window.location.hash == '#smart-receipt') {
            app.currentView.addClientModal && app.currentView.addClientModal.refresh();
        }
    };

    /**
     * Called when App received push notification
     * */
    var pushQueue = null;
    var lastPush = null;
    window.openPushGameMessage = function (item, openMessageDetails) {
        if (pushQueue == null) {
            pushQueue = new Queue();
        }
        lastPush = pushQueue.append(function () {
            if (sessionStorage.getItem('userId') && sessionStorage.getItem('userId') == item.userGlobalId) {
                if (gameMessagesList.where({'globalMessageId': item.globalMessageId}).length == 0) {
                    gameMessagesList.unshift(item);
                }
            }
            if (openMessageDetails) {
                var returnToList = openMessageDetails;
                if (Backbone.history && Backbone.history.fragment !== 'login') openMessageDetails = false;
                Backbone.history.navigate('game-messages/' + item.globalMessageId + '/false' +(openMessageDetails && '/' +returnToList || ''), {
                    trigger: true
                });
            } else {
                var pushModal = $('#push-modal');
                if(Backbone.history.fragment !== "login") {
                    if(!pushModal.hasClass('opened')) {
                        pushModal.addClass('opened').fadeIn(300, function(){
                            pushModal.on('touchend', function(evt){
                                Backbone.history.navigate('game-messages', {trigger: true});
                                pushModal.removeClass('opened').hide();
                                evt.preventDefault();
                                clearTimeout(notificationTimeout);
                            });
                            var notificationTimeout = setTimeout(function(){
                                pushModal.fadeOut(300, function(){
                                    pushModal.removeClass('opened').off('touchend');
                                    clearTimeout(notificationTimeout);
                                });
                            }, 5000);
                        });
                    }
                }
            }
            setTimeout(this.resolve.bind(this), 1000);
        }).always(function () {
            if (lastPush.state() != 'pending') {
                pushQueue = null;
            }
        });
    };
});
if (window.IS_DEVICE === undefined) {
    window.IS_DEVICE = false;
    window.DEVICE_TYPE = null;
    window.DEVICE_TOKEN = null;
}

window.API_BASE_URL = sessionStorage.getItem('API_BASE_URL');
window.IMAGES_BASE_URL = sessionStorage.getItem('IMAGES_BASE_URL');

function toastMessage(message) {
    if (window.IS_DEVICE) {
        NativeQueue.exec('toastMessage', {
            message: message
        });
    } else {
        alert(message);
    }
}

function nativeAlert(title, message) {
    if (window.IS_DEVICE) {
        NativeQueue.exec('alert', {
            title: title,
            message: message
        });
    } else {
        alert(message);
    }
}

function registerNotificationToken() {
    window.IS_DEVICE && NativeQueue.exec('registerNotificationToken', {});
}

/**
 * Fired when network connection status changed.
 * Used by native code.
 */
function connectionStatusChanged(isOnline) {
    if (isOnline != window.IS_ONLINE) {
        if (isOnline && window.IS_DEVICE && (window.DEVICE_TOKEN === undefined || window.DEVICE_TOKEN == '')) {
            registerNotificationToken();
        }
    }
    window.IS_ONLINE = isOnline;
}

function ajaxSetup(callback) {
    $.ajaxSetup({
        dataType: 'json',
        beforeSend: function (xhr, obj) {
            if(window.IS_ONLINE !== undefined && !window.IS_ONLINE) {
                nativeAlert('error', 'No network connection');
                return false;
            }
            if (sessionStorage.getItem('subdomain')) {
                obj.url += (obj.url.indexOf('?') == -1 ? '?' : '&') + 'subdomain=' + sessionStorage.getItem('subdomain');
            }

            obj.url += (obj.url.indexOf('?') == -1 ? '?' : '&') + 'format=json';

            if (sessionStorage.getItem('sessionId')) {
                xhr.setRequestHeader('Authorization', 'Basic ' + sessionStorage.getItem('sessionId'));
            }
        },
        error: function (xhr, status, error) {
            if (xhr.status == 401) {
                sessionStorage.setItem('sessionId', '');
                Backbone.history.navigate('login', {trigger: true});
            } else if ([0, 404].indexOf(xhr.status) != -1) {
                nativeAlert('error', 'Unable to connect to the server.');
            } else if (xhr.responseJSON && xhr.responseJSON.responseStatus) {
                nativeAlert(status, xhr.responseJSON.responseStatus.message);
            } else {
                nativeAlert(status, error);
            }
        }
    });
    if (callback) callback();
}

function getSettingCall(d) {
    d = d || $.Deferred();

    return getSettings().done(function (res) {
        window.SETTINGS = res;
        $.global.preferCulture(res.cultureInfo);

        var locale = res.cultureInfo.toLowerCase();
        if (locale == 'en-ca') {
            locale = 'en-us';
        }
        moment.locale(locale);
        window.SETTINGS.timeZoneOffset = window.SETTINGS.timeZoneOffset.replace(/^([0-9])/,'+$1').replace(/^([+|-]?\d{2}:\d{2}).*/, '$1');

        getEmployeeForUser().done(function (data){
            window.SETTINGS['userEmployee'] = data;
            localStorage.setItem('userEmployee', data ? data.id : undefined);
        });
        // window.API_BASE_URL = "http://localhost:6060/mobile/api/";
    }).fail(d && d.reject.bind(d));
}

function getPermissionsCall(d) {
    getPermissions().done(function (res) {
        window.PERMISSIONS.set(_.object(
            _.pluck(res, 'securityKey'), _.map(res, function (o) {
                return {
                    isVisible: o['isVisible'],
                    isElevatable: o['isElevatable'],
                    isReadonly: o['isReadonly']
                };
            })
        ));
        sessionStorage.setItem('menuPermitions', JSON.stringify({
            'evo.launchpad.registergroup': window.PERMISSIONS.get('evo.launchpad.registergroup')
        }));
    }).fail(d && d.reject.bind(d));
}

function login(usernameParam, passwordParam, urlParam) {

    var d = $.Deferred();

    //if token is undefined try to register device at Notification Service
    if (window.IS_ONLINE && window.IS_DEVICE && window.DEVICE_TOKEN === undefined) {
        d.reject();
        registerNotificationToken();
        nativeAlert("Notification Registration Failed", "Application will try to register on next login");
        return d.promise();
    }

    sessionStorage.setItem('sessionId', '');
    sessionStorage.setItem('userId', '');
    sessionStorage.setItem('userFullName', '');

    $.ajaxSetup({
        dataType: 'json',
        beforeSend: function (xhr, obj) {
        },
        error: function (xhr, status, error) {
        }
    });
    $.ajax({
        type: 'POST',
        url: window.API_BASE_URL + 'auth/credentials/?subdomain=' + urlParam,
        data: {
            'username': usernameParam,
            'password': passwordParam,
            'rememberMe': true
        },
        timeout: 10 * 1000
    }).done(function (res) {
        sessionStorage.setItem('sessionId', btoa(usernameParam + ':' + passwordParam));
        sessionStorage.setItem('userId', res.userId);
        sessionStorage.setItem('userFullName', res.displayName);
        sessionStorage.setItem('subdomain', urlParam);

        ajaxSetup(function (d) {
            getSettingCall(d);
            getPermissionsCall(d);
        });

        NativeQueue.exec(function () {
            var d1 = $.Deferred();
            if (window.IS_DEVICE) {
                $.ajax({
                    type: 'POST',
                    url: window.API_BASE_URL + 'UserDevice/' + window.DEVICE_TYPE + '/' + window.DEVICE_ID + '?Token=' + window.DEVICE_TOKEN,
                    dataType: 'text'
                }).done(d1.resolve.bind(d1)).fail(d1.reject.bind(d1));
            } else {
                d1.resolve();
            }
            return d1.promise();
        }).done(d.resolve.bind(d)).fail(d.reject.bind(d));

    }).fail(d.reject.bind(d));

    return d.promise();
}

function registerDevice() {
    $.ajax({
        type: 'POST',
        url: window.API_BASE_URL + 'UserDevice/' + window.DEVICE_TYPE + '/' + window.DEVICE_ID + '?Token=' + window.DEVICE_TOKEN,
        dataType: 'text'
    })
}

function unregister(deviceToken) {
    if (window.IS_DEVICE) {
        $.ajax({
            type: 'DELETE',
            url: window.API_BASE_URL + 'UserDevice/' + window.DEVICE_TYPE + '/' + deviceToken,
            dataType: 'text'
        });
    }
}

function getSettings() {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Settings/'
        });
    });
}

function getPermissions() {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Security/'
        });
    });
}

function userCanPerformAction(userName, password, securityKey) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'UserCanPerformAction',
            data: {
                userName: userName,
                password: password,
                securityKey: securityKey
            }
        });
    });
}

function getDesktops() {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Desktops/' + localStorage.getItem('username')
        });
    });
}

function getWidgets(desktopIdParam) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Widgets/' + desktopIdParam
        });
    });
}

function getWidgetProperties(widgetIdParam) {
    return $.ajax({
        method: 'GET',
        url: window.API_BASE_URL + 'WidgetProps/'
    });
}

function getDesktopData(desktopIdParam) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Desktop/' + desktopIdParam + '/Data'
        });
    });
}

function getSales(period) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'SalesDashboards/' + period
        });
    });
}

function gameMessageMarkAsRead(id) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'GameMessages/MarkViewed/' + id,
            dataType: 'text'
        });
    });
}

function getGameMessagesCount(onlyNewMessages) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'GameMessages/Count/?messageViewType=false'
        });
    });
}

function getGameMessages(pageNumber, pageSize, firstMessageId) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'GameMessages/List/',
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 10
            }
        });
    })
}

function getGameImageLocalUrl(url, callback) {
    window.IS_DEVICE && NativeQueue.exec('getGameImageLocalUrl', {
        url: url
    }).done(function () {
        callback && callback.apply(null, arguments);
    });
}

function lockScreenToPortraitMode(isLocked) {
    window.IS_DEVICE && NativeQueue.exec('lockScreenToPortraitMode', {
        isLocked: isLocked
    }).done(function () {
        window.isLocked = isLocked;
    });
}

function onLoadFinished() {
    window.IS_DEVICE && NativeQueue.exec('onLoadFinished', {});
}

function onUserLoggedIn() {
    window.IS_DEVICE && NativeQueue.exec('onUserLoggedIn', {});
}

function onGameMessagesCountChanged(count) {
    window.IS_DEVICE && NativeQueue.exec('onGameMessagesCountChanged', {
        count: count
    });
}

function openUrlInBrowser(url) {
    if (window.IS_DEVICE) {
        NativeQueue.exec('openUrlInBrowser', {
            url: url
        });
    } else {
        window.open(url, '_blank');
    }
}

function redirectToUrl(url) {
    if (window.IS_DEVICE) {
        NativeQueue.exec('redirectToUrl', {
            url: 'html5/' + url
        });
    } else {
        location.href = url;
    }
}

function getWorkstations(filter) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Workstations',
            data: {
                filter: filter
            }
        });
    });
}

function getWorkstationDrawerState(drawerId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/State'
        });
    });
}

function getDrawerHistory(drawerId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/History',
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function openDrawer(drawerId, data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/Open',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function closeDrawer(drawerId, data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/Close',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getPaymentsState(drawerId, paymentId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/PaymentTypePayments/' + paymentId
        });
    });
}

function getPaysInOut(drawerId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'PayInOutPayments/' + drawerId
        });
    });
}

function addUpdatePayInOut(type, drawerId, data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: type,
            url: window.API_BASE_URL + 'PayInOutPayments/' + drawerId,
            contentType: 'application/json; charset=utf-8',
            dataType: 'json',
            data: data
        });
    });
}

function deletePayInOut(drawerId, paymentId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'DELETE',
            url: window.API_BASE_URL + 'PayInOutPayments/' + drawerId + '/' + paymentId
        });
    });
}

function getPaymentTypes(filter, type) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'PayInOutTypes/' + type,
            data: {
                filter: filter
            }
        })
    });
}

function getClient(clientId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Client/' + clientId
        });
    });
}

function getClients(filter, clientAccountChargable, isGiftCardPurchaseForRequest, isRefundForClientAccountRequest, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        var data = {
            filter: filter,
            PageNumber: pageNumber || 0,
            PageSize: pageSize || 20,
            IsGiftCardPurchaseForRequest: isGiftCardPurchaseForRequest || false,
            IsRefundForClientAccountRequest: isRefundForClientAccountRequest || false
        };

        if (!_.isUndefined(clientAccountChargable))
            data.ClientAccountChargable = clientAccountChargable;

        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Clients/',
            data: data
        })
    });
}

function getClientAlerts(clientId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Alerts/Client/' + clientId
        })
    });
}

function getTodayAppointments(filter, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Appointments/',
            data: {
                filter: filter,
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        })
    });
}

function getAppointmentsDetails(clientId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'AppointmentsDetails/' + clientId
        })
    });
}

function getServicesTypes(pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'ServiceCats/',
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function getEmployee(employeeId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Employee/' + employeeId
        });
    });
}

function filterEmployees(filter, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last')
            return $.Deferred().promise();

        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Employees/',
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function filterEmployeesWithParameter(filter, param, pageNumber, pageSize) {
    return filterEmployees(filter, pageNumber, pageSize);
}

function getServices(id, employeeGlobalId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Services/' + id,
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20,
                employeeGlobalId: employeeGlobalId || ''
            }
        });
    });
}

function filterServices(filter, employeeGlobalId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Services/',
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20,
                employeeGlobalId: employeeGlobalId || ''
            }
        });
    });
}

function getServiceByEmployee(serviceId, employeeId, async) {
    return NativeQueue.exec(function () {
        return $.ajax({
            async: _.isUndefined(async) ? true : async,
            method: 'GET',
            url: window.API_BASE_URL + 'Service/' + serviceId + '/' + employeeId
        });
    });
}

function filterProducts(filter, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Products/',
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function getProductsCat(vendorId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'ProductCats/' + (vendorId || ''),
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function getProducts(manufacturer, catId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Products/' + (manufacturer || '') + '/' + (catId || ''),
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function generateGiftCardNumber() {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'GenerateGiftCardNumber',
            dataType: 'text'
        });
    });
}

function getGiftTypes(filter) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'GiftTypes/',
            data: {
                filter: filter
            }
        })
    });
}

function getAppointmentTypes(filter) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'AppointTypes/',
            data: {
                filter: filter
            }
        })
    });
}

function getAppointmentNotes(clientId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'AppointmentsNotes/' + clientId
        })
    });
}

function saleCalculationRequest(data, async) {
    return NativeQueue.exec(function () {
		async = async || true;
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'CalculateSale/',
            dataType: 'json',
			async: async,
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function saleFinalizeRequest(data, drawerId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'FinalizeSale/' + drawerId,
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getProductAlerts(productId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Alerts/Product/' + productId
        });
    });
}

function getServiceAlerts(serviceId, employeeId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Alerts/Service/' + serviceId + '/' + employeeId
        });
    });
}

function getDiscountsFor(filter, id, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Discounts/' + id,
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function getValidDisountsFor(filter, itemId, clientId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + ['AvailableDiscounts',itemId, clientId].join('/'),
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 20
            }
        });
    });
}

function getValidDisountsForSaleLines(data) {
	return NativeQueue.exec(function () {
        return $.ajax({
            method: 'POST',
            url: window.API_BASE_URL + 'DiscountsForSaleLines/',
			dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: JSON.stringify(data)
        });
    });
}

function getValidDisountsFlowList(filter, data, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }

		pageNumber = pageNumber || 0;
        pageSize = pageSize || 20;

        return $.ajax({
            method: 'POST',
            url: window.API_BASE_URL + 'DiscountsFlowList?' + ['filter=' + filter, 'PageNumber=' + pageNumber, 'PageSize=' + pageSize].join('&'),
			dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: JSON.stringify(data)
        });
    });
}

function setManualDiscount(data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'ManualDiscount/',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getCheckoutPaymentTypes() {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'PaymentTypes'
        });
    });
}

function getGiftCards(userOrCard) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'GiftCards/' + userOrCard,
            data: {
                showEmpty: true
            }
        });
    });
}

function getEmployeeForUser() {
	return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'Employees/Me',
			dataType: 'json'
        });
	});
}

function lockUnlockDrawer(drawerId, shouldLock) {
	return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'Drawer/' + drawerId + '/IsLockAction/' + shouldLock,
            dataType: 'json',
            contentType: 'application/json; charset=utf-8'
        });
	});
}

function getSalesOnHold(showDeleted, clientGlobalId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'SalesOnHoldInfo',
            dataType: 'json',
            data: {
                showDeleted: showDeleted ? showDeleted : false,
                clientGlobalId: clientGlobalId ? clientGlobalId : ''
            }
        });
    });
}

function getSaleOnHold(saleTempId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'GET',
            url: window.API_BASE_URL + 'SaleOnHold',
            dataType: 'json',
            data: {
                saleTempId: saleTempId
            }
        });
    });
}

function saleHoldRequest(data, drawerId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'HoldSale/' + drawerId,
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function deleteHoldSale(drawerId, holdSaleId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'DELETE',
            url: window.API_BASE_URL + 'HoldSale/' + drawerId + '/' + holdSaleId
        });
    });
}

function restoreDeletedHoldSale(holdSaleId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'RestoreHoldSale/' + holdSaleId
        });
    });
}


function getRefundGiftItems(number, isTransaction, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        if(isTransaction) {
            return $.ajax({
                method: 'GET',
                url: window.API_BASE_URL + 'RefundItems/',
                data: {
                    PageNumber: pageNumber || 0,
                    PageSize: pageSize || 20,
                    transactionNumber: number || ''
                }
            });
        } else {
            return $.ajax({
                method: 'GET',
                url: window.API_BASE_URL + 'RefundGiftItems/',
                data: {
                    PageNumber: pageNumber || 0,
                    PageSize: pageSize || 20,
                    giftNumber: number || ''
                }
            });
        }

    });
}

//function getRefundItems(transactionNumber, pageNumber, pageSize) {
//    return NativeQueue.exec(function () {
//        if (pageNumber == 'last') {
//            return $.Deferred().promise();
//        }
//        return $.ajax({
//            method: 'GET',
//            url: window.API_BASE_URL + 'RefundItems/',
//            data: {
//                PageNumber: pageNumber || 0,
//                PageSize: pageSize || 100,
//                transactionNumber: transactionNumber || ''
//            }
//        });
//    });
//}

function getRefundProductServiceItems(transactionNumber, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'RefundProductServiceItems/',
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 100,
                transactionNumber: transactionNumber || ''
            }
        });
    });
}

function getRefundProductServiceItemsById(clientGlobalId, itemId, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'RefundProductServiceItems/',
            data: {
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 100,
                clientGlobalId: clientGlobalId || '',
                itemId: itemId || ''
            }
        });
    });
}

function saveClient(data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'SaveDeepClient',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getClientData(clientId) {
    return NativeQueue.exec(function () {
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'ClientDeep/' + clientId
        });
    });
}

function getVisitNoteTypes(filter, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'VisitNoteTypes',
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 100
            }
        });
    });
}

function sendEmailReceipt(data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'EmailReceipt',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getTransactions(filterInfo) {
    return NativeQueue.exec(function () {
        if (filterInfo.pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Transactions',
            data: {
                transactionNumber: filterInfo.transactionNumber || '',
                start: filterInfo.startDate,
                end: filterInfo.endDate,
                drawerId: filterInfo.drawerId || '',
                saleState: filterInfo.saleState || '',
                PageNumber: filterInfo.pageNumber || 0,
                PageSize: filterInfo.pageSize || 20
            }
        });
    });
}

function voidTransaction(data) {
    return NativeQueue.exec(function () {
        return $.ajax({
            type: 'POST',
            url: window.API_BASE_URL + 'VoidTransaction',
            dataType: 'json',
            contentType: 'application/json; charset=utf-8',
            data: data
        });
    });
}

function getDrawers(filter, pageNumber, pageSize) {
    return NativeQueue.exec(function () {
        if (pageNumber == 'last') {
            return $.Deferred().promise();
        }
        return $.ajax({
            method: 'GET',
            url: window.API_BASE_URL + 'Drawers',
            data: {
                filter: filter || '',
                PageNumber: pageNumber || 0,
                PageSize: pageSize || 100
            }
        });
    });
}
