define([
    'backbone'
], function (Backbone) {
    'use strict';

    return Backbone.Router.extend({
        before: function (params, next) {
            var path = Backbone.history.location.hash;
            if (path != '#login' && !sessionStorage.getItem('sessionId')) {
                Backbone.history.navigate('login', {trigger: true, replace: true});
            } else {
                return next();
            }
        },
        after: function () {
        },
        route: function (route, name, callback) {
            if (!_.isRegExp(route)) route = this._routeToRegExp(route);
            if (_.isFunction(name)) {
                callback = name;
                name = '';
            }
            if (!callback) callback = this[name];

            var router = this;

            Backbone.history.route(route, function (fragment) {
                var args = router._extractParameters(route, fragment);
                var next = function () {
                    callback && callback.apply(router, args);
                    router.trigger.apply(router, ['route:' + name].concat(args));
                    router.trigger('route', name, args);
                    Backbone.history.trigger('route', router, name, args);
                    router.after.apply(router, args);
                };
                router.before.apply(router, [args, next]);
            });

            return this;
        },
        changeView: function (view) {
            if (this.currentView === view) {
                this.currentView.refresh();
                return;
            }
            if (this.currentView) {
                this.currentView.hide();
            }
            this.currentView = view;
            if (!this.currentView.$$rendered) {
                this.currentView.render();
                this.currentView.$$rendered = true;
            }
            this.currentView.show();
        }
    });
});