define([
    'backbone',
    'jquery'
], function (Backbone, $) {
    'use strict';

    return Backbone.View.extend({

        events: {
            'change .pseudo-textarea': 'convertContentEditable',
            'blur .pseudo-textarea': 'convertContentEditable',
            'touchstart .button': 'touchStartButton', // from baseView
            'touchend .button': 'touchEndButton', // from baseView
            'focus [data-input-blur=true]': 'inputFocusOutside' // from baseView
        },

        convertContentEditable: function(e) {
            var $element = $(e.target);
            var inputId = '#' + $element.data('input-id');
            $element.html(this.getHtmlFromText(this.getTextFromHtml($(e.target))));
            if(inputId) {
                $(inputId).val(this.getTextFromHtml($element)).trigger('change');
            }
        },

        hide: function () {
            this.$el.find('input, select').prop('disabled', true);
            this.$el.addClass('hidden');
        },

        show: function () {
            this.$el.find('input, select').prop('disabled', false);
            this.$el.removeClass('hidden');
        },

        refresh: function () {
        },

        clearErrorOnBlur: function(e) {
            var elem = $(e.currentTarget);
            if (elem.val())
                this.removeError(elem);
        },

        removeError: function(elem) {
            elem.closest('.error').removeClass('error');
        },

        inputFocusOutside: function(e) {
            $(document).on('click.inputfocus tap.inputfocus', function(e) {
                if($(e.target).closest('[data-input-blur=true]').length == 0) {
                    $('[data-input-blur=true]:focus').blur();
                    $(document).off('click.inputfocus tap.inputfocus');
                }
            });
            e.preventDefault();
        },

        touchStartButton: function (e) {
            var $target = $(e.target);
            if(!$target.hasClass('.button')) {
                $target = $target.closest('.button');
            }
            if($target.hasClass('inactive')) return false;
            if ($(e.currentTarget).hasClass('disabled')) {
                e.preventDefault();
                return false;
            }
            $(':focus').blur();
            $target.addClass('active');
        },

        touchEndButton: function (e) {
            var $target = $(e.target);
            if(!$target.hasClass('.button')) {
                $target = $target.closest('.button');
            }
            $target.removeClass('active');
        },

        getTextFromHtml: function($pseudoTextarea) {
            var text = $pseudoTextarea.html();
            text = text.replace(/<div><br><div\/>/g, '');
            text = text.replace(/<\/div><div>/g, '\n');
            text = text.replace(/<div>/g, '\n');
            text = text.replace(/\n<div>/g, '');
            text = text.replace(/<\/div>/g, '');
            text = text.replace(/(\n<br>)+/g, '\n');
            text = text.replace(/(<br>)+/g, '\n');
            text = text.replace(/<br>$/g, '');
            return text;
        },

        getHtmlFromText: function(text) {
            return text.replace(/\n/g, '<br\/>');
        },

        close: function () {
            if (this.childViews) {
                this.childViews.close();
            }

            this.unbind();
            this.off(); // Unbind all local event bindings
            this.stopListening();
            this.model && this.model.off();
            this.remove(); // Remove view from DOM
            delete this.$el; // Delete the jQuery wrapped object variable
            delete this.el; // Delete the variable reference to this node
            delete this.model;
        },

        isValidInputLength: function($input) {
            if($input.val().length > 256) {
                return false;
            } else {
                return true;
            }
        }

    });
});