define([
    'jquery',
    'views/yes-no-modal',
    'views/client-modal',
    'text!../templates/search-plugin.html',
    'bridge'
], function(jQuery, YesNoModal, ClientModal, SearchPluginTemplate){
    'use strict';

    (function($){
        var defaults = {
            filter : '',
            list: [],
            prevSaved: '',
            scroll: undefined,
            pluginTemplate: _.template(SearchPluginTemplate),
            template: _.template('<div class="option" data-id="<%= id%>"><%= displayName %></div>'),
            large: false, // use large or fullscreen
            fullScreen: false,
            filterFunctionParam: undefined,
            emptyMessage: '',
            noCloseButton: false,
            pageNumber: 0,
            pageSize: 20,
            bottomButtonsFixed: false,
            manualSearch: false,
            onlyProperValues: false, // allow values only from the list
            selectedCallBack: undefined,
            HasDefaultItem: false,
            defaultItem: undefined
    };
		
		var searchTimerHandle = null;
		var requestNumber = 0;
		
        var methods = {
            _options: {},
            init : function(receivedOptions) {
                var options = _.extend({}, defaults, receivedOptions || {});
                var $parent = $(this).parent();
                var clientControlsTemplate = _.template('<div class="client-search-controls adding"><div class="client-search-controls-add button green">Add Client</div><div class="client-search-controls-edit button green">Edit</div></div>');
                if (!options.noCloseButton) {
                    $parent.prepend($('<div class="search-close-icon"></div>').toggleClass('visible', $(this).data('id') ? true : false));
                }
                if (options.clientControls) {
                    $parent.prepend(clientControlsTemplate());
                }
                $(this).addClass('search-plugin-input');
                var ctx = this[0];
                methods.setOptions(ctx, options);

                // This is makes autoopen of search dropdown.
                //if (options.HasDefaultItem && options.defaultItem)
                //    // TODO: Think how to replace this spike
                //    setTimeout(function() {
                //        methods.doneCallback(requestNumber, [options.defaultItem], options, ctx, '');
                //    }, 0);
                //else
                //    methods.filter(ctx, '', options);

                methods._options[ctx.id]['defaultItem'] = options.defaultItem;

                methods.scrollSetup(ctx, options);

                methods.resizeEvent = function () {
                    if ($parent.find('.search-plugin-box').hasClass('visible')) {
                        methods.changeToFullScreen(ctx, options, true);
                    } else {
                        methods.changeToFullScreen(ctx, options, false);
                    }
                };
                if(options.fullScreen) {
                    methods.changeToFullScreen(ctx, options);
                    $(window).on('orientationHandlerCallback', methods.resizeEvent);
                }

                function filter(e) {
                    setTimeout(function() { methods.filter(this, e.target.value, options); }.bind(this), 0);
                }

                this.on({
                    focus: function(e) {
                        $(this).removeClass('error');
                        methods.selectValue(this);
                        options.pageNumber = 0;

						$parent.find('.search-close-icon').addClass('visible');
						$('.search-plugin-box').removeClass('visible'); // hide all boxes to show only one at a time

                        if (options.HasDefaultItem) {
                            if (!options.defaultItem)
                                filter.call(this, e); // TODO find out how to fix :focus
                            else 
                                methods.doneCallback(requestNumber, [options.defaultItem], options, ctx, '');
                        } else
                            filter.call(this, e); // TODO find out how to fix :focus

                        if ($(e.target).data('id')) {
                            $(e.target).parent().find('.client-search-controls').removeClass('adding').addClass('editing');
                        }
                    },
                    blur: function (e) {
                        var $target = $(e.target);
                        if ($target.data('id')) {
                            var needle = _.findWhere(options.list, {id: $target.data('id')});
                            if (needle) {
                                if (!needle.displayName) {
                                    needle.displayName = needle['lastName'] + ', ' + needle['firstName'];
                                }
                                $target.val(needle.displayName);
                                options.prevValue = needle.displayName;
                            } 
                        } else if (options.onlyProperValues) {
                            $target.val('');
                        }

                        if ($target.val() === '') {
                            $target.parent().find('.search-close-icon').removeClass('visible');
                            $target.removeClass('error').closest('.form-group').removeClass('error');
                            $target.parent().find('.client-search-controls').addClass('adding').removeClass('editing');
                        }

                        options.parentScroll.enable();
                    },
                    input: function (e) {
                        $(this).data('changed', true);
                        if (options.prevValue === e.target.value)
                            return true;
						
						
                        if (searchTimerHandle !== null){
							clearTimeout(searchTimerHandle);
							searchTimerHandle = null;
						}
						
						searchTimerHandle = setTimeout(function () {
							options.prevValue = e.target.value;
							options.pageNumber = 0;
							methods.filter(this, e.target.value, options);
						}.bind(this), 450);
                    }
                });
				
				var box = $parent.find('.search-plugin-box');
				function getTarget(e){
					var $target = $(e.target);
                    if (!$target.hasClass('option'))
                        return $target.parent();
					return $target;
				}
				box.on({
                    touchstart: function (e) {
                        getTarget(e).addClass('active');
                    },
                    touchend: function (e) {
                        getTarget(e).removeClass('active');
                    }
                });

				$parent.on('tap', '.option', function (e) {
                    var $target = getTarget(e);
                    methods.hideSelectBox(this, options);
                    $(this).data('id', $target.data('id')).blur();
                    $parent.find('.search-close-icon').addClass('visible');

				    options.defaultItem = methods.getSelectedObject.apply($(ctx));

				    if (options.selectedCallBack)
				        options.selectedCallBack(options.defaultItem);
				    
				    this.trigger('optionSelected', options.defaultItem);
                }.bind(this));

                $parent.on('touchstart', '.search-close-icon', function (e) {
                    methods.hideSelectBox(ctx, options);
                    $(ctx).val('').data('id', '').blur();
                    $(e.target).removeClass('visible');
                    if(options.fullScreen) {
                        methods.resizeEvent();
                    }
                    this.trigger('search-clear');
                    this.trigger('search-close');
                    options.defaultItem = undefined;
                }.bind(this));

                $parent.on('touchend', '.client-search-controls-add', function () {
                    new ClientModal({
                        inputId: $(ctx).attr('id')
                    });
                }.bind(this));

                $parent.on('touchend', '.client-search-controls-edit', function () {
                    var clientId = $(ctx).data('id');
                    if (clientId) {
                        if (clientId == window.SETTINGS.walkInMale.id || clientId == window.SETTINGS.walkInFemale.id) {
                            new YesNoModal({
                                question: "This is a system object that can't be edited",
                                positiveText: 'Ok',
                                negativeText: '',
                                noCallback: true
                            }).render();
                             return false;
                        }
                        getClientData(clientId).done(function (data) {
                            new ClientModal(_.extend({
                                inputId: $(ctx).attr('id')
                            }, data));
                        }.bind(this));
                    } else {
                        return false;
                    }
                }.bind(this));

                return this;
            },

            changeToFullScreen: function(ctx, options, extended) {
                var $mainWrap = $('#main-wrapper'),
                    $scroll = $(options.parentScroll.wrapper).children('div').children('div'),
                    mHeight = $mainWrap.outerHeight() - $scroll.outerHeight() - 70;
                if(extended) {
                    mHeight += $(ctx).siblings('.search-plugin-box').outerHeight();
                }
                if (options.bottomButtonsFixed) {
                    //mHeight = $scroll.outerHeight() - 100; // buttons and inputs
                    mHeight -= 20;
                }
                if (options.clientControls && $(this).data('id')) {
                    mHeight -= 50;
                }
                $(ctx).siblings('.search-plugin-box').css({maxHeight: mHeight});

            },

            scrollSetup: function (ctx, options) {
                $(ctx).parent().append(options.pluginTemplate({
                    id: $(ctx).attr('id'),
                    large: options.large,
                    fullScreen: options.fullScreen
                }));
                options.scroll = new IScroll('#'+$(ctx).attr('id')+'-scroll', {
                    tap: true,
                    probeType: 2,
                    scrollbars: true
                });
                options.scroll.on('beforeScrollStart', function () {
                    this.refresh();
                    $(ctx).blur();
                });
                options.scroll.on('scroll', function () {
                    this._triggerRefresh = this.y > 30;
                    $(this.wrapper).toggleClass('active-refresh', this.y > 0);
                    if (options.pageNumber != 'last') {
                        this._triggerLoadNext = this.y < 0 && (this.y - this.maxScrollY) < -30;
                        this._scrollToY = this.y;
                        $(this.wrapper).toggleClass('active-loading', this.y < 0 && (this.y - this.maxScrollY) < 0);
                    }
                });
                options.scroll.on('beforeScrollEnd', function () {
                    $(this.wrapper).removeClass('active-refresh active-loading refreshing loading');
                    if (this._triggerRefresh) {
                        this.enabled = false;
                        this.hasVerticalScroll = false;
                        $(this.wrapper).addClass('refreshing');
                        this._execEvent('needRefresh');
                        return;
                    }
                    if (this._triggerLoadNext) {
                        this.enabled = false;
                        $(this.wrapper).addClass('loading');
                        this._execEvent('needNext');
                    }
                });
                options.scroll.on('needRefresh', function () {
                    // small delay to shown visual effect
                    setTimeout(function () {
                        this._triggerRefresh = false;
                        options.pageNumber = 0;
                        methods.filter(ctx, ctx.value, options, this);
                    }.bind(this), 300);
                });
                options.scroll.on('needNext', function () {
                    // small delay to shown visual effect
                    setTimeout(function () {
                        this._triggerLoadNext = false;
                        methods.filter(ctx, ctx.value, options, this);
                    }.bind(this), 300);
                });

                options.parentScroll.on('beforeScrollStart', function (e) {
                    if (e) {
                        if ($(e.target).closest('.search-plugin-box').length > 0) {
                            this.disable();
                        }
                    }
                });
                $(options.parentScroll.wrapper).on('touchstart', function (e) {
                    if ($(e.target).closest('.search-plugin-box').length == 0) {
                        options.parentScroll && options.parentScroll.enable();
                    }
                });
            },

            filter: function (ctx, filter, options, scroll) {
				requestNumber++;
				var request = requestNumber;

                if (options.manualSearch) {
                    var r = options.filterFunction(filter, methods._options[ctx.id]['filterFunctionParam']);
                    methods.manualSearchCallback(ctx, filter, options, scroll, r);
                } else {
                    options.filterFunction(filter,  methods._options[ctx.id]['filterFunctionParam'],
                        options.pageNumber, options.pageSize)
                        .done(function (r) {
                            methods.doneCallback(request, r, options, ctx, filter);
                        })
                        .always(function () {
                            methods.alwaysCallback(scroll);
                        });
                }
            },

            manualSearchCallback: function (ctx, filter, options, scroll, r) {
                options.list = _.uniq(_.flatten(r), function (item) {
                    return JSON.stringify(item);
                });
                if (!methods._options[ctx.id]) {
                    methods.setOptions(ctx, options);
                }
                methods._options[ctx.id]['list'] = options.list;
                methods.filterCallback(ctx, options, filter);
                methods.alwaysCallback(scroll);
            },

            doneCallback: function (request, r, options, ctx, filter) {
                if(request != requestNumber)
                    return;

                if (r.status === undefined) {
                    if (options.pageNumber === 0) {
                        options.list = r;
                    } else {
                        options.list.push(r);
                        options.list = _.uniq(_.flatten(options.list), function (item) {
                            return JSON.stringify(item);
                        });
                    }
                    options.pageNumber += 1;

                    if (!methods._options[ctx.id]) {
                        methods.setOptions(ctx, options);
                    }
                    methods._options[ctx.id]['list'] = options.list;
                } else {
                    options.list = [];
                    options.pageNumber = 'last';
                }
                methods.filterCallback(ctx, options, filter);
            },

            alwaysCallback: function (scroll) {
                if (scroll) {
                    scroll.enabled = true;
                    $(scroll.wrapper).removeClass('refreshing loading');
                }
            },

            filterCallback: function (ctx, options, filter){
                var $box = $(ctx).parent().find('.search-plugin-content'),
                    list = options.list,
                    $input = $(ctx);

                $box.empty();
                list = _.reject(list, function (listItem) {
					if(methods._options[ctx.id] !== undefined)
						return _.indexOf(methods._options[ctx.id].removeList, listItem.id) != -1;
					else return false;
                });
                
                $input.trigger('gotList', list.length);

                if (list.length) {
                    var opts = [];
                    _.each(list, function (p) {
                        opts.push(options.template(p));
                    });
                    $box.append(opts);
                    if ($input.is(':focus') || options.HasDefaultItem) {
                        methods.showSelectBox(ctx, options);
                    }
                    if (options.scroll) {
                        options.scroll.refresh();
                        if (options.pageNumber === 0) {
                            options.scroll.scrollTo(0, 0);
                        }
                    }
                } else {
                    if (filter === '' && options.emptyMessage !== '') {
                        $box.append('<p style="text-align: center;">'+ options.emptyMessage +'</p>');
                        if ($input.is(':focus')) {
                            methods.showSelectBox(ctx, options);
                        }
                    } else {
                        methods.hideSelectBox(ctx, options);
                    }
                    //$input.data('id', '');
                }
            },

            selectValue: function (ctx) {
                if ($(ctx).is('[readonly]')) {
                    e.preventDefault();
                    $(ctx).blur();
                } else {
                    setTimeout(function() {
                        ctx.setSelectionRange(0, 9999);
                    }, 0);
                }
            },

            showSelectBox: function(ctx, options) {
                var $box = $(ctx).parent().find('.search-plugin-content');
                var $parent = $(ctx).parent();
                $box.closest('.search-plugin-box').addClass('visible');
                $(document).on('tap.searchBox', function(e) {
                    if($(e.target).closest('.search-plugin-box').length == 0 && $(e.target).closest(ctx).length == 0) {
                        methods.hideSelectBox(ctx, options);
                        $(ctx).blur();
                    }
                });
                if (options.clientControls) {
                    $parent.find('.client-search-controls').show();
                }
            },

            hideSelectBox: function (ctx, options){
                var $parent = $(ctx).parent();
                $(ctx).parent().find('.search-plugin-box').removeClass('visible');
                options.parentScroll && options.parentScroll.enable();
                $(document).off('tap.searchBox');
                if (options.clientControls) {
                    $parent.find('.client-search-controls').hide();
                }
            },

            getOptions: function (ctx) {
                return methods._options[ctx[0]['id']];
            },

            setOptions: function (ctx, options) {
                var param = options ? options.filterFunctionParam : undefined;
                methods._options[ctx.id] = {
                    filterFunctionParam: param,
                    list: [],
                    removeList: [],
                    clientControls: options ? options.clientControls : undefined
                };
            },

            getList: function () {
                return methods._options[this[0].id]['list'];
            },

            getSelectedObject: function () {
                var list = methods._options[this[0].id]['list'];
                var goptions = methods.getOptions(this);
                return _.findWhere(list, { id: this.data('id') }) || (goptions || {}).defaultItem;
            },

            initializeAdding: function () {
                var ctx = this[0];

                methods.hideSelectBox(this, methods.getOptions(this));
                new ClientModal({
                    inputId: $(ctx).attr('id')
                });
            },

            changeFilterFunctionParam: function (newValue) {
				try {
					methods._options[this[0].id]['filterFunctionParam'] = newValue;
				}
				catch(err) {
                    //console.log(err.message);
                }
            },

            updateRemoveList: function (list) {
                if (!methods._options[this[0].id]) {
                    methods.setOptions(this[0]);
                }
                methods._options[this[0].id].removeList = list;
                return this;
            },

            destroy: function () {
                // this - all selected inputs
                var $parent;
                _.each(this, function (input) {
                    if ($(this).hasClass('search-plugin-input')) {
                        delete methods._options[input.id];
                        $(window).off('orientationHandlerCallback', methods.resizeEvent);
                        $(this).off('focus blur keyup');
                        $parent = $(this).parent();
                        $parent.off('tap touchstart touchend');
                        $parent.find('.search-plugin-box').off('touchstart touchend');
                    }
                });
                return this;
            }
        };

        $.fn.searchPlugin = function(methodOrOptions) {
            if ( methods[methodOrOptions] ) {
                return methods[ methodOrOptions ].apply( this, Array.prototype.slice.call( arguments, 1 ));
            } else if ( typeof methodOrOptions === 'object' || ! methodOrOptions ) {
                // Default to "init"
                return methods.init.apply( this, arguments );
            } else {
                $.error( 'Method ' +  methodOrOptions + ' does not exist on jQuery.searchPlugin' );
            }
        };
    })(jQuery);

});
