/**
 * Created by petermomot on 3/31/15.
 */
define([
    'backbone'
], function (Backbone) {
    'use strict';

    var CheckoutLineModel = Backbone.Model.extend({
        defaults: {
            isRefund: false,
            lineNumber: 1,
            paymentType: {},
            amount: 0
        }
    });

    var CheckoutCollection = Backbone.Collection.extend({
        model: CheckoutLineModel,

        beforeAdd: function (object) {
            var typeEnum = object.paymentType.paymentTypeEnum;
            var m = _.filter(this.models, function(obj) {
                if (typeEnum === 1462) {
                    return obj.get('paymentType').paymentTypeEnum === object.paymentType.paymentTypeEnum;
                } else if (typeEnum === 1448) {
                    return obj.get('paymentType').paymentTypeEnum === object.paymentType.paymentTypeEnum &&
                        obj.get('giftCard').id === object.giftCard.id;
                } else {
                    return obj;
                }
            });
            if (typeEnum === 1462 || typeEnum === 1448) { // cash
                if (m.length == 1) {
                    m = m[0];
                    m.set('amount', m.get('amount') + object.amount);
                    var gCard = m.get('giftCard');
                    if (gCard) {
                        gCard['remaining'] = gCard['value'] - gCard['totalUsages'] - m.get('amount');
                        m.set('giftCard', gCard);
                    }
                } else {
                    object.lineNumber = this.models.length + 1;
                    if (object['giftCard']) {
                        var remaining = object['giftCard']['value'] - object['giftCard']['totalUsages'];//
                        remaining = remaining - object['amount'];
                        object['giftCard']['remaining'] = +remaining.toFixed(2);
                    }
                    this.add(object);
                }
            } else {
                object.lineNumber = this.models.length + 1;
                this.add(object);
            }
        },

        getAmount: function () {
            return _.reduce(this.models, function (memo, m) {
                return memo + m.get('amount')
            }, 0);
        },

        getGiftCardsAmount: function () {
            var gCards = _.filter(this.models, function (m) {
                return m.get('giftCard');
            });
            return _.reduce(gCards, function (memo, m) {
                return memo + m.get('amount');
            }, 0);
        },

        updateLineNumbers: function () {
            _.each(this.models, function (model, index) {
                model.set('lineNumber', index+1);
            });
        },

        beforeCalc: function () {
            var models = this.toJSON();
            _.each(models, function (m) {
                delete m['id'];
            }.bind(this));
            return models;
        },

        updateModel: function (newModel) {
            var m = this.get(newModel['id']);
            m.set('amount', newModel['amount']);
            if (m.get('checkNumber')) {
                m.set('checkNumber', newModel['checkNumber']);
            }
        },

        beforeRemove: function(modelId) {
            this.remove(this.get(modelId));
        },

        checkSignature: function() {
            var models = this.toJSON(),
                isNeedSignature = false;
            _.each(models, function(elem){
                if(elem.paymentType.paymentTypeEnum === 1460) {
                    isNeedSignature = true;
                }
            });
            return isNeedSignature;
        },

        setSignature: function(signatureDots) {
            _.each(this.models, function(model){
                if(model.get('paymentType')['paymentTypeEnum'] === 1460) {
                    model.set('signatureCapture', signatureDots);
                }
            });
        }
    });

    return new CheckoutCollection;
});