/**
 * Created by petermomot on 3/20/15.
 */
define([
    'backbone'
], function (Backbone) {
    'use strict';

    var ClientModel = Backbone.Model.extend({
        defaults: {
            firstName: '',
            lastName: '',
            imageId: '',
            primaryPhoneNumber: '',
            primaryEmailAddress: '',
            enableClientAccountCharging: true,
            balance: 0,
            clientNoteSummary: '',
            clientNoteCount: 0,
            alerts : null,
            alertCount: 0
        },

        initialize: function (attributes, options) {
            var notes = this.get("clientNoteSummary").trim();
            this.set("clientNoteSummary", $('<div/>').html(notes).text());
        }
    });

    var ClientsCollection = Backbone.Collection.extend({
        model: ClientModel,

        getLastClientId: function () {
            var clients = this.toJSON();
            if (clients.length) {
                return clients[clients.length - 1].id;
            } else {
                return undefined;
            }
        },

        getLastSelectedClient: function (clientId, removed) {
            removed = removed ? 1 : 0;
            var clients = this.toJSON();
            if (clientId) {
                var client = _.findWhere(clients, { id: clientId });
                return client ? client : {};
            } else {
                if (this.length - removed > 0) {
                    return clients[clients.length - 1];
                } else {
                    return {};
                }
            }
        },

        getNonWalkInClientsIds: function () {
            return _.pluck(_.filter(this.toJSON(), function (client) {
                return client.id !== window.SETTINGS.walkInMale.id && client.id !== window.SETTINGS.walkInFemale.id;
            }), 'id');
        }
    });

    return new ClientsCollection;
});