/**
 * Created by petermomot on 2/17/15.
 */
define([
    'backbone'
], function (Backbone) {
    'use strict';

    var DrawerModel = Backbone.Model.extend({
        defaults: function () {
            return {
                id: '',
                isOpen: '',
                status: '',
                cashInDrawer: 0,
                actualCashAmount: 0,
                differenceAsPayIn: false,
                paymentTypeStates: [],
                paysInOut: [],
                paysInOutUpd: [],
                cashTotal: 0,
                checkTotal: 0,
                checkCount: 0,
                closeReason: '',
                totalAmountDifference: 0,
                totalActualAmount: 0
            }
        },

        initialize: function () {
            this.listenTo(this, 'change:paymentTypeStates', function () {
                var paymentTypeStates = this.get('paymentTypeStates');
                if (paymentTypeStates.length) {
                    this.set('cashTotal', this.get('cashInDrawer') + _.findWhere(paymentTypeStates, {'shortcut': 'CASH'})['totalPayments']);
                    this.set('checkTotal', _.findWhere(paymentTypeStates, {'shortcut': 'CHECK'})['totalPayments']);
                    this.set('checkCount', _.findWhere(paymentTypeStates, {'shortcut': 'CHECK'})['countPayments']);
                } else {
                    this.set('cashTotal', 0);
                    this.set('checkTotal', 0);
                    this.set('checkCount', 0);
                }
                this.set('totalAmountDifference', 0);
                this.set('totalActualAmount', 0);
                _.each(paymentTypeStates, function(elem) {
                    _.extend(elem, {
                        actualAmount: 0,
                        amountDifference: 0 - (elem.catDisplayName === 'Cash' ? this.get('cashTotal') : elem.totalPayments)
                    });
                    if (elem.catDisplayName === 'Gift Card') {
                        elem.actualAmount = -elem.amountDifference;
                        elem.amountDifference = 0;
                    }
                    this.set('totalAmountDifference', this.get('totalAmountDifference') + elem.amountDifference);
                }.bind(this));
                var paymentTypes = _.uniq(_.pluck(paymentTypeStates, 'catDisplayName'));
                var paymentStatesByType = {};
                _.each(paymentTypes, function(elem) {
                    paymentStatesByType[elem.toString()] = {
                        collection: _.where(paymentTypeStates, {catDisplayName: elem}),
                        getAmountDifference: function() {
                            var summ = 0;
                            _.each(this.collection, function(elem) {
                                summ += elem.amountDifference;
                            });
                            return summ;
                        }
                    };
                });
                this.set('paymentStatesByType', paymentStatesByType);
            });
            this.listenTo(this, 'change:paysInOut', function () {
                this.set('paysInOutUpd', _.map(this.get('paysInOut'), function (pay) {
                    if (pay['type'] === undefined) {
                        pay['type'] = pay['amount'] > 0 ? 'PayIn' : 'PayOut';
                    }
                    return pay;
                }.bind(this)));
            });
        }
    });

    return new DrawerModel;
});