﻿/**
 * Created by petermomot on 7/2/15.
 */
define([
    'backbone',
    'views/yes-no-modal'
], function (Backbone, YesNoModal) {
    'use strict';

    var HoldSaleModel = Backbone.Model.extend({
        defaults: {
            totalPrice: 0,
            saleTempId: '',
            dateHolded: '',
            payingClient: {
                firstName: '',
                lastName: ''
            },
            creator: {
                displayName: ''
            },
            deleted: false,
            sale: {}
        }
    });

    var OnHoldCollection = Backbone.Collection.extend({
        model: HoldSaleModel,
        saleLineEmployees: [],
        tipEmployees: {},

        getByTempId: function (tempId) {
            return this.findWhere({ saleTempId: tempId })
        },

        removeByTempId: function (tempId) {
            this.remove(this.getByTempId(tempId));
        },

        prepareForExtract: function (tempId) {
            
           
            getSaleOnHold(tempId).done(function (r) {
                if (r.status === undefined) {

                    if (!r.saleTempId) {
                        new YesNoModal({
                            question: "This on hold transaction is already finalized",
                            positiveText: 'Ok',
                            negativeText: '',
                            noCallback: true
                        }).render();

                        this.removeByTempId(tempId);
                        return false;
                    }

                    this.activeModel = this.getByTempId(tempId);
                    this.activeModel.set('sale', r);
                    var m = this.activeModel.get('sale');
                    m.clients = [];
                    m.customTips = [];
                    this.peopleFilterFunc(m.clients, _.pluck(m.saleLines, 'client'), 0);
                    this.trigger('readyToReturn', this.findCustomTips().toJSON().sale);
                }
            }.bind(this));
        },

        // recursive get of unique objects by id
        peopleFilterFunc: function (filtered, toFilter, index) {
            if (index == toFilter.length) return;
            if (!_.findWhere(filtered, { 'id': toFilter[index].id })) {
                filtered.push(toFilter[index]);
            }
            this.peopleFilterFunc(filtered, toFilter, index + 1);
        },

        findCustomTips: function () {
            var sale = this.activeModel.get('sale'),
                saleLinesEmployees = _.pluck(sale.saleLines, 'employee');

            sale.customTips = _.reject(sale.tips, function (t) {
                return _.findWhere(saleLinesEmployees, { id: t.employee.id });
            });

            return this.activeModel;
        }
    });

    return OnHoldCollection;

});