/**
 * Created by petermomot on 4/22/15.
 */
define([
    'backbone',
    'views/permissions-modal'
], function (Backbone, PermissionsModal) {
    'use strict';

    var PermissionsModel = Backbone.Model.extend({
        isVisible: function (attr) {
            var o = this.get(attr);
            return o ? o.isVisible : true
        },

        isElevatable: function (attr) {
            var o = this.get(attr);
            return o ? o.isElevatable : false
        },

        isReadonly: function (attr) {
            var o = this.get(attr);
            return o ? o.isReadonly : false
        },

        navigateToPermissionsModal: function (e, callback) {
            var securityKey = $(e.target).closest('[data-security-key]').data('security-key');
            if(this.isReadonly(securityKey)) {
                return false;
            }
            if (this.isElevatable(securityKey)) {
                new PermissionsModal({
                    user: sessionStorage.getItem('userFullName'),
                    securityKey: securityKey,
                    positiveText: 'OK',
                    negativeText: 'Cancel',
                    positiveCallback: callback,
                    negativeCallback: function () {
                        this.closeView();
                    }
                }).render();
                return;
            }
            callback();
        },

        getVisibleReceiptSubMenu: function () {
            var types = [];
            if (this.isVisible('evo.register.saletab')) {
                types.push({
                    url: 'sales',
                    title: 'Sale',
                    securityKey: 'evo.register.saletab'
                });
            }
            if (this.isVisible('evo.register.adjustmentstab')) {
                types.push({
                    url: 'adjustments',
                    title: 'Adjustment',
                    securityKey: 'evo.register.adjustmentstab'
                });
            }
            if (this.isVisible('evo.register.refundtab')) {
                types.push({
                    url: 'refund',
                    title: 'Refund',
                    securityKey: 'evo.register.refundtab'
                });
            }
            if (this.isVisible('evo.register.onholdtab')) {
                types.push({
                    url: 'onhold',
                    title: 'On Hold',
                    securityKey: 'evo.register.onholdtab'
                });
            }
            if (this.isVisible('evo.register.transactionstab')) {
                types.push({
                    url: 'transactions',
                    title: 'Transactions',
                    securityKey: 'evo.register.transactionstab',
                });
            }
            return types;
        },

        getVisibleSaleTypes: function () {
            var types = [];
            if (this.isVisible('evo.register.saletabservicestab')) {
                types.push({
                    url: 'services',
                    title: 'Services',
                    securityKey: 'evo.register.saletabservicestab'
                });
            }
            if (this.isVisible('evo.register.saletabproductstab')) {
                types.push({
                    url: 'products',
                    title: 'Products',
                    securityKey: 'evo.register.saletabproductstab'
                });
            }
            if (this.isVisible('evo.register.saletabgiftcardtab')) {
                types.push({
                    url: 'gift-sales',
                    title: 'Gift Card',
                    securityKey: 'evo.register.saletabgiftcardtab'
                });
            }
            if(window.SETTINGS.allowClientAccounts) {
                if (this.isVisible('evo.register.saletabaccounttab')) {
                    types.push({
                        url: 'account-sales',
                        title: 'Account',
                        securityKey: 'evo.register.saletabaccounttab'
                    });
                }
            }
            return types;
        },

        getVisiblePaymentTypes: function () {
            var types = [];
            if (this.isVisible('evo.register.paymenttabcashtab')) {
                types.push({
                    url: 'cash',
                    title: 'Cash',
                    securityKey: 'evo.register.paymenttabcashtab'
                });
            }
            if (this.isVisible('evo.register.paymenttabcreditdebittab')) {
                types.push({
                    url: 'credit',
                    title: 'Credit/Debit',
                    securityKey: 'evo.register.paymenttabcreditdebittab'
                });
            }
            if (this.isVisible('evo.register.paymenttabchecktab')) {
                types.push({
                    url: 'check',
                    title: 'Check',
                    securityKey: 'evo.register.paymenttabchecktab'
                });
            }
            if (this.isVisible('evo.register.paymenttabgiftcardtab')) {
                types.push({
                    url: 'gift',
                    title: 'Gift Card',
                    securityKey: 'evo.register.paymenttabgiftcardtab'
                });
            }
            if(window.SETTINGS.allowClientAccounts) {
                if (this.isVisible('evo.register.paymenttabaccounttab')) {
                    types.push({
                        url: 'account',
                        title: 'Account',
                        securityKey: 'evo.register.paymenttabaccounttab'
                    });
                }
            }
            types.push({
                url: 'other',
                title: 'Other',
                securityKey: 'evo.register.paymenttabothertab'
            });
            return types;
        },

        getVisibleRefundTypes: function () {
            var types = [];
            if (this.isVisible('evo.register.refundtabclientnametab')) {
                types.push({
                    url: 'client-name-refund',
                    title: 'Client Name',
                    securityKey: 'evo.register.paymenttabcashtab'
                });
            }
            if (this.isVisible('evo.register.refundtabtransactionnumbertab')) {
                types.push({
                    url: 'transaction-refund',
                    title: 'Transaction #',
                    securityKey: 'evo.register.paymenttabchecktab'
                });
            }
            if (this.isVisible('evo.register.refundtabgiftcardnumbertab')) {
                types.push({
                    url: 'giftcard-refund',
                    title: 'Gift Card #',
                    securityKey: 'evo.register.paymenttabcreditdebittab'
                });
            }
            return types;
        }
    });
    window.PERMISSIONS = new PermissionsModel();

    return window.PERMISSIONS;
});