/**
 * Created by petermomot on 3/20/15.
 */
define([
    'backbone',
    'data/consts'
], function (Backbone, Consts) {
    'use strict';

    var SaleLineModel = Backbone.Model.extend({
        defaults: {
            isRefund: false,
            lineNumber: 0,
            client: {},
            employee: {},
            item: {},
            appointType: {
                id: '',
                shortcut: '',
                displayName: '',
                description: ''
            },
            discounts: [],
            calcFullCommission: false,
            quantity: 1,
            price: 0
        }
    });

    var SaleLinesCollection = Backbone.Collection.extend({
        model: SaleLineModel,

        beforeCalc: function () {
			this.updateLineNumbers();
            var models = this.toJSON();
            _.each(models, function (line) {
                if (line.appointType.id === '') {
                    delete line['appointType'];
                }
            });
            return models;
        },

        updateLineNumbers: function () {
            _.each(this.models, function (model, index) {
                model.set('lineNumber', index+1);
            });
        },

        afterCalc: function (r) {
            _.each(this.models, function (model, index) {
                model.set(r['saleLines'][index], {add: false}); // fires 'add' event on collection
            });
        },

        checkOnRefundItems: function () {
            var refunds = this.where({ isRefund: true });
            return refunds.length > 0;
        },

        getServicesForClients: function (clients) {
            var lines = this.toJSON(),
                services = _.filter(lines, function (line) {
                    return (line.item.type === Consts.SaleLineTypes.SERVICE && _.indexOf(clients, line.client.id) !== -1)
                });
            return  _.groupBy(services, function (service) {
                return service.client.id;
            });
        }

    });

    return new SaleLinesCollection;
});