define([
    'jquery',
    'views/modal',
    'views/yes-no-modal',
    'text!../../templates/client-modal.html',
    'text!../../templates/visit-note.html',
    'bridge'
], function ($, ModalView, YesNoModal, Template, VisitNoteTemplate) {
    'use strict';

    var ClientModel = Backbone.Model.extend({
        defaults: {
            firstName: '',
            middleName: '',
            lastName: '',
            isActive: true,
            isRestricted: false,
            //gender: 0,
            birthday: '',
            addresses: [],
            emails: [],
            phones: [],
            visitNotes: []
        }
    });

    var NoteModel = Backbone.Model.extend({
        defaults: {
            employee: null,
            noteType: null,
            dateCreated: null,
            text: ''
        }
    });

    var VisitNotesCollection = Backbone.Collection.extend({
        model: NoteModel
    });

    return ModalView.extend({
        el: '#client-modal .content',
        template: _.template(Template),
        scrollbars: false,
        scrollWrap: '#client-modal-scroll-wrap',
        serviceTemplate: _.template('<div class="option" data-id="<%= id %>"><%= shortcut %> : <%= displayName %></div>'),
        employeeTemplate: _.template('<div class="option" data-id="<%= id %>"><%= code %> : <%= lastName %>, <%= firstName %></div>'),
        visitNoteTemplate: _.template(VisitNoteTemplate),
        visitNotes: new VisitNotesCollection(),
        isNew: true,

        initialize: function (clientData) {
            this.model = new ClientModel();
            this.inputId = clientData['inputId'];
            delete clientData['inputId'];
            if(clientData.filterVisitNotesBy) {
                this.filterVisitNotesBy = clientData.filterVisitNotesBy;
            }
            if(clientData) {
                this.isNew = false;

                var offset = window.SETTINGS.timeZoneOffset;
                _.each(clientData.visitNotes || [], function(visitNote) {
                    visitNote.dateCreated = moment.utc(visitNote.dateCreated.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset).format('L @ h:mm A');
                });

                this.model.set(clientData);
                if (clientData['showVisitNotes']) {
                    this.navToNotes = true;
                    delete clientData['showVisitNotes'];
                }
            }
            this.render();
        },

        events: _.extend({
            'change input, select': 'checkChanges',
            'input input': 'capitalizeFirstLetter',
            'touchend .client-modal-cancel, #client-modal-close .client-modal-button-close': 'closeView',
            'change #client-modal-selector': 'changeTab',
            'tap #add-visit-note': 'addNote',
            'change #client-form-month': 'renderDays',
            'change #client-form-year': 'renderDays',
            'touchend #client-modal-save': 'saveClientCallback',
            'optionSelected [id|=visit-note-employee]': 'noteEmployeeSelect',
            'optionSelected [id|=visit-note-type]': 'noteTypeSelect',
            'search-clear [id|=visit-note-employee]': 'noteEmployeeClear',
            'search-clear [id|=visit-note-type]': 'noteTypeClear',
            'optionSelected #filter-service-search': 'filterVisitNotes',
            'search-clear #filter-service-search': 'showAllVisitNotes',
            'change [id|=visit-note-text-input]': 'noteTextEdit',
            'tap .remove-visit-note': 'removeNote',
            'input input, [id|=visit-note-text]': 'removeError',
            'tap #client-form-empty-birth': 'removeBirth',
            'tap #client-form-empty-email': 'removeEmail',
            'tap #client-form-empty-phone': 'removePhone',
            'tap #client-form-empty-address': 'removeAddress'
        }, ModalView.prototype.events),

        render: function () {
            ModalView.prototype.render.call(this);

            this.$('#filter-service-search').searchPlugin({
                parentScroll: this.scroll,
                filterFunction: getVisitNoteTypes,
                template: this.serviceTemplate
            });

            this.$('#client-form-phone').inputFormat({
                type: 'int',
                allowEmpty: true,
                places: 10
            });

            if(!this.isNew) {
                this.renderData();
                this.renderVisitNotes();
                if ($('html').hasClass('mobile')) this.$('.client-modal-body').css("height", "calc(100% - 165px)");
            }

            this.scroll.refresh();

            return this;
        },

        closeView: function () {
            if (this.visitNotes) {
                this.visitNotes.reset();
                this.visitNotes = undefined;
            }
            if (this.model) {
                this.model.clear().set(this.model.defaults);
            }
            if(this.filterVisitNotesBy) {
                this.trigger('notesQueue');
            }
            ModalView.prototype.closeView.call(this);
        },

        capitalizeFirstLetter: function(e) {
            var $target = $(e.target);
            if(this.$('#client-form-email, #client-form-phone').filter(e.target).length) {
                return false;
            }
            if($target.val().length == 1) {
                $target.val($target.val().toUpperCase());
            }
        },

        checkChanges: function(e) {
            var val;
            if($(e.target).closest('.input-format').length) {
                val = $(e.target).inputFormat('getValue');
            } else {
                val = $(e.target).val();
            }
            if(val) {
                $(e.target).addClass('changed');
            } else {
                $(e.target).removeClass('changed');
            }
            this.removeError(e);
        },

        showAllVisitNotes: function() {
            this.$('.visit-note').removeClass('note-hidden');
        },

        filterVisitNotes: function(e, noteType) {
            if(this.visitNotes.length) {
                this.$('.visit-note').each(function(index, elem) {
                    if ($(elem).find('[id|=visit-note-type]').data('id') != noteType.id) {
                        $(elem).addClass('note-hidden');
                    } else {
                        $(elem).removeClass('note-hidden');
                    }
                });
            }
        },

        renderData: function() {
            // birthday
            if(this.model.get('birthday')) {
                this.$('#client-form-year').find('option[value=' + moment.utc(this.model.get('birthday')).format('YYYY') + ']').prop('selected', true);
                this.$('#client-form-month').find('option[value=' + moment.utc(this.model.get('birthday')).format('M') + ']').prop('selected', true);
                this.$('#client-form-day').find('option[value=' + moment.utc(this.model.get('birthday')).format('D') + ']').prop('selected', true);
            }

            // gender
            if(this.model.get('gender')) {
                this.$('#client-form-gender').find('option[value=' + this.model.get('gender') + ']').prop('selected', true);
            }

            // primary email
            if(this.model.get('emails') && this.model.get('emails').length) {
                var primaryEmail = _.findWhere(this.model.get('emails'), {
                    isPrimary : true
                });
                if(primaryEmail) {
                    this.$('#client-form-email-type').addClass('changed').find('option[value=' + primaryEmail.type + ']').prop('selected', true);
                    this.$('#client-form-email').val(primaryEmail.address || '').addClass('changed');
                }
            }

            // primary phone
            if(this.model.get('phones') && this.model.get('phones').length) {
                var primaryPhone = _.findWhere(this.model.get('phones'), {
                    isPrimary : true
                });
                if(primaryPhone) {
                    this.$('#client-form-phone-type').addClass('changed').find('option[value=' + primaryPhone.type + ']').prop('selected', true);
                    this.$('#client-form-phone').inputFormat('setValue', primaryPhone.number || '').addClass('changed');
                }
            }

            // primary address
            if(this.model.get('addresses') && this.model.get('addresses').length) {
                var primaryAddress = _.findWhere(this.model.get('addresses'), {
                    isPrimary : true
                });
                if(primaryAddress) {
                    this.$('#client-form-address-type').addClass('changed').find('option[value=' + primaryAddress.type + ']').prop('selected', true);
                    this.$('#client-form-address1').val(primaryAddress.address1 || '').addClass(primaryAddress.address1 ? 'changed' : '');
                    this.$('#client-form-address2').val(primaryAddress.address2 || '').addClass(primaryAddress.address2 ? 'changed' : '');
                    this.$('#client-form-city').val(primaryAddress.city || '').addClass(primaryAddress.city ? 'changed' : '');
                    this.$('#client-form-state').val(primaryAddress.state || '').addClass(primaryAddress.state ? 'changed' : '');
                    this.$('#client-form-zip').val(primaryAddress.zipCode || '').addClass(primaryAddress.zipCode ? 'changed' : '');
                    this.$('#client-form-country').val(primaryAddress.country || '').addClass(primaryAddress.country ? 'changed' : '');
                }
            }
        },

        renderEachNote: function(filterNoteType) {
            _.each(this.model.get('visitNotes'), function(data) {
                this.addNote(data, filterNoteType);
            }, this);
        },

        renderVisitNotes: function() {
            if(this.model.get('visitNotes') && this.model.get('visitNotes').length) {
                if(this.filterVisitNotesBy) {
                    getVisitNoteTypes().done(function(data) {
                        var currentNoteType = _.findWhere(data, {id: this.filterVisitNotesBy});
                        this.$('#filter-service-search').val(currentNoteType.displayName);
                        this.renderEachNote(currentNoteType);
                    }.bind(this));
                } else {
                    this.renderEachNote();
                }
            }
            if (this.navToNotes) {
                setTimeout(function () {
                    this.$('#client-modal-selector').val('#client-form-visit').change();
                }.bind(this), 0);
            }
        },

        renderDays: function() {
            var $daysSelect = this.$('#client-form-day');
            var $monthSelect = this.$('#client-form-month');
            var $yearSelect = this.$('#client-form-year');
            var days = $daysSelect.find('option').length - 1;
            var newDays;
            var i;

            if($monthSelect.val()) {
                if($yearSelect.val()) {
                    newDays = moment.utc($yearSelect.val() + "-" + $monthSelect.val(), "YYYY-MM").daysInMonth();
                } else {
                    newDays = moment.utc("2012-" + $monthSelect.val(), "YYYY-MM").daysInMonth();
                }
            }
            if(days < newDays) {
                for(i = days; i <= newDays; i++) {
                    $daysSelect.append('<option value="' +  i + '">' +  i + '</option>');
                }
            } else if(days > newDays) {
                for(i = 0; i < (days - newDays); i++) {
                    $daysSelect.find('option:last-child').remove();
                }
            }
        },

        changeTab: function(e) {
            this.$(e.target.value).show().siblings('.client-form-tab').hide();
            this.scroll.refresh();
        },

        addNote: function(data, filterNoteType) {
            var note;
            if(!data.target) {
                note = new NoteModel(data);
            } else {
                note = new NoteModel({
                    dateCreated: moment.utc(moment().toISOString()).format('L @ h:mm A'),
                    noteType: this.$('#filter-service-search').searchPlugin('getSelectedObject')
                });
            }
            var dataId = note.cid;
            var hidden = {hidden: false};
            if(filterNoteType) {
                hidden.hidden = data.noteType.id !== filterNoteType.id;
            }
            this.$('#visit-notes').prepend(this.visitNoteTemplate(_.extend({
                dataId: note.cid
            }, hidden, note.toJSON())));
            this.visitNotes.add(note);

            this.$('#visit-note-employee-' + dataId).searchPlugin({
                filterFunction: filterEmployeesWithParameter,
                parentScroll: this.scroll,
                template: this.employeeTemplate
            });
            this.$('#visit-note-type-' + dataId).searchPlugin({
                parentScroll: this.scroll,
                filterFunction: getVisitNoteTypes,
                template: this.serviceTemplate
            });
            if(!data.target) {
                this.$('#visit-note-text-' + dataId).html(this.getHtmlFromText(data.text));
                this.$('#visit-note-employee-' + dataId).data('id', data.employee.id).val(data.employee.lastName + ', ' + data.employee.firstName);
            }
            if(data.target && window.SETTINGS['userEmployee']) {
                this.$('#visit-note-employee-' + dataId).data('id', window.SETTINGS['userEmployee'].id).val(window.SETTINGS['userEmployee'].lastName + ', ' + window.SETTINGS['userEmployee'].firstName);
                this.visitNotes.get(dataId).set('employee', window.SETTINGS['userEmployee']);
            }
            if (note.get('noteType')) {
                this.$('#visit-note-type-' + dataId).data('id', note.get('noteType').id).val(note.get('noteType').displayName);
            }
        },

        removeNote: function(e) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.remove(dataId);
            this.$('#visit-note-'+ dataId).remove();
        },

        removeBirth: function() {
            this.$('#client-form-month, #client-form-day, #client-form-year').find('option:first-child').prop('selected', true);
        },

        removeEmail: function() {
            this.$('#client-form-email-type').removeClass('error changed').find('option:first-child').prop('selected', true);
            this.$('#client-form-email').val('').removeClass('error changed');
        },

        removePhone: function() {
            this.$('#client-form-phone-type').removeClass('error changed').find('option:first-child').prop('selected', true);
            this.$('#client-form-phone').inputFormat('setValue', '').closest('.input-format').removeClass('error changed');
        },

        removeAddress: function() {
            this.$('#client-form-address-type').removeClass('error changed').find('option:first-child').prop('selected', true);
            this.$('#client-form-address1').val('').removeClass('error changed');
            this.$('#client-form-address2').val('').removeClass('error changed');
            this.$('#client-form-city').val('').removeClass('error changed');
            this.$('#client-form-state').val('').removeClass('error changed');
            this.$('#client-form-zip').val('').removeClass('error changed');
            this.$('#client-form-country').val('').removeClass('error changed');
        },

        noteEmployeeSelect: function(e, employee) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.get(dataId).set('employee', employee);
            this.removeError(e);
        },

        noteTypeSelect: function(e, type) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.get(dataId).set('noteType', type);
            this.removeError(e);
        },

        noteEmployeeClear: function(e) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.get(dataId).unset('employee');
            this.removeError(e);
        },

        noteTypeClear: function(e) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.get(dataId).unset('noteType');
            this.removeError(e);
        },

        noteTextEdit: function(e) {
            var dataId = $(e.target).closest('.visit-note').data('id');
            this.visitNotes.get(dataId).set('text', $(e.target).val().replace(/&nbsp;/g, ' '));
        },

        saveName: function() {
            this.model.set('firstName', this.$('#client-form-first-name').val());
            this.model.set('middleName', this.$('#client-form-middle-name').val());
            this.model.set('lastName', this.$('#client-form-last-name').val());
        },

        saveBirthday: function() {
            var year = this.$('#client-form-year').val();
            var month = this.$('#client-form-month').val();
            var day = this.$('#client-form-day').val();
            if(year && month && day) {
                this.model.set('birthday', year + '-' + month + '-' + day);
            }
        },

        saveGender: function() {
            var gender = this.$('#client-form-gender').val();
            if(gender) {
                this.model.set('gender', gender);
            }
        },

        saveEmail: function() {
            var emailType = this.$('#client-form-email-type').val();
            var email = this.$('#client-form-email').val();
            var setEmail = function() {
                this.model.set('emails', [{
                    type: emailType,
                    isPrimary: true,
                    address: email
                }]);
            }.bind(this);

            if (emailType) {
                if (this.isNew || !this.model.get('emails').length) {
                    setEmail();
                } else {
                    var primaryEmail = _.findWhere(this.model.get('emails'), { isPrimary: true });
                    if (primaryEmail) {
                        primaryEmail.type = emailType;
                        primaryEmail.address = email;
                    } else {
                        setEmail();
                    }
                }
            } else {
                if (!this.deletePrimaryEntity(this.model.get('emails')))
                    return 'There must be at least one primary email, primary email will not be deteted';
            }
            return '';
        },
        deletePrimaryEntity: function(array) {
            if (array.length == 1) {
                var index = _.findIndex(array, function(e) { return e.isPrimary; });
                array.splice(index, 1);
                return true;
            }
            else if(array.length == 0)
                return true;
            else
                return false;
        },
        savePhone: function() {
            var phoneType = this.$('#client-form-phone-type').val();
            var phone = this.$('#client-form-phone').inputFormat('getValue');
            var setPhone = function() {
                this.model.set('phones', [{
                    type: phoneType,
                    isPrimary: true,
                    number: phone
                }]);
            }.bind(this);

            if (phoneType) {
                if (this.isNew || !this.model.get('phones').length) {
                    setPhone();
                } else {
                    var primaryPhone = _.findWhere(this.model.get('phones'), { isPrimary: true });
                    if (primaryPhone) {
                        primaryPhone.type = phoneType;
                        primaryPhone.number = phone;
                    } else {
                        setPhone();
                    }
                }
            } else {
                if (!this.deletePrimaryEntity(this.model.get('phones')))
                    return 'There must be at least one primary phone, primary phone will not be deteted';;
            }

            return '';
        },

        saveAddress: function() {
            var addressType = this.$('#client-form-address-type').val();
            var address1 = this.$('#client-form-address1').val();
            var address2 = this.$('#client-form-address2').val();
            var city = this.$('#client-form-city').val();
            var state = this.$('#client-form-state').val();
            var zip = this.$('#client-form-zip').val();
            var country = this.$('#client-form-country').val();
            var setAddress = function() {
                this.model.set('addresses', [{
                    isPrimary: true,
                    type: addressType,
                    address1: address1,
                    address2: address2,
                    city: city,
                    state: state,
                    zipCode: zip,
                    country: country
                }]);
            }.bind(this);
            if (addressType) {
                if (this.isNew || !this.model.get('addresses').length) {
                    setAddress();
                } else {
                    var primaryAddress = _.findWhere(this.model.get('addresses'), { isPrimary: true });
                    if (primaryAddress) {
                        primaryAddress.type = addressType;
                        primaryAddress.address1 = address1;
                        primaryAddress.address2 = address2;
                        primaryAddress.city = city;
                        primaryAddress.state = state;
                        primaryAddress.zipCode = zip;
                        primaryAddress.country = country;
                    } else {
                        setAddress();
                    }
                }
            } else {
                if (!this.deletePrimaryEntity(this.model.get('addresses')))
                    return 'There must be at least one primary address, primary address will not be deteted';
            }

            return "";
        },

        saveVisitNotes: function() {
            this.model.set('visitNotes', this.visitNotes.toJSON());
        },

        isValidVisitNotes: function() {
            var valid = true;
            this.visitNotes.forEach(function(element) {
                var dataId = element.cid;
                if(!element.get('employee')) {
                    this.$('#visit-note-employee-' + dataId).addClass('error');
                    valid = false;
                }
                if(!element.get('noteType')) {
                    this.$('#visit-note-type-' + dataId).addClass('error');
                    valid = false;
                }
                if(!element.get('text').trim()) {
                    this.$('#visit-note-text-' + dataId).addClass('error');
                    valid = false;
                }
            }, this);
            return valid;
        },

        isValidFields: function($fields, required, $notRequiredFields) {
            var validate = function() {
                var isValid = true;
                $fields.each(function(index, element) {
                    if(!$(element).hasClass('changed')) {
                        isValid = false;
                        if($(element).closest('.input-format').length) {
                            $(element).closest('.input-format').addClass('error');
                        } else {
                            $(element).addClass('error');
                        }
                    }
                });
                return isValid;
            };

            if(required || $fields.filter('.changed').length != 0 || ($notRequiredFields && $notRequiredFields.filter('.changed').length)) {
                return {
                    valid: validate(),
                    finished: false
                };
            } else {
                $fields.each(function(index, element) {
                    if($(element).closest('.input-format').length) {
                        $(element).closest('.input-format').removeClass('error');
                    } else {
                        $(element).removeClass('error');
                    }
                });
            }
            return {
                valid: true,
                finished: true
            };
        },

        isValid: function() {
            var valid = true;
            this.$('[required]').each(function(index, elem) {
                if(!$(elem).val()) {
                    $(elem).addClass('error');
                    valid = false;
                }
            });

            if(!this.isValidFields(this.$('#client-form-email-type, #client-form-email')).finished) {
                if(!/^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$/.test(this.$('#client-form-email').val())) {
                    this.$('#client-form-email').addClass('error');
                    valid = false;
                } else {
                    this.$('#client-form-email').removeClass('error');
                }
            }

            if(!this.isValidFields(this.$('#client-form-phone-type, #client-form-phone'), window.SETTINGS.requireClientPhoneNumber).finished) {
                if(this.$('#client-form-phone').val().length < 10) {
                    this.$('#client-form-phone').closest('.input-format').addClass('error');
                    valid = false;
                } else {
                    this.$('#client-form-phone').closest('.input-format').removeClass('error');
                }
            }

            var validAddress = this.isValidFields(this.$('#client-form-address-type, #client-form-address1, #client-form-city, #client-form-state, #client-form-zip'), false, this.$('#client-form-address2, #client-form-country')).valid;
            valid = valid ? validAddress : false;

            var $errors = this.$('.error');
            if ($errors.length) this.scroll && this.scroll.scrollToElement($errors.closest('.client-modal-form-row')[0]);

            return valid;
        },

        removeError: function(e) {
            $(e.target).closest('.error').removeClass('error');
        },

        saveClientCallback: function() {
            var validData = this.isValid();
            var validNotes = this.isValidVisitNotes();
            var warnmessages = [];
            if (validData && validNotes) {
                this.saveName();
                this.saveBirthday();
                this.saveGender();
                warnmessages.push(this.saveEmail());
                warnmessages.push(this.savePhone());
                warnmessages.push(this.saveAddress());

                warnmessages = _.reject(warnmessages, function (str) { return str == ''; });

                var messagesQueueCallBack = function () {
                    if (!warnmessages.length)
                        return;

                    var index = _.findIndex(warnmessages, function (e) { return e != ''; });
                    var message = warnmessages[index];
 
                    var yesNo = new YesNoModal({
                        question: message,
                        positiveText: 'Ok',
                        negativeText: '',
                        positiveCallback: function () {
                            yesNo.closeView();
                            warnmessages = warnmessages.length > 1 ? warnmessages.slice(index + 1) : [];
                            messagesQueueCallBack();
                        }.bind(this)
                    }).render();
                }.bind(this);

                messagesQueueCallBack();

                this.saveVisitNotes();
                saveClient(JSON.stringify({ Client: this.model.toJSON() })).done(function (r) {
                    if (r.status === undefined && this.inputId) {
                        var callbackName = this.isNew ? 'clientCreated' : 'clientEdited';
                        $('#' + this.inputId).trigger(callbackName, _.extend(this.model.toJSON(), r));
                    }
                    
                    this.closeView();
                }.bind(this));
            } else {
                if (!validData && this.$('#client-form-general:visible').length == 0) {
                    this.$('#client-modal-selector').val('#client-form-general').change();
                } else if (!validNotes && this.$('#client-form-visit:visible').length == 0) {
                    this.$('#client-modal-selector').val('#client-form-visit').change();
                }
            }
        }

    });
});