define([
    'common/base-view',
    'views/widget',
    'text!../../templates/desktop.html'
], function (BaseView, WidgetView, desktopTemplate) {
    'use strict';
    return BaseView.extend({
        tagName: 'div',
        className: 'desktop-scroll',
        template: _.template(desktopTemplate),

        childViews: null,

        initialize: function () {
            this.childViews = {};
            this.rotateEventHandler = this.rotateEvent.bind(this);

            this.listenTo(this.model.widgets, 'add', this.onWidgetAdded);
            this.listenTo(this.model.widgets, 'change', this.onWidgetChanged);
            this.listenTo(this.model.widgets, 'remove', this.onWidgetRemoved);
            this.listenTo(this.model.widgets, 'sort', this.onWidgetsSorted);
        },

        onWidgetAdded: function (widget, list) {
            var view = new WidgetView({model: widget});
            this.childViews[widget.id] = view;
            this.$('.desktop').append(view.render().$el);

            view.$('.tile-text').quickfit({
                min: minFontSizeTileText,
                max: maxFontSizeTileText,
                truncate: true
            });
        },

        onWidgetChanged: function (widget, list) {
            var widgetToUpdateFrom = _.find(list, function (el) { return el.id == widget.id; });
            if (widgetToUpdateFrom) {
                var viewToChange = this.childViews[widget.id];
                if (viewToChange) {
                    viewToChange.model.set('displayValue', widgetToUpdateFrom.displayValue);
                    viewToChange.render();
                    viewToChange.$('.tile-text').quickfit({
                        min: minFontSizeTileText,
                        max: maxFontSizeTileText,
                        truncate: true
                    });
                }
            }
        },

        onWidgetRemoved: function (widget, list) {
            this.childViews[widget.id].close();
            delete this.childViews[widget.id];
        },

        onWidgetsSorted: function (list) {
            list.each(function (widget, i) {
                if (i == 0) {
                    this.$('#' + widget.get('id')).prependTo(this.$('.desktop'));
                } else {
                    this.$('#' + widget.get('id')).insertAfter(this.$('#' + list.at(i - 1).get('id')));
                }
            }.bind(this));
        },

        render: function () {
            var that = this;

            $(window).on('orientationHandlerCallback', this.rotateEventHandler);

            this.$el.html(this.template(this.model.toJSON()));

            this.$el.data('scroll', this.scroll = new IScroll(this.el, {
                tap: true,
                probeType: 2
            })
            .on('beforeScrollStart', function () {
                this.refresh();
            }).on('scroll', function () {
                $(this.wrapper).toggleClass('active-refresh', this.y > 0);
                if (this._triggerRefresh = this.y > 40) {
                    this.y = 40;
                }
            }).on('beforeScrollEnd', function () {
                $(this.wrapper).removeClass('active-refresh refreshing');
                if (this._triggerRefresh) {
                    this.enabled = false;
                    this.hasVerticalScroll = false;
                    this.y = 0;
                    $(this.wrapper).addClass('refreshing');
                    this._execEvent('needRefresh');
                }
            }).on('needRefresh', function () {
                that.refreshData().always(function () {
                    $(this.wrapper).removeClass('refreshing');
                    this._triggerRefresh = false;
                    this.enabled = true;
                    this.scrollTo(0, 0, this.options.bounceTime, this.options.bounceEasing);
                }.bind(this));
            }));

            this.model.widgets.each(function (widget, i, list) {
                this.onWidgetAdded(widget, list);
            }.bind(this));

            return this;
        },

        close: function (views, args) {
            $(window).off('orientationHandlerCallback', this.rotateEventHandler);

            this.scroll.destroy();
            delete this.scroll;

            _.each(this.childViews, function (view) {
                view.close();
            });
            delete this.childViews;

            BaseView.prototype.close.call(this);
        },

        rotateEvent: function () {
            this.scroll && this.scroll.refresh();
        },

        refreshData: function () {
            return window.app.getDesktop(this.model.get('id'));
        }

    });
});
