/**
 * Created by petermomot on 3/9/15.
 */
define([
    'jquery',
    'inputFormat',
    'views/yes-no-modal',
    'text!../../../templates/drawer/drawer-amount-left-modal.html'
], function ($, inputFormat, YesNoModal, DrawerAmountLeftTemplate) {
    'use strict';

    return YesNoModal.extend({
        el: '#drawer-amount-left-modal .content',
        scrollWrap: '#drawer-amount-left-scroll-wrap',
        template: _.template(DrawerAmountLeftTemplate),

        initialize: function (options) {
            this.model = new Backbone.Model(options);
            this.listenTo(this.model, 'change:amountLeft', function () {
                this.model.set('cashToDeposit', this.model.get('actualCash') - this.model.get('amountLeft'));
                if (this.model.get('cashToDeposit') < 0) {
                    this.model.set('cashToDeposit', 0);
                }
            });
            this.listenTo(this.model, 'change:cashToDeposit', function () {
                this.$('#cash-to-deposit').html($.global.format(this.model.get('cashToDeposit'), 'c'));
            });
            if (this.model.get('difference') < 0) {
                this.model.set('amountLeft', this.model.get('actualCash'));
            } else {
                this.model.set('amountLeft', this.model.get('defaultLeft'));
            }
            this.events = _.extend({}, YesNoModal.prototype.events, this.events);
        },

        events: {
            'blur #amount-left': 'changeAmountLeft'
        },

        render: function () {
            YesNoModal.prototype.render.call(this);
            this.$('#amount-left').inputFormat();
            return this;
        },

        changeAmountLeft: function (e) {
            this.model.set('amountLeft', $(e.target).inputFormat('getValue'));
        },

        tapButton: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();

            if (e.target.classList.contains('cancel')) {
                this.closeView();
                return false;
            }
            this.model.set('amountLeft', this.$('#amount-left').inputFormat('getValue'));
            if (this.model.get('amountLeft') <= this.model.get('actualCash')) {
                this.trigger('closeDrawer', {
                    cashLeftInDrawer: this.model.get('amountLeft'),
                    depositCashAmount: this.model.get('cashToDeposit'),
                    depositCheckAmount: this.model.get('actualCheck')
                });
                this.closeView();
            } else {
                new YesNoModal({
                    question: 'Amount left in drawer cannot be greater than actual cash amount',
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
            }
        }
    });
});
