/**
 * Created by petermomot on 2/18/15.
 */
define([
    'jquery',
    'moment',
    'common/base-view',
    'data/permissions',
    'views/modal',
    'text!../../../templates/drawer/drawer-history.html',
    'text!../../../templates/drawer/drawer-history-item-mobile.html',
    'text!../../../templates/drawer/drawer-history-item-tablet.html',
    'bridge'
], function ($, moment, BaseView, PermModel, ModalView, HistoryTemplate, HistoryItemMobileTemplate, HistoryItemTabletTemplate) {
    'use strict';

    var HistoryPopupModel = Backbone.Model.extend({
        defaults: {
            'isTablet': false
        }
    });

    var HistoryModel = Backbone.Model.extend({
        defaults: {
            'id': '',
            'opened': '',
            'closed': '',
            'openedBy': '',
            'closedBy': '',
            'openNote': '',
            'closeNote': '',
            'cashInDrawer': '',
            'differenceTotal': '',
            'openedClass' : '',
            'closedClass' : ''
        },

        initialize: function () {
            var offset = window.SETTINGS.timeZoneOffset;
            this.set('openedUpd', moment.utc(this.get('opened').replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset).format('L h:mm A'));
            if (this.get('closed') === '') {
                this.set('closedUpd', '');
            } else {
                this.set('closedUpd', moment.utc(this.get('closed').replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset).format('L h:mm A'));
            }

            if (this.get('cashInDrawer') !== '') {
                if($.global.format(this.get('cashInDrawer'), 'd') > 0) {
                    this.set('openedClass', 'positive');
                } else if($.global.format(this.get('cashInDrawer'), 'd') < 0) {
                    this.set('openedClass', 'negative');
                }
                this.set('cashInDrawer', $.global.format(this.get('cashInDrawer'), 'c'));
            }

            if (this.get('differenceTotal') !== '') {
                if($.global.format(this.get('differenceTotal'), 'd') > 0) {
                    this.set('closedClass', 'positive');
                } else if($.global.format(this.get('differenceTotal'), 'd') < 0) {
                    this.set('closedClass', 'negative');
                }
                this.set('differenceTotal', $.global.format(this.get('differenceTotal'), 'c'));
            }
        }
    });

    var HistoryCollection = Backbone.Collection.extend({
        model: HistoryModel
    });

    var HistoryItemView = Backbone.View.extend({
        className: 'history-item',
        initialize: function (options) {
            this.template = options.isTablet ? _.template(HistoryItemTabletTemplate) : _.template(HistoryItemMobileTemplate);
        },
        render: function () {
            this.$el.html(this.template(this.model.toJSON()));
            return this;
        }
    });

    var history = new HistoryCollection();
    var popupModel = new HistoryPopupModel();

    return ModalView.extend({
        el: '#drawer-history-modal .content',
        model: popupModel,
        collection: history,
        isTablet: $('.device.tablet').is(':visible'),
        template: _.template(HistoryTemplate),
        scrollWrap: '#history-scroll-wrap',
        childViews: [],
        $appendTo: '',
        collectionLoaded: false,
        pageNumber: 0,
        pageCount: 20,
        requestInProcess: false,

        initialize: function (options) {
            this.workstationId = options.workstationId;
            this.model.set('isTablet', this.isTablet);
            this.listenTo(this.collection, 'add', this.historyItemAdded);
            this.rotateEventHandler = this.setHeight.bind(this);
            $(window).on("orientationHandlerCallback", this.rotateEventHandler);
            this.events = _.extend({}, BaseView.prototype.events, this.events);
        },

        events: {
            'touchend #drawer-back': 'drawerBack'
        },

        render: function () {
            ModalView.prototype.render.call(this);

            this.$appendTo = this.$('.history-items');
            this.getData();

            return this;
        },

        renderEmpty: function () {
            this.$appendTo.html('<div class="empty">History for this drawer is empty</div>');
        },

        initializeIScroll: function () {
            // TODO add loading spinners
            $(document).on('touchmove', preventTouchForIScroll);
            var that = this;
            this.scroll = new IScroll(that.scrollWrap, {
                tap: true,
                scrollbars: 'custom'
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
            this.scroll.on('scrollEnd', function () {
                if (that.requestInProcess) {
                    return;
                }
                if (this.y - this.maxScrollY < 180 && that.pageNumber != 'last') {
                    that.getData();
                }
            });
        },

        getData: function () {
            this.requestInProcess = true;
            getDrawerHistory(this.workstationId, this.pageNumber, this.pageCount)
                .done(this.onDataGet.bind(this))
                .always(function () {
                    this.requestInProcess = false;
                }.bind(this));
        },

        onDataGet: function (data) {
            if (data.length == 0) {
                this.pageNumber = 'last';
                if (this.collection.length == 0) {
                    this.renderEmpty();
                }
            } else {
                this.collection.add(data);
                this.pageNumber += 1;
            }
        },

        historyItemAdded: function (item) {
            if (!this.collectionLoaded) {
                this.$('.history-items').empty();
                this.collectionLoaded = true;
            }
            var itemView = new HistoryItemView({isTablet: this.isTablet, model: item}).render();
            this.$appendTo.append(itemView.$el);
            this.childViews.push(itemView);
            this.scroll.refresh();
        },

        removeAllItems: function () {
            // TODO remove all models from memory

            _.each(this.childViews, function (view) {
                view.remove();
            });
            delete this.childViews;
            this.childViews = [];
        },

        drawerBack: function (e) {
            // this.closeView();
            PermModel.navigateToPermissionsModal(e, function () {
                window.history.back(); // return to drawer
            }.bind(this));
        },

        closeView: function (e, route) {
            if (e) {
                e.preventDefault();
                e.stopPropagation();
            }
            this.removeAllItems();
            this.collection.reset();
            $(document).off('touchmove', preventTouchForIScroll);
            ModalView.prototype.closeView.call(this, e, route);
        }
    })
});