define([
    'jquery',
    'common/base-view',
    '../modal',
    'text!../../../templates/drawer/drawer-mobile-transactions.html',
    'text!../../../templates/drawer/drawer-states-history.html',
    'bridge'
], function ($, BaseView, ModalView, TransactionsTemplate, DrawerStatesHistory) {
    'use strict';

    return ModalView.extend({
        el: '#transactions-modal .content',
        scrollWrap: '#drawer-transactions-modal-scroll-wrap',
        scrollbars: false,
        template: _.template(TransactionsTemplate),
        transactions: new Backbone.Collection(),

        initialize: function (options) {
            this.model = new Backbone.Model(options);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        events: {
            'tap #close-drawer-transactions': 'onCloseTransactionsClick',
            'tap .pay-history-button': 'onSetOneOkClick'
        },

        render: function () {
            ModalView.prototype.render.call(this);
            this.showPaymentsStates();
            return this;
        },

        generateTransactionList: function(elements) {
            var list = $('<ul/>');
            var offset = window.SETTINGS.timeZoneOffset;
            var historyItemTemplate = _.template(DrawerStatesHistory);
            _.each(elements, function (elem) {
                elem.displayName = this.model.get('displayName');
                elem.date = moment.utc(elem.date.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset);
                list.append(historyItemTemplate(elem));
            }.bind(this));
            $('#generated-transactions').html(list);
        },

        onCloseTransactionsClick: function(e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.trigger('checkedTransaction', this.transactions);
            this.closeView();
        },

        showPaymentsStates: function () {
            var elements = this.model.get('paymentsState');
            this.generateTransactionList(elements);
        },

        onSetOneOkClick: function (e) {
            var $checkBtn = $(e.target),
                id  = $checkBtn.data('payment-id');
            var pto = this.model.get('paymentsState');
            var singleAmount = _.where(pto, {'id': id})[0];

            if (!pto) return;

            if($checkBtn.hasClass('checked')) {
                singleAmount.checked = false;
                $checkBtn.removeClass('checked');
            } else {
                singleAmount.checked = true;
                $checkBtn.addClass('checked');
            }
            this.transactions = pto;
        }
    });
});