/**
 * Created by petermomot on 2/17/15.
 */
define([
    'jquery',
    'inputFormat',
    'moment',
    'common/base-view',
    'data/permissions',
    'data/drawer-payment-states',
    'text!../../../templates/drawer/drawer.html',
    'text!../../../templates/drawer/pay-in-out.html',
    'views/drawer/greater-amount-modal',
    'views/drawer/payment-modal',
    'views/drawer/drawer-close-diff-modal',
    'views/drawer/drawer-amount-left-modal',
    'views/drawer/drawer-mobile-transactions',
    'views/yes-no-modal',
    'text!../../../templates/drawer/drawer-states-accordion-wrapper.html',
    'text!../../../templates/drawer/drawer-states-accordion-item.html',
    'text!../../../templates/drawer/drawer-states-right.html',
    'text!../../../templates/drawer/drawer-states-history.html',
    'bridge'
], function ($, inputFormat, moment,
             BaseView, PermModel, PaymentStatesCollection,
             DrawerTemplate, PayInOutTemplate,
             GreaterAmountModal, PaymentModal, DrawerCloseDiffModal,
             DrawerAmountLeftModal, TransactionModal, YesNoModal,
             DrawerStatesWrapper, DrawerStatesItem, DrawerStatesRight, DrawerStatesHistory) {
    'use strict';

    return BaseView.extend({
        el: '.drawer',
        template: _.template(DrawerTemplate),
        payInOutTemplate: _.template(PayInOutTemplate),
        greaterAmountModal: undefined,
        paymentModal: undefined,
        closeDiffModal: undefined,
        openTransactionModal: undefined,
        transactionId: undefined,
        amountLeftModal: undefined,
        yesNoModal: undefined,
        actualAmountHelper: 0,

        onRotateHandler: null,
        scroll: null,
        summaryScroller: [],

        currentPaymentTypeObject: null,

        initialize: function () {
            this.onRotateHandler = this.onRotate.bind(this);

			this.unlockHandler = function(){
				this.unlockDrawer.apply(this).always(function() {
					Backbone.history.off('route', this.unlockHandler );
				}.bind(this));
			}.bind(this);

            this.listenTo(this.model, 'change:status', function () {
                this.$('.drawer-status').html(this.model.get('status'));
            });
            this.listenTo(this.model, 'change:isOpen', function () {
                var isOpen = this.model.get('isOpen');
                this.$el.toggleClass('opened', isOpen);
                this.$el.toggleClass('closed', !isOpen);
            });
            this.listenTo(this.model, 'change:cashInDrawer', function () {
                var cash = $.global.format(this.model.get('cashInDrawer'), 'c');
                this.$('#expected-cash').html(cash);
                this.$('#opening-cash').html(cash);
                this.calcDifference();
            });
            this.listenTo(this.model, 'change:actualCashAmount', function () {
                var actualCash = this.model.get('actualCashAmount');
                this.$('#actual-cash').inputFormat('setValue', actualCash);
                var $actAmInput = this.$('#actual-amount-input');
                if ($actAmInput.length) {
                    $actAmInput.inputFormat('setValue', actualCash);
                }
                this.calcDifference();
            });
            this.listenTo(this.model, 'change:cashTotal', function () {
                this.$('#cash-total').html($.global.format(this.model.get('cashTotal'), 'c'));
            });
            this.listenTo(this.model, 'change:checkTotal', function () {
                this.$('#check-total').html($.global.format(this.model.get('checkTotal'), 'c'));
            });
            this.listenTo(this.model, 'change:checkCount', function () {
                this.$('#check-count').html(this.model.get('checkCount'));
            });
            this.listenTo(this.model, 'change:paysInOutUpd', function () {
                var renderedPays = [];
                _.each(this.model.get('paysInOutUpd'), function (pay) {
                    renderedPays.push(this.payInOutTemplate(_.extend(pay, {PermModel: PermModel})));
                }.bind(this));
                this.$('#pays').html(renderedPays);
            });
            this.listenTo(this.model, 'change:paymentTypeStates', function () {
                this.buildAccordion();
                this.model.set('totalPayments', 0);
                _.each(this.model.get('paymentTypeStates'), function (elem) {
                    this.model.set('totalPayments', this.model.get('totalPayments') + elem.totalPayments);
                }, this);
            });
            this.listenTo(this.model, 'change:totalAmountDifference', function () {
                var difference = this.model.get('totalAmountDifference');
                var $summaryDifference = $('#summary-difference');
                $summaryDifference.text($.global.format(difference, 'c')).removeClass('positive negative');
                if (difference > 0) {
                    $summaryDifference.addClass('positive');
                } else if (difference < 0) {
                    $summaryDifference.addClass('negative');
                }
            });
            this.listenTo(this.model, 'change:totalActualAmount', function () {
                $('#summary-total').text($.global.format(this.model.get('totalActualAmount'), 'c'));
            });

            this.events = _.extend({}, BaseView.prototype.events, this.events);
        },

        events: {
            'tap .drawer-history': 'showDrawerHistory',
            'tap #drawer-open-btn': 'onOpenDrawerClick',
            'tap #drawer-close-btn': 'onCloseDrawerClick',
            'tap #show-transactions': 'onShowTransactionsClick',
            'tap #close-drawer-save, #open-drawer-save': 'drawerSubmit',
            'tap #close-drawer-cancel, #open-drawer-cancel': 'onCancelCloseClick',
            'touchend #open-drawer-save' : 'drawerSubmit',
            'touchend #open-drawer-cancel' : 'onCancelCloseClick',
            'blur #actual-cash': 'actualCashBlur',
            'tap a.header': 'accordion',
            'tap .drawer-summary-item': 'showPaymentsStates',
            'tap #drawer-pay-done': 'hidePaymentsStates',
            'tap #set-all-ok .icon': 'onSetAllOkClick',
            'tap .pay-history-button': 'onSetOneOkClick',
            'change #actual-amount-input': 'onActualAmountChange',
            'blur #actual-amount-input': 'onActualAmountBlur',
            'tap .button-pay-in, .button-pay-out': 'createPay',
            'tap .button-edit': 'updatePay',
            'tap .button-del': 'deletePay',
            'touchend #drawer-back': 'drawerBack'
        },

        render: function () {
            this.$el.empty().html(this.template(_.extend(this.model.toJSON(), {PermModel: PermModel})));
            this.$('input[type="text"], input[type="number"]').inputFormat();

            this.scroll = new IScroll(this.$('.drawer-form').get(0), {
                tap: true,
                disableMouse: true
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
            this.$('.summary-scroller').each(function (index, el) {
                if (!this.summaryScroller[index]) {
                    this.summaryScroller[index] = new IScroll(el, {
                        tap: true
                    });
                    this.summaryScroller[index].on('beforeScrollStart', function () {
                        var scrollTop = $(el).scrollTop();
                        if (scrollTop !== 0) {
                            this.scrollTo(0, scrollTop*-1);
                            $(el).scrollTop(0);
                        }
                        this.refresh();
                    });
                }
            }.bind(this));

            return this;
        },

        show: function () {
            $(document).on('touchmove', preventTouchForIScroll);
            BaseView.prototype.show.call(this);
            $(window).on("orientationHandlerCallback", this.onRotateHandler);
            this.model.set('id', JSON.parse(localStorage.getItem('workstation'))['drawerId']);
            this.refresh();
        },

        hide: function () {
            $(document).off('touchmove', preventTouchForIScroll);
            BaseView.prototype.hide.call(this);
            $(window).off("orientationHandlerCallback", this.onRotateHandler);
            this.$('input[type="text"]').val('').trigger('change');
            this.actualCashBlur();
        },

        refresh: function () {
            this.getDrawerState(this.model.get('id'), true);
            this.refreshUI();
        },

        drawerBack: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                window.history.back(); // return to register menu
            }.bind(this));
        },

        refreshUI: function () {
            this.$('>.active').removeClass('active');
            this.$('.drawer-form').addClass('active');
            this.$('.drawer-back-button').show();
            this.$('.pseudo-textarea').html('');
            this.hidePaymentsStates();
            this.scroll.scrollTo(0, 0);
        },

        setLoading: function (loading) {
            this.$el.toggleClass('loading', loading === true);
        },

        getDrawerState: function (drawerId, loading) {
            this.setLoading(loading);
            getWorkstationDrawerState(drawerId).done(function (data) {
                this.model.set(data);
                this.model.set('actualCashAmount', 0);
                this.setLoading(true);
                getPaysInOut(data.id).done(function (data) {
                    var offset = window.SETTINGS.timeZoneOffset;
                    this.model.set('paysInOut', _.map(data || [], function (payment) {
                        payment.date = moment.utc(payment.date.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset);
                        return payment;
                    }));
                    this.refreshUI();
                }.bind(this)).always(this.setLoading.bind(this));
            }.bind(this)).always(this.setLoading.bind(this));
        },

        showDrawerHistory: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                Backbone.history.navigate('drawer-history/' + this.model.get('id'), { trigger: true });
            }.bind(this));
        },

        onOpenDrawerClick: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.lockDrawer().add(function (data) {

                    getWorkstationDrawerState(this.model.get('id')).done(function (data) {
                        if (data.isOpen) {

                            new YesNoModal({
                                question: 'The state of this drawer has been modified.  Please review the current state.',
                                positiveText: 'OK',
                                negativeText: '',
                                positiveCallback: function () {
                                    Backbone.history.navigate('register', { trigger: true, replace: true });
                                }
                            }).render();
                            
                        } else {
                            this.model.set('startEditDate', moment.utc().utcOffset(window.SETTINGS.timeZoneOffset).format());
                            this.model.set('differenceAsPayIn', false);
                            this.model.set('actualCashAmount', 0);
                            this.calcDifference();
                            this.$('#drawer-notes').empty();

                            this.$('>.active').removeClass('active');
                            this.$('.drawer-back-button').hide();
                            this.$('.drawer-summary-wrapper.closed').addClass('active');
                            _.each(this.summaryScroller, function (scroll) { scroll.scrollTo(0, 0); });
                        }
                    }.bind(this)).always(this.setLoading.bind(this));

                }.bind(this));
            }.bind(this));
        },

		lockDrawer: function(){
			var callbacks = $.Callbacks("memory stopOnFalse" );

			callbacks.add(function(data){
					if(data) {
						// this.setLoading.bind(this);
						this.setLoading(false);
						new YesNoModal({
							question: data.message,
							positiveText: 'OK',
							noCallback: true,
							negativeText: '',
							positiveCallback: function () {

							}.bind(this)
						}).render();
						return false;
					}
					else Backbone.history.on('route', this.unlockHandler );
				}.bind(this));

			lockUnlockDrawer(this.model.get('id'), true)
				.done(callbacks.fire)
				.fail(this.setLoading.bind(this));

			return callbacks;
		},

		unlockDrawer: function(){
			return lockUnlockDrawer(this.model.get('id'), false)
		},

		showCloseDrawer: function () {
		    this.setLoading(true);
		    getWorkstationDrawerState(this.model.get('id')).done(function (data) {
		        if (!data.isOpen) {

		            new YesNoModal({
		                question: 'The state of this drawer has been modified.  Please review the current state.',
		                positiveText: 'OK',
		                negativeText: '',
		                positiveCallback: function () {
		                    Backbone.history.navigate('register', { trigger: true, replace: true });
		                }
		            }).render();
		        } else {
		            this.model.set('startEditDate', moment.utc().utcOffset(window.SETTINGS.timeZoneOffset).format());
		            this.$('>.active').removeClass('active');
		            this.$('.drawer-back-button').hide();
		            this.$('.drawer-summary-wrapper.opened').addClass('active');
		            this.$('.drawer-summary-item:first').trigger('tap', [false]);
		            this.scroll.scrollTo(0, 0);
		            _.each(this.summaryScroller, function (scroll) {
		                scroll.scrollTo(0, 0);
		            });
		        }
		    }.bind(this)).always(this.setLoading.bind(this));
		},

		onCloseDrawerClick: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                var self = this;
		        this.setLoading(true);

		        this.lockDrawer().add(function (data) {

		            getSalesOnHold().done(function (data) {
		                if (data.length) {
		                    new YesNoModal({
		                        question: 'There are currently transactions on hold. ' +
                                'Would you like to view them before closing your drawer?',
		                        positiveText: 'Yes',
		                        positiveCallback: function () {
		                            Backbone.history.navigate('smart-receipt/showHold', { trigger: true, replace: true });
		                        },
		                        negativeText: 'No',
		                        negativeCallback: function () {
		                            this.closeView();
		                            self.showCloseDrawer();
		                        }
		                    }).render();
		                } else self.showCloseDrawer();
		            });
                    // .always(this.setLoading.bind(this));

		        }.bind(this));
		    }.bind(this));
        },

        onCancelCloseClick: function (e) {
			this.unlockDrawer().always(function(data) {
			    Backbone.history.off('route', this.unlockHandler);
			    if (e.target.id.indexOf('close') == 0) {
			        if ($('html').hasClass('tablet')) {
			            this.$('.drawer-summary-item:first').trigger('tap');
			        }
			        this.getDrawerState(this.model.get('id'), false);
			    }
				this.refreshUI();
			}.bind(this));
        },

        drawerSubmit: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                var diffObject = this.actualCashBlur();
                $(':focus').blur();
                if (this.model.get('isOpen')) {
                    if (diffObject.diffAmount != 0) {
                        this.closeDiffModal = new DrawerCloseDiffModal(
                            _.extend(diffObject, {closeReason: this.model.get('closeReason')})
                        ).render();
                        this.differenceModalListener();
                    } else {
                        this.showAmountLeftModal();
                    }
                } else {
                    
                    if (diffObject.diffClass == 'more') {
                        this.openGreaterAmountModal();
                    } else if (diffObject.diffClass == 'less') {
                        this.yesNoModal = new YesNoModal({
                            question: 'The Actual Cash Amount entered is lower than Expected Cash Amount. Would you like to save with this discrepancy?',
                            positiveText: 'Yes',
                            positiveCallback: function () {
                                this.openDrawer();
                                this.yesNoModal.closeView();
                            }.bind(this),
                            negativeText: 'No',
                            negativeCallback: function () {
                                this.closeView();
                            }
                        }).render();
                    } else {
                        this.openDrawer();
                    }
                }
            }.bind(this));
        },

        openDrawer: function () {
            var data = JSON.stringify({
                'drawerState': {
                    'startEditDate': this.model.get('startEditDate'),
                    'actualCashAmount': this.model.get('actualCashAmount'),
                    'differenceAsPayIn': this.model.get('differenceAsPayIn'),
                    'reason': this.$('#drawer-notes').val(),
                    'cashAudit': {
                        'reason': ''
                    }
                }
            });
            this.setLoading(true);
            openDrawer(this.model.get('id'), data).always(function () {
                this.setLoading(false);
                this.model.set('paysInOut', []);
                Backbone.history.navigate('register', {trigger: true, replace: true});
            }.bind(this));
        },

        closeDrawer: function () {
            var data = {
                "drawerState": {
                    'startEditDate': this.model.get('startEditDate'),
                    "cashAudit": {
                        "reason": "String"
                    },
                    "cashLeftInDrawer": this.model.get('cashLeftInDrawer'),
                    "depositCashAmount": this.model.get('depositCashAmount'),
                    "depositCheckAmount": this.model.get('depositCheckAmount'),
                    "reason": this.model.get('closeReason'),
                    "paymentTypeStates": this.model.get('paymentTypeStates')
                }
            };
            this.setLoading(true);
            closeDrawer(this.model.get('id'), JSON.stringify(data)).always(function () {
                this.setLoading(false);
                this.model.set('paysInOut', []);
                Backbone.history.navigate('register', {trigger: true, replace: true});
            }.bind(this));
        },

        actualCashBlur: function () {
            if (this.model.get('isOpen') == false) {
                this.model.set('actualCashAmount', this.$('#actual-cash').inputFormat('getValue'));
            }
            return this.calcDifference();
        },

        calcDifference: function () {
            var diff = this.model.get('actualCashAmount') - this.model.get('cashInDrawer'),
                diffClass = '';
            if (this.model.get('isOpen')) {
                diff = this.model.get('totalAmountDifference');
            }
            if (diff > 0) {
                diffClass = 'more';
            } else if (diff < 0) {
                diffClass = 'less';
            }
            this.$('#difference-cash')
                .html($.global.format(diff, 'c'))
                .closest('.form-group').removeClass('less more').addClass(diffClass);
            return {
                diffAmount: diff,
                diffClass: diffClass
            };
        },

        openGreaterAmountModal: function () {
            this.greaterAmountModal = new GreaterAmountModal().render();
            this.listenToOnce(this.greaterAmountModal, 'asPayInEvent', function (e) {
                this.greaterAmountModal = undefined;
                this.model.set('differenceAsPayIn', e);
                this.openDrawer();
            }.bind(this));
        },

        differenceModalListener: function () {
            this.listenToOnce(this.closeDiffModal, 'returnReason', function (diffObject) {
                this.closeDiffModal = undefined;
                this.model.set('closeReason', diffObject['reason']);
                if (diffObject['_continue']) {
                    this.showAmountLeftModal();
                }
            }.bind(this));
        },

        showAmountLeftModal: function () {
            this.amountLeftModal = new DrawerAmountLeftModal({
                actualCash: this.model.get('actualCashAmount'),
                actualCheck: this.model.get('checkTotal'),
                defaultLeft: window.SETTINGS['defaultCashLeftInDrawer'],
                difference: this.model.get('actualCashAmount') - window.SETTINGS['defaultCashLeftInDrawer'] // actualCash - defaultLeftValue
            }).render();
            this.listenTo(this.amountLeftModal, 'closeDrawer', function (closeObject) {
                delete this.amountLeftModal;
                this.model.set('cashLeftInDrawer', closeObject['cashLeftInDrawer']);
                this.model.set('depositCashAmount', closeObject['depositCashAmount']);
                this.model.set('depositCheckAmount', closeObject['depositCheckAmount']);
                this.closeDrawer();
            }.bind(this));
        },

        paymentListener: function (type, callback) {
            this.listenToOnce(this.paymentModal, 'sendObject', function (json) {
                this.paymentModal = undefined;
                if (!_.isEmpty(json)) {
                    addUpdatePayInOut(type, this.model.get('id'), JSON.stringify(json)).done(function (data) {
                        if (json.payment.type == 'PayOut') {
                            json.payment.amount = Math.abs(json.payment.amount) * -1;
                        }
                        //callback(json.payment);
                        this.getDrawerState(this.model.get('id'), false);
                    }.bind(this));
                    if (json.payment.type == 'PayOut') {
                        json.payment.amount = Math.abs(json.payment.amount) * -1;
                    }
                }
            }.bind(this));
        },

        createPay: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.paymentModal = new PaymentModal({
                    type: e.target.classList.contains('button-pay-in') ? 'PayIn' : 'PayOut',
                    cashInDrawer: this.model.get('cashTotal')
                }).render();
                this.paymentListener('PUT', function (r) {
                    this.model.set('paysInOut', _.union(this.model.get('paysInOut'), [r]));
                }.bind(this));
            }.bind(this));
        },

        updatePay: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                var payment = _.findWhere(this.model.get('paysInOutUpd'), {'id': $(e.target).data('id')});
                this.paymentModal = new PaymentModal(
                    _.extend(payment, {
                        cashInDrawer: this.model.get('cashTotal')
                    })
                ).render();
                this.paymentListener('POST', function (r) {
                    this.model.set('paysInOut', _.union(_.reject(this.model.get('paysInOut'), {id: r.id}), [r]));
                }.bind(this));
            }.bind(this));
        },

        deletePay: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                var paymentId = $(e.target).data('id');
                this.yesNoModal = new YesNoModal({
                    question: 'Are you sure you want to delete?',
                    positiveText: 'Yes',
                    positiveCallback: {
                        triggerEvent: 'yesNoEvent',
                        sendProperty: true
                    },
                    negativeText: 'No',
                    negativeCallback: {
                        triggerEvent: 'yesNoEvent',
                        sendProperty: false
                    }
                }).render();
                this.listenToOnce(this.yesNoModal, 'yesNoEvent', function (response) {
                    this.yesNoModal = undefined;
                    if (response) {
                        deletePayInOut(this.model.get('id'), paymentId).always(function () {
                            this.model.set('paysInOut', _.reject(this.model.get('paysInOut'), {id: paymentId}));
                            this.getDrawerState(this.model.get('id'), false);
                        }.bind(this));
                    }
                }.bind(this));
            }.bind(this));
        },

        buildAccordion: function () {
            var wrapperTemplate = _.template(DrawerStatesWrapper);
            var itemTemplate = _.template(DrawerStatesItem);
            $('#drawer-states-accordion').html(_.map(this.model.get('paymentStatesByType'), function (elem, key) {
                var accordionElement = $('<div/>').attr('class', 'drawer-summary-accordion');
                var accordionItems = $('<ul/>');
                var totalDifference = 0;
                _.each(elem.collection, function (innerElem) {
                    totalDifference += innerElem.amountDifference;
                    accordionItems.append(itemTemplate(innerElem));
                });
                accordionElement.append(wrapperTemplate({
                    title: key,
                    itemsCount: elem.collection.length,
                    amountDifference: elem.getAmountDifference()
                }));
                accordionElement.append(accordionItems);
                return (elem.collection.length > 1) ? accordionElement : accordionItems;
            }));
        },

        accordion: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            var self = this;
            var $parent = $(e.target).closest('.drawer-summary-accordion');
            if ($parent.hasClass('opened')) {
                $parent.removeClass('opened').find('ul').slideUp(function () {
                    self.summaryScroller[0].refresh();
                });
            } else {
                $parent.siblings('.drawer-summary-accordion').has('.header').removeClass('opened').find('ul').slideUp();
                $parent.addClass('opened').find('ul').slideDown(function () {
                    self.summaryScroller[0].refresh();
                });
            }
        },

        setAccordionValues: function (valueType) {
            var pto = this.currentPaymentTypeObject;
            if (!pto) {
                return;
            }

            var $relatedAccordionItem = this.$('.drawer-summary-item[data-payment-id=' + pto.id + ']');

            var itemHtmlNode = $relatedAccordionItem.find('.' + valueType);

            itemHtmlNode.text($.global.format(pto[valueType], 'c'));
            if (valueType === 'amountDifference') {
                if (parseFloat(pto[valueType]).toFixed(2) > 0) {
                    itemHtmlNode.removeClass('negative').addClass('positive');
                } else if (parseFloat(pto[valueType]).toFixed(2) < 0) {
                    itemHtmlNode.removeClass('positive').addClass('negative');
                } else {
                    itemHtmlNode.removeClass('negative positive');
                }
            }
            $relatedAccordionItem.parent().siblings('.header').find('.accordion-header-' + valueType).each(function (i, el) {
                var summary = _.reduce(this.model.get('paymentStatesByType')[pto.catDisplayName].collection, function (memo, elem) {
                    return memo + elem[valueType];
                }, 0).toFixed(2);
                var headerHtmlNode = $(el);
                headerHtmlNode.text($.global.format(+summary, 'c'));
                if (valueType === 'amountDifference') {
                    if (parseFloat(summary).toFixed(2) > 0) {
                        headerHtmlNode.removeClass('negative').addClass('positive');
                    } else if (parseFloat(summary).toFixed(2) < 0) {
                        headerHtmlNode.removeClass('positive').addClass('negative');
                    } else {
                        headerHtmlNode.removeClass('negative positive');
                    }
                }
            }.bind(this));

            var totalAmount = 0;
            var totalDifference = 0;
            _.each(this.model.get('paymentTypeStates'), function (elem) {
                totalAmount += elem.actualAmount;
                totalDifference += elem.amountDifference;
            });
            this.model.set('totalAmountDifference', totalDifference);
            this.model.set('totalActualAmount', totalAmount);
        },

        onShowTransactionsClick: function() {
            var pto = this.currentPaymentTypeObject;
            this.openTransactionModal = new TransactionModal({
                paymentsState: pto.paymentsState.toJSON(),
                displayName: pto.displayName
            }).render();
            this.listenTo(this.openTransactionModal, 'checkedTransaction', this.onSetResponseOk);
        },

        generateTransactionList: function(elements, displayName) {
            var check = false;
            var offset = window.SETTINGS.timeZoneOffset;
            var list = $('<ul/>');
            var historyItemTemplate = _.template(DrawerStatesHistory);
            _.each(elements, function (elem) {
                elem.displayName = displayName;
                elem.date = moment.utc(elem.date.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset);
                list.append(historyItemTemplate(elem));
                check = true;
            });
            $('#drawer-pay-history').html(list);
            return check;
        },

        showPaymentsStates: function (e, activate) {
            this.$('.drawer-summary-item').removeClass('selected');
            $(e.target).closest('.drawer-summary-item').addClass('selected');
            var checkViewTransactions = false;
			setTimeout(function(){
				var id = $(e.target).closest('.drawer-summary-item').data('payment-id');
                this.transactionId = id;
				this.currentPaymentTypeObject = _.where(this.model.get('paymentTypeStates'), {'id': id})[0];

				getPaymentsState(this.model.get('id'), id).done(function (data) {
					var pto = this.currentPaymentTypeObject;
					if (!pto) return;

					if (!pto.paymentsState) {
					    pto.paymentsState = new PaymentStatesCollection(data)
					} else {
					    pto.paymentsState.add(data, { merge: true });
					}

					if (activate !== false) {
						this.$('.list.drawer-summary').removeClass('active');
						this.$('.form.drawer-summary').addClass('active');
					}

					pto.actualAmount = pto.actualAmount || 0;
					pto.amountDifference = pto.amountDifference || 0;

					this.$('#drawer-states-right').html(_.template(DrawerStatesRight)(_.extend({
						cashTotal: this.model.get('cashTotal'),
						PermModel: PermModel,
						allChecked: pto.paymentsState.isAllChecked()
					}, pto)));

				    if(!this.generateTransactionList(pto.paymentsState.toJSON(), pto.displayName)) {
                        $('#show-transactions').hide();
                    }

				    this.$('#actual-amount-input').inputFormat();
				    
				    if (pto.displayName == "Gift Card") {
				        pto.paymentsState.setChecked(true);
				        this.$('#actual-amount-input').hide();
				    }

					this.summaryScroller[1].scrollTo(0, 0);
					this.summaryScroller[2].refresh();
					this.summaryScroller[2].scrollTo(0, 0);
				}.bind(this));
			}.bind(this),100);
        },

        hidePaymentsStates: function (e) {
            if (e) {
                e.preventDefault();
                e.stopPropagation();
                e.stopImmediatePropagation();
            }
            this.$('.list.drawer-summary').addClass('active');
            this.$('.form.drawer-summary').removeClass('active');
        },

        onActualAmountChange: function (e) {
            var pto = this.currentPaymentTypeObject;
            if (!pto) {
                return;
            }

            pto.actualAmount = $(e.currentTarget).inputFormat('getValue');
            pto.amountDifference = (pto.actualAmount - (pto.shortcut == 'CASH' ? this.model.get('cashTotal') : pto.totalPayments)).toFixed(2) * 1;
            if (pto.shortcut === 'CASH') {
                this.model.set('actualCashAmount', pto.actualAmount);
            }

            var amountDiffClass = '';
            if(pto.amountDifference > 0) amountDiffClass = 'positive';
            else if(pto.amountDifference < 0) amountDiffClass = 'negative';

            this.$('#amount-difference').html($.global.format(pto.amountDifference, 'c'));
            this.$('#amount-difference').removeClass('negative positive').addClass(amountDiffClass);

            this.setAccordionValues('amountDifference');
            this.setAccordionValues('actualAmount');
        },

        onActualAmountBlur: function (e) {
            if ($(e.target).inputFormat('getValue') === this.currentPaymentTypeObject.paymentsState.getTotal(true)) {
                return;
            }
            this.actualAmountHelper = $(e.target).inputFormat('getValue');
            this.currentPaymentTypeObject.paymentsState.setChecked(false);
            this.$('#set-all-ok, .pay-history-button').removeClass('checked');
        },

        onSetAllOkClick: function () {
            var $setAllOkBtn = this.$('#set-all-ok');
            var pto = this.currentPaymentTypeObject;

            if (!pto) return;

            if($setAllOkBtn.hasClass('checked')) {
                pto.paymentsState.setChecked(false);
                this.actualAmountHelper = 0;
                this.$('.pay-history-button').removeClass('checked');
                this.$('#actual-amount-input').inputFormat('setValue', 0);
                $setAllOkBtn.removeClass('checked');
            } else {
                pto.paymentsState.setChecked(true);
                this.$('.pay-history-button').addClass('checked');
                this.$('#actual-amount-input').inputFormat('setValue', pto.paymentsState.getTotal());
                $setAllOkBtn.addClass('checked');
            }
        },

        onSetOneOkClick: function (e) {
            var $checkBtn = $(e.target),
                id  = $checkBtn.data('payment-id'),
                pto = this.currentPaymentTypeObject.paymentsState,
                actualCashAmount = $('#actual-amount-input').inputFormat('getValue'),
                singleAmount = pto.get(id);

            if (!pto) return;

            if($checkBtn.hasClass('checked')) {
                this.$('#actual-amount-input').inputFormat('setValue', actualCashAmount - singleAmount.get('amount'));
                singleAmount.set('checked', false);
                $checkBtn.removeClass('checked');
            } else {
                this.$('#actual-amount-input').inputFormat('setValue', actualCashAmount + singleAmount.get('amount'));
                singleAmount.set('checked', true);
                $checkBtn.addClass('checked');
            }

            this.isAllOk(pto);
        },

        isAllOk: function (paymentsStateCollection) {
            var isAllOk = paymentsStateCollection.isAllChecked();
            if(isAllOk) {
                this.$('#actual-amount-input').inputFormat('setValue', paymentsStateCollection.getMaxAmount());
                this.$('#set-all-ok').addClass('checked');
                this.actualAmountHelper = 0;
            } else {
                this.$('#set-all-ok').removeClass('checked');
            }
        },

        recalculateTransactions: function() {
            if (!this.currentPaymentTypeObject) return;
            var total = this.currentPaymentTypeObject.paymentsState.getTotal(true);
            this.$('#actual-amount-input').inputFormat('setValue', this.actualAmountHelper + total);
        },

        onSetResponseOk: function(elements) {
            this.currentPaymentTypeObject.paymentsState.updateChecked(elements);
            this.recalculateTransactions();
            this.isAllOk(this.currentPaymentTypeObject.paymentsState);
        },

        onRotate: function () {
            this.scroll.scrollTo(0, 0);
            _.each(this.summaryScroller, function (scroll) {
                scroll.scrollTo(0, 0);
            });
        }

    });
});