/**
 * Created by petermomot on 3/2/15.
 */
define([
    'jquery',
    'moment',
    'inputFormat',
    'searchPlugin',
    'data/permissions',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/drawer/payment-modal.html',
    'bridge'
], function ($, moment, inputFormat, searchPlugin, PermModel, ModalView, YesNoModal, PaymentTemplate) {
    'use strict';

    var Model = Backbone.Model.extend({
        defaults: function () {
            return {
                id: '',
                date: moment.utc().utcOffset(window.SETTINGS.timeZoneOffset),
                payInOutTypeId: '',
                payInOutPaymentType: 1745,
                payInOutTypeDisplayName: '',
                reason: '',
                amount: 0
            }
        }
    });

    return ModalView.extend({
        el: '#payment-modal .content',
        template: _.template(PaymentTemplate),
        scrollWrap: '#payment-scroll-wrap',
        filter: '',
        tapTime: new Date(),

        initialize: function (options) {
            this.model = new Model(options);
            this.model.set('amount', Math.abs(this.model.get('amount')));
            this.model.set('reason', this.getHtmlFromText(this.model.get('reason')));
            this.events = _.extend(ModalView.prototype.events, this.events);
        },

        render: function () {
            ModalView.prototype.render.call(this);
            this.$('#payment-amount').inputFormat();
            this.$('#payment-select-input').searchPlugin({
                filterFunction: getPaymentTypes,
                filterFunctionParam: this.model.get('type'),
                parentScroll: this.scroll
            });
            var $paymentFormGroup = this.$('.payment-select-group'), paymentType = this.model.get('type');
            switch (paymentType) {
                case 'PayOut':
                    if (!PermModel.isVisible('evo.register.drawertab_modifypayoutlookup')) {
                        $paymentFormGroup.addClass('hidden');
                    }
                    break;
                case 'PayIn':
                    if (!PermModel.isVisible('evo.register.drawertab_modifypayinlookup')) {
                        $paymentFormGroup.addClass('hidden');
                    }
                    break;
            }
            return this;
        },
        
        events: {
            'change #payment-amount': 'removeErrorHighlight',
            'tap .button': 'saveAndClose',
            'submit .payment-form': 'formSubmit'
        },

        removeErrorHighlight: function(e) {
            $(e.target).closest('.input-format').removeClass('error');
        },

        validateForm: function () {
            var $form = this.$('.form'),
                serialized = $form.serializeArray(),
                toSend = _.object(_.pluck(serialized, 'name'), _.pluck(serialized, 'value')),
                errors = false,
                $formGroup = '';

            toSend['date'] = moment(this.model.get('date')).format();
            toSend['type'] = this.model.get('type');
            toSend['payInOutPaymentType'] = +toSend['payInOutPaymentType'];
            toSend['payInOutTypeId'] = this.$('#payment-select-input').data('id');
            toSend['payInOutTypeDisplayName'] = $('#payment-select-input').val();
            toSend['amount'] = $.global.parseFloat(toSend['amount']);

            _.each(toSend, function (value, key, list) {
                $formGroup = $form.find('[name=' + key + ']').parent();
                if (!value && $formGroup.hasClass('required')) {
                        $formGroup.addClass('error');
                        errors = true;
                } else {
                    $formGroup.removeClass('error');
                }
            });

            if (toSend['amount'] <= 0) {
                $('#payment-amount').closest('.input-format').addClass('error');
/*
                 new YesNoModal({
                     question: 'Amount should be greater than 0',
                     positiveText: 'Ok',
                     negativeText: '',
                     noCallback: true
                 }).render();
*/
                errors = true;
            }

            if (errors) {
/*
                if (this.$('.payment-select-group').hasClass('error')) {
                    new YesNoModal({
                        question: 'Please select a pay ' + (this.model.get('type') == 'PayOut' ? 'out' : 'in') + ' type',
                        positiveText: 'Ok',
                        negativeText: '',
                        noCallback: true
                    }).render();
                }
*/
                return false;
            }

            if (this.model.get('type') == 'PayOut') {
                if (this.model.get('cashInDrawer') + this.model.get('amount') - toSend['amount'] < 0) {
                    // we want to pay out more than we have
                    new YesNoModal({
                        question: "Amount can't be greater than cash in the drawer",
                        positiveText: 'Ok',
                        negativeText: '',
                        noCallback: true
                    }).render();
                    return false;
                }
            }

            if (!this.isValidInputLength($('#payment-reason'))) {
                new YesNoModal({
                    question: "The field Reason must be a string or array type with a maximum length of '256'",
                    positiveText: 'Ok',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }

            return toSend;
        },

        saveAndClose: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.$(':focus').blur();
            if (new Date() - this.tapTime < 300) {
                this.tapTime = new Date();
                return false;
            }
            this.tapTime = new Date();
            if (e.target.classList.contains('cancel')) {
                this.trigger('sendObject', {});
                this.closeView();
                return false;
            }
            $('#payment-reason').val(this.getTextFromHtml($('#payment-reason-input')));
            var data = this.validateForm();
            if (data) {
                this.trigger('sendObject', {
                    "payment": data
                });
                this.closeView();
            }
        },

        formSubmit: function (e) {
            this.saveAndClose(e);
        }

    });
});