/**
 * Created by petermomot on 12/16/14.
 */
define([
    'common/base-view',
    'data/game-messages',
    'text!../../templates/game-messages.html',
    'text!../../templates/game-message.html',
    'bridge',
    'iscroll'
], function (BaseView, gameMessagesList, gameMessagesTemplate, gameMessageTemplate) {
    'use strict';

    var GameMessageView = Backbone.View.extend({
        tagName: 'div',
        className: 'game-message-box',
        template: _.template(gameMessageTemplate),

        initialize: function () {
            this.model.on('change:viewed', function () {
                this.$el.toggleClass('new');
            }, this);
            this.model.on('change:imageUrl', function () {
                this.$('.img').css({
                    'background-image': 'url(\'' + this.model.get('imageUrl') + '\')',
                    'background-size': '100% 100%'
                });
            }, this);
        },

        events: {},

        render: function () {
            this.$el
                .html(this.template(this.model.toJSON()))
                .attr('data-id', this.model.get('globalMessageId'));

            this.$('.img').css({
                'background-image': 'url(\'' + this.model.get('imageUrl') + '\')',
                'background-size': '100% 100%'
            });

            if (this.model.get('viewed') == false) {
                this.$el.addClass('new');
            }
            if ($('.device.tablet').is(':visible')) {
                this.$('.text').css('font-size', window.messageFontSize)
            }

            return this;
        },

        close: function () {
            if (this.childViews) {
                this.childViews.close();
            }

            this.unbind();
            this.off(); // Unbind all local event bindings
            this.model && this.model.off();
            delete this.$el; // Delete the jQuery wrapped object variable
            delete this.el; // Delete the variable reference to this node
            delete this.model;
        }

    });

    var GameMessagesView = BaseView.extend({
        tagName: 'div',
        el: '.game-messages',
        template: _.template(gameMessagesTemplate),
        scroll: undefined,
        requestInProcess: false,
        pageNumber: 0,
        pageSize: 10,
        childViews: [],
        openedMessageTime: new Date(),

        initialize: function () {
            this.render();
            this.listenTo(gameMessagesList, 'add', this.renderMessage);
            this.listenTo(gameMessagesList, 'reset', this.emptyElement);
        },

        events: {
            //'tap .game-message-box': 'showMessageDetails',
            'touchstart .game-message-box': 'touchMessage',
            'touchend .game-message-box': 'touchEndMessage'
        },

        render: function () {
            this.$el.empty().html(this.template());
            this.$$rendered = true;
            return this;
        },

        renderMessage: function (item) {
            this.$('.game-messages-gif, .empty').remove();
            var messageView = new GameMessageView({model: item});
            if (gameMessagesList.indexOf(item) == 0) {
                this.$el.find('.messages').prepend(messageView.render().$el);
            } else {
                // add push message to end of list
                this.$el.find('.messages').append(messageView.render().$el);
            }
            this.childViews.push(messageView);
            this.refresh();
        },

        renderEmpty: function () {
            this.$('.game-messages-gif, .empty').remove();
            this.$('.messages').append('<div class="empty">There are no messages for you</div>');
        },

        initializeIScroll: function () {
            var that = this;
            this.scroll = new IScroll(this.$('.messages-scroll').get(0), {
                tap: true,
                probeType: 2,
                scrollbars: 'custom'
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
            this.scroll.on('scroll', function () {
                this._triggerRefresh = this.y > 40;
                $(this.wrapper).toggleClass('active-refresh', this.y > 0);
                if (this._triggerRefresh = this.y > 40) {
                    this.y = 40;
                }
                if (that.pageNumber != 'last') {
                    $(this.wrapper).toggleClass('active-loading', this.y < 0 && (this.y - this.maxScrollY) < 0);
                    if (this._triggerLoadNext = this.y < 0 && (this.y - this.maxScrollY) < -40) {
                        this._scrollToY = this.y;
                        this.y = this.maxScrollY - 40;
                    }
                }
            });
            this.scroll.on('beforeScrollEnd', function () {
                $(this.wrapper).removeClass('active-refresh active-loading refreshing loading');
                if (this._triggerRefresh) {
                    this.enabled = false;
                    this.hasVerticalScroll = false;
                    this.y = 0;
                    $(this.wrapper).addClass('refreshing');
                    this._execEvent('needRefresh');
                    return;
                }
                if (this._triggerLoadNext) {
                    this.moved = false;
                    this.enabled = false;
                    this.y = this._scrollToY;
                    $(this.wrapper).addClass('loading');
                    this._execEvent('needNext');
                    return;
                }
            });
            this.scroll.on('needRefresh', function () {
                // small delay to shown visual effect
                setTimeout(function () {
                    this._triggerRefresh = false;
                    that.requestInProcess = true;
                    getGameMessages(that.pageNumber = 0, that.pageSize)
                        .done(that.onGameMessagesGet.bind(that))
                        .always(function () {
                            $(this.wrapper).removeClass('refreshing');
                            that.requestInProcess = false;
                            this.enabled = true;
                            this.scrollTo(0, 0, this.options.bounceTime, this.options.bounceEasing);
                        }.bind(this));
                }.bind(this), 300);
            });
            this.scroll.on('needNext', function () {
                // small delay to shown visual effect
                setTimeout(function () {
                    this._triggerLoadNext = false;
                    that.requestInProcess = true;
                    getGameMessages(that.pageNumber, that.pageSize, gameMessagesList.first().get('globalMessageId'))
                        .done(that.onGameMessagesGet.bind(that))
                        .always(function () {
                            $(this.wrapper).removeClass('loading');
                            that.requestInProcess = false;
                            this.enabled = true;
                            // uncomment if we need to scroll to show first loaded message
                            // this.scrollTo(0, this.y - $('.game-message-box:first').height(), 'auto');
                        }.bind(this));
                }.bind(this), 300);
            });
        },

        show: function () {
            $(document).on('touchmove', preventTouchForIScroll);
            BaseView.prototype.show.call(this);
            this.getData();
        },

        hide: function () {
            $(document).off('touchmove', preventTouchForIScroll);
            BaseView.prototype.hide.call(this);
        },

        getData: function () {
            this.requestInProcess = true;
            this.pageNumber = 0;
            gameMessagesList.reset();
            getGameMessages(this.pageNumber, this.pageSize)
                .done(this.onGameMessagesGet.bind(this))
                .always(function () {
                    this.requestInProcess = false;
                    this.refresh();
                    this.scroll.scrollTo(0, 0);
                }.bind(this));
        },

        refresh: function () {
            !this.scroll && this.initializeIScroll();
        },

        onGameMessagesGet: function (data) {
            var offset = window.SETTINGS.timeZoneOffset;
            _.each(data, function(item) {
                item.displayDate = moment.utc(item.displayDate.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset).format('L h:mm A');
            });
            if (data.length < this.pageSize) {
                this.pageNumber = 'last';
            } else {
                this.pageNumber += 1;
            }
            if (data.length == 0) {
                if (gameMessagesList.length == 0) {
                    this.renderEmpty();
                }
            } else {
                _.each(data, function (item) {
                    if (gameMessagesList.where({'globalMessageId': item.globalMessageId}).length == 0) {
                        gameMessagesList.add(item);
                    }
                });
            }
        },

        showMessageDetails: function (e) {
            var $box = $(e.target).closest('.game-message-box');
            var messageId = $box.data('id');

            Backbone.history.navigate('game-messages/' + messageId + '/true', {trigger: true});
        },

        touchMessage: function (e) {
            var $box = $(e.target).closest('.game-message-box');
            this.$el.find('.game-message-box').removeClass('active');
            $box.addClass('active');

            this.touchPosition = $(e.target).offset();
        },

        touchEndMessage: function (e) {
            var $box = $(e.target).closest('.game-message-box');
            setTimeout(function () {
                $box.removeClass('active');
            }, 200);

            if (this.touchPosition && (Math.abs($(e.target).offset().top - this.touchPosition.top) < 40)
                && (new Date() - this.openedMessageTime > 700)) {
                this.openedMessageTime = new Date();
                e.stopPropagation();
                this.showMessageDetails(e);
            }
            this.touchPosition = undefined;
        },

        emptyElement: function () {
            this.$el
                .prepend('<img alt="loading..." class="loading-gif game-messages-gif" style="" src="images/load_clock.gif"/>')
                .find('.messages').empty();
            _.each(this.childViews, function (item) {
                item.close();
            })
        }

    });

    return GameMessagesView;
});
