/**
 * Created by petermomot on 12/16/14.
 */
define([
    'jquery',
    'common/base-view',
    'text!../../templates/login-content.html',
    'text!../../templates/login-server-option.html',
    'bridge'
], function ($, BaseView, loginContentTemplate, loginServerOptionTemplate) {
    'use strict';

    return BaseView.extend({
        el: '#login',
        template: _.template(loginContentTemplate),
        optionTemplate: _.template(loginServerOptionTemplate),
        timeoutId: 0,
        touchStartLogin: false,
        formElements: null,

        events: function () {
            return _.extend(BaseView.prototype.events, {
                'change #domain-field': 'domainChange',
                'focus #domain-field': 'domainFocus',
                'touchstart #submit-field': 'touchStartLoginSubmit',
                'touchend #submit-field': 'touchEndLoginSubmit',
                'submit .login-form': 'loginSubmit',
                'touchend .label-fix': 'labelFix',
                'keyup #url-field, #login-field, #pass-field': 'clearErrorOnBlur',
                'change #login-field, #pass-field': 'clearErrorOnBlur'
            });
        },

        getDomainList: function () {
            if (!window.IS_DEVICE && window.SERVERS_URL === undefined) {
                window.SERVERS_URL = "https://s3.amazonaws.com/mymeevo/servers.json";
                //window.SERVERS_URL = "http://localhost:6060/mobile/tests/servers.json";
            }
            this.$('.login-form').addClass('in-work');

            $.ajax({
                type: 'GET',
                dataType: 'json',
                url: window.SERVERS_URL + "?t="+ (new Date()).getTime(),
                timeout: 10000
            }).done(function (data) {
                localStorage.setItem('domainList', JSON.stringify(data));
                this.getDomainListSuccess(data)
            }.bind(this))
                .fail(this.getDomainListLocal.bind(this))
                .always(this.removeInWork.bind(this));
        },

        getDomainListLocal: function () {
            var domainList = localStorage.getItem('domainList');
            if (domainList != null) {
                this.getDomainListSuccess(JSON.parse(domainList));
            } else {
                nativeAlert('error', 'An internet connection is required to select a Meevo server.');
            }
        },

        getDomainListSuccess: function (data) {
            if (!data) { return false;}
            data = _.sortBy(data, function (server) {
                return server.sortOrder;
            });
            var $select = $('#domain-field');
            $select.empty();
            var options = [];

            _.each(data, function (s) {
                options.push(this.optionTemplate({s: s}));
            }.bind(this));
            $select.append(options);

            if ($select.find("option[value='" + localStorage.getItem('domain') + "']").length > 0) {
                $select.val(localStorage.getItem('domain'));
            }
            this.domainChange();
        },

        render: function () {
            this.$el.empty().html(this.template());
            this.$el.find('#login-field').val(localStorage.getItem('username'));
            this.$el.find('#url-field').val(localStorage.getItem('subdomain'));
            this.$el.find('#pass-field').val(localStorage.getItem('userpass') || '');
            if(localStorage.getItem('hasFeature')) {
                this.$('#feature-field').val(localStorage.getItem('featureDomain'));
                this.$('form').addClass('feature');
            }
            var $rememberMe = this.$('#remember-me');
            $rememberMe.siblings('.label').css({
                paddingLeft: $rememberMe.outerWidth() + 5
            });
            $rememberMe.css({
                top: '50%',
                marginTop: -$rememberMe.outerHeight()/2
            });
            this.formElements = this.$('#login-field, #pass-field, #remember-me, #url-field, #domain-field, #feature-field');

            $('html').removeClass('loading');
            return this;
        },

        hide: function () {
            BaseView.prototype.hide.call(this);
            this.$el.find('#pass-field').val('');
        },

        show: function () {
            BaseView.prototype.show.call(this);
            $(document).off('touchmove', preventTouchForIScroll);
            this.getDomainList();
        },

        domainChange: function (e) {
            var $option = '';
            if (e === undefined) {
                // triggered programmatically
                $option = this.$(':selected');
            } else {
                $option = $(e.target).find(':selected');
            }
            $('#domain-field').closest('.error').removeClass('error');
            if ($option.data('status') == 'inactive') {
                nativeAlert($option.data('message-title'), $option.data('message'));
            }
            sessionStorage.setItem('API_BASE_URL', '' + $option.data('url') + $option.data('apiPath'));
            sessionStorage.setItem('IMAGES_BASE_URL', '' + $option.data('url') + $option.data('imagePath'));
            sessionStorage.setItem('BASE_URL', $option.data('url'));
            window.API_BASE_URL = sessionStorage.getItem('API_BASE_URL');
            window.IMAGES_BASE_URL = sessionStorage.getItem('IMAGES_BASE_URL');
        },

        domainFocus: function (e) {
            if ($(e.target).find('option').length == 1 && this.$(':selected').val() == '') {
                this.getDomainList();
                $(e.target).blur();
            }
        },

        touchStartLoginSubmit: function (e) {
            e.preventDefault();
            $(e.target).addClass('active');

            this.touchStartLogin = false;
            clearTimeout(this.timeoutId);
            this.timeoutId = setTimeout(function () {
                this.touchStartLogin = true;
                var $form = $(e.target).closest('form');
                var hasFeature = $form.hasClass('feature');
                var $option = $form.find(':selected');
                var $urlField = $('#url-field');
                var $featureField = $('#feature-field');

                if (hasFeature) { // hide additional input
/*
                    var match = $featureField.val().match(/\/\/(\w*)\./);
                    if (match != null && match.length == 2) {
                        $urlField.val(match[1]);
                    }
*/
                    $urlField.val(localStorage.getItem('subdomain'));
                    this.domainChange();
                } else { // show additional input
                    if(localStorage.getItem('featureDomain')) {
                        $featureField.val(localStorage.getItem('featureDomain'));
                    } else {
                        var url = $urlField.val();
                        if ($option.data('url')) {
                            url = $option.data('url').replace(/^(http|https):\/\//, function(str, p1, offset, s) {
                                return str + $urlField.val() + '.'
                            });
                        }
                        $featureField.val(url);
                    }
                }

                $form.toggleClass('feature');
            }.bind(this), 10000);
        },

        touchEndLoginSubmit: function (e) {
            e.preventDefault();
            $(e.target).removeClass('active');
            if (!this.touchStartLogin) {
                this.tapLoginSubmit();
            }
        },

        tapLoginSubmit: function (e) {
            clearTimeout(this.timeoutId);
            this.$('.form').submit();
        },

        loginSubmit: function (e) {
            e.preventDefault();
            var $target = $(e.target),
                serialized = $target.serializeArray(),
                errors = false,
                $formGroup = '',
                hasFeature = $target.hasClass('feature');

            window.hasFeature = hasFeature;

            if ($target.hasClass('in-work')) {
                return false;
            }

            var toSend = _.object(_.pluck(serialized, 'name'), _.pluck(serialized, 'value'));
            _.each(toSend, function (value, key, list) {
                $formGroup = $target.find('[name=' + key + ']').parent();
                if (value == '') {
                    if (!hasFeature && key != 'feature' || hasFeature && key != 'url') {
                        $formGroup.addClass('error');
                        errors = true;
                    }
                } else {
                    $formGroup.removeClass('error');
                }
            });
            if (errors) {
                var $errorGroup = this.$('.error:first');
                if (!$errorGroup.hasClass('select')) {
                    $errorGroup.find('input').focus();
                }
                return false;
            }


            var $option = this.$(':selected');
            var $fullLinkField = $('#feature-field');
            if (hasFeature) {
                sessionStorage.setItem('API_BASE_URL', $fullLinkField.val() + '/mobile/api/');
                sessionStorage.setItem('IMAGES_BASE_URL', $fullLinkField.val() + '/images/');
                sessionStorage.setItem('BASE_URL', $fullLinkField.val());
                window.API_BASE_URL = sessionStorage.getItem('API_BASE_URL');
                window.IMAGES_BASE_URL = sessionStorage.getItem('IMAGES_BASE_URL');
                toSend['url'] = toSend['feature'].match(/\/\/(\w*)\./)[1];
            }

            $target.addClass('in-work');
            this.formElements.prop('disabled', true);
            this.$('.form-group.select').addClass('inactive');
            login(toSend['username'], toSend['password'], toSend['url'])
                .done(this.afterLoginSuccess.bind(this))
                .fail(function (xhr, status, error) {
                    if(xhr.status == 0) {
                        nativeAlert('error', 'Unable to connect to the server.');
                    } else {
                        nativeAlert('Login Error', 'Invalid user name, password or domain');
                    }
                }.bind(this))
                .always(this.removeInWork.bind(this));
            return true;
        },

        removeInWork: function () {
            this.$el.find('.login-form').removeClass('in-work');
            this.formElements.prop('disabled', false);
            this.$('.form-group.select').removeClass('inactive');
        },

        afterLoginSuccess: function () {
            localStorage.setItem('username', this.$('#login-field').val());
            localStorage.setItem('subdomain', this.$('#url-field').val());
            localStorage.setItem('domain', this.$('#domain-field :selected').html());
            if (window.hasFeature) {
                localStorage.setItem('domain', sessionStorage.getItem('BASE_URL').substring(sessionStorage.getItem('BASE_URL').indexOf('.')));
            }

            if(this.$('.login-form').hasClass('feature')) {
                localStorage.setItem('featureDomain', $('#feature-field').val());
                localStorage.setItem('hasFeature', true);
            } else {
                localStorage.removeItem('hasFeature');
            }
            var workstation = localStorage.getItem('workstation');
            if (workstation) {
                workstation = JSON.parse(workstation);
                if (sessionStorage.getItem('userId') != workstation['userId'] ||
                    localStorage.getItem('subdomain') != workstation['subdomain'] ||
                    localStorage.getItem('domain') != workstation['domain']) {
                    localStorage.removeItem('workstation');
                }
            }
            if($('#remember-me').is(':checked')) {
                localStorage.setItem('userpass', this.$('#pass-field').val());
            } else {
                localStorage.removeItem('userpass');
            }

            Backbone.history.navigate('menu', {trigger: true, replace: true});

            onUserLoggedIn();
        },

        labelFix: function(e) {
            var $checker = $(e.target).closest('.label-fix').find(':checkbox');
            if($(e.target).closest(':checkbox').length == 0) {
                if(!$checker.prop('disabled')) {
                    $checker.prop('checked', !$checker.prop('checked'));
                }
            }
        }

    });
});
