/**
 * Created by petermomot on 9/11/14.
 */
/**
 * Menu Application
 * @param {Object} Model List of Backbones models
 * @param {Object} View List of Backbone views
 * @param {Object} Collection List of Backbones collections
 * */
define([
    'common/base-view',
    'data/consts',
    'data/permissions',
    'text!../../templates/menu-content.html',
    'text!../../templates/menu-item.html',
    'bridge'
], function (BaseView, Consts, PermModel, menuContentTemplate, menuItemTemplate) {
    'use strict';

    var MenuItemModel = Backbone.Model.extend({
        defaults: function () {
            return {
                "title": "",
                "url": "",
                "locked": false,
                "counter": false,
                "count": "",
                "key": ""
            }
        }
    });

    var MenuItemsList = Backbone.Collection.extend({
        model: MenuItemModel
    });

    var menuList = new MenuItemsList();

    function processMenuItems() {
        _.each(Consts.MenuItems, function (item) {
            if (!(localStorage.getItem("domain") === ".meevocloud.co.uk" && item.title === 'Register')) {
                menuList.add(item, { silent: true });
            }
        });
    }

    if(_.isEmpty(window.PERMISSIONS.toJSON())) {
        window.PERMISSIONS.set(JSON.parse(sessionStorage.getItem('menuPermitions')));
        processMenuItems();
    } else {
        processMenuItems();
    }

    var MenuItemView = Backbone.View.extend({
        tagName: 'div',
        className: 'group-choose',
        template: _.template(menuItemTemplate),

        initialize: function () {
            this.model.on('change:count', function () {
                var count = this.model.get('count');
                if (count > 0) {
                    if (count > 99) {
                        count = '99+';
                    }
                    this.$el.find('.count').removeClass('empty').html(count);
                } else {
                    this.$el.find('.count').addClass('empty');
                }
            }, this);
        },

        events: {
            'tap .group-choose-body': 'groupChoose'
        },

        render: function () {
            this.$el.html(this.template(this.model.toJSON()));
            if (this.model.get('locked')) {
                this.$el.addClass('locked');
            }
            $('html').removeClass('loading');
            return this;
        },

        groupChoose: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            PermModel.navigateToPermissionsModal(e, function () {
                var $target = $(e.target);
                if ($target.hasClass('count')) {
                    $target = $target.parent();
                }
                Backbone.history.navigate($target.data('url'), {trigger: true});
            }.bind(this));
        }
    });

    return BaseView.extend({
        el: '.menu-items',
        template: _.template(menuContentTemplate),

        render: function () {
            this.$el.empty().html(this.template());
            var menu = [];

            menuList.each(function (item) {
                var skip = (item.get('url') === 'register' && !PermModel.isVisible('evo.launchpad.registergroup'));

                if (!skip && (item.toJSON().key ? PermModel.isVisible(item.toJSON().key) : true)) {
                    menu.push(new MenuItemView({
                        model: item
                    }).render().$el);
                }
            });
            this.$('.menu-scroll').html(menu);

            this.scroll = new IScroll(this.el, {
                tap: true
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });

            return this;
        },

        hide: function () {
            BaseView.prototype.hide.call(this);
            $(document).off('touchmove', preventTouchForIScroll);
        },

        show: function () {
            BaseView.prototype.show.call(this);
            $(document).on('touchmove', preventTouchForIScroll);
            this.refresh();
        },

        refresh: function() {
            setTimeout(function () {
                getGameMessagesCount(true).done(function (data) {
                    menuList.findWhere({"url": "game-messages"}).set('count', data);
                    onGameMessagesCountChanged(data);
                });
            }, 100);
        }

    });
});
