/**
 * Created by petermomot on 10/7/14.
 */
define([
    'common/base-view',
    'data/permissions',
    'factories/bread-crumbs',
    'factories/modal-buttons-bottom'
], function (BaseView, PermModel, BreadCrumbsFactory, ModalButtonsBottom) {
    'use strict';

    return BaseView.extend({
        tagName: 'div',
        scroll: null,
        routeHandler: null,
        scrollbars: 'custom',
        clientModal: undefined,

        events: _.extend({
            'touchend .close': 'closeView',
            'clientCreated input[type="text"]': 'clientCreatedCallback',
            'clientEdited input[type="text"]': 'clientEditedCallback'
        }, BaseView.prototype.events),

        render: function () {
            var p = {PermModel: PermModel || window.PERMISSIONS},
                o = this.model ? _.extend(this.model.toJSON(), p) : p,
                $mainContent,
                bCrumbsEvents = {},
                mButtonsEvents = {};

            this.$el.html(this.template(o));
            this.$el.closest('.modal').removeClass('hidden');

            this.$('textarea[data-autoresize]').each(function() {
                var offset = this.offsetHeight - this.clientHeight;
                var resizeTextarea = function(el) {
                    $(el).css('height', 'auto').css('height', el.scrollHeight + offset);
                };
                $(this).on('input', function() { resizeTextarea(this); }).removeAttr('data-autoresize');
            });

            this.setHeight();
            if (!this.withoutScroll) {
                this.initializeIScroll();
            }

            this.routeHandler = function () {
                this.closeView(null, Backbone.history.fragment);
            }.bind(this);

            Backbone.history.on('route', this.routeHandler);

            $mainContent = this.$el.children().first();
            if (this.breadCrumbs || this.modalButtonsBottom) {

                if (this.breadCrumbs) {
                    bCrumbsEvents = this.enableBreadCrumbs($mainContent);
                }
                if (this.modalButtonsBottom) {
                    mButtonsEvents = this.addButtonsBottom($mainContent);
                }

                this.events = _.extend({}, this.events, bCrumbsEvents, mButtonsEvents);
                this.delegateEvents(); // refresh events
            }

            this.onRender();

            return this;
        },
        onPhone: function() {
            return $('html').hasClass('mobile');
        },
        isWithWorkstation: function() {
            var isWithWorkstation = true;
            if(window.SETTINGS['creditPaymentService'] === "ElementPS") {
                //var storage = JSON.parse(localStorage.getItem('workstation'));
                //if(!storage['hasCreditPaymentDevice']) {
                    isWithWorkstation = false;
                //}
            }
            return isWithWorkstation;
        },
        setupSearchField : function(domId, options, entity, populatedCallBack) {
            if (entity && (window.SETTINGS.walkInFemale.id != entity.id && window.SETTINGS.walkInMale.id != entity.id)) {
                $('#' + domId).val(entity.displayName);
                $('#' + domId).data('id', entity.id);

                options = _.clone(options);
                options.HasDefaultItem = true;
                options.defaultItem = entity;
                $('#' + domId).searchPlugin(options);

                if (populatedCallBack)
                    populatedCallBack();

                return;
            }
            $('#' + domId).searchPlugin(options);
        },

        clientCreatedCallback: function (e, r) {},

        clientEditedCallback: function (e, r) {},

        enableBreadCrumbs: function ($mainContent) {
            var bCrumbs = BreadCrumbsFactory({
                breadCrumbs: this.breadCrumbs
            });

            $mainContent.addClass(bCrumbs.modalContentClass);
            this.$el.prepend(bCrumbs.html);

            this.breadCrumbsGo = bCrumbs.touchCallback;
            return bCrumbs.events;
        },

        addButtonsBottom: function ($mainContent) {
            var mButtons = ModalButtonsBottom(_.extend(this.modalButtonsBottom, {
                events: this.events
            }));

            $mainContent.addClass(mButtons.modalContentClass);
            this.$el.append(mButtons.html);

            return mButtons.events;
        },

        onRender: function () {},

        setHeight: function () {
            // in some modals this is rotate event
            this.scroll && this.scroll.refresh();
        },

        initializeIScroll: function () {
            var wrapper;

            if (this.scrollWrap !== undefined) {
                wrapper = this.$(this.scrollWrap).get(0);
            } else {
                if (this.breadCrumbs || this.modalButtonsBottom) {
                    this.$el.wrapInner('<div class="scroll-outside"><div class="scroll-wrap"><div></div></div></div>');
                } else {
                    this.$el.wrapInner('<div class="scroll-wrap"><div></div></div>');
                }

                wrapper = this.$('.scroll-wrap').get(0);
            }

            this.scroll = new IScroll(wrapper, {
                tap: true,
                disableMouse: true,
                scrollbars: this.scrollbars
            });
            this.scroll.on('beforeScrollStart', function () {
                for (var scrolled = $(wrapper); scrolled.not("body").size(); scrolled = scrolled.parent()) {
                    var scrollTop = scrolled.scrollTop();
                    if (scrollTop !== 0) {
                        this.scrollTo(0, scrollTop * -1);
                        scrolled.scrollTop(0);
                    }
                }
                this.refresh();
            });
        },

        destroyScroll: function () {
            if (this.scroll) {
                this.scroll.destroy();
            }
            this.scroll = null;
        },

        cancel: function () {
            this.closeView();
        },

        closeView: function (e, route) {
            if (e) {
                e.preventDefault();
                e.stopPropagation();
            }

            $(window).off("orientationHandlerCallback", this.setHeight);

            this.$('input').searchPlugin('destroy').inputFormat('destroy');

            this.destroyScroll();

            this.$el.closest('.modal').addClass('hidden');

            Backbone.history.off('route', this.routeHandler);

            this.unbind();
            this.off(); // Unbind all local event bindings
            this.model && this.model.off();
            this.collection && this.collection.off();

            // instead this.remove
            this.stopListening();
            this.$el.replaceWith(this.$el.empty().clone());

            delete this.$el; // Delete the jQuery wrapped object variable
            delete this.el; // Delete the variable reference to this node
            delete this.model;
            delete this.collection;

            if (e) {
                window.history.back();
            }

            return true;
        }

    });
});

