/**
 * Created by petermomot on 2/16/15.
 */
define([
    'common/base-view',
    'text!../../templates/register.html',
    'views/yes-no-modal',
    'data/permissions',
    'bridge'
], function (BaseView, RegisterTemplate, YesNoModal, PermModel) {
    'use strict';

    var RegisterModel = Backbone.Model.extend({
        defaults: function () {
            return {
                'workstation': JSON.parse(localStorage.getItem('workstation')),
                'drawerOpened': false
            }
        }
    });
    var registerModel = new RegisterModel();

    return BaseView.extend({
        el: '.register',
        template: _.template(RegisterTemplate),
        model: registerModel,
        timeoutId: undefined,
        timeoutCleared: false,

        initialize: function () {
            this.listenTo(this.model, 'change:workstation', function () {
                this.$('.workstation-top').html(this.model.get('workstation')['displayName']);
            }.bind(this));
        },

        events: {
            'touchstart .workstation-top': 'touchStartWorkstation',
            'touchend .workstation-top': 'touchEndWorkstation',
            'tap .drawer-register': 'showDrawer',
            'tap .sales-register': 'showSales',
            'tap .on-hold-register': 'showHold'
        },

        render: function () {
            if (!this.$$rendered) {
                this.setWorkstation();
            }
            this.$el.empty().html(this.template(_.extend(this.model.toJSON(), {PermModel: PermModel})));

            this.scroll = new IScroll(this.el, {
                tap: true
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });

            return this;
        },

        show: function () {
            BaseView.prototype.show.call(this);
            this.refresh();
        },

        refresh: function () {
            this.setWorkstation();
        },

        touchStartWorkstation: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();

            this.timeoutId = setTimeout(function () {
                this.timeoutCleared = true;
                Backbone.history.navigate('workstation', {trigger: true});
            }.bind(this), 1000);
        },

        touchEndWorkstation: function (e) {
            if (!this.timeoutCleared) {
                clearTimeout(this.timeoutId);
                this.timeoutId = undefined;
                toastMessage('Please hold this to select other workstation');
            }
            this.timeoutCleared = false;
        },

        setWorkstation: function () {
            this.model.set('workstation', JSON.parse(localStorage.getItem('workstation')));
        },

        getDrawerState: function (drawerId, callback) {
            if (drawerId) {
                getWorkstationDrawerState(drawerId)
                    .done(function (data) {
                        if (String(data.status).toLowerCase().indexOf('closed') != -1
                            || String(data.status).toLowerCase().indexOf('opened') != -1) {
                            this.model.set('drawerOpened', data['isOpen']);
                            callback();
                        }
                    }.bind(this));
            }
        },

        showDrawer: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                Backbone.history.navigate('drawer', {trigger: true});
            }.bind(this));
        },

        showSales: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.getDrawerState(this.model.get('workstation')['drawerId'], function () {
                    if (this.model.get('drawerOpened')) {
                        Backbone.history.navigate('smart-receipt', {trigger: true});
                    } else {
                        setTimeout(function(){
                            new YesNoModal({
                                question: 'Please open your drawer before proceeding.',
                                positiveText: 'Ok',
                                positiveCallback: function(){
                                    Backbone.history.navigate('drawer', {trigger: true, replace: true});
                                },
                                negativeText: ''
                            }).render();
                        },0);
                    }
                }.bind(this));
            }.bind(this));
        },

        showHold: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                Backbone.history.navigate('smart-receipt/showHold', { trigger: true });
            }.bind(this));
        }

    });
});