define([
    'jquery',
    'searchPlugin',
    'views/modal',
    'text!../../../templates/smart-receipt/add-client-modal.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'bridge'
], function ($, searchPlugin, ModalView, AddClientModalTemplate, UserSearchTemplate) {
    'use strict';

    var SomeModel = Backbone.Model.extend({
        defaults: {
            id: '',
            firstName: '',
            lastName: '',
            removeButton: false
        }
    });

    return ModalView.extend({
        el: '#add-client-modal .content',
        template: _.template(AddClientModalTemplate),
        scrollWrap: '#add-client-modal-scroll-wrap',
        events: {
            'tap .add-female, .add-male' : 'addWalkInClient',
            'tap .search-plugin-content .option' : 'addClient',
            'tap #add-client-circle' : 'clientAddCircle'
        },
        modalButtonsBottom: { // works with isNeedToRemove function
            phone: false,
            phoneButtons: [
                { id: 'remove-client-button', className: 'red', size: '50', value: 'Remove' },
                { id: 'button-ok', className: 'red-dark', size: '50', value: 'Cancel' }
            ],
            tabletButtons: [
                { id: 'remove-client-button', className: 'red', size: '50', value: 'Remove' },
                { id: 'button-ok', className: 'red-dark', size: '50', value: 'Cancel' }
            ],
            touchCallBacks: {
                'remove-client-button': 'removeClient',
                'button-ok': 'cancel'
            }
        },
        initialize: function (options) {
            this.client = options.client;
            this.model = new SomeModel(options);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        isNeedToRemove: function() {
            if(!this.model.get('removeButton')) {
                this.$('#remove-client-button').hide();
                this.$('#button-ok').removeClass('col-50').addClass('col-100');
            }
        },

        render: function(){
            ModalView.prototype.render.call(this);

            this.isNeedToRemove();

            this.$('#client-search').searchPlugin({
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                     return getClients(filter, clientAccountChargable, undefined, undefined, pageNumber, pageSize);
                },
                parentScroll: this.scroll,
                fullScreen: true,
                template: _.template(UserSearchTemplate),
                clientControls: true,
                HasDefaultItem: true,
                defaultItem: this.client
            });

            return this;
        },

        clientAddCircle: function (e) {
            this.$('#client-search').searchPlugin('initializeAdding');
            e.preventDefault();
            e.stopPropagation();
        },

        clientCreatedCallback: function (e, client) {
            this.trigger('closeEvent', {
                type: 'add',
                client: client
            });
            this.closeView();
        },

        clientEditedCallback: function (e, client) {
            this.addClientTriggers(client);
        },

        addWalkInClient: function(e){
            var $target = $(e.target).hasClass('button') ? $(e.target) : $(e.target).parent();
            var client = window.SETTINGS['walkInFemale'];
            if($target.hasClass('add-male')){
                client = window.SETTINGS['walkInMale'];
            }
            if (this.model.get('id')) {
                this.trigger('closeEvent', {
                    type: 'change',
                    client: {
                        oldId: this.model.get('id'),
                        newClient: client
                    }
                });
            } else {
                this.trigger('closeEvent', {
                    type: 'add',
                    client: client
                });
            }
            this.closeView();
        },

        addClient: function(e){
            var userList = this.$('#client-search').searchPlugin('getList');
            var $target = $(e.target);
            if (!$target.hasClass('option')) {
                $target = $target.parent();
            }
            var client = _.findWhere(userList, {id: $target.data('id')});

            this.addClientTriggers(client);
        },

        addClientTriggers: function (client) {
            if (this.model.get('id')) {
                this.trigger('closeEvent', {
                    type: 'change',
                    client: {
                        oldId: this.model.get('id'),
                        newClient: client
                    }
                });
            } else {
                this.trigger('closeEvent', {
                    type: 'add',
                    client: client
                });
            }
            this.closeView();
        },

        removeClient: function(){
            this.trigger('closeEvent', {
                type: 'remove',
                client: this.model.get('id')
            });
            this.closeView();
        }

    });
});
