define([
    'jquery',
    'searchPlugin',
    'views/modal',
    'text!../../../templates/smart-receipt/adjustments-modal.html',
    'text!../../../templates/smart-receipt/adjustment-template.html',
    'bridge'
], function($, searchPlugin, ModalView, AdjustmentsModalTemplate, AdjustmentTemplate){
    'use strict';

    return ModalView.extend({
        el: '#adjustments-modal .content',
        template: _.template(AdjustmentsModalTemplate),
        adjustmentTemplate: _.template(AdjustmentTemplate),
        scrollWrap: '#adjustments-modal-scroll-wrap',
        scrollbars: false,
        events: {
            'touchend #predefined-discount': 'predefinedDiscount',
            'touchend #manual-discount': 'manualDiscount',
            'touchend .delete-discount' : 'deleteDiscount',
            'tap .edit-discount' : 'editDiscount',
            //'touchend #ok': 'customClose'
        },
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                { id: 'ok', className: 'green', size: '100', value: 'OK' }
            ],
            tabletButtons: [
                { id: 'ok', className: 'green', size: '100', value: 'OK' }
            ],
            touchCallBacks: {
                'ok': 'customClose'
            }
        },

        initialize: function() {
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            this.onPhone = $('html').hasClass('mobile');
            this.modalButtonsBottom.phone = this.onPhone;
            this.el = this.onPhone ? '#adjustments-modal .content' : '#tablet-adjustments-modal .content';
            this.setElement(this.el);
            // TODO update discounts list on sales lines update
        },

        render: function(){
            var tempTemplate = '';

            ModalView.prototype.render.call(this);

            _.each(this.collection.models, function (model) {
                tempTemplate += this.adjustmentTemplate(model.toJSON());
            }.bind(this));
            this.$('#discounts-list').append(tempTemplate);

            this.scroll.on('beforeScrollStart', function () {
                $('#adjustments-modal-scroll-wrap').css('bottom', $('.modal-buttons').outerHeight() + 10);
                this.refresh();
            });

            return this;
        },



        predefinedDiscount: function(){
            this.triggerEvent('showPredefined');
        },

        manualDiscount: function(){
            this.triggerEvent('showManual');
        },

        deleteDiscount: function(e){
            var discountId = $(e.target).closest('.discount-info').attr('data-discount-id'),
                discountObj = {};
            $(e.target).closest('.discount-info').remove();
            _.each(this.collection.models, function(model){
                if (model.get('id') == discountId){
                    discountObj = model.toJSON();
                }
            });
            this.trigger('adjustmentEvent', {
                type: 'delDiscount',
                discountId: discountId,
                discount: discountObj
            });
        },

        editDiscount: function(e){
            this.triggerEvent('showManual', {
                discountId: $(e.target).closest('.discount-info').attr('data-discount-id')
            });
        },

        triggerEvent: function (type, o) {
            o = o || {};
            this.trigger('adjustmentEvent', _.extend(o, {
                type: type
            }));
            this.closeView();
            setTimeout(function () { $('#discount-search').trigger('blur'); }, 5);
        },

        customClose: function () {
            if (this.onPhone) {
                this.trigger('adjustmentEvent', {type: 'cancel'});
                this.closeView();
            } else {
                this.trigger('adjustmentEvent', {type: 'doneLeave'});
            }
        }
    });
});