/**
 * Created by petermomot on 4/1/15.
 */
define([
    'jquery',
    'inputFormat',
    'searchPlugin',
    'views/modal',
    'views/yes-no-modal',
    'data/gift-cards',
    'text!../../../templates/smart-receipt/checkout-modal.html',
    'text!../../../templates/smart-receipt/checkout-credit-payment-block.html',
    'text!../../../templates/smart-receipt/checkout-other-payment-block.html',
    'text!../../../templates/smart-receipt/checkout-gift-line.html',
    'text!../../../templates/smart-receipt/checkout-gift-client.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'bridge'
], function ($, inputFormat, searchPlugin, ModalView, YesNoModal, GiftCards,
             CheckoutTemplate, CreditPaymentTemplate, OtherPaymentTemplate, GiftLineTemplate, GiftClientTemplate,
             UserSearchTemplate) {
    'use strict';

    var Model = Backbone.Model.extend({
        defaults: {
            amount: 0,
            id: '',
            isRefund: false,
            lineNumber: 0,
            paymentType: {
                catDisplayName: '',
                displayName: '',
                id: '',
                paymentTypeEnum: 0,
                shortcut: ''
            }
        }
    });

    return ModalView.extend({
        template: _.template(CheckoutTemplate),
        creditPaymentTemplate: _.template(CreditPaymentTemplate),
        otherPaymentTemplate: _.template(OtherPaymentTemplate),
        giftLineTemplate: _.template(GiftLineTemplate),
        clientCardsTemplate: _.template(GiftClientTemplate),
        scrollbars: false,
        paymentTypes: [],
        balance: 0,
        clientAccount: undefined,
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                {
                    id: 'checkout-cancel-payment',
                    className: 'red cancel',
                    size: '20',
                    value: ''
                },
                {
                    id: 'checkout-back',
                    className: 'yellow back',
                    size: '20',
                    value: ''
                },
                {
                    id: 'checkout-submit',
                    className: 'green',
                    size: '60',
                    value: 'Add payment'
                }
            ],
            tabletButtons: [
                {
                    id: 'checkout-cancel-payment',
                    className: 'red',
                    size: '100',
                    value: 'cancel'
                }
            ],
            touchCallBacks: {
                'checkout-cancel-payment': 'checkoutCancel',
                'checkout-back': 'checkoutBack',
                'checkout-submit': 'beforeSubmit'
            }
        },

        initialize: function (options) {
            this.setBalance(options.balance);
            this.payingClient = _.find(options.clients, function (n) {
                return n.id != window.SETTINGS.walkInFemale.id
                    && n.id != window.SETTINGS.walkInMale.id
                    && n.enableClientAccountCharging === true;
            });
            this.checkouts = options.checkouts;
            this.paymentTypes = options.paymentTypes;

            this.checkoutsLiveReference = options.checkoutsLiveReference;
            this.clients = options.clients;

            this.beforeModelSet(options);
            this.giftCards = GiftCards;
            this.listenTo(this.giftCards, 'add', this.giftCardAdded);
            this.listenTo(this.giftCards, 'change', this.giftCardChange);
            this.listenTo(this.giftCards, 'remove', this.giftCardRemoved);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            this.el = this.model.get('phone') ? '#checkout-modal .content' : '#tablet-checkout-modal .content';
            this.setElement(this.el);
        },

        setBalance: function(val) {
            this.balance = val;

            // renew GC "Use" field
            if(this.giftCards && this.giftCards.length) {
                _.each(this.giftCards.toJSON(), function(elem) {
                    $('.checkout-gift-line input[data-id=' + elem.id + ']').inputFormat('setValue', elem.remaining < val ? elem.remaining : val);
                });
            }

            this.balanceChange();
        },

        beforeModelSet: function (options) {
            var o = {
                phone: $('html').hasClass('mobile'),
                title: options.title,
                url: options.url
            };

            if (options.originalModel) {
                o = _.extend(o, options.originalModel);
                this.initialAmount = options.initialAmount;
            }
            this.model = new Model(o);

            if (o.phone) {
                if (this.model.get('lineNumber') === 0) {
                    this.breadCrumbs = ['Checkout', 'Payment', options.title];
                } else {
                    this.modalButtonsBottom.phoneButtons = [
                        {
                            id: 'checkout-cancel-payment',
                            className: 'red',
                            size: '50',
                            value: 'cancel'
                        },
                        {
                            id: 'checkout-submit',
                            className: 'green',
                            size: '50',
                            value: 'save'
                        }
                    ];
                }
                
                this.modalButtonsBottom.phone = true;
            }
        },

        render: function () {
            var $giftClientInput = this.$('#checkout-gift-client');

            ModalView.prototype.render.call(this);
            this.$el.append('<div id="no-credit-reader"><div class="middle-aligned">Mobile credit card processing is not currently supported in My Meevo. Please finalize this transaction on another workstation.</div></div>');
            this.renderPaymentTypes();
            this.checkoutBalanceCaption(this.balance);

            this.addPlugins();
            if (this.model.get('lineNumber')) {
                this.$('.checkout-form').hide();
                this.$('.checkout-edit-form').show();

                var displayName = this.model.get('paymentType')['displayName'],
                    paymentTypeEnum = this.model.get('paymentType')['paymentTypeEnum'];
                if (paymentTypeEnum == 1460) { // credit
                    displayName = 'Charge';
                } else if (paymentTypeEnum == 1463) {
                    this.$('.checkout-edit-check').show();
                    this.$('#checkout-edit-check').inputFormat('setValue', this.model.get('checkNumber'));
                }
                this.$('.checkout-edit-type').html(displayName);
            }
            this.$('.checkout-form').addClass(this.model.get('url'));

            //gift cards form
            if (this.model.get('url') == 'gift') {
                if (this.model.get('phone')) {
                    this.$('.scroll-outside').attr('style', function (i, s) { return s ? s : '' + 'padding-bottom: 0 !important;' });
                    this.$('.modal-buttons-bottom').hide();
                    this.$('.cancel-gifts-button').show();
                    this.$('.before-bottom-buttons').removeClass('before-bottom-buttons');
                }
                if (!$giftClientInput.val() && !$giftClientInput.data('id') && window.SETTINGS['allowClientSearchWhenRedeemingGiftCard']) {
                    var counter = 0;
                    _.each(this.clients, function (client) {
                        this.searchGiftCards(client.id, false, null, function (cards) {
                            $giftClientInput.data('id', client.id);
                            this.addCards(cards);
                        }.bind(this))
                            .always(function () {
                                if (++counter == this.clients.length) {
                                    $giftClientInput.data('id', '');
                                }
                            }.bind(this));
                    }.bind(this));
                }
            }

            if(!this.isWithWorkstation() && this.model.get('url') == 'credit') {
                var $reader = this.$('#no-credit-reader');
                $reader && $reader.addClass('visible');
                this.$('#checkout-cancel-payment, #checkout-back').css({zIndex: 15});
            }

            return this;
        },

        addPlugins: function () {
            var options = {
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                     return getClients(filter, clientAccountChargable, undefined, undefined, pageNumber, pageSize);
                },
                parentScroll: this.scroll,
                template: _.template(UserSearchTemplate),
                large: true
            };


            var func = this.getAccountBalance;
            if (this.$('#checkout-account').length)
                this.setupSearchField('checkout-account', _.extend(options, {filterFunctionParam: 1}), this.payingClient, function() {
                    func.call(this, { target: document.getElementById('checkout-account') });
                }.bind(this));

            if (this.$('#checkout-gift-client').length)
                this.setupSearchField('checkout-gift-client', options, undefined);

            this.$('#checkout-amount').inputFormat({
                allowMinus: false
            });
            if (this.model.get('lineNumber')) {
                this.$('#checkout-edit-amount').inputFormat({
                    allowMinus: false
                });
            }
            this.$('#checkout-check').inputFormat({
                type: 'int',
                places: 10,
                allowEmpty: true
            });
            this.$('#checkout-edit-check').inputFormat({
                type: 'int',
                places: 10
            });
            this.$('#checkout-gift-number').inputFormat({
                type: 'any',
                places: 20,
                allowEmpty: true
            });
        },

        events: {
            'tap .set-balance-amount': 'setUseBalanceAmount',
            'tap #checkout-use-balance': 'checkoutUseBalance',
            'tap .checkout-checkbox': 'checkoutCheckbox',
            'blur #checkout-account': 'getAccountBalance',
            'optionSelected #checkout-gift-client': 'onClientSelected',
            'tap #checkout-gift-client-search': 'giftClientSearch',
            'tap #checkout-gift-card-search': 'giftCardSearch',
            'tap .show-zero-gift-cards': 'showZeroGiftCards',
            'tap .checkout-add-gift': 'addGiftPayment',
            'tap .delete-cards-for': 'removeClientCards',
            'change #checkout-amount': 'onChangeCheckoutAmount',
            'tap #checkout-reset': 'checkoutReset',
            'tap .button.save': 'saveEdition',
            'tap #cancel-gifts-button': 'checkoutBack',
            'blur #checkout-check': 'checkEmptyValue',
            'input #checkout-edit-amount, #checkout-edit-check': 'checkEmptyValue',
            'touchend #close-no-credit-reader': 'checkoutBack'
        },

        checkEmptyValue: function (e) {
            if($(e.target).val().trim().length) {
                $(e.target).closest('.error').removeClass('error');
            }
        },

        onClientSelected: function(e) {
            $(e.target).closest('.error').removeClass('error');
        },

        onChangeCheckoutAmount: function() {
            var $checkOut = this.$('#checkout-amount');
            if($checkOut.closest('.amount-group').hasClass('error')) {
                if($checkOut.inputFormat('getValue')) {
                    $checkOut.closest('.amount-group').removeClass('error');
                }
            }
        },

        checkoutBalanceCaption: function(balance) {
            var captionText = (balance < 0) ? 'Change Due:' : 'Balance Due:';
            this.setBalance(balance);
            this.$('.checkout-due-group').html(
                "<span class='caption'>" + captionText + "</span>" +
                "<span class='balance'>" + $.global.format(Math.abs(balance), 'c') + "</span>"
            );
        },

        setUseBalanceAmount: function(e) {
            var toAdd = +$(e.target).data('amount') + this.$('#checkout-amount').inputFormat('getValue');
            this.$('#checkout-amount').inputFormat('setValue', toAdd);
        },

        renderPaymentTypes: function () {
            var creditPayments = [], otherPayments = [];
            _.each(this.paymentTypes, function (p) {
                p['shortcut'] = p['shortcut'].trim();
                p = _.extend({imageId: ''}, p);
                if (p['paymentTypeEnum'] == 1460 || p['paymentTypeEnum'] == 1461) { // credit/debit
                    creditPayments.push(this.creditPaymentTemplate(p));
                } else if (p['paymentTypeEnum'] == 1518) { // other
                    otherPayments.push(this.otherPaymentTemplate(p));
                }
            }.bind(this));
            this.$('.credit-group').html(creditPayments);
            this.$('.other-group').html(otherPayments);
        },

        balanceChange: function() {
            if (this.balance > 0) {
                this.$('#checkout-use-balance').removeClass('inactive');
            } else {
                this.$('#checkout-use-balance').addClass('inactive');
            }
        },

        checkoutUseBalance: function () {
            if (this.balance > 0) {
                this.$('#checkout-amount').inputFormat('setValue', this.balance.toFixed(2));
                this.$('#checkout-submit').trigger('touchend');
            }
        },

        checkoutCheckbox: function (e) {
            var $target = $(e.target);
            if (!$target.hasClass('checkout-checkbox')) {
                $target = $target.closest('.checkout-checkbox');
            }
            if ($target.hasClass('checked')) { return false; }
            this.$('.checkout-checkbox').removeClass('checked');
            $target.addClass('checked');
            $target.closest('.form-group').removeClass('error');
        },

        getAccountBalance: function (e) {
            var $input = $(e.target), $balance = this.$('#checkout-account-balance');
            if ($input.data('id')) {
                $input.parent().removeClass('error');
                this.clientAccount = $input.searchPlugin('getSelectedObject');
                if (!this.clientAccount) {
                    $input.parent().addClass('error');
                    $balance.text('');
                    return false;
                } else {
                    setTimeout(function () {
                        $balance.empty()
                            .text($.global.format(this.clientAccount.balance, 'c'))
                            .removeClass('negative positive');
                        if (this.clientAccount.balance > 0) {
                            $balance.addClass('positive');
                        } else if (this.clientAccount.balance < 0) {
                            $balance.addClass('negative');
                        }
                    }.bind(this), 10);
                }
            } else {
                $balance.text('');
            }
        },

        giftClientSearch: function (e) {
            $(':focus').blur();
            var clientId = this.$('#checkout-gift-client').data('id');
            if (clientId) {
                $(e.target).parent().removeClass('error');
                this.searchGiftCards(clientId, true, function() {
                    new YesNoModal({
                        question: "Client doesn't have a gift card",
                        positiveText: 'OK',
                        negativeText: '',
                        noCallback: true
                    }).render();
                });
            } else
                $(e.target).parent().addClass('error');
        },

        searchGiftCards: function (searchTerm, shouldAdd, noCardsCallback, hasCardsCallback) {
           return getGiftCards(searchTerm).done(function (r) {

               if (r.length) {
                   if(shouldAdd)
                    this.addCards(r);

                   if (hasCardsCallback)
                       hasCardsCallback(r);

               } else if (noCardsCallback)
                   noCardsCallback();
           }.bind(this));
        },

        addCards: function (r) {
            this.$el && this.$('.checkout-gift-cards').addClass('less-padding');
            this.giftCards.beforeAdd(r, _.filter(this.checkoutsLiveReference.toJSON(), function (c) { return c.giftCard; }));
        },

        giftCardSearch: function (e) {
            $(':focus').blur();
            var giftNumber = this.$('#checkout-gift-number').val();
            if (giftNumber) {
                $(e.target).parent().removeClass('error');
                this.searchGiftCards(giftNumber, true, function () {
                    new YesNoModal({
                        question: "Please enter a valid gift card number",
                        positiveText: 'OK',
                        negativeText: '',
                        noCallback: true
                    }).render();
                });
            } else {
                $(e.target).parent().addClass('error');
            }
        },

        showZeroGiftCards: function () {
            this.$('#show-zero-gift-cards').toggleClass('checked')
                .closest('.gift-card-group').toggleClass('checked');
        },

        giftCardAdded: function (card) {
            var existingCard = _.filter(_.map(this.checkoutsLiveReference.models, function(el) { return el.toJSON(); }), function (c) {
                if (c['giftCard']) {
                    return c['giftCard']['id'] === card.get('id');
                }
            })[0];
            if (existingCard) {
                card.set('remaining', existingCard['giftCard']['remaining']);
            }
            if (card.get('remaining') < 0) {
                return false;
            }
            var clientId = card.get('client').id,
                $clientCards = this.$('#cards-'+clientId);
            if ($clientCards.length === 0) {
                this.$('.checkout-gift-cards').append(this.clientCardsTemplate(card.get('client')));
            }

            this.$('#cards-'+ clientId +' .cards-by-client').append(this.giftLineTemplate(card.toJSON()));
            var val = card.get('remaining') - this.balance < 0 ? card.get('remaining') : (this.balance < 0 ? 0 : this.balance);
            this.$('input[data-id=' + card.get('id') + ']').inputFormat({
                startValue: val
            });
        },

        giftCardRemoved: function (card) {
            this.$('.checkout-gift-line[data-id=' + card.get('id') + ']').remove();
            var clientId = card.get('client').id,
                $clientCards = this.$('#cards-'+clientId);
            if ($clientCards.find('.cards-by-client').html() === '') {
                $clientCards.remove();
                this.scroll.refresh();
            }
        },

        giftCardChange: function (card) {
            $(".checkout-gift-line[data-id='" + card.get('id') + "']").find('.remaining').html($.global.format(card.get('remaining'), 'c'));
        },

        removeClientCards: function (e) {
            this.giftCards.removeByClientId($(e.target).data('id'));
            if (this.$('.checkout-gift-cards').children().length == 0) this.$('.checkout-gift-cards').removeClass('less-padding');
        },

        addGiftPayment: function (e) {
            var $target = $(e.target),
                card = this.giftCards.get($target.data('id')),
                $input = $target.closest('.line').find('input'),
                val = $input.inputFormat('getValue'),
                payment = _.findWhere(this.paymentTypes, {paymentTypeEnum: 1448});

            if (val <= 0) {
                new YesNoModal({
                    question: "Please enter an amount greater than zero.",
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            } else if (val > card.get('remaining')) {
                $input.addClass('error');
                new YesNoModal({
                    question: "Please enter an amount less than the amount remaining.",
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }
            $input.removeClass('error');
            this.customClose({
                type: 'add',
                checkout: {
                    paymentType: payment,
                    amount: val,
                    giftCard: card.toJSON()
                }
            });
            card.set('remaining', +parseFloat(card.get('remaining') - val).toFixed(2));
            $target.closest('.checkout-gift-line').remove();
        },

        checkoutValidate: function (catDisplayName) {
            var $form = this.$('.checkout-form'),
                toSend = {},
                errors = false,
                $formGroup = '';

            toSend['checkout-amount'] = this.$('#checkout-amount').inputFormat('getValue');

            switch(catDisplayName) {
                case 'Credit/Debit':
                    var $checkedLabel = this.$('.card-type.checked label');
                    var cardCheckedValue = $checkedLabel.html();
                    if (cardCheckedValue) {
                        toSend[$checkedLabel.attr('for')] = cardCheckedValue;
                    } else {
                        toSend[this.$('.card-type .img').attr('id')] = cardCheckedValue;
                    }
                    break;
                case 'Check':
                    toSend['checkout-check'] = +this.$('#checkout-check').val();
                    break;
                case 'Account':
                    toSend['checkout-account'] = this.$('#checkout-account-balance').html();
                    break;
                case 'Other':
                    var checkedOtherVal = this.$('.other-type.checked').attr('id');
                    if (checkedOtherVal) {
                        toSend[checkedOtherVal] = checkedOtherVal.substr(15);
                    } else {
                        toSend[this.$('.other-type').attr('id')] = checkedOtherVal;
                    }
                    break;
            }

            _.each(toSend, function (value, key) {
                $formGroup = $form.find('#'+key).closest('.form-group');
                if (!value && $formGroup.hasClass('required')) {
                    $formGroup.addClass('error');
                    errors = true;
                } else {
                    $formGroup.removeClass('error');
                }
            });
            if (errors) {
                //this.$('.error:first input').focus();
                return false;
            }

            return toSend;
        },

        beforeSubmit: function (e) {
            this.$(':focus').blur();
            if (this.model.get('id')) {
                this.saveEdition();
            } else {
                this.checkoutSubmit(e);
            }
        },

        checkoutSubmit: function (e) {
            e.preventDefault();
            var payment,
                catDisplayName = this.model.get('title'),
                additionalFields = {};

            var data = this.checkoutValidate(catDisplayName);
            if (!data) {
                return false;
            }

            switch (catDisplayName) {
                case 'Cash':
                    payment = _.findWhere(this.paymentTypes, {catDisplayName: catDisplayName});
                    break;
                case 'Credit/Debit':
                    payment = _.findWhere(this.paymentTypes, {shortcut: this.$('.card-type.checked label').html()});
                    break;
                case 'Check':
                    payment = _.findWhere(this.paymentTypes, {catDisplayName: catDisplayName});
                    additionalFields = {
                        checkNumber: data['checkout-check']
                    };
                    break;
                case 'Account':
                    payment = _.findWhere(this.paymentTypes, {catDisplayName: catDisplayName});
                    additionalFields = {
                        client: this.clientAccount
                    };
                    break;
                case 'Other':
                    payment = _.findWhere(this.paymentTypes, {displayName: this.$('.other-type.checked').html()});
                    break;
            }
            this.customClose({
                type: 'add',
                checkout: _.extend({
                    paymentType: payment,
                    amount: data['checkout-amount']
                }, additionalFields)
            });
        },

        checkoutReset: function (e) {
            $(e.target).closest('form').trigger('reset');
            $(e.target).find('.form-group').removeClass('error');
            this.$('.checkout-checkbox').removeClass('checked');
            setTimeout(function () {
                this.$('#checkout-account-balance').text('').removeClass('negative positive');
                this.$('#checkout-amount').inputFormat('setValue', 0);
                this.$('#checkout-check').inputFormat('setValue', '');
                this.$('#checkout-gift-number').inputFormat('setValue', '');
                this.$('.search-close-icon').removeClass('visible');
            }.bind(this), 0);
            this.giftCards.removeAll();
        },

        saveEdition: function () {
            var $amountInput = this.$('#checkout-edit-amount'),
                amountInputVal = $amountInput.inputFormat('getValue'),
                $checkNumberInput = this.$('#checkout-edit-check'),
                checkNumber = $checkNumberInput.inputFormat('getValue'),
                errors = false,
                gCard = this.model.get('giftCard'), remaining;
            if (gCard) {
                remaining = gCard['remaining'] + +this.initialAmount;
                gCard['remaining'] = gCard['value'] - gCard['totalUsages'] - amountInputVal;
                this.model.set('giftCard', gCard);
            }

            if (!amountInputVal) {
                $amountInput.closest('.form-group').addClass('error');
                errors = true;
            }

            if (this.model.get('checkNumber')) {
                if (!checkNumber) {
                    $checkNumberInput.closest('.form-group').addClass('error');
                    errors = true;
                } else {
                    $checkNumberInput.closest('.form-group').removeClass('error');
                    this.model.set('checkNumber', checkNumber);
                    errors = false;
                }
            }
            if (gCard) {
                if (remaining < amountInputVal) {
                    new YesNoModal({
                        question: "Please enter an amount less than the amount remaining.",
                        positiveText: 'OK',
                        negativeText: '',
                        noCallback: true
                    }).render();
                    $amountInput.closest('.form-group').addClass('error');
                    errors = true;
                    gCard['remaining'] = gCard['value'] - gCard['totalUsages'] - +this.initialAmount;
                    this.model.set('giftCard', gCard);
                } else {
                    $amountInput.closest('.form-group').removeClass('error');
                    this.model.set('giftCard', gCard);
                    this.model.set('amount', amountInputVal);
                }
            } else if(amountInputVal) {
                $amountInput.closest('.form-group').removeClass('error');
                this.model.set('amount', amountInputVal);
                errors = false;
            }

            if (errors) {
                return false;
            }
            this.customClose({
                type: 'edit',
                checkout: this.model.toJSON()
            });
        },

        customClose: function (response) {
            if (this.model.get('phone')) {
                if (response) {
                    if (response.type) {
                        if (response.type != 'returnToMenu') {
                            this.trigger('checkoutEvent', response);
                        }
                    } else {
                        this.trigger('checkoutEvent', {
                            type: 'back'
                        });
                    }
                }
                this.closeView();
            } else {
                if (response) {
                    this.trigger('checkoutEvent', _.extend({
                        doneLeave: true
                    }, response));
                } else {
                    this.trigger('checkoutEvent', {type: 'cancel'});
                }
                this.$el && this.$('#checkout-reset').trigger('tap');
            }
        },

        closeView: function () {
            ModalView.prototype.closeView.call(this);
            this.giftCards.removeAll();
        },

        checkoutBack: function () {
            this.trigger('checkoutEvent', {
                type: 'back'
            });
            this.closeView();
        },

        checkoutCancel: function () {
            this.customClose();
        }
    });
});
