define([
    'jquery',
    'searchPlugin',
    'views/modal',
    'text!../../../templates/smart-receipt/client-from-appointment-modal.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'bridge'
], function ($, searchPlugin, ModalView, AddClientTemplate, UserSearchTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#client-from-appointment-modal .content',
        template: _.template(AddClientTemplate),
        scrollWrap: '#client-from-appointment-scroll-wrap',
        scrollbars: false,
        events: {
            'gotList #appointment-search': 'gotAppointList',
            'tap #add-client-button': 'addClient',
            'tap #add-client-cancel': 'cancel'
        },
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                { id: 'add-client-button', className: 'green', size: '50', value: 'Add' },
                { id: 'add-client-cancel', className: 'red', size: '50', value: 'Cancel' }
            ],
            tabletButtons: [
                { id: 'add-client-button', className: 'green', size: '50', value: 'Add' },
                { id: 'add-client-cancel', className: 'red', size: '50', value: 'Cancel' }
            ],
            touchCallBacks: {
                'add-client-button': 'addClient',
                'add-client-cancel': 'cancel'
            }
        },

        initialize: function() {
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        onRender: function () {
            this.$el.attr('class', 'content loading-appointments');

            this.$('#appointment-search').searchPlugin({
                filterFunction: getTodayAppointments,
                parentScroll: this.scroll,
                fullScreen: true,
                template: _.template(UserSearchTemplate),
                emptyMessage: 'There are no clients with appointments today'
            });
            getTodayAppointments().done(function (r) {
                if (r.status === undefined) {
                    this.gotAppointList(null, r.length);
                }
            }.bind(this));

            return this;
        },

        gotAppointList: function (e, length) {
            var wasLoading = this.$el.hasClass('loading-appointments');
            this.$el.removeClass('has-appointments no-appointments loading-appointments').addClass(length > 0 ? 'has-appointments' : 'no-appointments');
            if (length == 0) {
                this.$('.empty').html('There are no clients with appointments today');
            }
            if (wasLoading) this.$('#appointment-search').focus();
        },

        addClient: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            var $input = this.$('#appointment-search');
            var user;

            if ($input.data('id')) {
                $input.parent().removeClass('error');
                user = $input.searchPlugin('getSelectedObject');
                if (!user) {
                    $input.parent().addClass('error');
                    return false;
                }
            } else {
                $input.parent().addClass('error');
                return false;
            }

            this.trigger('closeEvent', user);
            this.closeView();
        }

    });
});
