/**
 * Created by petermomot on 3/23/15.
 */
define([
    'jquery',
    'inputFormat',
    'searchPlugin',
    'data/consts',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/discount-sale-modal.html',
    'text!../../../templates/smart-receipt/discount-item.html',
    'text!../../../templates/smart-receipt/available-saleline-template.html',
    'bridge'
], function ($, inputFormat, searchPlugin, Consts, ModalView, YesNoModal, DiscountSaleTemplate, DiscountItemTemplate, SaleLineTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#discount-sale-modal .content',
        template: _.template(DiscountSaleTemplate),
        discountItemTemplate: _.template(DiscountItemTemplate),
        saleLineTemplate: _.template(SaleLineTemplate),
        scrollWrap: '#discount-sale-scroll-wrap',
        scrollbars: false,
        yesNoModal: undefined,
        discountEffectEnum: {
            percent: 1674,
            dollar: 1675,
            specific: 1676,
            spread: 1724
        },
        editedId: undefined,
        selectedSaleLines: [],
        allSelectedSaleLines: [],
        discountForEdit: {},
		IsCalculatingNow: false,
		UpdateCompleted: true,
		
		initialize: function (options) {
            this.model = new Backbone.Model(options.model);
            this.selectedSaleLines = [];
            this.allSelectedSaleLines = [];
            this.discountForEdit = {};
            this.onPhone = $('html').hasClass('mobile');
            
            // TODO update discounts list on sales lines update

            if (options.multipleDiscount) {
                this.el = this.onPhone ? '#discount-sale-modal .content' : '#tablet-discount-sale-modal .content';
                this.setElement(this.el);
                this.multipleDiscount = true;
                this.saleLines = options.saleLines;
                this.model.set('discounts', []);
                this.model.set('item', {});

                if(options.discountForEdit){
                    this.discountForEdit = options.discountForEdit.toJSON();
                }

            }else{
                this.multipleDiscount = false;
                this.model.set('allDiscounts', options.discounts);
                this.listenTo(this.model, 'change:discounts', this.renderDiscounts);
            }
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        render: function(){
            ModalView.prototype.render.call(this);
			
            if ( !this.multipleDiscount ){
                this.renderDiscounts();
            } else {
                this.$('.predefined').hide();
                this.$('.spread-discount').show();
                this.$('.select-all').show();
            }

            if (this.multipleDiscount){
                this.renderSaleLines(this.saleLines);
            }

            this.addPlugins();

            $(':focus').blur();
            this.enableFirst();

            setTimeout(function() {
                $('.discount-placeholder').hide();
            }, 600);

            if (!_.isEmpty(this.discountForEdit)) {
                this.setDataForEditing(this.discountForEdit);
            }

            return this;
        },

        addPlugins: function() {
		
            if (!this.multipleDiscount) {
                if(this.model.get('client').id) {
                    this.$('#discount-coupon-value').searchPlugin({
                        filterFunction: function(filter, pageNumber, pageSize){
                            return getValidDisountsFor(filter, this.model.get('item').id, this.model.get('client').id, pageNumber, pageSize);
                        }.bind(this),
                        parentScroll: this.scroll
                    });

                    this.$('#discount-coupon-value').searchPlugin('updateRemoveList', _.pluck(this.model.get('discounts'), 'discountId'));
                } else {
                    this.couponDisabled = true;
                }
            }
            else this.$('#discount-spread-value').inputFormat();

            this.$('#discount-percent-value').inputFormat({
                type: 'percentage',
                places: 4,
                allowMinus: false
            });
            this.$('#discount-dollar-value').inputFormat({
                allowMinus: false
            });
            this.$('#discount-specific-value').inputFormat({
                allowMinus: false
            });
        },

        renderDiscounts: function () {
            var discounts = this.model.get('discounts'),
                price = this.model.get('item').price,
                rendered = [];

            _.each(discounts, function (d, index) {
                price = price - d['priceAdj'];
                d = _.extend(d, _.findWhere(this.model.get('allDiscounts'), {id: d.discountId}), {
                    isLast: index+1 === d.order,
                    subTotal: price
                });
                rendered.push(this.discountItemTemplate(d));
            }.bind(this));
            this.$('#discount-coupon-value').searchPlugin('updateRemoveList', _.pluck(discounts, 'discountId'));

            if (rendered.length) {
                this.$('#discount-items').html(rendered);
                this.$('.discounts').removeClass('hidden')
            } else {
                this.$('.discounts').addClass('hidden');
            }
        },

        renderSaleLines: function(saleLines){
            var tempTemplate = '';

            _.each(saleLines, function(line){
                switch (line.item.type) {
                    case Consts.SaleLineTypes.SERVICE:
                    case Consts.SaleLineTypes.PRODUCT:
                        tempTemplate += this.saleLineTemplate(line);
                        break;
                }
            }.bind(this));

            this.$('#sale-lines-list').html(tempTemplate);

        },

        events: {
            'tap .radio-input': 'tapRadioInput',
            'tap .radio-line label': 'tapRadioLabel',
            'tap .commission label': 'checkCommission',
            'tap #discount-commission': 'checkCommission',
            'tap .button-edit': 'editDiscount',
            'tap .button-del': 'delDiscount',
            'touchend #apply-discount': 'applyDiscount',
            'touchend #cancel-discount': 'customClose',
            'tap .available-sale-line' : 'selectSaleLine',
            'touchstart .discount-placeholder' : 'preventFocus',
            'tap .select-all' : 'selectAll'
        },

        preventFocus: function() {
            this.$('.discount-placeholder').hide();
        },

        enableFirst: function () {
            this.$('.radio-input').removeClass('checked');
            this.$('#discount-percent').addClass('checked');
            this.$('#cancel-discount').html('close');
            if (!this.multipleDiscount) this.$('.radio-line.form-group').show();
            this.changeType();
            this.editedId = undefined;
        },

        changeType: function () {
            var $target = this.$('.radio-input.checked'),
                $closestLine = $target.closest('.radio-line');

            this.$('.radio-line').addClass('disabled').removeClass('error');
            this.$('.discount-value').prop('disabled', true);
            $closestLine.removeClass('disabled');
            this.blurInputs();

            $closestLine.find('.discount-value').prop('disabled', false).show();
			$closestLine.find('.input-format').css('background', '');
            if ($target.data('value') === 'coupon')
                this.$('.not-coupon').hide();
            else
                this.$('.not-coupon').show();
        },

        tapRadioInput: function (e) {
            if(this.couponDisabled && $(e.target).closest('.predefined').length) {
                return false;
            }
			this.removeErrors();
            this.$('.radio-input').removeClass('checked');
            $(e.target).addClass('checked');
            this.changeType();
        },

        tapRadioLabel: function (e) {
            if(this.couponDisabled && $(e.target).closest('.predefined').length) {
                return false;
            }
			this.removeErrors();
            this.$('.radio-input').removeClass('checked');
            this.$('#'+$(e.target).attr('for')).addClass('checked');
            this.changeType();
        },
		
		removeErrors: function(){
		    this.$("div.form-group.radio input[type='" + window.inputType + "']").parent().removeClass('error');
		},

        blurInputs: function () {
            _.each(this.$('.discount-value'), function (input) {
                var $i = $(input);
                switch($i.attr('id')) {
                    case 'discount-percent-value':
                    case 'discount-dollar-value':
                    case 'discount-specific-value':
                        $i.inputFormat('setValue', 0).hide();
                        break;
                    case 'discount-spread-value':
                        if(this.multipleDiscount)
                            $i.inputFormat('setValue', 0).hide();
                        break;
                    case 'discount-coupon-value':
                        $i.val('').data('id', '');
                        $i.parent().find('.search-plugin-box').removeClass('visible');
                        break;
                }
                $i.blur().attr('disabled', true);
            }.bind(this));
            this.$('#discount-reason').val('');
            this.$('#discount-reason-input').html('').blur();
            this.$('#discount-commission').removeClass('checked');
        },

        checkCommission: function () {
            this.$('#discount-commission').toggleClass('checked');
        },

        editDiscount: function (e) {
            this.editedId = $(e.target).data('id');
            var d = _.findWhere(this.model.get('allDiscounts'), {id: this.editedId});
            var radioVal = _.invert(this.discountEffectEnum)[d['discountEffectEnum']];
            this.blurInputs();
            this.$('#discount-reason').val(d.reason);
            this.$('#discount-reason-input').html(this.getHtmlFromText(d.reason)).blur();
            this.$('#discount-commission').toggleClass('checked', d.commissionOnBasePrice);
            this.$('#cancel-discount').html('cancel');
            this.$('.radio-line').addClass('disabled').removeClass('error');
            this.$('.radio-line.form-group').hide();
            this.$('.radio-input').removeClass('checked');

            this.$('#discount-'+radioVal+'-value').prop('disabled', false).inputFormat('setValue', d.amount).show()
				.closest('.radio-line').removeClass('disabled')
				.find('.radio-input').addClass('checked');
        },

        delDiscount: function (e) {
			if(this.IsCalculatingNow)
				return;
            this.IsCalculatingNow = true;

            this.enableFirst();
            this.trigger('discountEvent', {
                type: 'delDiscount',
                discountId: $(e.target).data('id'),
				deleteDoneCallBack: function(){
					this.IsCalculatingNow = false;
                }.bind(this)
            });
        },

        applyDiscount: function (e) {

            if(this.multipleDiscount && !this.selectedSaleLines.length) {
                new YesNoModal({
                    question: 'Select an item to discount',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }

            if (!this.isValidInputLength($('#discount-reason'))) {
                new YesNoModal({
                    question: "The field Reason must be a string or array type with a maximum length of '256'",
                    positiveText: 'Ok',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }

            var needle, $input, radioVal = this.$('.radio-input.checked').data('value'), val;
            this.$('.discount-value').blur();

            switch (radioVal) {
                case 'percent':
                case 'dollar':
                case 'specific':
                case 'spread':
                    $input = this.$('#discount-'+radioVal+'-value');
                    val = $input.inputFormat('getValue');
                    if (val) {
                        $input.parent().removeClass('error');
						
						if(this.IsCalculatingNow)
							return false;
							
                        this.addManualDiscount(val, this.discountEffectEnum[radioVal], e);
						
                    } else {
                        $input.parent().addClass('error');
                        return false;
                    }
                    break;
                case 'coupon':
                    $input = this.$('#discount-coupon-value');
                    if ($input.data('id')) {
                        $input.parent().removeClass('error');
                        needle = $input.searchPlugin('getSelectedObject');
                        if (!needle) {
                            $input.parent().addClass('error');
                            return;
                        }
                        this.addDiscount(needle);
                    } else {
                        $input.parent().addClass('error');
                    }
                    break;
            }
        },

        addManualDiscount: function(amount, effect, e) {
            var d, dataNew = {
                amount: amount,
                commissionOnBasePrice: this.$('#discount-commission').hasClass('checked'),
                discountEffectEnum: effect,
                reason: this.getTextFromHtml(this.$('#discount-reason-input'))
            };

            if (this.editedId) {
                d = {
                    discount: _.extend(
                        _.findWhere(this.model.get('allDiscounts'), {id: this.editedId}),
                        dataNew
                    )
                };
            } else {
                d = { discount: dataNew };
            }
			if(!this.IsCalculatingNow){
				this.IsCalculatingNow = true;
				setManualDiscount(JSON.stringify(d))
					.done(function (r) {
						if (r.status === undefined){
							this.UpdateCompleted = false;
							this.addDiscount(r, e);
							}
					}.bind(this))
					.always(function(){
						this.IsCalculatingNow = false;
					}.bind(this));
			}
        },

        selectSaleLine: function(e){
            if( $(e.target).hasClass('selected') ){
                this.selectedSaleLines = _.reject(this.selectedSaleLines, function(id){
                    return id == $(e.target).attr('data-saleline-id');
                });
            }else{
                this.selectedSaleLines.push($(e.target).attr('data-saleline-id'));
            }
            $(e.target).toggleClass('selected');
        },

        selectAll: function(e){
            var allLines = _.pluck(_.filter(this.saleLines, function(elem){
                if(elem.item.type == Consts.SaleLineTypes.SERVICE || elem.item.type == Consts.SaleLineTypes.PRODUCT) {
                    return true;
                }
            }), 'id');
            this.selectedSaleLines = allLines !== undefined ? allLines : [];
            this.$('.available-sale-line')
                .removeClass('selected')
                .addClass('selected');

        },

        setDataForEditing: function(discount, e){
            var radioVal = _.invert(this.discountEffectEnum)[discount['discountEffectEnum']];
            this.$('#discount-reason').val(discount.reason);
            this.$('#discount-reason-input').html(this.getHtmlFromText(discount.reason)).blur();
            this.$('#discount-commission').toggleClass('checked', discount.commissionOnBasePrice);
            this.$('.radio-line').addClass('disabled').removeClass('error');
            this.$('.discount-value').prop('disabled', true);
            this.$('.radio-line.form-group').hide();
            this.$('.radio-input').removeClass('checked');
            this.$('#discount-'+radioVal+'-value')
                .prop('disabled', false)
                .inputFormat('setValue', discount.amount).show()
                .closest('.radio-line').removeClass('disabled')
                .find('.radio-input').addClass('checked');

            var saleLinesObjList = _.filter(this.saleLines, function(line){
                var isSelected = false;

                _.each(line.discounts, function(searchedDiscount){
                    if( searchedDiscount.discountId == discount.id){
                        isSelected = true;
                        return;
                    }
                });

                return isSelected;
            });
            this.allSelectedSaleLines = _.pluck(saleLinesObjList, 'id');
            this.selectedSaleLines = _.pluck(saleLinesObjList, 'id');
            _.each(this.selectedSaleLines, function(saleLineID){
                _.each($('.available-sale-line'), function(el){
                    if($(el).attr('data-saleline-id') == saleLineID){
                        $(el).addClass('selected');
                    }
                });
            });
        },

        addDiscount: function (data, e) {
            var that = this;
            if(this.multipleDiscount){
                if(_.isEmpty(this.discountForEdit)){
                    this.trigger('applyEvent', {
                        type: 'addDiscount',
                        discount: data,
                        saleLines : this.selectedSaleLines
                    });
                }else{
                    var saleLinesForEditing = _.filter(this.allSelectedSaleLines, function(id){
                        var forEditing = false;
                        _.each(that.selectedSaleLines, function(selectedId){
                            if(selectedId == id){
                                forEditing = true;
                                return;
                            }
                        });
                        return forEditing;
                    });
                    var saleLinesForDeleting = _.reject(this.allSelectedSaleLines, function(id){
                        var forDeleting = false;
                        _.each(that.selectedSaleLines, function(selectedId){
                            if(selectedId == id){
                                forDeleting = true;
                                return;
                            }
                        });
                        return forDeleting;
                    });
                    var saleLinesForAdding = _.reject(that.selectedSaleLines, function(id){
                        var forAdding = false;
                        _.each(that.allSelectedSaleLines, function(selectedId){
                            if(selectedId == id){
                                forAdding = true;
                                return;
                            }
                        });
                        return forAdding;
                    });

                    data.id = this.discountForEdit.id;

                    this.trigger('applyEvent', {
                        discount: data,
                        discountId: data.id,
                        saleLinesForAdding: saleLinesForAdding,
                        saleLinesForDeleting: saleLinesForDeleting,
                        saleLinesForEditing: saleLinesForEditing
                    });
                }

                this.customClose();
            } else {
                if (this.editedId) {
                    this.trigger('discountEvent', {
                        type: 'editDiscount',
                        discount: data
                    });
                } else {
                    if (data.amount === 0) {
                        new YesNoModal({
                            question: 'Discount amount should not be empty or zero',
                            positiveText: 'Ok',
                            negativeText: '',
                            noCallback: true
                        }).render();
                        return false;
                    }
                    this.trigger('discountEvent', {
                        type: 'addDiscount',
                        discount: data
                    });
                }
                this.enableFirst();
            }
        },

        customClose: function (e) {
            if (e && $(e.target).html() == 'cancel') {
                this.enableFirst();
            } else {
                if(!this.multipleDiscount && (this.IsCalculatingNow || !this.UpdateCompleted))
                    return false;

                this.trigger('discountEvent', {
                    type: 'calc'
                });
                // applyEvent - for manual adjustments
                this.trigger('applyEvent', {});
                this.trigger('discountEvent', {type: 'makeUndefined'});
                this.closeView();
                
            }
        }
    });
});
