/**
 * Created by petermomot on 3/23/15.
 */
define([
    'jquery',
    'data/consts',
    'inputFormat',
    'searchPlugin',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/edit-sale-modal.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'bridge'
], function ($, consts, inputFormat, searchPlugin, ModalView, YesNoModal, EditSaleTemplate, UserSearchTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#edit-sale-modal .content',
        template: _.template(EditSaleTemplate),
        scrollWrap: '#edit-sale-scroll-wrap',
        scrollbars: false,
        serviceTemplate: _.template('<div class="option" data-id="<%= id %>"><%= shortcut %> : <%= displayName %></div>'),
        employeeTemplate: _.template('<div class="option" data-id="<%= id %>"><%= code %> : <%= lastName %>, <%= firstName %></div>'),
        yesNoModal: undefined,
        events: {
            'tap #remove-sale-line': 'removeSaleLine',
            'blur #edit-purchased-for': 'changePurchaseFor',
            'blur #edit-card-price, #edit-card-value': 'changeCardAmounts',
            'blur #edit-service-search': 'changeService',
            'blur #edit-sold-by': 'changeEmployee',
            'blur #edit-service-price': 'changePrice',
            'blur #edit-app-type': 'changeAppointmentType',
            'tap .button-plus': 'plusQuantity',
            'tap .button-minus': 'minusQuantity',
            'focus #edit-quantity': 'focusQuantity',
            'blur #edit-quantity': 'changeQuantity',
            'tap #edit-service-discount': 'editDiscount'
        },
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                { id: 'remove-sale-line', className: 'red', size: '50', value: 'Remove' },
                { id: 'cancel-sale-line', className: 'green', size: '50', value: 'OK' }
            ],
            tabletButtons: [
                { id: 'remove-sale-line', className: 'red', size: '50', value: 'Remove' },
                { id: 'cancel-sale-line', className: 'green', size: '50', value: 'OK' }
            ],
            touchCallBacks: {
                'remove-sale-line': 'removeSaleLine',
                'cancel-sale-line': 'onOkHandler'
            }
        },

        initialize: function () {
            this.giftCard = this.model.get('item')['displayName'] === 'GC';
            this.listenTo(this.model, 'change:employee', function () {
				if(this.model.get('item').type !== consts.SaleLineTypes.PRODUCT)
					this.$('#edit-service-search').searchPlugin('changeFilterFunctionParam', this.model.get('employee')['id']);
            }.bind(this));
            this.listenTo(this.model, 'change:item', function () {
                this.$('#edit-service-price').inputFormat('setValue', this.model.get('item')['price']);
                if (this.giftCard) {
                    this.$('#edit-card-price').inputFormat('setValue', this.model.get('item')['price']);
                    this.$('#edit-card-value').inputFormat('setValue', this.model.get('item')['value']);
                }
            }.bind(this));
            this.listenTo(this.model, 'change:discounts', this.discountNumber);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
			this.onOkHandler = this.onOkButton.bind(this);
        },

        render: function(){
            ModalView.prototype.render.call(this);

            this.addPlugins();
            this.discountNumber();
            if (this.giftCard) {
                this.$('.edit-sale').addClass('gift-card');
            }
            return this;
        },

        addPlugins: function () {
            if (this.giftCard) {
                this.$('#edit-purchased-for').searchPlugin({
                    filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                         return getClients(filter, clientAccountChargable, undefined, undefined, pageNumber, pageSize);
                    },
                    parentScroll: this.scroll,
                    template: _.template(UserSearchTemplate)
                });

                this.$('#edit-card-price').inputFormat();
                this.$('#edit-card-value').inputFormat();
            }
            var searchOptions = {
                parentScroll: this.scroll,
                template: this.serviceTemplate
            };
            switch(this.model.get('item').type) {
                case consts.SaleLineTypes.SERVICE:
                    _.extend(searchOptions, {
                        filterFunction: filterServices,
                        filterFunctionParam: this.model.get('employee')['id']
                    });
                    this.$('#edit-service-search').searchPlugin(searchOptions);
                    break;
                case consts.SaleLineTypes.PRODUCT:
                    _.extend(searchOptions, {
                        filterFunction: filterProducts
                    });
                    this.$('#edit-service-search').searchPlugin(searchOptions);
                    break;
            }


            this.$('#edit-sold-by').searchPlugin({
                filterFunction: filterEmployeesWithParameter,
                parentScroll: this.scroll,
                template: this.employeeTemplate
            });
			this.$('#edit-sold-by').data('id','');


            this.$('#edit-app-type').searchPlugin({
                filterFunction: getAppointmentTypes,
                parentScroll: this.scroll,
                template: this.serviceTemplate
            });

            this.$('#edit-service-price').inputFormat();
            this.$('#edit-quantity').inputFormat({
                type: 'int',
                places: 4
            });

			$('#cancel-sale-line').on('tap', this.onOkHandler);
        },

        touchStartButton: function (e) {
            $(e.target).addClass('active');
        },

        touchEndButton: function (e) {
            $(e.target).removeClass('active');
        },

        discountNumber: function () {
            var count = this.model.get('discounts').length;
            if (count) {
                this.$('#discount-number').addClass('visible').html(count);
            } else {
                this.$('#discount-number').removeClass('visible').html('');
            }
        },

        changeService: function (e) {
            try {
                var service = $(e.target).searchPlugin('getSelectedObject');
                if (service) {
                    if (this.model) {
                        this.model.set('item', service);
                        this.trigger('changeEvent', {
                            type: 'service',
                            item: service
                        });
                    }
                }
            } catch (e) {
            }
        },

        changePurchaseFor: function (e) {
            var client = $(e.target).searchPlugin('getSelectedObject');
            if (client && this.model) {
                this.trigger('changeEvent', {
                    type: 'purchase-for',
                    client: client
                });
            }
        },

        changeCardAmounts: function (e) {
            var $target = $(e.target),
                val = $target.inputFormat('getValue'),
                id = $target.attr('id'),
                name = id.substr(id.length-5);
            $target.closest('.form-group').find('input').removeClass('error');
            if (val !== this.model.get('item')[name]) {
                this.trigger('changeEvent', {
                    type: 'card-'+name,
                    amount: val,
                    name: name
                });
            }
        },

        changeEmployee: function (e) {
            var employee = $(e.target).searchPlugin('getSelectedObject');
            if (employee) {
				$('#cancel-sale-line').off('tap', this.onOkHandler).addClass('disabled');

                this.model.set('employee', employee);
                if (this.model.get('item')['type'] == consts.SaleLineTypes.SERVICE) {
                    getServiceByEmployee(this.model.get('item')['id'], employee.id).done(function (r) {
                        if (this.model) {
                            if (r.status === undefined) {
                                this.model.set('item', r)
                            }
                            this.changeEmployeeCallback(employee);
                        }
                    }.bind(this))
					.always(function(){	$('#cancel-sale-line').on('tap', this.onOkHandler).removeClass('disabled'); }.bind(this));
                } else {
                    this.changeEmployeeCallback(employee);
					$('#cancel-sale-line').on('tap', this.onOkHandler).removeClass('disabled');
                }
            }
        },
        changeEmployeeCallback: function (employee) {
            this.trigger('changeEvent', {
                type: 'employee',
                employee: employee,
                item: this.model.get('item')
            });
        },

        changePrice: function (e, close) {
            var val = this.$('#edit-service-price').inputFormat('getValue');
            val = val ? val : 0;
            if (val !== this.model.get('item')['price']) {
				$('#cancel-sale-line').off('tap', this.onOkHandler).addClass('disabled');
                var d = {
                    discount: {
                        amount: val,
                        commissionOnBasePrice: false,
                        discountEffectEnum: 1676,
                        reason: ''
                    }
                };
                setManualDiscount(JSON.stringify(d)).done(function (r) {
                    if (r.status === undefined) {
                        this.trigger('changeEvent', {
                            type: 'discount',
                            discount: r
                        });
                    }
                    if (close) {
                        this.closeView(e, true);
                    }
                }.bind(this))
				.always(function(){
					$('#cancel-sale-line').on('tap', this.onOkHandler).removeClass('disabled');
				}.bind(this));
            } else {
                this.trigger('changeEvent', {
                    type: 'tryToRemoveDiscount'
                });
                if (close) {
                    this.closeView(e, true);
                }
            }
        },

        changeAppointmentType: function (e) {
            this.trigger('changeEvent', {
                type: 'appointment',
                appointment: $(e.target).searchPlugin('getSelectedObject')
            });
        },

        removeSaleLine: function (e) {
            this.trigger('changeEvent', {
                type: 'remove',
                id: this.model.get('id')
            });
            this.closeView(e);
        },

        plusQuantity: function () {
            var $input = this.$('#edit-quantity');
            $input.inputFormat('setValue', $input.inputFormat('getValue')+1);

            this.trigger('changeEvent', {
                type: 'quantity',
                quantity: $input.inputFormat('getValue')
            });
        },

        minusQuantity: function () {
            var $input = this.$('#edit-quantity');
            $input.blur().inputFormat('setValue', $input.inputFormat('getValue')-1);
            var val = $input.inputFormat('getValue');
            if (val === 0) {
                this.zeroValue();
            } else {
                this.trigger('changeEvent', {
                    type: 'quantity',
                    quantity: val
                });
            }
        },

        changeQuantity: function (e) {
            var val = $(e.target).inputFormat('getValue');
            if (val <= 0) {
                this.zeroValue();
                return;
            }
            this.trigger('changeEvent', {
                type: 'quantity',
                quantity: val
            });
        },

        zeroValue: function () {
            this.yesNoModal = new YesNoModal({
                question: 'Do you want to remove this item?',
                positiveText: 'Yes',
                positiveCallback: {
                    triggerEvent: 'yesNoEvent',
                    sendProperty: true
                },
                negativeText: 'No',
                negativeCallback: {
                    triggerEvent: 'yesNoEvent',
                    sendProperty: false
                }
            }).render();
            this.listenToOnce(this.yesNoModal, 'yesNoEvent', function (r) {
                if (r) {
                    this.$('#remove-sale-line').trigger('tap');
                } else {
                    this.$('#edit-quantity').inputFormat('setValue', 1);
                    this.trigger('changeEvent', {
                        type: 'quantity',
                        quantity: 1
                    });
                }
            }.bind(this));
        },

        editDiscount: function () {
            this.closeView(null, {
                type: 'showDiscounts',
                id: this.model.get('id')
            });
        },

        validateEditing: function() {
            var $searchFields = this.giftCard ? $('#edit-purchased-for, #edit-sold-by') : $('#edit-service-search, #edit-sold-by, #edit-app-type');
            var valid = true;
            $searchFields.each(function (index, element) {
                if (element.id == 'edit-app-type') {
                    if (element.value.length && $(element).data('changed') && !$(element).searchPlugin('getSelectedObject')) {
                        valid = false;
                        $(element).addClass('error');
                    }
                } else if (!element.value.length || ($(element).data('changed') && !$(element).searchPlugin('getSelectedObject'))) {
                    valid = false;
                    $(element).addClass('error');
                }
            });
            return valid;
        },

        onOkButton: function () {
            if (this.model.get('isRefund')) {
                this.closeView();
                return true;
            }
            if(this.$('#edit-quantity').inputFormat('getValue') <= 0) {
                this.zeroValue();
                return;
            }
            if(this.validateEditing()) {
                this.closeView();
            }
        },

        closeView: function (e, someParam) {
            if (this.$('input:focus').attr('id') == 'edit-service-price' && someParam === undefined) {
                this.changePrice(e, true);
            } else {
                this.$('input:focus').blur();
                this.trigger('changeEvent', {
                    type: 'calc',
                    someParam: someParam
                });
                ModalView.prototype.closeView.call(this);
            }
        }

    });
});
