/**
 * Created by petermomot on 7/20/15.
 */
define([
    'jquery',
    'moment',
    'data/permissions',
    'views/smart-receipt/on-hold-modal',
    'views/yes-no-modal',
    'views/smart-receipt/on-hold-deleted-modal',
    'bridge'
], function ($, moment, PermModel,
             HoldBaseModalView, YesNoModal, HoldDeletedModal) {
    'use strict';

    return HoldBaseModalView.extend({
        el: '#tablet-on-hold-modal .content',
        transactionsType: '',
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                {
                    id: 'on-hold-cancel',
                    className: 'red cancel',
                    size: '20',
                    value: ''
                },
                {
                    id: 'on-hold-show-deleted',
                    className: 'violet-light',
                    size: '80',
                    value: 'show deleted',
                    visible: PermModel.isVisible('evo.register.onholdtab_showdeleted'),
                    securityKey: 'evo.register.onholdtab_showdeleted'
                }
            ],
            tabletButtons: [
                {
                    id: 'on-hold-cancel',
                    className: 'red',
                    size: '100',
                    value: 'cancel'
                }
            ],
            touchCallBacks: {
                'on-hold-cancel': 'tapCancel',
                'on-hold-show-deleted': 'showDeleted'
            }
        },

        initialize: function (options) {
            var o = _.extend({}, options, {
                phone: $('html').hasClass('mobile'),
                PermModel: PermModel
            });
            if (o.phone) {
                this.el = '#on-hold-modal .content';
                this.setElement(this.el);
                this.breadCrumbs = ['Register', 'On Hold'];
                this.modalButtonsBottom.phone = true;
            }
            this.model = new Backbone.Model(o);
            HoldBaseModalView.prototype.initialize.call(this, options);

            this.events = _.extend({}, HoldBaseModalView.prototype.events, this.events);
        },

        addListeners: function () {
            HoldBaseModalView.prototype.addListeners.call(this);
            this.listenTo(this.holdCollection, 'remove', this.holdItemRemoved);
            this.listenTo(this.holdCollection, 'readyToReturn', function (modelToReturn) {
                this.customClose({
                    type: 'extract',
                    holdItem: modelToReturn
                });
            }.bind(this));
        },

        events: {
            'tap .hold-item-remove': 'holdItemRemove',
            'tap .hold-item-extract': 'holdItemExtract'
        },

        customClose: function (response) {
            this.holdDeletedModal = undefined;
            if (this.model.get('phone')) {
                if (response) this.trigger('holdEvent', response);
                else this.trigger('holdEvent', { type: 'cancel' });
                this.closeView();
                this.resetAll();
            } else {
                if (response) {
                    this.trigger('holdEvent', _.extend({
                        doneLeave: true
                    }, response));
                } else {
                    this.trigger('holdEvent', {
                        type: 'cancel',
                        doneLeave: true
                    });
                }
            }
        },

        showDeleted: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.$el.parent().addClass('hidden');
                this.holdDeletedModal = new HoldDeletedModal({

                }).render();
                this.listenTo(this.holdDeletedModal, 'holdDeletedEvent', function (r) {
                    switch (r.type) {
                        case 'back':
                            this.holdDeletedModal = undefined;
                            this.transactionsType = '',
                            this.$el.parent().removeClass('hidden');
                            this.getSalesAndRender();
                            break;
                        case 'returnToMenu':
                            this.holdDeletedModal = undefined;
                            this.transactionsType = '',
                            this.$el.parent().removeClass('hidden');
                            this.customClose({ type: 'returnToMenu' });
                            break;
                        case 'refreshList':
                            this.transactionsType = ' deleted',
                            this.getSalesAndRender();
                            break;
                    }
                }.bind(this));
            }.bind(this));
        },

        holdItemRemove: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.yesNoModal = new YesNoModal({
                    question: 'Are you sure you want to delete?',
                    positiveText: 'Yes',
                    positiveCallback: function () {
                        this.yesNoModal.closeView();
                        this.deleteSale($(e.target).data('id'));
                    }.bind(this),
                    negativeText: 'No',
                    negativeCallback: function () {
                        this.closeView();
                    }
                }).render();
            }.bind(this));
        },

        holdItemExtract: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                if (this.model.get('restored')) {
                    this.yesNoModal = new YesNoModal({
                        question: 'Warning: Current Transaction Will Be Overwritten When This Transaction is Returned. Would You Like to Continue?',
                        positiveText: 'Yes',
                        positiveCallback: function () {
                            this.yesNoModal.closeView();
                            this.holdCollection.prepareForExtract($(e.target).data('id'));
                        }.bind(this),
                        negativeText: 'No',
                        negativeCallback: function () {
                            this.closeView();
                        }
                    }).render();
                } else {
                    this.holdCollection.prepareForExtract($(e.target).data('id'));
                }
            }.bind(this));
        },

        getSalesCallback: function (r) {
            this.holdCollection && this.holdCollection.add(r);
        },

        deleteSale: function (saleTempId) {
            deleteHoldSale(JSON.parse(localStorage.getItem('workstation'))['drawerId'], saleTempId).done(function (r) {
                if (r) {
                    this.holdCollection.removeByTempId(saleTempId);
                }
            }.bind(this));
        },

        holdItemRemoved: function (holdItem) {
            this.$('.on-hold-line[data-id="' + holdItem.get('saleTempId') + '"]').remove();
            if (this.holdCollection.length === 0 && this.$el)
                this.$('#hold-items-wrap').html('<div class="empty">There are no on hold transactions</div>');
        }
    });
});