﻿/**
 * Created by petermomot on 6/15/15.
 */
define([
    'jquery',
    'moment',
    'data/permissions',
    'data/hold-transactions',
    'views/modal',
    'text!../../../templates/smart-receipt/on-hold-modal.html',
    'text!../../../templates/smart-receipt/on-hold-line.html',
    'bridge'
], function ($, moment, PermModel, OnHoldCollection, ModalView,
             HoldModalTemplate, HoldItemTemplate) {
    'use strict';

    return ModalView.extend({
        template: _.template(HoldModalTemplate),
        holdItemTemplate: _.template(HoldItemTemplate),
        yesNoModal: undefined,
        holdCollection: new OnHoldCollection(),

        initialize: function (options) {
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            this.addListeners();
        },

        onRender: function () {
            this.getSalesAndRender();
        },

        addListeners: function () {
            this.listenTo(this.holdCollection, 'add', this.holdItemAdded);
            this.listenTo(this.holdCollection, 'reset', this.holdCollectionReset);
        },

        tapCancel: function () {
            this.customClose();
        },

        getSalesAndRender: function () {
            this.holdCollection && this.holdCollection.reset();
            getSalesOnHold(this.transactionsType !== '').done(function (r) {
                if (r.status === undefined) {
                    if (r.length == 0) {
                        this.$el && this.$('#hold-items-wrap').html('<div class="empty">There are no' + this.transactionsType + ' on hold transactions</div>');
                    } else {
                        this.$el && this.$('#hold-items-wrap').empty();
                        this.getSalesCallback(r);
                    }
                }
            }.bind(this));
        },

        holdItemAdded: function (holdItem) {
            if (this.$('.on-hold-line[data-id="' + holdItem.get('saleTempId') + '"]').length) return false; // prevent duplicates in list
            var item = holdItem.toJSON(),
                offset = window.SETTINGS.timeZoneOffset;
            item.dateHolded = moment.utc(item.dateHolded.replace(/Z?(\+.*)?(\-[0-9]{2}\:[0-9]{2})?$/, '') + offset).utcOffset(offset);
            this.$el && this.$('#hold-items-wrap').append(this.holdItemTemplate(_.extend(item, {
                PermModel: PermModel,
                moment: moment
            })));
        },

        holdCollectionReset: function () {
            this.scroll && this.scroll.scrollTo(0, 0);
            this.$el && this.$('#hold-items-wrap').html('<div class="empty">Loading' + this.transactionsType + ' on hold transactions</div>');
        },

        resetAll: function () {
            this.holdCollection.reset();
            this.holdCollection = undefined;
            this.yesNoModal = undefined;
        }
    });
});