define([
    'jquery',
    'searchPlugin',
    'data/consts',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/predefined-discount-modal.html',
    'text!../../../templates/smart-receipt/available-saleline-template.html',
    'bridge'
], function($, searchPlugin, Consts, ModalView, YesNoModal,
            PredefinedDiscountModalTemplate, AvailableSaleLineTemplate){
    'use strict';

    return ModalView.extend({
        el: '#predefined-discount-modal .content',
        template: _.template(PredefinedDiscountModalTemplate),
        saleLineTemplate: _.template(AvailableSaleLineTemplate),
        scrollWrap: '#predefined-discount-modal-scroll-wrap',
        scrollbars: false,
        selectedDiscount: undefined,
        selectedSaleLines: [],
        availableSaleLines: [],
        events: {
            'tap .option': 'selectDiscount',
            'tap .available-sale-line': 'selectSaleLine',
            'focus #discount-search': 'hideSaleLines',
            'blur #discount-search': 'changeDiscount',
            'touchend .discount-placeholder' : 'preventFocus',
            'touchend #select-all': 'selectAll'
        },
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                { id: 'apply', className: 'green', size: '50', value: 'Apply' },
                { id: 'cancel', className: 'red', size: '50', value: 'Cancel' }
            ],
            tabletButtons: [
                { id: 'apply', className: 'green', size: '50', value: 'Apply' },
                { id: 'cancel', className: 'red', size: '50', value: 'Cancel' }
            ],
            touchCallBacks: {
                'apply': 'applyDiscount',
                'cancel': 'closeModal'
            }
        },

        initialize: function(options){
            this.selectedSaleLines = [];
            this.discountsRequestData = options.couponsRequestData;
			this.adjustmentsList = options.adjustmentsList;
            this.modalButtonsBottom.phone = this.onPhone;
            this.onPhone = $('html').hasClass('mobile');
            this.el = this.onPhone ? '#predefined-discount-modal .content' : '#tablet-predefined-discount-modal .content';
            this.setElement(this.el);

            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        render: function(){
            ModalView.prototype.render.call(this);

            this.scroll.on('beforeScrollStart', function (e) {
                $('#predefined-discount-modal-scroll-wrap').css('bottom', $('.modal-buttons-bottom').outerHeight() + 10);
                this.refresh();
            });

            this.$('#discount-search').searchPlugin({
                filterFunction: function(filter, pageNumber, pageSize, a){
                    return getValidDisountsFlowList(filter, this.getRequestData(), pageSize, a);
				}.bind(this),
                parentScroll: this.scroll
            });

            setTimeout(function() {
                $('.discount-placeholder').hide();
            }, 600);

            return this;
        },

        getRequestData: function() {           
            var items = _.filter(this.discountsRequestData.toJSON(), function (value) {
                if(value.item.type == 1425 || value.item.type == 1426)
                    if (value.client.id)
                        if (value.item.id)
                            return true;

                return false;
            });

            return {
                ClientIds: _.map(items, function (value) { return value.client.id; }),
                ItemIds: _.map(items, function (value) { return value.item.id; }),
                Applied: _.map(items, function (value) { return { ItemId: value.item.id, DiscountIds: _.map(value.discounts, function (el) { return el.discountId; }) } })
            }
        },

        preventFocus: function() {
            this.$('.discount-placeholder').hide();
        },

        hideSaleLines: function () {
            this.$('#sale-lines-list').hide();
        },

        changeDiscount: function (e) {
            if (!$(e.target).data('id')) {
                this.availableSaleLines = [];
                this.renderSaleLines();
            } else {
                this.selectDiscount(e);
            }
            this.$('#sale-lines-list').show();
        },

        selectDiscount: function(e){
            var discountId = $(e.target).data('id');
            this.selectedDiscount = discountId;

            getValidDisountsForSaleLines(this.getRequestData()).done(function (data) {
				this.availableSaleLines = _.reject(this.collection.models, function(model){
					if(model.get('item')['type'] == Consts.SaleLineTypes.GIFTCARD)
						return true;

					if(model.get('item')['type'] == Consts.SaleLineTypes.ACCOUNT)
						return true;

					if (_.findWhere(model.get('discounts'), {discountId: discountId}))
						return true;

					var availableDiscounts = _.find(data, function(value){ return value.itemId === model.get('item')['id']; })
					if(!availableDiscounts)
						return true;

					if (!_.find(availableDiscounts.discountIds, function(value){ return value === discountId }))
						return true;

					return false;
				});
				this.renderSaleLines();
			}.bind(this));
        },

        renderSaleLines: function(){
            var tempTemplate = '';
            _.each(this.availableSaleLines, function(line){
                tempTemplate += this.saleLineTemplate(line.toJSON())
            }.bind(this));

            this.$('#sale-lines-list').html(tempTemplate);
        },

        selectAll: function(){
            var allLines = _.pluck(this.availableSaleLines, 'id');
            this.selectedSaleLines = allLines !== undefined ? allLines : [];
            this.$('.available-sale-line')
                .removeClass('selected')
                .addClass('selected');
        },

        selectSaleLine: function(e){
            if( $(e.target).hasClass('selected') ){
                this.selectedSaleLines = _.reject(this.selectedSaleLines, function(id){
                    return id == $(e.target).attr('data-saleline-id');
                });
            }else{
                this.selectedSaleLines.push($(e.target).attr('data-saleline-id'));
            }
            $(e.target).toggleClass('selected');
        },

        applyDiscount: function(e){
            if(!this.selectedSaleLines.length) {
                new YesNoModal({
                    question: 'Select an item to discount',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }

            var needle = this.$('#discount-search').searchPlugin('getSelectedObject');

            if (needle.amount === 0) {
                new YesNoModal({
                    question: 'Discount amount should not be empty or zero',
                    positiveText: 'Ok',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }

            this.trigger('closeEvent', {
                discount: needle,
                type: 'addDiscount',
                saleLines : this.selectedSaleLines
            });
        },

        closeModal: function () {
            this.trigger('closeEvent', {});
        }
    });
});