define([
    'jquery',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/print-modal.html',
    'bridge'
], function ($, ModalView, YesNoModal, PrintModalTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#print-modal .content',
        template: _.template(PrintModalTemplate),
        scrollWrap: '#print-modal-scroll-wrap',
        yesNoModal: undefined,
        options: undefined,

        initialize: function (options) {
            this.model = new Backbone.Model(options);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            if(this.options === undefined)
                this.options = options;
        },

        events: {
            'tap #btn-email': 'emailAction',
            'tap #btn-cancel': 'cancelAction',
            'tap #email-cancel': 'closeEmailForm',
            'tap #email-send': 'triggerSubmitEmailForm',
            'submit #email-form': 'submitEmailForm'
        },

        onRender: function () {
            if (this.model.get('action') == 'email') {
                this.$('#btn-email').trigger('tap');
            }
        },

        emailAction: function (e) {
            var $elem = $(e.target);
            if(!$elem.hasClass('.button')) {
                $elem = $elem.closest('.button');
            }
            if($elem.hasClass('opened')) {
                this.$('#email-form').stop(true, true).slideUp(200, function(){
                    $elem.removeClass('opened');
                });
            } else {
                this.$('#email-form').stop(true, true).slideDown(200, function(){
                    $elem.addClass('opened');
                });
            }
            e.preventDefault();
        },

        closeEmailForm: function (e) {
            this.$('#email-form').stop(true, true).slideUp(200, function(){
                this.$('#btn-email').removeClass('opened');
            }.bind(this));
            e.preventDefault();
        },

        triggerSubmitEmailForm: function (e) {
            setTimeout(function () {
                this.$el && this.$('#email-form-submit').click();
            }.bind(this), 0);
        },

        submitEmailForm: function (e) {
            e.preventDefault();
            if (this.reguestInProcess) return false;

            var $emailField = this.$('#print-email'),
                email = $emailField.val();
            $emailField.removeClass('error');
            if(!email) {
                $emailField.addClass('error');
            } else if(!/^[\w-\.]+@([\w-]+\.)+[\w-]{2,4}$/.test(email)) {
                $emailField.addClass('error');
                this.yesNoModal = new YesNoModal({
                    question: "You must provide a valid email address!",
                    className: '',
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
            } else {
                this.reguestInProcess = true;
                this.$el && this.$('#email-send').addClass('disabled');
                sendEmailReceipt(JSON.stringify({
                    SaleId: this.model.get('saleId'),
                    EmailAddress: email
                }))
                    .done(function (r) {
                        if (r === true) {
                            this.closeView();
                        }
                }.bind(this))
                    .always(function () {
                        this.reguestInProcess = false;
                        this.$el && this.$('#email-send').removeClass('disabled');
                }.bind(this));
            }
            /*this.closeView(undefined, undefined, {
                type: 'email',
                email: this.$('#print-email').val(),
                saleId: this.model.get('saleId')
            });*/
        },

        cancelAction: function (e) {
            this.closeView(undefined, undefined, {
                type: 'noEmail'
            });
        },

        closeView: function (e, route, response) {
            this.yesNoModal = undefined;
            if (response) this.trigger('printModalEvent', response);
            else this.trigger('printModalEvent', {
                type: 'close'
            });
            ModalView.prototype.closeView.call(this, e);
        }

    });
});
