﻿define([
    'jquery',
    'data/consts',
    'data/permissions',
    'views/modal',
    'text!../../../templates/smart-receipt/refund-checkout-modal.html',
    'text!../../../templates/smart-receipt/checkout-credit-payment-block.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'views/smart-receipt/signature-modal',
    'bridge'
], function ($, Consts, PermModel, ModalView, Template, CreditPaymentTemplate, UserSearchTemplate, SignatureView) {
    'use strict';

    var RefundCheckoutModel = Backbone.Model.extend({
        defaults: {
            refundValue: 0,
            payingClient: {
                id: '',
                firstName: '',
                lastName: ''
            }
        }
    });

    return ModalView.extend({
        el: '#refund-checkout-modal .content',
        template: _.template(Template),
        withoutScroll: true,
        model: new RefundCheckoutModel(),
        creditPaymentTemplate: _.template(CreditPaymentTemplate),
        paymentObject: {
            lineNumber: 1,
            isRefund: true,
            paymentType: {}
        },

        initialize: function (options) {
            this.model.set('refundValue', options.refundValue);
            this.model.set('payingClient', options.payingClient);
            this.paymentObject.amount = options.refundValue;
            this.paymentTypes = options.paymentTypes;
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            this.render();
        },

        events: {
            'touchend .refund-checkout-cancel': 'closeView',
            'touchend .refund-checkout-variant': 'showRefundVariant',
            'touchend .refund-variant-cancel': 'showMain',
            'touchend #close-no-credit-reader': 'showMain',
            'touchend .checkout-checkbox': 'checkoutCheckbox',
            'touchend #charge-back-reset': 'chargeBackReset',
            'touchend #refund-card-number-generate': 'cardNumberGenerate',

            // checkout
            'touchend #checkout-charge-back': 'checkoutChargeBack',
            'touchend #checkout-client-account': 'checkoutClientAccount',
            'touchend #checkout-gift-cart': 'checkoutGiftCard'
        },

        render: function () {
            ModalView.prototype.render.call(this);



            this.$('.refund-checkout-body').append('<div id="no-credit-reader"><div class="middle-aligned" style="font-size: ' +( this.onPhone() ? 16 : 28 )+ 'px;">Mobile credit card processing is not currently supported in My Meevo. Please finalize this transaction on another workstation.</div></div>');

            this.renderCardTypes();

            this.clientAccountScroll = new IScroll('#refund-checkout-client-account', {
                tap: true,
                bounce: false,
                disableMouse: true,
                scrollbars: 'custom'
            });

            this.$('#refund-giftcard-number').inputFormat({
                type: 'any',
                places: 20,
                allowEmpty: true
            });

            this.$('#refund-client-search').searchPlugin({
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                    return getClients(filter, clientAccountChargable, undefined, true, pageNumber, pageSize);
                },
                fullScreen: false,
                template: _.template(UserSearchTemplate),
                parentScroll: this.clientAccountScroll,
                bottomButtonsFixed: true
            });

            this.$('.refund-checkout-variants').addClass('buttons-' + this.$('.refund-checkout-variant').length);

            if (!this.isWithWorkstation()) {
                var $reader = this.$('#no-credit-reader');
                $reader && $reader.addClass('visible');
                this.$('.modal-buttons-bottom').css({ zIndex: 15 });
            }


            return this;
        },

        checkoutChargeBack: function(e) {
            e.preventDefault();
            e.stopPropagation();
            var $selectedType = this.$('.card-type.checkout-checkbox.checked');
            var finalize = function() {
                this.paymentObject.paymentType = _.findWhere(this.paymentTypes, {shortcut: $selectedType.data('shortcut')});
                this.closeView(null, null, {
                    type: 'refundCheckout',
                    paymentObject: this.paymentObject
                });
            }.bind(this);
            if ($selectedType.length) {
                if($selectedType.data('type') == '1460' && window.SETTINGS.signatureCaptureEnabled) {
                    this.onSignatureModal = new SignatureView({
                        total: this.model.get('refundValue')
                    }).render();
                    this.listenToOnce(this.onSignatureModal, 'changeAsTipWithSignature', function(response){
                        this.paymentObject.signatureCapture = response;
                        finalize();
                    });
                } else {
                    finalize();
                }
            } else {
                this.$('.credit-group').addClass('error');
            }
        },

        checkoutCashBack: function() {
            this.paymentObject.paymentType = _.findWhere(this.paymentTypes, {paymentTypeEnum: 1462});
            this.closeView(null, null, {
                type: 'changeDue',
                paymentObject: this.paymentObject
            });
        },

        checkoutClientAccount: function() {
            var $input = this.$('#refund-client-search'),
                client = $input.searchPlugin('getSelectedObject'),
                payingClient = this.model.get('payingClient');
            if ($input.data('id') == payingClient.id) client = payingClient;

            if (client) {
                $input.removeClass('error');
                this.addSaleLine({
                    type: Consts.SaleLineTypes.ACCOUNT,
                    displayName: 'Client Account Sale',
                    price: -this.model.get('refundValue'),
                    note: '',
                    client: client
                });
            } else {
                $input.addClass('error');
            }
        },

        checkoutGiftCard: function() {
            var $input = this.$('#refund-giftcard-number'),
                cardNumber = $input.inputFormat('getValue');
            if (cardNumber) {
                $input.closest('.form-group').removeClass('error');
                this.addSaleLine({
                    type: Consts.SaleLineTypes.GIFTCARD,
                    displayName: 'GC',
                    number: cardNumber,
                    dateValidFrom: moment().toISOString(),
                    price: -this.model.get('refundValue'),
                    value: -this.model.get('refundValue'),
                    client: this.model.get('payingClient')
                });
            } else {
                $input.closest('.form-group').addClass('error');
            }
        },

        addSaleLine: function (item) {
            var newSaleLine = {
                isSaleLinePayment: true,
                client: this.model.get('payingClient'),
                item: item
            };
            this.closeView(null, null, {
                type: 'refundSaleLineCheckout',
                newSaleLine: newSaleLine
            });
        },

        renderCardTypes: function() {
            var creditPayments = [];
            _.each(this.paymentTypes, function (p) {
                p['shortcut'] = p['shortcut'].trim();
                p = _.extend({imageId: ''}, p);
                if (p['paymentTypeEnum'] == 1460 || p['paymentTypeEnum'] == 1461) { // credit/debit
                    creditPayments.push(this.creditPaymentTemplate(p));
                }
            }.bind(this));
            this.$('.credit-group').html(creditPayments);
        },

        closeView: function(e, fragment, response) {
            this.trigger('refundCheckoutEvent', _.extend({
                type: 'close'
            }, response));
            ModalView.prototype.closeView.call(this);
        },

        showRefundVariant: function(e) {
            e.preventDefault();
            e.stopPropagation();
            PermModel.navigateToPermissionsModal(e, function () {
                if(!this.isWithWorkstation() && $(e.target).data('url') == '#refund-checkout-charge-back') {
                    this.$('#no-credit-reader') && this.$('#no-credit-reader').addClass('visible');
                }
                if($(e.target).data('url') == '#refund-checkout-cash-back') {
                    this.checkoutCashBack();
                } else {
                    this.$($(e.target).data('url')).addClass('active').siblings().removeClass('active');
                }
                if ($(e.target).data('url') == '#refund-checkout-charge-back' && this.$('#no-credit-reader').hasClass('visible')) {
                    this.$('#refund-checkout-charge-back').addClass('show-only-cancel');
                }
            }.bind(this));
        },

        showMain: function() {
            this.$('.refund-checkout-section').removeClass('active');
            this.$('#refund-checkout-main').addClass('active');
            this.$('#refund-checkout-charge-back').removeClass('show-only-cancel');
        },

        checkoutCheckbox: function (e) {
            var $target = $(e.target).closest('.checkout-checkbox');
            if ($target.hasClass('checked')) { return false; }
            this.$('.checkout-checkbox').removeClass('checked');
            $target.addClass('checked').parent().removeClass('error');
        },

        chargeBackReset: function() {
            this.$('.checkout-checkbox').removeClass('checked');
        },

        cardNumberGenerate: function () {
            generateGiftCardNumber().done(function (r) {
                if (r.status === undefined) {
                    this.$('#refund-giftcard-number').inputFormat('setValue', r);
                }
            }.bind(this));
        }

    });
});