﻿define([
    'jquery',
    'moment',
    'data/permissions',
    'views/modal',
    'views/yes-no-modal',
    'views/smart-receipt/refund-modal',
    'text!../../../templates/smart-receipt/refund-item-modal.html',
    'bridge'
], function ($, moment, PermModel, ModalView, YesNoModal, RefundBaseModal, RefundItemModalTemplate) {
    'use strict';

    return RefundBaseModal.extend({
        el: '#refund-item-modal .content',
        template: _.template(RefundItemModalTemplate),
        searchValue: undefined,
        isTransaction: undefined,
        //scrollWrap: '#refund-modal-scroll-wrap',
        //scrollbars: true,
        modalButtonsBottom: {
            phone: true,
            phoneButtons: [
                {
                    id: 'refund-item-cancel',
                    className: 'red',
                    size: '100',
                    value: 'cancel'
                }
            ],
            touchCallBacks: {
                'refund-item-cancel': 'customClose'
            }
        },

        events: {
            'tap .button-refund': 'refundButtonClick',
            'tap .refund-return-check, .refund-item-to-shelf label': 'checkReturnToShelf'
        },

        initialize: function(options) {
            var o = _.extend({}, options, {
                phone: $('html').hasClass('mobile')
            });
            this.breadCrumbs = ['Register', 'Refund', options.title];
            this.currentTab = options.currentTab;
            this.refundedItems = options.refundedItems;
            this.model = new Backbone.Model(o);
            this.events = _.extend({}, RefundBaseModal.prototype.events, this.events);
            this.addListeners();
        },

        onRender: function() {
            this.searchValue = this.model.get('number');
            this.isTransaction = this.model.get('isTransaction');

            this.$('#refund-sub-items').empty();
            switch (this.currentTab) {
                case 'tab-client':
                    this.searchCardsClient(this.searchValue.clientID.id, this.searchValue.itemID.id);
                    break;
                case 'tab-transaction':
                case 'tab-giftcard':
                    this.searchCards(this.searchValue, this.isTransaction);
                    break;
            }

            return this;
        },

        loadNextPage: function () {
            switch (this.currentTab) {
                case 'tab-client':
                    if (this.searchValue.clientID !== undefined && this.searchValue.itemID !== undefined)
                        this.searchCardsClient(this.searchValue.clientID.id, this.searchValue.itemID.id);
                    break;
                case 'tab-transaction':
                case 'tab-giftcard':
                    if(this.searchValue !== '')
                        this.searchCards(this.searchValue, this.isTransaction);
                    break;
            }
        },

        extractEvent: function (modelToReturn) {
            this.trigger('extractMobile', modelToReturn);
            this.customClose();
        },

        customClose: function (response) {
            if (response && response.type == 'returnToMenu') {
                this.trigger('returnToMenu');
            }
            this.refundGlobalCollection = undefined;
            this.yesNoModal = undefined;
            this.closeView();
        }
    });
});