define([
    'jquery',
    'moment',
    'searchPlugin',
    'data/consts',
    'data/permissions',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/refund-modal.html',
    'text!../../../templates/smart-receipt/refund-line.html',
    'bridge'
], function ($, moment, searchPlugin, Consts, PermModel, ModalView, YesNoModal,
             RefundModalTemplate, RefundLineTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#tablet-refund-modal .content',
        template: _.template(RefundModalTemplate),
        refundItemTemplate: _.template(RefundLineTemplate),
        serviceTemplate: _.template('<div class="option" data-id="<%= id %>"><%= shortcut %> : <%= displayName %></div>'),
        employeeTemplate: _.template('<div class="option" data-id="<%= id %>"><%= lastName %>, <%= firstName %></div>'),
        currentTab: 'tab-client',
        yesNoModal: undefined,
        pageNumber: 0,
        pageSize: 5,
        requestInProcess: false,
        refundedItems: [],
        isProduct: false,
        refundGlobalCollection: new Backbone.Collection(),

        initialize: function (options) {
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        events: {
            'blur input': 'scrollFix'
        },

        initializeIScroll: function () {
            var that = this,
                wrapper;

            this.$el.wrapInner('<div class="scroll-outside"><div class="scroll-wrap"><div></div></div></div>');
            wrapper = this.$('.scroll-wrap').get(0);
            this.scroll = new IScroll(wrapper, {
                tap: true,
                disableMouse: true,
                probeType: 2,
                scrollbars: this.scrollbars
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
            this.scroll.on('scroll', function () {
                if (that.pageNumber != 'last') {
                    this._triggerLoadNext = this.y < 0 && (this.y - this.maxScrollY) < -30;
                    this._scrollToY = this.y;
                    $(this.wrapper).toggleClass('active-loading', this.y < 0 && (this.y - this.maxScrollY) < 0);
                }
            });
            this.scroll.on('beforeScrollEnd', function () {
                $(this.wrapper).removeClass('active-loading loading');
                if (this._triggerLoadNext) {
                    this.enabled = false;
                    $(this.wrapper).addClass('loading');
                    // small delay to show visual effect
                    setTimeout(function () {
                        this._triggerLoadNext = false;
                        this.enabled = true;
                        that.loadNextPage();
                    }.bind(this), 100);
                }
            });
        },

        addListeners: function () {
            this.listenTo(this.refundGlobalCollection, 'add', this.refundItemAdded);
            this.listenTo(this.refundGlobalCollection, 'emptyEvent', this.refundsCollectionEmpty);
            this.listenTo(this.refundGlobalCollection, 'notEmptyEvent', this.refundsCollectionNotEmpty);
            this.listenTo(this.refundGlobalCollection, 'change:hidden', function (m) {
                var $refLine = this.$('.refund-item[data-id="' + m.get('saleLine').refundSaleLineId + '"]');
                if (m.get('hidden')) {
                    $refLine.addClass('hidden');
                } else {
                    $refLine.removeClass('hidden');
                }
            }.bind(this));
        },

        refundItemAdded: function (refundItem) {
            refundItem.set('hidden', _.indexOf(this.refundedItems, refundItem.get('saleLine').refundSaleLineId) != -1);
            refundItem.set('currentTab', this.currentTab);
            var $elem = (!this.model.get('phone')) ? this.$('#refund-items') : this.$('#refund-sub-items');
            this.$el && $elem.append(this.refundItemTemplate(
                _.extend(refundItem.toJSON(), {PermModel: PermModel})
            ));
            this.scroll && this.scroll.refresh();
        },

        refundsCollectionEmpty: function () {
            this.$el && this.$('#refund-sub-items').html('<div class="empty">There are no transactions for this criteria</div>');
            this.scroll && this.scroll.refresh();
        },

        refundsCollectionNotEmpty: function () {
            this.$el && this.$('.empty').remove();
            this.scroll && this.scroll.refresh();
        },

        updateRefundedItems: function (items) {
            this.refundedItems = items;
            
            _.each(this.refundGlobalCollection.models, function (refundItem) {
                refundItem.set('hidden', _.indexOf(this.refundedItems, refundItem.get('saleLine').refundSaleLineId) != -1);
            }.bind(this));
        },

        refundButtonClick: function (e) {
            var itemID = $(e.target).closest('.refund-item').data('id');
            var isRefunded = _.contains(this.refundedItems, itemID);
            if(isRefunded) {
                new YesNoModal({
                    question: "This item is already being refunded.",
                    className: '',
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            } else if(!this.requestInProcess) {
                this.refundedItems.push(itemID);
                this.refundedItems = _.uniq(this.refundedItems);
                this.updateRefundedItems(this.refundedItems);
                this.trigger('refundListChanging', this.refundedItems);
                this.extractEvent(this.findModelByID(this.refundGlobalCollection, itemID).toJSON());
            }
        },

        searchDoneCallback: function (r) {
            if (!this.model) return false;
            if(!this.model.get('phone')) {
                if (r.length == 0) {
                    if (this.currentTab === 'tab-client') {
                        this.noRefundItems();
                    }
                    this.pageNumber = 'last';
                } else if (r.length) {
                    this.refundGlobalCollection && this.refundGlobalCollection.add(r);
                    this.pageNumber += 1;
                }
            } else {
                if (r.length == 0) {
                    if (this.refundGlobalCollection && this.refundGlobalCollection.length === 0) {
                        this.refundGlobalCollection.trigger('emptyEvent');
                    }
                    this.pageNumber = 'last';
                } else if (r.length) {
                    this.refundGlobalCollection && this.refundGlobalCollection.trigger('notEmptyEvent');
                    this.refundGlobalCollection && this.refundGlobalCollection.add(r);
                    this.pageNumber += 1;
                }
            }

        },

        setRequestHelper: function (check) {
            this.requestInProcess = check;
        },

        searchHelper: function () {
            if(this.pageNumber === 0) {
                this.$('#refund-items').empty();
            }
            this.scroll && this.scroll.refresh();
        },

        // Card searching
        searchCards: function (number, isTransaction) {
            if(!this.requestInProcess) {
                this.searchHelper();
                this.setRequestHelper(true);
                return getRefundGiftItems(number, isTransaction, this.pageNumber, this.pageSize)
                    .done(this.searchDoneCallback.bind(this))
                    .always(this.setRequestHelper.bind(this, false));
            }
        },

        searchCardsClient: function (clientGlobalId, itemId) {
            this.searchHelper();
            this.setRequestHelper(true);
            if(clientGlobalId === undefined && itemId === undefined) return false;
            return getRefundProductServiceItemsById(clientGlobalId, itemId, this.pageNumber, this.pageSize)
                .done(this.searchDoneCallback.bind(this))
                .always(this.setRequestHelper(false));
        },

        // ONLY MOBILE
        searchClientSubRequest: function (clientGlobalId, itemId) {
            this.setRequestHelper(true);
            if(clientGlobalId === undefined && itemId === undefined) return false;
            return getRefundProductServiceItemsById(clientGlobalId, itemId, this.pageNumber, this.pageSize)
                .done(function (r) {
                    if(r.length == 0) {
                        this.noRefundItems();
                    } else {
                        this.openSubModal('Client Name', this.getCurrentFieldValue(), false);
                    }
                }.bind(this))
                .always(this.setRequestHelper(false));
        },

        searchClickHelper: function () {
            this.pageNumber = 0;
            this.refundGlobalCollection && this.refundGlobalCollection.reset();
            this.$('#refund-items').empty();
        },

        // Client Search
        clickSearchClient: function (isTabChanging) {
            var selectedClient = this.getSelectedClient();
            var itemProduct = this.$('#refund-client-product').searchPlugin('getSelectedObject');
            var itemService = this.$('#refund-client-service').searchPlugin('getSelectedObject');
            if(this.model.get('phone')) {
                if(!selectedClient) {
                    this.$('#refund-client-name').addClass('error');
                } else {
                    if(!itemProduct && !itemService) {
                        this.$('#refund-client-product').addClass('error');
                        this.$('#refund-client-service').addClass('error');
                    }
                }
            }
            this.searchClickHelper();
            if(selectedClient !== undefined && (itemProduct !== undefined || itemService !== undefined)) {
                if(this.model.get('phone')) {
                    // TODO: optimize it
                    if(this.isProduct) {
                        this.searchClientSubRequest(selectedClient.id, itemProduct.id || itemService.id);
                    } else {
                        this.searchClientSubRequest(selectedClient.id, itemService.id || itemProduct.id);
                    }
                } else {
                    if(this.isProduct) {
                        this.searchCardsClient(selectedClient.id, itemProduct.id || itemService.id, false);
                    } else {
                        this.searchCardsClient(selectedClient.id, itemService.id || itemProduct.id, false);
                    }
                }
            }
        },

        // Giftcard Button Search
        clickSearchGiftcard: function (isTabChanging) {
            var $field = this.$('#refund-giftcard-find');
            this.searchClickHelper();
            this.checkingField($field, 'Gift Card #', false, isTabChanging);
        },

        // Transaction Button Search
        clickSearchTransaction: function (isTabChanging) {
            var $field = this.$('#refund-transaction-find');
            this.searchClickHelper();
            this.checkingField($field, 'Transaction #', true, isTabChanging);
        },

        // Main search button
        customSearch: function (e) {
            if($(e.target).hasClass('readonly')) {
                return false;
            }
            this.onRefundItemModal = undefined;
            switch(this.currentTab) {
                case 'tab-client':
                    this.clickSearchClient(false);
                    break;
                case 'tab-transaction':
                    this.clickSearchTransaction(false);
                    break;
                case 'tab-giftcard':
                    this.clickSearchGiftcard(false);
                    break;
            }
        },

        checkingField: function (field, tabName, tabState, isTabChanging) {
            field.closest('.form-group').removeClass('error');
            if (field.val() === '') {
                if(isTabChanging) {
                    this.$('#refund-items').empty();
                } else {
                    field.closest('.form-group').addClass('error');
                }
                return false;
            } else {
                if(this.model.get('phone')) {
                    this.openSubModal(tabName, field.val(), tabState);
                } else {
                    this.searchCards(field.val(), tabState);
                }
            }
        },

        changeReturnToShelfValue: function (collection, compareId) {
            _.forEach (collection.models, function(model) {
                if(model.get('saleLine')['refundSaleLineId'] === compareId) {
                    var saleLine = model.get('saleLine');
                    saleLine.returnToShelf = !saleLine.returnToShelf;
                    model.set("saleLine", saleLine);
                }
            });
        },

        checkReturnToShelf: function (e) {
            var elemID = $(e.target).closest('.refund-item').data('id');
            var $shelf = $(e.target).closest('.refund-item-to-shelf');
            if(!$shelf.hasClass('readonly')) {
                $shelf.find('.refund-return-check').toggleClass('checked');
                this.changeReturnToShelfValue(this.refundGlobalCollection, elemID);
            }
        },

        extractEvent: function (modelToReturn) {
            this.customClose({
                type: 'extract',
                refundItem: modelToReturn
            });
        },

        findModelByID: function (collection, modelID) {
            var modelOUT = undefined;
            _.forEach (collection.models, function(model) {
                var id = model.get('saleLine')['refundSaleLineId'];
                if( modelID === id ) {
                    modelOUT = model;
                }
            });
            return modelOUT;
        },

        customClose: function (response) {
            if (this.model.get('phone')) {
                if (response) this.trigger('refundEvent', response);
                else this.trigger('refundEvent', { type: 'cancel' });
                this.closeView();
                this.resetAll();
            } else {
                if (response) {
                    this.trigger('refundEvent', _.extend({
                        doneLeave: true
                    }, response));
                } else {
                    this.trigger('refundEvent', {
                        type: 'cancel',
                        doneLeave: true
                    });
                }
                //this.refundGlobalCollection.reset();
                //this.render();
            }
        },

        resetAll: function () {
            this.refundGlobalCollection = undefined;
            this.onRefundItemModal = undefined;
            this.yesNoModal = undefined;
            this.refundedItems = [];
        },

        scrollFix: function () {
            // dirty hack to update scroll wrap position, after keyboard was opened on small height screen
            // we need this only on android tablets with small height in landscape
            if ( !(window.innerHeight < 610 && $('html').hasClass('android tablet landscape')) ) {
                return;
            }
            var $scWrap = this.$('.scroll-wrap');
            setTimeout(function () {
                // if nothing is in focus
                if (this.$(':focus').length == 0) {
                    // add overflow inline property
                    $scWrap.css({
                        overflow: 'hidden'
                    });
                    // remove overflow inline property
                    setTimeout(function () {
                        $scWrap.css({
                            overflow: ''
                        });
                    }.bind(this), 300);
                }
            }.bind(this), 300);
        }
    });
});