﻿define([
    'jquery',
    'data/permissions',
    'views/modal',
    'text!../../../templates/smart-receipt/refund-no-item-modal.html',
    'bridge'
], function ($, PermModel, ModalView, RefundNoItemModalTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#refund-no-item-modal .content',
        template: _.template(RefundNoItemModalTemplate),
        scrollWrap: '#refund-no-item-modal-scroll-wrap',
        options: undefined,
        events: {
            'tap #btn-yes': 'yesAction',
            'tap #btn-no': 'noAction',
            'tap .refund-return-check, .refund-item-to-shelf label': 'checkReturnToShelf',
            'touchend .button-plus': 'plusQuantity',
            'touchend .button-minus': 'minusQuantity',
            'blur #edit-quantity': 'changeQuantity'
        },

        initialize: function (options) {
            this.model = new Backbone.Model(options);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            if( this.options === undefined )
                this.options = options;
            this.client = this.options.client;
            this.item = this.options.item;
        },

        onRender: function () {
            $(':focus').blur();
            this.$('#edit-quantity').inputFormat({
                type: 'int',
                places: 4
            });
            return this;
        },

        yesAction: function () {
            this.trigger('noRefundItem', {
                saleLine: {
                    client: this.client,
                    item: this.item,
                    quantity: this.$('#edit-quantity').inputFormat('getValue'),
                    returnToShelf: this.getReturnToShelfValue(),
                    isRefund: true
                }
            });
            this.closeView();
        },

        noAction: function () {
            this.closeView();
        },

        getReturnToShelfValue: function() {
            return (this.$('.refund-return-check').hasClass('checked')) ? true : false;
        },

        plusQuantity: function (e) {
            e.preventDefault();
            e.stopPropagation();
            var $input = this.$('#edit-quantity'),
                newVal = $input.inputFormat('getValue');
            $input.inputFormat('setValue',  +newVal + 1);
        },

        minusQuantity: function (e) {
            e.preventDefault();
            e.stopPropagation();
            var $input = this.$('#edit-quantity'),
                newVal = $input.inputFormat('getValue');
            $input.blur().inputFormat('setValue',  +newVal - 1);
            var val = $input.inputFormat('getValue');
            if (val === 0){
                this.$('#edit-quantity').inputFormat('setValue', 1);
            }
        },

        changeQuantity: function (e) {
            var val = $(e.target).inputFormat('getValue');
            if (val <= 0) {
                this.$('#edit-quantity').inputFormat('setValue', 1);
                return;
            }
        },

        checkReturnToShelf: function (e) {
            var $shelf = $(e.target).closest('.refund-item-to-shelf');
            if(!$shelf.hasClass('readonly')) {
                $shelf.find('.refund-return-check').toggleClass('checked');
            }
        }
    });
});
