﻿define([
    'jquery',
    'moment',
    'data/consts',
    'data/permissions',
    'views/yes-no-modal',
    'views/smart-receipt/refund-modal',
    'views/smart-receipt/refund-item-modal',
    'views/smart-receipt/refund-no-item-modal',
    'bridge'
], function ($, moment, Consts, PermModel, YesNoModal,
             RefundBaseModal, OnRefundItemModalView, OnRefundNoItemModalView) {
    'use strict';

    return RefundBaseModal.extend({
        onRefundItemModal: undefined,
        onRefundNoItemModal: undefined,
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                {
                    id: 'refund-cancel',
                    className: 'red cancel',
                    size: '20',
                    value: ''
                },
                {
                    id: 'refund-search',
                    className: 'violet',
                    size: '80',
                    value: 'search'
                }
            ],
            tabletButtons: [
                {
                    id: 'refund-cancel',
                    className: 'red',
                    size: '100',
                    value: 'cancel'
                }
            ],
            touchCallBacks: {
                'refund-cancel': 'customClose',
                'refund-search': 'customSearch'
            }
        },

        initialize: function (options) {
            var o = _.extend({}, options, {
                phone: $('html').hasClass('mobile'),
                PermModel: PermModel
            });
            if (o.phone) {
                this.breadCrumbs = ['Register', 'Refund'];
                this.modalButtonsBottom.phone = true;
                this.el = '#refund-modal .content';
                this.setElement(this.el);
            }
            this.refundedItems = options.refundedItems;
            this.model = new Backbone.Model(o);
            RefundBaseModal.prototype.initialize.call(this, o);
            this.events = _.extend({}, RefundBaseModal.prototype.events, this.events);
            this.addListeners();
        },

        events: {
            'touchstart .search-close-icon': 'changeClientName',
            'tap .refund-return-check, .refund-item-to-shelf label': 'checkReturnToShelf',
            'change #refund-search-by': 'changeRefundTabMobile',
            'tap .refund-tabs-menu li': 'changeRefundTab',
            'tap .button-refund': 'refundButtonClick',
            'tap #refund-transaction-submit, #refund-giftcard-submit': 'customSearch'
        },

        onRender: function() {
            $(this.$el).removeClass('client-selected');
            this.addPlugins();

            if (!this.model.get('phone')) {
                if (this.$('.refund-tabs-menu-item')) {
                    var $firstTab = this.$('.refund-tabs-menu-item[data-url]').first();
                    if (!$firstTab.hasClass('readonly')) {
                        $firstTab.trigger('tap');
                    } else {
                        $firstTab.removeClass('readonly').trigger('tap').addClass('readonly');
                    }
                }
            } else {
                this.$('#refund-search-by') && this.$('#refund-search-by').trigger('change');
            }

            this.changeClientName();

            return this;
        },

        loadNextPage: function () {
            var val = this.getCurrentFieldValue();
            switch (this.currentTab) {
                case 'tab-client':
                    if (val.clientID !== undefined && val.itemID !== undefined)
                        this.searchCardsClient(val.clientID.id, val.itemID.id, false);
                    break;
                case 'tab-transaction':
                    if (val !== '') this.searchCards(val, true);
                    break;
                case 'tab-giftcard':
                    if (val !== '') this.searchCards(val, false);
                    break;
            }
        },

        addPlugins: function () {
            var inputOptions = {
                type: 'any',
                places: 20,
                allowEmpty: true
            };

            if(this.$('#refund-transaction-find')) {
                this.$('#refund-transaction-find').inputFormat(inputOptions);
            }

            if(this.$('#refund-giftcard-find')) {
                this.$('#refund-giftcard-find').inputFormat(inputOptions);
            }

            var searchOptions = {
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                    return getClients(filter, clientAccountChargable, undefined, undefined, pageNumber, pageSize);
                },
                parentScroll: this.scroll,
                template: this.employeeTemplate,
                large: true
            };

            if(this.$('#refund-client-name')) {
                this.$('#refund-client-name').searchPlugin(_.extend({}, searchOptions, {
                    selectedCallBack: this.changeClientName.bind(this)
                }));
            }

            if(this.$('#refund-client-product')) {
                this.$('#refund-client-product').searchPlugin(_.extend({}, searchOptions, {
                    filterFunction: filterProducts,
                    template: this.serviceTemplate,
                    selectedCallBack: function(obj) {
                        if (obj){
                            this.isProduct = true;
                            this.$('#refund-client-product').removeClass('error');
                            this.$('#refund-client-service').removeClass('error');
                            if(!this.model.get('phone')) this.clickSearchClient(false);
                        }
                    }.bind(this)
                }));
            }

            if(this.$('#refund-client-service')) {
                this.$('#refund-client-service').searchPlugin(_.extend({}, searchOptions, {
                    filterFunction: filterServices,
                    template: this.serviceTemplate,
                    selectedCallBack: function(obj) {
                        if (obj) {
                            this.isProduct = false;
                            this.$('#refund-client-product').removeClass('error');
                            this.$('#refund-client-service').removeClass('error');
                            if(!this.model.get('phone')) this.clickSearchClient(false);
                        }
                    }.bind(this)
                }));
            }
        },

        // Changing tab: ONLY MOBILE
        changeClientName: function () {
            var obj = this.getSelectedClient();
            (obj) ? $(this.$el).addClass('client-selected') : $(this.$el).removeClass('client-selected');
            this.isProduct = this.$('#refund-client-product').searchPlugin('getSelectedObject') !== undefined;

            if(!this.model.get('phone')){
                this.clickSearchClient(false);
            }
        },

        // For tablet usage
        changeRefundTab: function (e) {
            e.preventDefault();
            e.stopPropagation();
            var $tab = $(e.target),
                url = $tab.data('url');
            if(!$tab.hasClass('selected') && !$tab.hasClass('readonly')) {
                $tab.siblings('li.refund-tabs-menu-item').removeClass('selected');
                $tab.addClass('selected');
                this.$('.refund-tabs-content .refund-tab').removeClass('current');
                this.$('#' + url).addClass('current');
                this.currentTab = $('#' + url).data('tab');
                this.renderCurrentTabCollection();
            }
            if (this.scroll) {
                this.scroll.refresh();
                this.scroll.scrollTo(0, 0);
            }
        },

        // For mobile usage
        changeRefundTabMobile: function (e) {
            var $option = $(e.target).find(':selected'),
                url = $option.data('url');
            this.$('.refund-tabs-content .refund-tab').removeClass('current');
            this.$('#' + url).addClass('current');
            this.currentTab = $('#' + url).data('tab');
            this.renderCurrentTabCollection();
            this.$('.form-group').removeClass('error');
            this.$('#refund-client-name').removeClass('error');
            this.$('#refund-client-product').removeClass('error');
            this.$('#refund-client-service').removeClass('error');
        },

        getSelectedClient: function () {
            var obj = this.$('#refund-client-name').searchPlugin('getSelectedObject'),
                clientFromModel = this.model.get('client');
            if (obj) {
                this.model.set('client', {});
            } else {
                if (clientFromModel.id && clientFromModel.id !== this.model.get('invisibleClientId')) obj = clientFromModel;
            }
            return obj;
        },

        getCurrentFieldValue: function () {
            var val = '';

            switch(this.currentTab) {
                case 'tab-client':
                    val = {
                        clientID: this.getSelectedClient(),
                        itemID: (this.isProduct) ?
                            this.$('#refund-client-product').searchPlugin('getSelectedObject') :
                            this.$('#refund-client-service').searchPlugin('getSelectedObject')
                    };
                    break;
                case 'tab-transaction':
                    val = this.$('#refund-transaction-find').val();
                    break;
                case 'tab-giftcard':
                    val = this.$('#refund-giftcard-find').val();
                    break;
            }

            return val;
        },

        renderCurrentTabCollection: function () {
            if(!this.model.get('phone')) {
                this.$('#refund-items').empty();
                switch(this.currentTab) {
                    case 'tab-client':
                        this.clickSearchClient(true);
                        break;
                    case 'tab-transaction':
                        this.clickSearchTransaction(true);
                        break;
                    case 'tab-giftcard':
                        this.clickSearchGiftcard(true);
                        break;
                }
            }
        },

        // Opens sub-modal with NO items
        noRefundItems: function () {
            this.onRefundNoItemModal = new OnRefundNoItemModalView({
                client: this.getSelectedClient(),
                item: (this.isProduct) ?
                    this.$('#refund-client-product').searchPlugin('getSelectedObject') :
                    this.$('#refund-client-service').searchPlugin('getSelectedObject')
            }).render();
            this.listenTo(this.onRefundNoItemModal, 'noRefundItem', this.extractEvent);
        },

        // Opens sub-modal with items : ONLY MOBILE
        openSubModal: function (title, number, isTransaction) {
            this.onRefundItemModal = new OnRefundItemModalView({
                title: title,
                number: number,
                isTransaction: isTransaction,
                currentTab: this.currentTab,
                refundedItems: this.refundedItems
            }).render();
            this.listenTo(this.onRefundItemModal, 'returnToMenu', function () {
                this.customClose({ type: 'returnToMenu' });
            }.bind(this));
            this.listenTo(this.onRefundItemModal, 'extractMobile', this.extractEvent);
        }
    });
});