/**
 * Created by petermomot on 5/5/15.
 */
define([
    'jquery',
    'views/smart-receipt/sales-modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/sale-accounts.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'bridge'
], function ($, SalesModalView, YesNoModal, InnerTemplate, UserSearchTemplate) {
    'use strict';

    return SalesModalView.extend({
        innerTemplate: _.template(InnerTemplate),

        initialize: function (options) {
            options.title = 'Account';
            this.payingClient = options.payingClient;
            SalesModalView.prototype.initialize.call(this, options);
            this.events = _.extend({}, SalesModalView.prototype.events, this.events);
        },

        afterInit: function (o) {
            o = _.extend(o, {
                accordionClass: 'sales-account-selection',
                accordionItemClass: 'accounts-accordion-item'
            });
            this.model = new Backbone.Model(o);
        },

        afterRender: function () {
            this.$('.accounts-accordion-item .sales-accordion-body').html(this.innerTemplate());
            this.$('.sales-accordion-item.employees').remove();
            SalesModalView.prototype.afterRender.call(this);

            this.$('#account-selection-price-total').inputFormat({
                clearButton: true,
                allowMinus: true
            });

            var func = this.getBalance;
            this.setupSearchField(
                'account-for', {
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                    return getClients(filter, true, undefined, true, pageNumber, pageSize);
                },
                parentScroll: this.salesScreenScroll[0],
                large: true,
                template: _.template(UserSearchTemplate)
                },
                this.payingClient,
                function() {
                     func.call(this, { target: document.getElementById('account-for') });
                }.bind(this));

            this.showSalesSection('.sales-account-selection');
            return this;
        },

        events: {
            'blur #account-for': 'getBalance',
            'submit #account-selection-form': 'setAccountSaleLine',
            'search-close #account-for': 'clearBalance',
            'change #account-for': 'changeAccount',
            'tap #pay-all': 'payAll',
            'keyup #account-selection-price-total': 'clearErrorOnBlur',
            'change #account-selection-price-total': 'clearErrorOnBlur',
        },

        changeAccount: function(e) {
            if(e.target.value.trim().length == 0) {
                this.clearBalance();
            }
        },

        getBalance: function (e) {
            var $input = $(e.target);
            if ($input.data('id')) {
                $input.parent().removeClass('error');
                this.userAccount = $input.searchPlugin('getSelectedObject');
                if (!this.userAccount) {
                    $input.parent().addClass('error');
                    return false;
                } else {
                    this.$('#account-balance')
                        .text($.global.format(this.userAccount.balance, 'c'))
                        .removeClass('negative positive error');
                    if (this.userAccount.balance > 0) {
                        this.$('#account-balance').addClass('positive');
                    } else if (this.userAccount.balance < 0) {
                        this.$('#account-balance').addClass('negative');
                    }
                }
            }
        },

        validateAccountForm: function (skipPriceValidation) {
            skipPriceValidation = skipPriceValidation || false;

            var $accountField = this.$('#account-for');
            var $valueField = this.$('#account-selection-price-total');
            var $noteField = this.$('#account-note');
            $noteField.val(this.getTextFromHtml(this.$('#account-note-input')));
            var value = parseFloat($valueField.inputFormat('getValue'));
            var note = $noteField.val().trim();
            var invalid = false;
            if(!this.userAccount) {
                $accountField.addClass('error');
                invalid = true;
            }
            if (!skipPriceValidation)
                if (!value) {
                    $valueField.closest('.input-format').addClass('error');
                    invalid = true;
                }

            if(invalid) {
                return false;
            }
            $accountField.removeClass('error');
            $valueField.closest('.input-format').removeClass('error');

            if (!this.isValidInputLength($('#account-note'))) {
                new YesNoModal({
                    question: "The field Notes must be a string or array type with a maximum length of '256'",
                    positiveText: 'Ok',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }

            return {
                client: this.userAccount,
                price: value,
                note: note
            };
        },

        setAccountSaleLine: function (e) {
            e && e.preventDefault();
            var item = this.validateAccountForm((e || {}).skipPriceValidation);
            if (item) {
                var newSaleLine = {};
                newSaleLine.client = _.findWhere(this.clients, {
                    id: this.lastSelected.get('clientId')
                });
                newSaleLine.item = _.extend({
                    type: 1545,
                    displayName: 'Client Account Sale',
                    shortcut: ''
                }, item);
                this.trigger('addedSaleLine', newSaleLine);
                this.customClose();
            }
        },

        payAll: function(){
            if(this.userAccount === undefined){
                new YesNoModal({
                    question: "You must select a client",
                    positiveText: 'Ok',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            }
            var valueToSet = 0;
            if(this.userAccount.balance < 0)
                valueToSet = Math.abs(this.userAccount.balance);

            this.$('#account-selection-price-total').inputFormat('setValue', valueToSet);

            this.setAccountSaleLine({ preventDefault: function() {}, skipPriceValidation: true });
        },

        clearBalance: function (){
            $('#account-balance')
                .text($.global.format(0, 'c'))
                .removeClass('negative positive error');
        }

    });
});
