/**
 * Created by petermomot on 5/5/15.
 */
define([
    'jquery',
    'data/consts',
    'views/smart-receipt/sales-modal',
    'text!../../../templates/smart-receipt/sale-gifts.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'views/yes-no-modal',
    'bridge'
], function ($, Consts, SalesModalView, GiftsTemplate, UserSearchTemplate, YesNoModal) {
    'use strict';

    return SalesModalView.extend({
        giftsTemplate: _.template(GiftsTemplate),
        msPerDay: 1000 * 60 * 60 * 24,
        selectedGC: null,

        initialize: function (options) {
            options.title = 'Gift Card';
            SalesModalView.prototype.initialize.call(this, options);
            this.events = _.extend({}, SalesModalView.prototype.events, this.events);
        },

        afterInit: function (o) {
            o = _.extend(o, {
                accordionClass: 'gift-cards-accordion',
                accordionItemClass: 'gift-cards-accordion-item'
            });
            this.model = new Backbone.Model(o);
        },

        afterRender: function () {
            this.$('.gift-cards-accordion-item .sales-accordion-body').html(this.giftsTemplate());
            SalesModalView.prototype.afterRender.call(this);

            this.$('#card-value').inputFormat({
                clearButton: true,
                allowMinus: false
            });
            this.$('#card-type').searchPlugin({
                filterFunction: getGiftTypes,
                parentScroll: this.salesScreenScroll[0],
				selectedCallBack : this.cardTypeChange
            });
            this.$('#card-type').on('optionSelected', function() {
                this.selectedGC = $('#card-type').searchPlugin('getSelectedObject');
                this.$('#card-value').inputFormat('setValue', this.selectedGC.defaultValue);
                this.$('.set-gift-card-amount').removeClass('selected');
            }.bind(this));
            this.$('#card-for').searchPlugin({
                filterFunction: function(filter, clientAccountChargable, pageNumber, pageSize) {
                     return getClients(filter, clientAccountChargable, true, undefined, pageNumber, pageSize);
                },
                parentScroll: this.salesScreenScroll[0],
                large: true,
                template: _.template(UserSearchTemplate),
                selectedCallBack: function(obj) {
                    if (obj)
                        SalesModalView.prototype.removeError($('#card-for'));
                }
            });
            this.$('#card-valid-from').val(this.prepareTodayForInput());
            this.$('#card-number').inputFormat({
                type: 'any',
                places: 20,
                allowEmpty: true
            });

            this.showGiftCardsSection();

            return this;
        },

        events: function () {
            return _.extend(SalesModalView.prototype.events, {
                'touchend .sales-accordion-item.gift-cards-accordion-item .header': 'showGiftCardsSection',
                'tap #card-number-generate': 'cardNumberGenerate',
                'tap .set-gift-card-amount': 'changeGiftValue',
                'tap .input-format-clear-icon': 'clearGiftValue',
                //'change #card-valid-from': 'changeFromDate',
                'blur #card-valid-from': 'blurFromDate',
                //'change #card-valid-to': 'changeToDate',
                'reset .gift-card': 'cardReset',
                'submit .gift-card': 'cardSubmit',
                'keyup #card-number': 'clearErrorOnBlur'
            });
        },

        cardTypeChange: function(obj){
		    if (obj) {
		        if (obj.dateExpiration) {
		            var date = new Date(Date.parse(obj.dateExpiration));
		            var day = ("0" + date.getDate()).slice(-2);
		            var month = ("0" + (date.getMonth() + 1)).slice(-2);
		            var year = date.getFullYear();
		            $('#card-valid-to').val([year, month, day].join('-'));
		        }
		        SalesModalView.prototype.removeError($('#card-type'));
		    }
        },

        showGiftCardsSection: function () {
            this.showSalesSection('.gift-cards-accordion');
            this.showAccordionItem('.gift-cards-accordion-item');
        },

        cardNumberGenerate: function () {
            generateGiftCardNumber().done(function (r) {
                if (r.status === undefined) {
                    this.$('#card-number').inputFormat('setValue', r);
                    this.$('#card-number').trigger('keyup');
                }
            }.bind(this));
        },

        changeGiftValue: function (e) {
            this.$('.set-gift-card-amount').removeClass('selected');
            $(e.target).addClass('selected');
            var amount = +$(e.target).data('amount');
            this.$('#card-value').inputFormat('setValue', amount).closest('.form-group').removeClass('error');
        },

        clearGiftValue: function () {
            this.$('.set-gift-card-amount').removeClass('selected');
            return true;
        },

        prepareTodayForInput: function () {
            var now = new Date();
            var day = ("0" + now.getDate()).slice(-2);
            var month = ("0" + (now.getMonth() + 1)).slice(-2);
            return now.getFullYear() + "-" + (month) + "-" + (day);
        },

        getDateFromInput: function (d) {
            d = d.split('-');
            return new Date(+d[0], +d[1] - 1, +d[2]);
        },

        changeFromDate: function (e) {
            if (!e.target.value) {
                e.target.value = this.prepareTodayForInput();
                return;
            }
            var dateFrom = this.getDateFromInput(e.target.value),
                diff = dateFrom.getTime() - new Date().getTime(),
                $dateTo = this.$('#card-valid-to');
            if (diff < 0 && this.msPerDay + diff < 0) {
                e.target.value = this.prepareTodayForInput();
            } else {
                if ($dateTo.val() != '') {
                    var dateTo = this.getDateFromInput($dateTo.val());
                    if (dateFrom.getTime() - dateTo.getTime() > 0) {
                        $dateTo.val(e.target.value);
                    }
                }
            }
        },

        blurFromDate: function (e) {
            if (!e.target.value) {
                e.target.value = this.prepareTodayForInput();
            }
        },

        changeToDate: function (e) {
            var dateTo = this.getDateFromInput(e.target.value);
            var dateFromVal = this.$('#card-valid-from').val(),
                dateFrom = this.getDateFromInput(dateFromVal);
            var diff = dateTo.getTime() - dateFrom.getTime();
            if (diff < 0) {
                e.target.value = dateFromVal;
            }
        },

        cardReset: function () {
            setTimeout(function () {
                this.$('#card-value').inputFormat('setValue', 0);
                this.$('#card-valid-from').val(this.prepareTodayForInput());
            }.bind(this), 0);
        },

        validateForm: function () {
            var $form = this.$('.gift-card'),
                serialized = $form.serializeArray(),
                toSend = _.object(_.pluck(serialized, 'name'), _.pluck(serialized, 'value')),
                errors = false,
                $formGroup = '',
                $cardFor = this.$('#card-for');

            toSend['card-value'] = this.$('#card-value').inputFormat('getValue');

            toSend['card-type'] = this.$('#card-type').data('id') ? {
                id : this.$('#card-type').data('id'),
                displayName : this.$('#card-type').val()
            }:undefined;

            toSend['card-for'] = $cardFor.searchPlugin('getSelectedObject');

            _.each(toSend, function (value, key) {
                $formGroup = $form.find('[name=' + key + ']').closest('.form-group');
                if (!value && $formGroup.hasClass('required')) {
                    $formGroup.addClass('error');
                    errors = true;
                } else {
                    $formGroup.removeClass('error');
                }
            });

            return errors ? false : toSend;
        },

        cardSubmit: function (e) {
            e && e.preventDefault();
            var dateFrom = this.getDateFromInput(this.$('#card-valid-from').val());
            var dateTo = this.getDateFromInput(this.$('#card-valid-to').val());
            var diff = dateTo.getTime() - dateFrom.getTime();
            if (diff < 0) {
                new YesNoModal({
                    question: 'The expiration date must be greater than or equal to the valid from date.',
                    positiveText: 'OK',
                    negativeText: '',
                    positiveCallback: function () {
                        this.closeView()
                    }
                }).render();
                return false;
            }

            this.$('input').blur();
            var data = this.validateForm();
            if (data) {
                this.lastSelected.set('giftGard', {
                    displayName: 'GC',
                    shortcut: 'GC',
                    type: Consts.SaleLineTypes.GIFTCARD,
                    number: data['card-number'],
                    client: data['card-for'],
                    giftType: data['card-type'],
                    dateValidFrom: data['card-valid-from'],
                    dateExpiration: data['card-valid-to'],
                    price: this.selectedGC.defaultPrice || data['card-value'],
                    value: data['card-value']
                });
                if (!this.setSaleLine()) {
                    this.buildEmployeeList();
                }
            }
        }
    });
});
