/**
 * Created by mzarichniuk on 4/7/2015.
 */
define([
    'jquery',
    'data/consts',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/sales-base-modal.html',
    'text!../../../templates/smart-receipt/sales-category.html',
    'bridge'
], function ($, Consts, ModalView, YesNoModal,
             SalesModalTemplate, SalesCategoryTemplate) {
    'use strict';

    return ModalView.extend({
        template: _.template(SalesModalTemplate),
        scrollbars: false,
        salesScreenScroll: [],
        withoutScroll: true,
        employees: undefined,
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                {
                    id: 'sale-cancel',
                    className: 'red cancel',
                    size: '20',
                    value: ''
                },
                {
                    id: 'sale-back',
                    className: 'yellow back',
                    size: '20',
                    value: ''
                },
                {
                    id: 'sale-submit',
                    className: 'green',
                    size: '60',
                    value: 'apply'
                }
            ],
            tabletButtons: [
                {
                    id: 'sale-submit',
                    className: 'green',
                    size: '50',
                    value: 'apply'
                },
                {
                    id: 'sale-cancel',
                    className: 'red',
                    size: '50',
                    value: 'cancel'
                }  
            ],
            touchCallBacks: {
                'sale-cancel': 'saleCancel',
                'sale-back': 'saleBack',
                'sale-submit': 'saleSubmit'
            }
        },

        initialize: function (options) {
            this.onPhone = $('html').hasClass('mobile');
            this.lastSelected = options.lastSelected;
            this.clients = options.clients;
            this.invisibleClientId = options.invisibleClientId;
            this.saleLinesCount = options.saleLinesCount;
            this.lastSelected.set('clientId', options.clientId);
            this.el = this.onPhone ? '#sales-modal .content' : '#tablet-sales-modal .content';
            this.setElement(this.el);

            var o = {
                phone: this.onPhone,
                title: options.title
            };
            if (o.phone) {
                this.breadCrumbs = ['Register', 'Sale', options.title];
                this.modalButtonsBottom.phone = true;
            }

            this.events = _.extend({}, ModalView.prototype.events, this.events);
            this.afterInit(o);
        },

        afterInit: function (o) {
            this.model = new Backbone.Model(o);
        },

        render: function () {
            ModalView.prototype.render.call(this);
            this.$el.removeClass('without-buttons-landscape');

            this.getSelectedEmployee();

            this.afterRender();
            return this;
        },

        afterRender: function () {
            this.scrollInit();
        },

        events: {
            'focus .sales-items-filter':'killPlaceHolder',
            'blur .sales-items-filter':'restorePlaceHolder',

            // employees
            'tap .services-item, .products-item': 'showEmployeeList',
            'touchend .sales-accordion-item.employees .header': 'showEmployeeList',
            'tap .sales-category.employee': 'selectEmployee',
            'keyup .filter-employees': 'filterEmployeeList',
            'focus .filter-employees':'killPlaceHolder',
            'blur .filter-employees':'restorePlaceHolder',
			'tap .choose-me-employee .me':'chooseMe',
        },

        manufacturersWasSet: false,
        manufacturer: null,

        scrollInit: function() {
            // It has collection with objects for each scroll screen.
            // In every object are 4 fields:
            // 1. current page number
            // 2. page size
            // 3. callback for refreshing scroll screen
            // 4. callback for loading next page
            var that = this;
            var alwaysCallbackRefresh = function() {
                $(this.wrapper).find('.sales-items-filter').val('');
                $(this.wrapper).removeClass('refreshing');
                that.requestInProcess = false;
                this.enabled = true;
                this.scrollTo(0, 0, this.options.bounceTime, this.options.bounceEasing);
            };
            var alwaysCallbackNext = function() {
                $(this.wrapper).removeClass('loading');
                that.requestInProcess = false;
                this.enabled = true;
            };
            this.scrollLoading = {
                'services cat': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        getServicesTypes(this.pageNumber = 0, this.pageSize)
                            .done(that.onGetServicesTypes.bind(that))
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        getServicesTypes(this.pageNumber, this.pageSize)
                            .done(that.onGetServicesTypes.bind(that))
                            .always(alwaysCallbackRefresh.bind(scroller));
                    }
                },
                'services': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        getServices(that.lastSelected.get('catId') || '', that.lastSelected.get('employee') && that.lastSelected.get('employee').id, this.pageNumber = 0, this.pageSize)
                            .done(function(data) {
                                that.onGetServices.call(that, data, true);
                            })
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        getServices(that.lastSelected.get('catId') || '', that.lastSelected.get('employee') && that.lastSelected.get('employee').id, this.pageNumber, this.pageSize)
                            .done(function(data) {
                                that.onGetServices.call(that, data, false);
                            })
                            .always(alwaysCallbackNext.bind(scroller));
                    }
                },
                'employees': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        filterEmployees('', this.pageNumber = 0, this.pageSize)
                            .done(function(data) {
                                that.onGetEmployees.call(that, data, true);
                            })
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        filterEmployees('', this.pageNumber, this.pageSize)
                            .done(function(data) {
                                that.onGetEmployees.call(that, data, false);
                            })
                            .always(alwaysCallbackNext.bind(scroller));
                    }
                },
                'products vendors': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        getProductsCat('', this.pageNumber = 0, this.pageSize)
                            .done(function(data) {
                                that.onGetProductsCat.call(that, data, true);
                            })
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        getProductsCat('', this.pageNumber, this.pageSize)
                            .done(function(data) {
                                that.onGetProductsCat.call(that, data, false);
                            })
                            .always(alwaysCallbackNext.bind(scroller));
                    }
                },
                'products cat': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        getProductsCat(that.lastSelected.get('venId'), this.pageNumber = 0, this.pageSize)
                            .done(function(data) {
                                that.onGetProductsCat.call(that, data, true, that.lastSelected.get('venId'));
                            })
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        getProductsCat(that.lastSelected.get('venId'), this.pageNumber, this.pageSize)
                            .done(function(data) {
                                that.onGetProductsCat.call(that, data, false, that.lastSelected.get('venId'));
                            })
                            .always(alwaysCallbackNext.bind(scroller));
                    }
                },
                'products': {
                    pageNumber: 0,
                    pageSize: 20,
                    refresh: function(scroller) {
                        getProducts(this.manufacturer, that.lastSelected.get('catId'), this.pageNumber = 0, this.pageSize)
                            .done(function(data) {
                                that.onGetProducts.call(that, data, true);
                            })
                            .always(alwaysCallbackRefresh.bind(scroller));
                    },
                    loadNext: function(scroller) {
                        getProducts(this.manufacturer, that.lastSelected.get('catId'), this.pageNumber, this.pageSize)
                            .done(function(data) {
                                that.onGetProducts.call(that, data, false);
                            })
                            .always(alwaysCallbackNext.bind(scroller));
                    }
                }
            };

            this.$('.smart-receipt-sales-section .sales-accordion-body').each(function (index, elem) {
                if (this.salesScreenScroll[index]) {
                    this.salesScreenScroll[index].destroy();
                }
                this.salesScreenScroll[index] = null;
                this.salesScreenScroll[index] = new IScroll(elem, {
                    tap: true,
                    probeType: 2,
                    disableMouse: true
                });
                this.salesScreenScroll[index].on('beforeScrollStart', function () {
                    var scrollTop = $(this.wrapper).scrollTop();
                    if (scrollTop !== 0) {
                        this.scrollTo(0, scrollTop*-1);
                        $(this.wrapper).scrollTop(0);
                    }
                    this.refresh();
                    $(this.wrapper).find('.sales-items-filter').blur();
                });
                this.salesScreenScroll[index].on('scroll', function () {
                    if($(this.wrapper).data('content')) {
                        this._triggerRefresh = this.y > 40;
                        $(this.wrapper).toggleClass('active-refresh', this.y > 0);
                        if (this._triggerRefresh = this.y > 40) {
                            this.y = 40;
                        }
                        if (that.scrollLoading[$(this.wrapper).data('content')].pageNumber != 'last') {
                            $(this.wrapper).toggleClass('active-loading', this.y < 0 && (this.y - this.maxScrollY) < 0);
                            if (this._triggerLoadNext = this.y < 0 && (this.y - this.maxScrollY) < -40) {
                                this._scrollToY = this.y;
                                this.y = this.maxScrollY - 40;
                            }
                        }
                    }
                });
                this.salesScreenScroll[index].on('beforeScrollEnd', function () {
                    if($(this.wrapper).data('content')) {
                        $(this.wrapper).removeClass('active-refresh active-loading refreshing loading');
                        if (this._triggerRefresh) {
                            this.enabled = false;
                            this.hasVerticalScroll = false;
                            this.y = 0;
                            $(this.wrapper).addClass('refreshing');
                            this._execEvent('needRefresh');
                        }
                        if (this._triggerLoadNext) {
                            this.moved = false;
                            this.enabled = false;
                            this.y = this._scrollToY;
                            $(this.wrapper).addClass('loading');
                            this._execEvent('needNext');
                        }
                    }
                });
                this.salesScreenScroll[index].on('needRefresh', function () {
                    setTimeout(function () {
                        this._triggerRefresh = false;
                        that.salesScreenScroll[index].requestInProcess = true;
                        that.scrollLoading[$(this.wrapper).data('content')].refresh(this);
                    }.bind(this), 300);
                });
                this.salesScreenScroll[index].on('needNext', function () {
                    setTimeout(function () {
                        this._triggerLoadNext = false;
                        that.salesScreenScroll[index].requestInProcess = true;
                        that.scrollLoading[$(this.wrapper).data('content')].loadNext(this);
                    }.bind(this), 300);
                });
            }.bind(this));
        },

        refreshScrolls: function() {
            _.each(this.salesScreenScroll, function (elem) {
                elem.scrollTo(0, 0);
                elem.refresh();
            });
        },

        setPageNumber: function(data, key) {
            if(_.isNumber(data)) {
                this.scrollLoading[key].pageNumber = data;
                return;
            }
            if(data.length < this.scrollLoading[key].pageSize) {
                this.scrollLoading[key].pageNumber = 'last';
            } else {
                this.scrollLoading[key].pageNumber++;
            }
        },

        setClient: function (clients) {
            return _.findWhere(clients, {id: this.lastSelected.get('clientId')});
        },

        setEmployee: function () {
            var employeeFromCollection = this.lastSelected.get('employee');
            if(!employeeFromCollection && window.SETTINGS['userEmployee'] && (window.SETTINGS['userEmployee'].id == this.lastSelected.get('employeeId'))) {
                employeeFromCollection = window.SETTINGS['userEmployee'];
            }
            return employeeFromCollection;
        },

        setSaleLine: function (param) {
            var newSaleLine = {},
                clients = this.clients;

            if (!this.lastSelected.get('clientId')) {
                if (clients.length > 0) {
                    this.lastSelected.set('clientId', clients[clients.length - 1].id);
                }
            } else if (this.lastSelected.get('clientId') === this.invisibleClientId) {
                if (clients.length > 0 && !_.findWhere(clients, {id: this.invisibleClientId})) {
                    this.lastSelected.set('clientId', clients[clients.length - 1].id);
                }
            }

            newSaleLine.client = this.setClient(clients);

            if (this.lastSelected.get('employeeId')) {
                if (this.lastSelected.get('itemId')) {
                    newSaleLine.item = _.findWhere(this.lastSelected.get('itemsArray'), {
                        id: this.lastSelected.get('itemId')
                    });
                    if(newSaleLine.item.type == Consts.SaleLineTypes.SERVICE) {
                        getServiceByEmployee(this.lastSelected.get('itemId'), this.lastSelected.get('employeeId'), false).done(function(data) {
                            newSaleLine.item = data;
                        }.bind(this));
                    }
                } else if (this.lastSelected.get('giftGard')) {
                    newSaleLine.item = this.lastSelected.get('giftGard');
                } else {
                    return false;
                }
            } else {
                return false;
            }

            newSaleLine.employee = this.setEmployee();
            this.newSaleLine = newSaleLine;
            if (!param) {
                this.saleSubmit()
            }
            return true;
        },

        customClose: function (r) {
            if (this.onPhone) {
                // upd this
                if (r) {
                    if (r.type == 'returnToMenu') {
                        this.trigger('closeEvent', 'returnToMenu');
                    }
                    if (r == 'returnToSubMenu') {
                        this.trigger('closeEvent', 'returnToSubMenu');
                    }
                }
                this.closeView();
            } else {
                this.trigger('closeEvent', 'doneLeave');
                this.afterRender();
            }
        },

        showSalesSection: function (selector) {
            this.$(selector).show().siblings('.smart-receipt-sales-section').hide();
            this.refreshScrolls();
        },

        showAccordionItem: function (selector) {
            var $parent = $(selector);
            $parent.siblings('.sales-accordion-item').removeClass('opened');
            $parent.addClass('opened').find('.sales-accordion-body');
        },

        killPlaceHolder: function (e) {
            var elem = $(e.target);
            var placeHolder = elem.attr('placeholder');
            elem.data('holder', placeHolder);
            elem.removeAttr('placeholder');
        },

        restorePlaceHolder: function (e) {
            var elem = $(e.target);
            var placeHolder = elem.data('holder');
            elem.attr('placeholder', placeHolder);
            elem.removeAttr('data-holder');
        },

        saleCancel: function () {
            var employeeId = this.lastSelected.get('employeeId');
            var employee = this.lastSelected.get('employee');
            var clientId = this.lastSelected.get('clientId');

            this.lastSelected.clear();
            this.lastSelected.set({
                employeeId: employeeId,
                clientId: clientId,
                employee: employee
            });

            this.getSelectedEmployee();
            this.customClose();
        },

        saleBack: function () {
            this.customClose('returnToSubMenu');
        },

        saleSubmit: function (e) {
            if (e) {
                // Gift Card Submit
                if(this.model.get('title') === "Gift Card") {
                    this.cardSubmit();
                }
                // Account Submit
                if(this.model.get('title') === "Account") {
                    this.setAccountSaleLine();
                }
            
                if (!this.setSaleLine(true)) return false;
            }
            
            this.trigger('addedSaleLine', this.newSaleLine);
            this.lastSelected.unset('itemId');
            this.lastSelected.unset('giftGard');
            this.newSaleLine = undefined;
            this.customClose();
        },

        closeView: function (r) {
            this.trigger('closeEvent', r ? r : 'close');
            ModalView.prototype.closeView.call(this);
        },

        // employee events
        getSelectedEmployee: function () {
            var selectedEmployee = undefined;
            if(this.lastSelected.get('employeeId') !== undefined && this.saleLinesCount > 0) {
                selectedEmployee = this.lastSelected.get('employee');
                if(!selectedEmployee && window.SETTINGS['userEmployee'] && (window.SETTINGS['userEmployee'].id == this.lastSelected.get('employeeId'))) {
                    selectedEmployee = window.SETTINGS['userEmployee'];
                }
                this.buildSelectedEmployeeTemplate(selectedEmployee);
            } else if(this.lastSelected.get('employeeId') !== undefined) {
                this.buildSelectedEmployeeTemplate(this.lastSelected.get('employee'));
            } else {
                this.$('.sales-accordion-item.employees .header').html('Employee');
            }
        },

        showEmployeeList: function (e) {
            if (e) {
                var serviceId = $(e.target).closest('.services-item').data('id');
                var productId = $(e.target).closest('.products-item').data('id');
                if (serviceId) {
                    this.lastSelected.set('itemId', serviceId);
                    $(e.target).closest('.sales-item').addClass('selected').siblings().removeClass('selected');
                }
                if (productId) {
                    this.lastSelected.set('itemId', productId);
                    $(e.target).closest('.sales-item').addClass('selected').siblings().removeClass('selected');
                }
            }
            if (!this.setSaleLine()) {
                this.buildEmployeeList();
            }
        },

        onGetEmployees: function(data, empty) {
            var $container = this.$('.employees ul');
            if(empty) {
                $container.empty();
                this.setPageNumber(0, 'employees');
            }
            this.setPageNumber(data, 'employees');
            var itemTemplate = _.template(SalesCategoryTemplate);
            _.each(data, function (elem) {
                $container.append(itemTemplate(_.extend({
                    itemClass: 'employee',
                    imageId: '',
                    displayName: elem.code
                }, elem)));
            });
            $container.find('[data-id=' + this.lastSelected.get('employeeId') + ']').addClass('selected');
			this.mergeEmployees(data);
        },

		mergeEmployees: function(data){
		
			var employees = this.employees;
			_.each(data, function(empl){
				if(!_.some(employees, function(elem){ return elem.id === empl.id; })) {
                    employees.push(empl);
                }
			});
			
			this.employees = employees;
		},

        buildEmployeeList: function () {
            this.showAccordionItem('.sales-accordion-item.employees');
            filterEmployees('')
                .done(function (data) {
                    if (!this.$el) return false;
                    this.$('.employees .sales-accordion-body').data('content', 'employees');
                    this.$('.employees .filter-employees').val('');
                    this.employees = data;
                    this.onGetEmployees(data, true);
                    this.showAccordionItem('.sales-accordion-item.employees');
                    this.refreshScrolls();
                    if (window.SETTINGS['userEmployee']) {
                        this.$('.choose-me-employee').show();
                    }
                }.bind(this));
        },

		chooseMe : function(e){
            this.lastSelected.set('employeeId', window.SETTINGS['userEmployee'].id);
            this.lastSelected.set('employee', window.SETTINGS['userEmployee']);
            this.buildSelectedEmployeeTemplate(window.SETTINGS['userEmployee']);
			if (!this.setSaleLine())
                this.showAccordionItem($(e.target).closest('.sales-accordion-item').siblings('.sales-accordion-item'));
		},
		
		buildSelectedEmployeeTemplate : function(employee){
			return this.$('.sales-accordion-item.employees .header').html(_.template('Employee > <span class="current-employee"><%= code %></span>')(employee));
		},
		
        filterEmployeeList: function (e) {
            var query = $(e.target).val();
            if(query === '') {
                this.buildEmployeeList();
            } else {
                filterEmployees(query)
                    .done(function (data) {
                        this.onGetEmployees(data, true);
                        this.refreshScrolls();
                    }.bind(this));
            }
        },

        selectEmployee: function (e) {
            var $employee = $(e.target).closest('.sales-category.employee');
            var employeeId = $employee.data('id');
            $employee.addClass('selected').siblings().removeClass('selected');
            var currentEmployeeObject = _.findWhere(this.employees, {id: employeeId});
            this.lastSelected.set('employeeId', employeeId);
            this.lastSelected.set('employee', currentEmployeeObject);
            this.buildSelectedEmployeeTemplate(currentEmployeeObject);
            if (!this.setSaleLine()) {
                this.showAccordionItem($(e.target).closest('.sales-accordion-item').siblings('.sales-accordion-item'));
            }
        }

    });
});
