/**
 * Created by petermomot on 5/5/15.
 */
define([
    'jquery',
    'views/smart-receipt/sales-modal',
    'text!../../../templates/smart-receipt/sales-category.html',
    'text!../../../templates/smart-receipt/sales-item.html',
    'text!../../../templates/smart-receipt/sale-services-products.html',
    'bridge'
], function ($, SalesModalView,
             SalesCategoryTemplate, SalesItemTemplate, InnerTemplate) {
    'use strict';

    return SalesModalView.extend({
        innerTemplate: _.template(InnerTemplate),

        initialize: function (options) {
            options.title = 'Products';
            SalesModalView.prototype.initialize.call(this, options);
            this.events = _.extend({}, SalesModalView.prototype.events, this.events);
        },

        afterInit: function (o) {
            o = _.extend(o, {
                accordionClass: 'products-accordion',
                accordionItemClass: 'products'
            });
            this.model = new Backbone.Model(o);
        },

        afterRender: function () {
            this.$el.addClass('without-buttons-landscape');
            this.$('.sales-accordion-item.products .sales-accordion-body').html(this.innerTemplate({ className: 'products' }));
            SalesModalView.prototype.afterRender.call(this);
            this.buildSalesProductsCat();
            return this;
        },

        events: {
            'touchend .sales-accordion-item.products .header': 'showSalesProductsVendor',
            'touchend .sales-accordion-item.products .header .products-vendor': 'showSalesProductsCat',
            'touchend .sales-accordion-item.products .header .products-cat': 'showProductsAccordionItem',
            'tap .products-vendor': 'showSalesProductsCat',
            'tap .products-cat': 'showSalesProducts',
            'input .sales-products-filter': 'filterSalesProducts'
        },

        showProductsAccordionItem: function (e) {
            e.stopPropagation();
            this.showAccordionItem('.sales-accordion-item.products');
        },

        showSalesProductsVendor: function () { // fire build vendors list
            this.buildSalesProductsCat();
        },

        showSalesProductsCat: function (e) { // fire build categories list after vendor select
            var $vendorItem = $(e.target).closest('.products-vendor');
            $vendorItem.addClass('selected').siblings().removeClass('selected');
            var vendorId = $vendorItem.data('id');
            var vendorName;
            if($vendorItem.is('.header .products-vendor')) {
                e.stopPropagation();
                vendorName = $vendorItem.text();
            } else {
                vendorName = $vendorItem.find('.title').text();
            }
            this.buildSalesProductsCat(vendorId, vendorName);
        },

        onGetProductsCat: function(data, empty, vendorId) {
            var $container = this.$('.products ul');
            if(empty) {
                $container.empty();
                this.setPageNumber(0, vendorId ? 'products cat' : 'products vendors');
            }
            this.setPageNumber(data, vendorId ? 'products cat' : 'products vendors');
            var itemTemplate = _.template(SalesCategoryTemplate);
            _.each(data, function (elem) {
                $container.append(itemTemplate(_.extend({
                    itemClass: vendorId ? 'products-cat' : 'products-vendor',
                    imageId: elem.imageId || ''
                }, elem)));
            });
        },

        buildSalesProductsCat: function (vendorId, vendorName) { // build products categories/vendors list
            getProductsCat(vendorId).done(function (data) {
                if (!this.$el) return false;
                this.lastSelected.unset('itemId');
                this.lastSelected.unset('catId');
                this.$('.products .sales-items-filter').val('');
                if(vendorId) {
                    this.lastSelected.set('venId', vendorId);
                    this.$('.products .sales-accordion-body').data('content', 'products cat');
                    this.$('.sales-accordion-item.products .header').html(
                        _.template(
                            'Products > <span class="products-vendor" data-id="<%= vendorId %>"><%= vendorName %></span>'
                        )({
                            vendorId: vendorId,
                            vendorName: vendorName
                        })
                    );
                    if (this.manufacturersWasSet) {
                        this.manufacturersWasSet = false;
                        this.manufacturer = vendorId;
                    }
                } else {
                    this.manufacturersWasSet = true;
                    this.$('.products .sales-accordion-body').data('content', 'products vendors');
                    this.$('.sales-accordion-item.products .header').html('Products');
                }
                this.showSalesSection('.products-accordion');
                this.onGetProductsCat(data, true, vendorId);
                this.showAccordionItem('.sales-accordion-item.products');
            }.bind(this));
        },

        showSalesProducts: function (e) { // fire build products list after category select
            var $catItem = $(e.target).closest('.products-cat');
            $catItem.addClass('selected').siblings().removeClass('selected');
            var catId = $catItem.data('id');
            var catName = $catItem.find('.title').text();
            this.buildSalesProducts(catId, catName);
        },

        onGetProducts: function(data, empty) {
            var $container = this.$('.products ul');
            if(empty) {
                $container.empty();
                this.setPageNumber(0, 'products');
            }
            this.setPageNumber(data, 'products');
            var itemTemplate = _.template(SalesItemTemplate);
            _.each(data, function (elem) {
                $container.append(itemTemplate(_.extend({itemClass: 'products-item'}, elem)));
            });
        },

        buildSalesProducts: function (catId, catName) { // build products list
            getProducts(this.manufacturer, catId).done(function (data) {
                if (!this.$el) return false;
                this.lastSelected.set('catId', catId);
                this.lastSelected.set('itemsArray', data);
                this.$('.products .sales-items-filter').val('');
                this.$('.sales-accordion-item.products .header').append(
                    _.template(
                        ' > <span class="products-cat" data-id="<%= catId %>"><%= catName %></span>'
                    )({
                        catId: catId,
                        catName: catName
                    })
                );
                this.$('.products .sales-accordion-body').data('content', 'products');
                this.showSalesSection('.products-accordion');
                this.onGetProducts(data, true);
                this.showAccordionItem('.sales-accordion-item.products');
            }.bind(this));
        },

        filterSalesProducts: function (e) {
            var query = $(e.target).val();
            if(query === '') {
                this.buildSalesProductsCat();
            } else {
                filterProducts(query).done(function (data) {
                    if (!this.$el) return false;
                    this.lastSelected.set('itemsArray', data);
                    this.$('.products .sales-accordion-body').data('content', 'products');
                    this.onGetProducts(data, true);
                    this.refreshScrolls();
                }.bind(this));
            }
        }
    });
});
