/**
 * Created by petermomot on 5/5/15.
 */
define([
    'jquery',
    'views/smart-receipt/sales-modal',
    'text!../../../templates/smart-receipt/sales-category.html',
    'text!../../../templates/smart-receipt/sales-item.html',
    'text!../../../templates/smart-receipt/sale-services-products.html',
    'bridge'
], function ($, SalesModalView,
             SalesCategoryTemplate, SalesItemTemplate, InnerTemplate) {
    'use strict';

    return SalesModalView.extend({
        innerTemplate: _.template(InnerTemplate),

        initialize: function (options) {
            options.title = 'Services';
            SalesModalView.prototype.initialize.call(this, options);
            this.events = _.extend({}, SalesModalView.prototype.events, this.events);
        },

        afterInit: function (o) {
            o = _.extend(o, {
                accordionClass: 'services-accordion',
                accordionItemClass: 'services'
            });
            this.model = new Backbone.Model(o);
        },

        afterRender: function () {
            this.$el.addClass('without-buttons-landscape');
            this.$('.sales-accordion-item.services .sales-accordion-body').html(this.innerTemplate({className: 'services'}));
            SalesModalView.prototype.afterRender.call(this);
            this.buildSalesServicesCategory();
            return this;
        },

        events: {
            'tap .services-category': 'showSalesServices',
            'touchend .sales-accordion-item.services .header': 'buildSalesServicesCategory',
            'touchend .sales-accordion-item.services .header .current-category': 'showServicesAccordionItem',
            'input .sales-services-filter': 'filterSalesServices'
        },

        showServicesAccordionItem: function(e) {
            e.stopPropagation();
            this.showAccordionItem('.sales-accordion-item.services');
        },

        onGetServicesTypes: function(data, empty) {
            var $container = this.$('.services ul');
            if(empty) {
                $container.empty();
                this.setPageNumber(0, 'services cat');
            }
            this.setPageNumber(data, 'services cat');
            this.model.set('services', data);
            var itemTemplate = _.template(SalesCategoryTemplate);
            _.each(data, function (elem) {
                $container.append(itemTemplate(_.extend({
                    itemClass: 'services-category',
                    imageId: ''
                }, elem)));
            });
        },

        buildSalesServicesCategory: function () {
            getServicesTypes().done(function (data) {
                if (!this.$el) return false;
                this.lastSelected.unset('itemId');
                this.lastSelected.unset('catId');
                this.$('.sales-accordion-item.services .header').html('Services');
                this.$('.services .sales-items-filter').val('');
                this.showSalesSection('.services-accordion');
                this.$('.services .sales-accordion-body').data('content', 'services cat');
                this.onGetServicesTypes(data, true);
                this.showAccordionItem('.sales-accordion-item.services');
            }.bind(this));
        },

        showSalesServices: function (e) {
            var $salesCategory = $(e.target).closest('.sales-category');
            this.buildSalesServices($salesCategory.data('id'));
            $salesCategory.addClass('selected').siblings().removeClass('selected');
        },

        onGetServices: function(data, empty) {
            var $container = this.$('.services ul');
            if(empty) {
                $container.empty();
                this.setPageNumber(0, 'services');
            }
            this.setPageNumber(data, 'services');
            this.lastSelected.set('itemsArray', data);
            var itemTemplate = _.template(SalesItemTemplate);
            _.each(data, function (elem) {
                $container.append(itemTemplate(_.extend({itemClass: 'services-item'}, elem)));
            });
        },

        buildSalesServices: function (id) {
            var category = _.findWhere(this.model.get('services'), {'id': id});
            getServices(id, this.lastSelected.get('employee') && this.lastSelected.get('employee').id).done(function (data) {
                if (!this.$el) return false;
                this.lastSelected.set('catId', id);
                this.$('.services .sales-items-filter').val('');
                this.$('.sales-accordion-item.services .header').html(
                    _.template(
                        'Services > <span class="current-category"><%= displayName %></span>'
                    )(_.findWhere(this.model.get('services'), {id: id}))
                );
                this.showSalesSection('.services-accordion');
                this.$('.services .sales-accordion-body').data('content', 'services');
                category.services = data;
                this.onGetServices(data, true);
                this.showAccordionItem('.sales-accordion-item.services');
            }.bind(this));
        },

        filterSalesServices: function (e) {
            var query = $(e.target).val();
            if(query === '') {
                this.buildSalesServicesCategory();
            } else {
                filterServices(query, this.lastSelected.get('employee') && this.lastSelected.get('employee').id).done(function (data) {
                    this.$('.services .sales-accordion-body').data('content', 'services');
                    this.onGetServices(data, true);
                    this.refreshScrolls();
                }.bind(this));
            }
        }

    });
});
