﻿define([
    'jquery',
    'signature',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/signature.html',
    'bridge'
], function ($, Signature, ModalView, YesNoModal, SignatureTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#signature-modal .content',
        template: _.template(SignatureTemplate),
        scrollWrap: '#signature-modal-scroll-wrap',
        scrollbars: false,
        yesNoModal: undefined,
        options: undefined,
        signaturePad: undefined,
        penColor: "rgb(0, 0, 0)",
        backgroundColor: "rgb(255, 255, 255)",
        events: {
            'tap #signature-clear'  : 'clearSignature',
            'tap #signature-save'   : 'saveSignature',
            'tap #signature-close'  : 'cancelSignature'
        },

        initialize: function (options) {
            this.model = new Backbone.Model(options);
            this.events = _.extend({}, ModalView.prototype.events, this.events);
            if(this.options === undefined)
                this.options = options;
        },

        render: function() {
            ModalView.prototype.render.call(this);

            $(':focus').blur();

            this.addPlugins();

            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });

            return this;
        },

        addPlugins: function() {
            var canvas = this.$('#signature-canvas')[0];
            this.signaturePad = new Signature(canvas, {
                dotSize: 1,
                minWidth: 1,
                maxWidth: 1,
                penColor: this.penColor,
                backgroundColor: this.backgroundColor
            });
            this.signaturePad && this.signaturePad.clear();
            this.resizeCanvas(canvas);
            //$(window).on('resize orientationchange', function(){
            //    this.resizeCanvas(canvas);
            //}.bind(this));
        },

        resizeCanvas: function(canvas) {
            var ratio = Math.max(window.devicePixelRatio || 1, 1);
            canvas.width = canvas.offsetWidth * ratio;
            canvas.height = canvas.offsetHeight * ratio;
            canvas.getContext("2d").scale(ratio, ratio);
        },

        saveSignature: function() {
            if (this.signaturePad.isEmpty()) {
                this.yesNoModal = new YesNoModal({
                    question: "Please enter your signature",
                    className: '',
                    positiveText: 'OK',
                    negativeText: '',
                    noCallback: true
                }).render();
                return false;
            } else {
                var points = this.signaturePad.getExternalPoints();
                if(!this.isContainLine(points)) {
                    this.yesNoModal= new YesNoModal({
                        question: "Payment signature must contains at least one line!",
                        className: '',
                        positiveText: 'OK',
                        negativeText: '',
                        noCallback: true
                    }).render();
                    return false;
                } else {
                    this.trigger('changeAsTipWithSignature', points);
                    this.cancelSignature();
                }
            }
        },

        isContainLine: function(points) {
            var withLine = false;

            _.each(points, function(elem) {
                if(elem.length > 0) withLine = true;
            });

            return withLine;
        },

        clearSignature: function() {
            this.signaturePad && this.signaturePad.clear();
        },

        cancelSignature: function() {
            this.resetAll();
            this.closeView();
        },

        resetAll: function() {
            this.signaturePad && this.signaturePad.off();
            this.signaturePad = undefined;
            this.yesNoModal = undefined;
        }
    });
});
