define([
    'jquery',
    // data
    'data/clients',
    'data/discounts',
    'data/checkouts',
    'data/consts',
    'data/permissions',
    'data/hold-transactions',
    // views
    'common/base-view',
    'views/yes-no-modal',
    'views/smart-receipt/add-client-modal',
    'views/smart-receipt/client-from-appointment-modal',
    'views/smart-receipt/client-notes-modal',
    'views/smart-receipt/change-tip-modal',
    'views/smart-receipt/tip-modal',
    'views/smart-receipt/adjustments-modal',
    'views/smart-receipt/predefined-discount-modal',
    'views/smart-receipt/print-modal',
    'views/smart-receipt/signature-modal',
    'views/smart-receipt/edit-sale-modal',
    'views/smart-receipt/discount-sale-modal',
    'views/smart-receipt/checkout-modal',
    'views/smart-receipt/sub-menu-modal',
    'views/smart-receipt/on-hold-list-modal',
    'views/smart-receipt/refund-search',
    'views/smart-receipt/refund-checkout-modal',
    'views/smart-receipt/transactions-filter-modal',
    'views/client-modal',
    // SalesModals
    'views/smart-receipt/sales-services-modal',
    'views/smart-receipt/sales-products-modal',
    'views/smart-receipt/sales-gifts-modal',
    'views/smart-receipt/sales-accounts-modal',
    // templates
    'text!../../../templates/smart-receipt/smart-receipt.html',
    'text!../../../templates/smart-receipt/add-client-modal.html',
    'text!../../../templates/smart-receipt/client.html',
    'text!../../../templates/smart-receipt/sale-line.html',
    'text!../../../templates/smart-receipt/sales-category.html',
    'text!../../../templates/smart-receipt/sales-item.html',
    'text!../../../templates/smart-receipt/user-search.html',
    'text!../../../templates/smart-receipt/checkout-line.html',
    'text!../../../templates/smart-receipt/sub-menu-buttons.html',
    'text!../../../templates/smart-receipt/sub-menu-types.html',
    'text!../../../templates/smart-receipt/sub-menu-types.html',
    'bridge'
], function ($,
             // data
             clientsCollection, discountsCollection, checkoutsCollection, Consts, PermModel, OnHoldCollection,
             // views
             BaseView, YesNoModal, AddClientModalView, ClientFromAppointmentModal, ClientNotesModalView,
             ChangeAsTipModal, AddTipModal, AdjustmentsModalView, PredefinedDiscountModalView, PrintView, SignatureView,
             EditSaleModal, DiscountSaleModal, CheckoutModalView, SubMenuView, OnHoldModalView,
             OnRefundModalView, RefundCheckoutModalView, TransactionsModal, ClientModal,
             //SalesModalView,
             SalesServicesModalView, SalesProductsModalView,
             SalesGiftsModalView, SalesAccountsModalView,
             // templates
             smartReceiptTemplate, addClientModalTemplate, ClientTemplate,
             SaleLineTemplate, SalesCategoryTemplate, SalesItemTemplate,
             UserSearchTemplate, CheckoutLineTemplate,
             SubMenuButtonsTemplate, subMenuTypesTemplate) {
    'use strict';

    var LastSelected = Backbone.Model.extend({
        defaults: {
            catId: undefined,
            venId: undefined,
            clientId: undefined,
            itemId: undefined,
            itemsArray: undefined,
            giftGard: undefined
        }
    });

    return BaseView.extend({
        el: '.smartReceipt',
        template: _.template(smartReceiptTemplate),
        clientTemplate: _.template(ClientTemplate),
        saleLineTemplate: _.template(SaleLineTemplate),
        checkoutLineTemplate: _.template(CheckoutLineTemplate),
        subMenuTypesTemplate: _.template(subMenuTypesTemplate),
        receiptSubMenuTemplate: _.template(SubMenuButtonsTemplate),
        addClientModal: undefined,
        salesScreenScroll: [],
        tabletSubMenuType: undefined,
        tabletSubMenuScroll: undefined,
        lastSelected: undefined,
        clientNotesModal: undefined,
        editSaleModal: undefined,
        prevPayingClientId: undefined,
        discountModelId: undefined,
        checkoutScroll: undefined,
        changeAsATipModal: undefined,
        refundCheckoutModal: undefined,
        addTipModal: undefined,
        onHoldModal: undefined,
        onRefundModal: undefined,
        onPrintModal: undefined,
        onSignatureModal: undefined,
        transactionsModal: undefined,
        refundedItems: [],
        paymentTypes: [],
        customTips: [],
        tipModalData: {
            tipAmount : 0,
            tipType: 'percentage'
        },
        queriedFinalize: false,
        invisibleClientId: "invisibleClient",
        holdPayingClientId: '',
        calculationInProgress: false,

        events: {
            'tap .sales-accordion .sales-accordion-item .header': 'salesAccordion',
            'tap .client .client-name': 'tapClientName',
            'tap .sale-line': 'tapSaleLine',
            'tap .button-del': 'saleDelete',
            'tap .checkout-line': 'tapCheckoutLine',
            'tap .button-checkout-del': 'checkoutLineDelete',
            'touchend .button.hold': 'holdSale',

            // navigation events
            'touchend .receipt-sub-menu .button': 'navigateThroughReceiptSubMenu',
            'tap .sub-menu-nav-item': 'navigateThroughReceiptSubMenu',
            'touchend #add-new-client': 'navigateToClientModal',
            'touchend #today-appointments': 'navigateToTodayAppointmentsModal',
            'tap .add-sale-line-button': 'navigateToReceiptSubMenu',
            'tap .button-edit': 'navigateToSaleEdit',
            'tap .button-discount': 'navigateToSaleDiscount',
            'tap .sale-line .notification': 'navigateToSaleNotification',
            'tap .appoint-note-alert, .alert': 'navigateToClientNoteSummary',
            'touchend .button.checkout': 'navigateToCheckout',
            'touchend .button.refund': 'navigateToRefundCheckout',
            'tap .button-checkout-edit': 'navigateToCheckoutLineEdit',
            'tap #add-checkout-line-button': 'navigateToCheckoutSubMenu',
            'touchend .button.no-sale': 'navigateToNoSales',
            'touchend #edit-sale': 'refresh', // navigate back from checkout to edit sale lines
            'touchend #finalize-sale': 'beforeNavigateToChangeModal',
            'touchend #add-tip, #checkout-tip': 'navigateToTips',

            'touchend .pseudo-sub-menu-item': 'toggleSubmenuDropDown',
            'touchend .receipt-submenu-drop-down-inner .sub-menu-item': 'submenuItemSelected'
        },

        initialize: function () {
            this.clients = clientsCollection;
            this.discounts = discountsCollection;
            this.checkouts = checkoutsCollection;
            this.clientHoldTransactions = new OnHoldCollection();
            this.lastSelected = new LastSelected();
            this.tips = [];
            var $html = $('html');
            this.onPhone = $html.hasClass('mobile');
            this.onLandscape = $html.hasClass('landscape');
            this.onRotateHandler = this.onRotate.bind(this);

            this.addListeners();

            this.events = _.extend({}, BaseView.prototype.events, this.events);
        },

        addListeners: function () {
            // this.collection events
            this.listenTo(this.collection, 'add', this.saleLineAdded);
            this.listenTo(this.collection, 'change:client', this.saleLineAdded);
            this.listenTo(this.collection, 'change:employee', this.saleLineChangeEmployee);
            this.listenTo(this.collection, 'change:quantity', this.saleLineChangeQuantity);
            this.listenTo(this.collection, 'change:price', this.saleLineChangePrice);
            this.listenTo(this.collection, 'change:discounts', this.saleLineChangeDiscounts);
            this.listenTo(this.collection, 'change:alerts', this.saleLineChangeAlerts);

            // commented because Backbone triggered change callback not every time in this collection
            // this.listenTo(this.collection, 'change:item', this.saleLineChangeItem);

            this.listenTo(this.collection, 'remove', this.saleLineRemoved);
            // this.clients events
            this.listenTo(this.clients, 'add', this.clientAdded);
            this.listenTo(this.clients, 'change:alertCount', this.clientAlertsChanged);
            //this.listenTo(this.clients, 'remove', this.clientRemoved);
            // commented because Backbone triggered remove callback not every time in this collection
            this.listenTo(this.clients, 'reset', this.clientsReset);
            // this.model events
            this.listenTo(this.model, 'change:totalTax1', this.changeTax1);
            this.listenTo(this.model, 'change:totalTax2', this.changeTax2);
            this.listenTo(this.model, 'change:totalTax3', this.changeTax3);
            this.listenTo(this.model, 'change:totalPrice', this.changeTotalPrice);
            this.listenTo(this.model, 'change:totalTips', this.changeTotalTips);
            this.listenTo(this.model, 'change:totalPayments', this.changeTotalPayments);
            this.listenTo(this.model, 'change:payingClientGlobalId', this.changePayingClient);
            this.listenTo(this.model, 'change:totalRefundTax1', this.changeRefundTax1);
            this.listenTo(this.model, 'change:totalRefundTax2', this.changeRefundTax2);
            this.listenTo(this.model, 'change:totalRefundTax3', this.changeRefundTax3);
            // this.checkouts events
            this.listenTo(this.checkouts, 'add', this.checkoutLineAdded);
            this.listenTo(this.checkouts, 'change:amount', this.checkoutLineChangeAmount);
            this.listenTo(this.checkouts, 'remove', this.checkoutLineRemoved);
            this.listenTo(this.checkouts, 'reset', this.checkoutsReset);
            // this.clientHoldTransactions events
            this.listenTo(this.clientHoldTransactions, 'readyToReturn', this.holdTransactionExtract);
        },

        // this.collection events
        saleLineAdded: function (line, collection, options) { // this.collection, 'add'; this.collection, 'change:client'
            line.set('lineNumber', this.collection.length);
            if(this.salesModal) {
                this.salesModal.saleLinesCount = this.collection.length;
            }
            this.scroll && this.scroll.refresh();
            if (line.get('invisibleLine')) return false;
            if (!line.get('id')) {
                var id = line.get('client')['id'] ? line.get('client')['id'] : this.invisibleClientId;
                line.set('id', line.get('client')['id'] + '_' + new Date().getTime());
                this.getSaleLineAlerts(line);
                if(_.isUndefined(options[0])) {
                    this.reCalcSale();
                } else {
                    options[0] && this.reCalcSale();
                }
            }
            var l = line.toJSON();
            if (this.$('#' + line.get('id')).length == 0) {
                this.$('.sale-lines-body[data-client-id="' + (l.client['id'] ? l.client['id'] : this.invisibleClientId) + '"]')
                    .append(this.saleLineTemplate(_.extend({
                        alerts: undefined,
                        Consts: Consts
                    }, l)))
                    .closest('.client').removeClass('invisible');
                if(line.get('isRefund') === true) {
                    this.refundedItems.push(line.get('refundSaleLineId'));
                    this.refundedItems = _.uniq(this.refundedItems);
            }
                this.checkRefundTaxesVisibility();
            }
            this.$('.button.no-sale, .button.hold').removeClass('inactive');
        },

        saleLineChangeEmployee: function (line) { // this.collection, 'change:employee'
            this.$('#' + line.id).find('.sale-line-employee').html(line.get('employee')['code']);
        },

        saleLineChangeQuantity: function (line) { // this.collection, 'change:quantity'
            this.$('#' + line.id).find('.sale-line-quantity').html(line.get('quantity'));
        },

        saleLineChangePrice: function (line) { // this.collection, 'change:price'
            this.$('#' + line.id).find('.sale-line-price').html($.global.format(line.get('price'), 'c'));
        },

        saleLineChangeDiscounts: function (line) { // this.collection, 'change:discounts'
            var discountsIds = _.pluck(line.get('discounts'), 'discountId'),
                $line = this.$('#' + line.id);
            if (discountsIds.length) {
                var discounts = _.map(this.discounts.models, function (d) {
                    if (_.indexOf(discountsIds, d.get('id')) != -1) {
                        return d.toJSON();
                    }
                }.bind(this));
                discounts = _.reject(discounts, function (d) {
                    return d === undefined;
                });
                var displayNames = _.pluck(discounts, 'displayName');
                $line.find('.sale-line-discounts').html(displayNames.join(', '));
                $line.find('.discount-number').addClass('visible').html(discountsIds.length);
            } else {
                $line.find('.sale-line-discounts').html('');
                $line.find('.discount-number').removeClass('visible').html('');
            }
        },

        saleLineChangeAlerts: function (line) { // this.collection, 'change:alerts'
            var $alerts = this.$('#' + line.id).find('.notification'),
                alerts = line.get('alerts');
            if (alerts.length) {
                $alerts.removeClass('hidden').data('alert', alerts[0]['alertMessage']).html(alerts.length);
            } else {
                $alerts.addClass('hidden').data('alert', '').html('');
            }
        },

        saleLineChangeItem: function (line) { // this.collection, 'change:item'
            this.getSaleLineAlerts(line);
            var $line = this.$('#'+line.get('id')),
                item = line.get('item');

            $line.find('.item-display-name').html(item['displayName'] + (item.type == Consts.SaleLineTypes.GIFTCARD ? ' ' + $.global.format(item['value'], 'c') + ' ' + item['number'].toUpperCase():' '));

            $line.find('.item-shortcut').html(item['shortcut']);
            if (item['client']) {
                $line.find('.item-purchased-for').html(
                    'Purchased for: ' + item['client']['lastName'] + ', ' + item['client']['firstName']
                );
            }
        },

        saleLineRemoved: function (line) { // this.collection, 'remove'
            var $line = this.$('#' + line.id);
            if(this.salesModal) {
                this.salesModal.saleLinesCount = this.collection.length;
            }
            this.scroll && this.scroll.refresh();
            if (!line.get('client')['id'] || line.get('client')['id'] == this.invisibleClientId) {
                var $client = $line.closest('.client');
                if ($client.find('.sale-lines-body').children().length == 1) {
                    $client.addClass('invisible');
                }
            }
            this.checkRefundTaxesVisibility();
            $line.remove();
            this.collection.updateLineNumbers();
            if (this.clients.models.length == 1 && this.clients.get(this.invisibleClientId) && this.collection.length == 0) {
                this.$('.button.no-sale, .button.hold').addClass('inactive');
            }

            if (this.collection.models.length == 0) {
                this.tips = [];
                this.customTips = [];
                this.refundedItems = [];
                if (this.onRefundModal) {
                    this.onRefundModal.updateRefundedItems([]);
                }
            } else {
                var employeeId = line.get('employee')['id'],
                    needToClearTip = true;

                var refundSaleID = line.get('refundSaleLineId');
                this.refundedItems = _.without(this.refundedItems, refundSaleID);
                if (this.onRefundModal) {
                    this.onRefundModal.updateRefundedItems(this.refundedItems);
                }

                _.each(this.collection.models, function(model){
                    if( model.get('employee')['id'] == employeeId ){
                        needToClearTip = false;
                    }
                });

                if( needToClearTip ){
                    var tipToClear = _.find(this.tips, {id : employeeId});
                    if( tipToClear !== undefined ){
                        this.tips = _.reject(this.tips, {id : employeeId});
                        this.model.set('totalTips', this.model.get('totalTips') - tipToClear.amount);
                    }
                }
            }
            this.reCalcSale();
        },

        // this.clients events
        clientAdded: function (client) { // this.clients, 'add'
            var clientId = client.get('id'),
                clientHtml = this.clientTemplate(_.extend(client.toJSON(), {
                    PermModel: PermModel,
                    additionalClass: clientId === this.invisibleClientId ? 'invisible' : ''
                }));

            if(this.salesModal) {
                this.salesModal.clients = this.clients.toJSON();
            }
            if (this.model.get('payingClientGlobalId') === clientId) {
                this.$('.clients-list').prepend(clientHtml);
            } else {
                this.$('.clients-list').append(clientHtml);
            }
            if (client.get('isRestricted') && window.SETTINGS.warnClientIsRestrictedAlert) {
                new YesNoModal({
                    question: 'Client Restricted',
                    positiveText : 'Ok',
                    className : 'modal-alert',
                    noCallback: true,
                    restricted: true,
                    negativeText: ''
                }).render();
            }
            if (
                (clientId !== window.SETTINGS['walkInFemale'].id)
                && (clientId !== window.SETTINGS['walkInMale'].id)
                && (clientId !== this.invisibleClientId)
            ) {
                getClientAlerts(clientId).done(function (r) {
                    if (r.status === undefined) {
                        client.set("alerts", r);
                        client.set('alertCount', r.length);
                    }
                });
            }
            if (this.clients.length > 1 || (this.clients.length == 1 && !this.clients.get(this.invisibleClientId))) {
                this.$('.button.no-sale, .button.hold').removeClass('inactive');
            }
        },

        clientAlertsChanged: function (client) { // this.clients, 'change:alertCount'
            var $clientHeader = this.$('.client[data-client-id=' + client.id + ']').find('.header'),
                $alerts = $clientHeader.find('.notification-alert'),
                alertCount = client.get('alertCount'),
                balance = client.get('balance');
            if(this.salesModal) {
                this.salesModal.clients = this.clients.toJSON();
            }
            if (alertCount) {
                if ($alerts.length) {
                    $alerts.html(alertCount);
                } else {
                    $clientHeader.append('<div class="button alert notification-alert">' + alertCount + '</div>');
                    $alerts = $clientHeader.find('.notification-alert');
                }
                if (balance > 0) {
                    $alerts.toggleClass("balance-alert-pos notification-alert");
                } else if (balance < 0) {
                    $alerts.toggleClass("balance-alert-neg notification-alert");
                }
            } else {
                $alerts.remove();
            }
        },

        clientRemoved: function (client, invisibleClientChanged) { // this.clients, 'remove'
            this.$('.client[data-client-id=' + client.id + ']').remove();
            this.scroll.scrollTo(0, 0);
            if (this.clients.models.length === 0) {
                this.$('.button.no-sale, .button.hold').addClass('inactive');
            }
            if (client.id === this.invisibleClientId && !invisibleClientChanged) {
                var invisibleClientLines = _.pluck(_.filter(this.collection.toJSON(), function (line) {
                    return line.id.indexOf(this.invisibleClientId) != -1;
                }.bind(this)), 'id');
                this.collection.remove(invisibleClientLines);
            }
            setTimeout(function () {
                if (this.clients.models.length === 0) {
                    this.addInvisibleClient();
                }
                if (this.clients.models.length == 1 && this.clients.get(this.invisibleClientId) && this.collection.length == 0) {
                    this.$('.button.no-sale, .button.hold').addClass('inactive');
                }
            }.bind(this), 10);
            if (this.lastSelected.get('clientId') == client.id) this.changeLastSelectedClient(undefined, true);
        },

        clientsReset: function () { // this.clients, 'reset'
            if(this.salesModal) {
                this.salesModal.clients = this.clients.toJSON();
            }
            this.$('.sales-accordion.client').remove();
            this.addInvisibleClient();
        },

        // this.model events
        changeTax1: function () { // this.model, 'change:totalTax1'
            this.$('.tax-first').html($.global.format(this.model.get('totalTax1'), 'c'));
        },

        changeTax2: function () { // this.model, 'change:totalTax2'
            this.$('.tax-second').html($.global.format(this.model.get('totalTax2'), 'c'));
        },

        changeTax3: function () { // this.model, 'change:totalTax3'
            this.$('.tax-third').html($.global.format(this.model.get('totalTax3'), 'c'));
        },

        changeTotalPrice: function () { // this.model, 'change:totalPrice'
            this.$('.total-price').html($.global.format(this.model.get('totalPrice'), 'c'));
        },

        changeTotalTips: function () { // this.model, 'change:totalTips'
            var tips = this.model.get('totalTips');
            if (tips) {
                this.$('.total-tips').html($.global.format(tips, 'c')).parent().show();
            } else {
                this.$('.row.tips').hide();
            }
        },

        changeTotalPayments: function () { // this.model, 'change:totalPayments'
            this.$('.total-payments').empty().text($.global.format(this.model.get('totalPayments'), 'c'));
            this.changeCheckoutPayments();
        },

        changePayingClient: function (model) {
            var clientId = model.get('payingClientGlobalId'),
                clientObject = this.clients.get(clientId),
                isRefunded = false;
            if (clientObject) isRefunded = clientObject.toJSON().isRefunded;

            if (!clientId || clientId == this.invisibleClientId
                || clientId == this.holdPayingClientId || isRefunded || this.stopHoldRequest) {
                this.stopHoldRequest = undefined;
                return false;
            }
            //this.checkIfHold(clientId);
        },

        checkIfHold: function(clientId) {
            this.holdPayingClientId = '';
            this.clientHoldTransactions.reset();
            getSalesOnHold(false, clientId).done(function (r) {
                if (r.status === undefined) {
                    if (r.length != 0) {
                        this.clientHoldTransactions.add(r);
                        this.yesNoModal = new YesNoModal({
                            question: 'This client has a transaction on hold. Would you like to ring up the transaction?',
                            positiveText: 'Yes',
                            positiveCallback: function () {
                                this.yesNoModal.closeView();
                                this.beforeGetFirstHoldTransaction();
                            }.bind(this),
                            negativeText: 'No',
                            negativeCallback: function () {
                                this.yesNoModal.closeView();
                                this.clientHoldTransactions.reset();
                            }.bind(this)
                        }).render();
                    }
                }
            }.bind(this));
        },

        changeRefundTax1: function () { // this.model, 'change:totalRefundTax1'
            this.$('.refund-tax-first').html($.global.format(this.model.get('totalRefundTax1'), 'c'));
        },

        changeRefundTax2: function () { // this.model, 'change:totalRefundTax2'
            this.$('.refund-tax-second').html($.global.format(this.model.get('totalRefundTax2'), 'c'));
        },

        changeRefundTax3: function () { // this.model, 'change:totalRefundTax3'
            this.$('.refund-tax-third').html($.global.format(this.model.get('totalRefundTax3'), 'c'));
        },

        // this.checkouts events
        checkoutLineAdded: function (line) { // this.checkouts, 'add'
            if (!line.get('id')) {
                line.set('id', line.get('paymentType')['id'] + '_' + new Date().getTime());
            }

                var l = line.toJSON();
                var $line = $(this.checkoutLineTemplate(l));
                if (l['paymentType']['paymentTypeEnum'] == 1448) {
                    var cardNumber = String(l['giftCard']['number']);

                    var reg = new RegExp("^GC[0-9]+");
                    if(reg.test(cardNumber)){
                        if (cardNumber[cardNumber.length - 4] === '0')
                            cardNumber = cardNumber.substr(cardNumber.length - 4);
                        else
                            cardNumber = cardNumber.substr(cardNumber.length - 5);
                    }
                    $line.find('.checkout-line-gift-card-num').html(' ' + cardNumber);
                }
            if (this.$('#' + l.id).length == 0) this.$('.checkout-lines').append($line);
                this.updatePaymentInfoHeight();
        },

        checkoutLineChangeAmount: function (line) { // this.checkouts, 'change:amount'
            this.$('#' + line.id).data('amount', line.get('amount')).find('.right-col').html($.global.format(line.get('amount'), 'c'));
        },

        checkoutLineRemoved: function (line) { // this.checkouts, 'remove'
            this.$('#' + line.id).remove();
            this.checkouts.updateLineNumbers();


            // Update remaining
            if (this.checkoutModal)
                if (this.checkoutModal.giftCards)
                    if (this.checkoutModal.giftCards.length)
                        this.checkoutModal.giftCards.update([line.toJSON()]);

            this.changeCheckoutPayments();

            this.updatePaymentInfoHeight();
        },

        checkoutsReset: function () { // this.checkouts, 'reset'
            this.$('.checkout-line:not(#checkout-balance)').remove();
            this.updatePaymentInfoHeight();
        },

        // view events
        render: function () {
            getCheckoutPaymentTypes().done(function(data){
                this.paymentTypes = data;
            }.bind(this));

            this.receiptSubMenu = PermModel.getVisibleReceiptSubMenu();
            this.saleTypesMenu = PermModel.getVisibleSaleTypes();
            this.paymentTypesMenu = PermModel.getVisiblePaymentTypes();
            // this.refundTypesMenu = PermModel.getVisibleRefundTypes();

            this.$el.empty().html(this.template(_.extend(this.model.toJSON(), {
                PermModel: PermModel,
                menu: this.receiptSubMenu,
                buttonsTemplate: this.receiptSubMenuTemplate
            })));
            this.model.set('payingClientGlobalId', this.invisibleClientId);
            if (!this.onPhone) {
                this.$('.sub-menu-item:first').trigger('touchend', true);
                this.renderTabletLandscapeRight();
                if (this.onLandscape) {
                    this.navigateToTabletRight();
                }
            }
            this.$('.button.checkout').addClass('inactive');

            this.scroll = new IScroll('.smart-receipt-scroll-wrapper', {
                tap: true,
                disableMouse: true,
                scrollbars: 'custom'
            });
            this.scroll.on('beforeScrollStart', function () {
                $('.smart-receipt-scroll-wrapper').css('bottom', $('.smart-receipt-wrapper .general').outerHeight());
                this.refresh();
            });

            this.addInvisibleClient();

            return this;
        },

        show: function () {
            $(document).on('touchmove', preventTouchForIScroll);
            BaseView.prototype.show.call(this);
            $(window).on("orientationHandlerCallback", this.onRotateHandler);
            this.refresh();
        },

        hide: function () {
            $(document).off('touchmove', preventTouchForIScroll);
            BaseView.prototype.hide.call(this);
            $(window).off("orientationHandlerCallback", this.onRotateHandler);
            if (!this.onPhone) {
                this.closeCheckoutModal();
                this.onHoldModal && this.onHoldModal.trigger('holdEvent', { type: 'cancel' });
                this.onRefundModal && this.onRefundModal.trigger('refundEvent', { type: 'cancel' });
                this.predefinedDiscountsModal && this.predefinedDiscountsModal.trigger('closeEvent', { type: 'cancel' });
                this.manualDiscounts && this.manualDiscounts.trigger('applyEvent', { type: 'cancel' });
                this.adjustmentModal && this.adjustmentModal.trigger('adjustmentEvent', { type: 'cancel' });
                this.transactionsModal && this.transactionsModal.trigger('transactionsEvent', { type: 'cancel' }); 
            }
        },

        refresh: function () {
            this.hideCheckoutLines();
            this.$('.drawer-name').html(JSON.parse(localStorage.getItem('workstation')).drawerDisplayName);
            if (!this.onPhone) {
                this.renderReceiptSubMenu('sales', this.noMenuTap);
                this.closeCheckoutModal();
                this.$('.sub-menu-nav-item.selected').removeClass('selected');
                this.hideTabletLandscapeRight(true);
            }

            getCheckoutPaymentTypes().done(function (r) {
                if (r.status === undefined) {
                    this.checkoutPaymentTypes = r;
                }
            }.bind(this));
            this.detectIfCanNotCheckout();
            this.checkRefundTaxesVisibility();
            this.noMenuTap = false;
            this.scroll && this.scroll.refresh();
        },

        salesAccordion: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.hideCheckoutLines();
            var $target = $(e.target);
            this.changeLastSelectedClient($target.closest('.client').data('client-id'));
            if ($target.is('.notification-alert', '.appoint-note-alert', '.client-name')) {
                return;
            }
            var $parent = $target.closest('.sales-accordion-item');
            if ($parent.hasClass('opened') || $parent.hasClass('active')) {
                $parent.removeClass('opened active').find('.sales-accordion-body').slideUp(function () {
                    _.each(this.salesScreenScroll, function (elem) {
                        elem.refresh();
                    });
                });
            } else {
                $parent.siblings('.sales-accordion-item').removeClass('opened active').find('.sales-accordion-body').slideUp();
                $parent.addClass('active').find('.sales-accordion-body').slideDown(function () {
                    _.each(this.salesScreenScroll, function (elem) {
                        elem.refresh();
                    });
                });
            }
        },

        tapClientName: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.changeLastSelectedClient($(e.target).data('client-id'));
            this.navigateToClientModal(e);
        },

        tapSaleLine: function (e) {
            if ($(e.target).hasClass('button')) {
                return false;
            }
            this.hideCheckoutLines();
            var el = $(e.target).closest('.sale-line');
            this.$('.sale-line.tapped').not(el).removeClass('tapped');
            el.toggleClass('tapped');
            if (!this.onPhone) {
                this.hideTabletLandscapeRight(true);
            }
        },

        saleDelete: function (e) {
            this.hideCheckoutLines();
            this.collection.remove(this.collection.get($(e.target).data('id')));
            if (!this.onPhone) {
                this.hideTabletLandscapeRight(true);
            }
        },

        tapCheckoutLine: function (e) {
            if ($(e.target).hasClass('button') || $(e.target).attr('id') == 'checkout-balance') {
                return false;
            }
            var el = $(e.target).closest('.checkout-line');
            if (el.attr('id') == 'checkout-balance') {
                return false;
            }
            //this.$('.checkout-line.tapped').not(el).removeClass('tapped');
            el.toggleClass('tapped');
            $(document).on('touchstart.checkout-tap', function(e) {
                if(!$(e.target).closest('.checkout-line').is(el)) {
                    el.removeClass('tapped');
                    $(document).off('touchstart.checkout-tap');
                }
            });
            if (!this.onPhone) {
                this.hideTabletLandscapeRight(true);
            }
        },

        checkoutLineDelete: function (e) {
            var id = $(e.target).data('id');
            if (!id) return;

            this.checkouts.beforeRemove(id);

            if (!this.onPhone) {
                this.hideTabletLandscapeRight(true);
                if (this.checkoutModal && this.checkoutModal.model.get('lineNumber')) {
                    this.closeCheckoutModal();
                    this.$('.sub-menu-nav-item.selected').removeClass('selected').trigger('tap', true);
                }
            }
        },

        holdSale: function (e) {
            if ($(e.target).hasClass('inactive')) return false;
            if(!this.holdProcessing) {
                this.holdProcessing = true;
                PermModel.navigateToPermissionsModal(e, function () {
                    if (this.collection.length == 0) {
                        new YesNoModal({
                            question: "Can't hold a sale without any sale lines.",
                            positiveText: 'OK',
                            noCallback: true,
                            negativeText: ''
                        }).render();
                        this.holdProcessing = false;
                        return false;
                    }
                    if (!this.model.get('payingClientGlobalId') || this.model.get('payingClientGlobalId') == this.invisibleClientId) {
                        new YesNoModal({
                            question: 'Please select a client for all sale lines',
                            positiveText: 'OK',
                            noCallback: true,
                            negativeText: ''
                        }).render();
                        this.holdProcessing = false;
                        return false;
                    }

                    var model = this.model.beforeCalc();
                    model['saleLines'] = this.collection.beforeCalc();
                    model['discounts'] = this.discounts.beforeCalc(_.pluck(this.collection.toJSON(), 'discounts'));
                    model['payments'] = this.checkouts.beforeCalc();
                    model['tips'] = this.tips;
                    var data = {
                        sale: model
                    };
                    saleHoldRequest(JSON.stringify(data), JSON.parse(localStorage.getItem('workstation'))['drawerId']).done(function (r) {
                        this.holdProcessing = false;
                        if (r.status === undefined) {
                            this.clearSales();
                            this.refresh();
                        }
                    }.bind(this)).always(function(){
                        this.holdProcessing = false;
                    }.bind(this));

                }.bind(this));
            }
        },

        // navigation events
        navigateThroughReceiptSubMenu: function (e, param) {
            e.preventDefault();
            e.stopPropagation();
            
            this.$('.receipt-submenu-drop-down-inner').hide().removeClass('opened');
            PermModel.navigateToPermissionsModal(e, function () {
                if (!param) { // not manually triggered
                    this.$('#register-tab-right').addClass('active').removeClass('hidden');
                }
                var $target = $(e.target),
                    url = $target.data('url');
                if ($target.hasClass('selected')) return false;
                if ($target.hasClass('sub-menu-item')) {
                    this.$('.sub-menu-item, .pseudo-sub-menu-item').removeClass('selected');
                } else {
                    this.$('.sub-menu-nav-item').removeClass('selected');
                }
                $target.addClass('selected');

                this.closeUnneededModals();

                switch (url) {
                    // select menu item
                    case 'sales':
                        this.renderTabletSubMenu('sales');
                        if (!param) { // not manually triggered
                            this.navigateToTabletRight();
                        }
                        break;
                    case 'adjustments':
                        this.navigateToAdjustments();
                        break;
                    case 'refund':
                        this.$('.sub-menu-types ul').hide();
                        this.navigateToRefundModal();
                        break;
                    case 'onhold':
                        this.$('.sub-menu-types ul').hide();
                        this.navigateToOnHoldModal();
                        break;
                    case 'transactions':
                        this.$('.sub-menu-types ul').hide();
                        setTimeout(function () {
                            $(':focus').blur();
                        }, 1000)
                        this.navigateToTransactionsModal();
                        break;
                    case 'payments':
                        this.renderTabletSubMenu('payments');
                        break;
                    // select sub menu item
                    case 'services':
                    case 'products':
                    case 'gift-sales':
                    case 'account-sales':
                        this.navigateToSales(url);
                        break;
                    case 'cash':
                    case 'credit':
                    case 'check':
                    case 'gift':
                    case 'account':
                    case 'other':
                        this.navigateToCheckoutModal({
                            url: url,
                            title: $target.html()
                        });
                        break;
                }
            }.bind(this));
        },

        navigateThroughPaymentsSubMenu: function () {
            this.subMenuModal = new SubMenuView({
                menu: this.paymentTypesMenu,
                breadCrumbs: ['Checkout', 'Payment'],
                colorScheme: 'violet',
                el: '#sub-menu-payments-modal .content'
            }).render();
            this.listenToOnce(this.subMenuModal, 'closeEvent', function (r) {
                this.subMenuSalesModal = undefined;
                var toSwitch = r.url ? r.url : r;
                switch (toSwitch) {
                    case 'cash':
                    case 'credit':
                    case 'check':
                    case 'gift':
                    case 'account':
                    case 'other':
                        this.navigateToCheckoutModal(r);
                        break;
                }
            });
        },

        navigateToClientModal: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.hideCheckoutLines();
                var id = $(e.target).attr('data-client-id') ? $(e.target).attr('data-client-id') : '';
                var options;
                if (id) {
                    options = _.extend(this.clients.get(id).toJSON(), {
                        removeButton: (id === this.invisibleClientId) ? false : true
                    });
                }

                

                if (options) {
                    if (options.id === 'invisibleClient')
                        options = {id: 'invisibleClient'};
                    else
                        options.client = options;
                } else
                    options = {};

                this.addClientModal = new AddClientModalView(options).render();
                this.listenToOnce(this.addClientModal, 'closeEvent', function (response) {
                    this.addClientModal = undefined;
                    switch (response.type) {
                        case 'add':
                            this.addClientToReceipt(response.client, false);
                            break;
                        case 'remove':
                            this.removeClientFromReceipt(response.client);
                            break;
                        case 'change':
                            this.changeClientInReceipt(response.client);
                            break;
                    }
                }.bind(this));
            }.bind(this));
        },

        navigateToTodayAppointmentsModal: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.hideCheckoutLines();
                this.todayAppointments = new ClientFromAppointmentModal().render();
                this.listenToOnce(this.todayAppointments, 'closeEvent', function (client) {
                    this.todayAppointments = undefined;
                    client.checkForAppointments = true;
                    this.addClientToReceipt(client, true);
                });
            }.bind(this));
        },

        navigateToReceiptSubMenu: function (e) {
            this.hideCheckoutLines();
            this.plusClientId = $(e.target).closest('.sales-accordion.client').data('client-id');
            this.changeLastSelectedClient(this.plusClientId);
            if (this.onPhone) {
                this.navigateToPhoneReceiptSubMenu();
            } else {
                this.navigateToTabletRight(true);
            }
        },

        navigateToTabletRight: function (addActiveClass) {
            var $tabRight = this.$('#register-tab-right');
            $tabRight.removeClass('hidden');
            if (addActiveClass) {
                $tabRight.addClass('active');
                this.$('.receipt-sub-menu .sub-menu-item:first').trigger('touchend');
            }
            this.createSubmenuDropDown();
        },

        createSubmenuDropDown: function() {
            if(!this.$('.receipt-submenu-drop-down').length) {
                var $dropdownListItems = this.$('.receipt-sub-menu .sub-menu-item[data-url=onhold]').add('.receipt-sub-menu .sub-menu-item[data-url=transactions]');
                $dropdownListItems.wrapAll('<div class="receipt-submenu-drop-down"/>');
                var $dropdownList = this.$('.receipt-sub-menu .receipt-submenu-drop-down');
                $dropdownList.prepend('<div class="pseudo-sub-menu-item"/>');
                var dropdownCaption = this.$('.receipt-sub-menu .pseudo-sub-menu-item');
                dropdownCaption.text($dropdownListItems.eq(0).text());
                $dropdownListItems.wrapAll('<div class="receipt-submenu-drop-down-inner"/>');
            }
        },

        toggleSubmenuDropDown: function(e) {
            var $dropDown = $(e.target).siblings('.receipt-submenu-drop-down-inner');

            if($dropDown.hasClass('opened')) {
                $dropDown.hide().removeClass('opened');
            } else {
                $(document).on('touchend.pseudomenu', function(e) {
                    if($(e.target).closest('.receipt-submenu-drop-down').length === 0) {
                        $dropDown.hide().removeClass('opened');
                        $(document).off('touchend.pseudomenu');
                    }
                    e.preventDefault();
                });
                $dropDown.show().addClass('opened');
            }
        },

        submenuItemSelected: function(e) {
            var $selectedItem = $(e.target);
            var $dropDown = $(e.target).closest('.receipt-submenu-drop-down-inner');
            var $title = $dropDown.siblings('.pseudo-sub-menu-item');
            $title.text($selectedItem.text()).addClass('selected');
        },

        navigateToSales: function (typeToShow) {
            var clientsJSON = this.clients.toJSON();
            var o = {
                clientId: this.plusClientId ? this.plusClientId : this.clients.getLastClientId(),
                payingClient: _.find(clientsJSON, function (client) { return client.id != window.SETTINGS.walkInFemale.id && client.id != window.SETTINGS.walkInMale.id; }.bind(this)),
                lastSelected : this.lastSelected,
                invisibleClientId : this.invisibleClientId,
                clients : clientsJSON,
                saleLinesCount : this.collection.length
            };

            if (!o.payingClient || o.payingClient.id == this.invisibleClientId)
                o.payingClient = undefined;

            switch(typeToShow) {
                case 'services':
                    this.salesModal = new SalesServicesModalView(o).render();
                    break;
                case 'products':
                    this.salesModal = new SalesProductsModalView(o).render();
                    break;
                case 'gift-sales':
                    this.salesModal = new SalesGiftsModalView(o).render();
                    break;
                case 'account-sales':
                    this.salesModal = new SalesAccountsModalView(o).render();
                    break;
            }
            this.listenTo(this.salesModal, 'closeEvent', this.closeSalesCallback);
            this.listenTo(this.salesModal, 'addedSaleLine', function(newSaleLine) {
                this.collection.add(newSaleLine);
            });
        },

        // Smart-Receipt Navigation
        navigateToPhoneReceiptSubMenu: function () {
            this.subMenuModal = new SubMenuView({
                menu: this.receiptSubMenu,
                breadCrumbs: ['Register'],
                colorScheme: 'grey',
                el: '#sub-menu-modal .content'
            }).render();
            this.listenToOnce(this.subMenuModal, 'closeEvent', function (r) {
                this.subMenuModal = undefined;
                var toSwitch = r.url ? r.url : r;
                switch (toSwitch) {
                    case 'adjustments':
                        this.navigateToAdjustments();
                        break;
                    case 'sales':
                        this.navigateToPhoneSalesSubMenu();
                        break;
                    case 'refund':
                        this.navigateToRefundModal();
                        break;
                    case 'onhold':
                        this.navigateToOnHoldModal();
                        break;
                    case 'transactions':
                        this.navigateToTransactionsModal();
                        break;
                }
            });
        },

        // Generate phone SALES sub-menu
        navigateToPhoneSalesSubMenu: function () {
            this.subMenuSalesModal = new SubMenuView({
                menu: this.saleTypesMenu,
                breadCrumbs: ['Register', 'Sales'],
                colorScheme: 'violet',
                el: '#sub-menu-sales-modal .content'
            }).render();
            this.listenToOnce(this.subMenuSalesModal, 'closeEvent', function (r) {
                this.subMenuSalesModal = undefined;
                var toSwitch = r.url ? r.url : r;
                switch (toSwitch) {
                    case 'services':
                    case 'products':
                    case 'gift-sales':
                    case 'account-sales':
                        this.navigateToSales(r.url);
                        break;
                    case 'register':
                        this.navigateToPhoneReceiptSubMenu();
                        break;
                }
            });
        },

        navigateToSaleEdit: function (e) {
            this.hideCheckoutLines();
            $(e.target).closest('.sale-line').removeClass('tapped');
            var isRefund = ($(e.target).closest('.sale-line').hasClass('refund-item')) ? true : false;
            this.editSaleModal = new EditSaleModal({
                model: new Backbone.Model(this.collection.get($(e.target).data('id')).toJSON()),
                isRefund: isRefund
            }).render();
            this.listenTo(this.editSaleModal, 'changeEvent', this.saleLineEdited);
        },

        navigateToSaleDiscount: function (e, id) {
            this.hideCheckoutLines(true);
            if (e) {
                $(e.target).closest('.sale-line').removeClass('tapped');
                id = $(e.target).data('id');
            }

            var originalModel = this.collection.get(id),
                options = {
                    model: originalModel.toJSON(),
                    discounts: this.discounts.toJSON()
                };
            this.discountSaleModal = new DiscountSaleModal(options).render();
            this.discountModelId = id;
            this.listenTo(this.discountSaleModal, 'discountEvent', this.saleLineDiscounted);
        },

        navigateToSaleNotification: function (e) {
            this.hideCheckoutLines();
            this.salesLineNotification = new YesNoModal({
                question: $(e.target).data('alert'),
                positiveText: 'Dismiss',
                className: 'modal-alert',
                positiveCallback: {
                    triggerEvent: 'yesNoEvent',
                    sendProperty: $(e.target).data('id')
                },
                negativeText: 'Ok',
                negativeCallback: {
                    triggerEvent: 'yesNoEvent',
                    sendProperty: false
                }
            }).render();
            this.listenToOnce(this.salesLineNotification, 'yesNoEvent', function (response) {
                if (response) {
                    //var responseItem  = this.collection.get(response).toJSON();
                    _.findWhere(this.collection.models, {id: response}).set('alerts', []);
                    // if(responseItem.item.type === Consts.SaleLineTypes.PRODUCT) {
                    //     this.collection.remove(this.collection.get(response));
                    //     if (!this.onPhone) {
                    //         this.hideTabletLandscapeRight(true);
                    //     }
                    // }

                }
            });
        },

        navigateToClientNoteSummary: function (e) {
            e.preventDefault();
            e.stopPropagation();
            e.stopImmediatePropagation();
            this.hideCheckoutLines();
            var clientId = $(e.target).siblings('.client-name').attr('data-client-id');
            this.clientNotesModal = new ClientNotesModalView({
                model: this.clients.find(function (model) {
                    return model.get('id') == clientId
                })
            }).render();
        },

        navigateToCheckout: function (e) {

            if (this.calculationInProgress) {
                new YesNoModal({
                    question: 'Calculation request still processing!',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }

            var ifCanNotCheckout = this.detectIfCanNotCheckout();
            if (ifCanNotCheckout.canNot) {
                if (this.queriedFinalize) {
                    new YesNoModal({
                        question: 'Finalize request still processing!',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                }
                if (this.collection.length && (!this.model.get('payingClientGlobalId') || this.model.get('payingClientGlobalId') == this.invisibleClientId)) {
                    new YesNoModal({
                        question: 'Please select a client for all sale lines',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                }
                return false;
            }
            PermModel.navigateToPermissionsModal(e, function () {
                if (
                    (!this.model.get('payingClientGlobalId') || this.model.get('payingClientGlobalId') == this.invisibleClientId) &&
                    this.model.get('totalTips')) {
                    new YesNoModal({
                        question: 'Please select the client giving this tip',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                }
                if (!this.model.get('payingClientGlobalId') || this.model.get('payingClientGlobalId') == this.invisibleClientId) {
                    new YesNoModal({
                        question: 'Please select a client for all sale lines',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                }

                var $wrapper = this.$('.smart-receipt-wrapper'),
                    $hasCheckoutClass = $wrapper.hasClass('show-checkout');

                if (!$hasCheckoutClass) this.$('.smart-receipt-wrapper').addClass('show-checkout');
                this.scroll._execEvent('beforeScrollStart');
                if (!this.onPhone) {
                    this.salesModal && this.salesModal.closeView();
                    this.salesModal = undefined;
                    if (!$hasCheckoutClass) this.renderReceiptSubMenu('payments');
                }
                if (this.checkoutScroll) {
                    this.checkoutScroll.refresh();
                    if (!$hasCheckoutClass) this.checkoutScroll.scrollTo(0, 0);
                } else {
                    this.checkoutScroll = new IScroll('#smart-receipt-checkout-scroll', {
                        tap: true,
                        bounce: false,
                        disableMouse: true,
                        scrollbars: 'custom'
                    });
                    this.checkoutScroll.on('beforeScrollStart', function () {
                        this.refresh();
                    });
                }
                if (this.checkouts.models.length == 0 || $hasCheckoutClass) {
                    if (!(!this.onPhone && this.onLandscape)) this.navigateToCheckoutSubMenu(); // not on tablet landscape
                }
            }.bind(this));
        },

        navigateToCheckoutSubMenu: function () {
            if (this.onPhone) {
                this.navigateThroughPaymentsSubMenu();
            } else {
                this.$('.sub-menu-nav-item:first').trigger('tap');
                this.navigateToTabletRight(true);
            }
        },

        navigateToRefundCheckout: function (e) {
            if (this.queriedFinalize) {
                new YesNoModal({
                    question: 'Finalize request still processing!',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }
            if (!this.model.get('payingClientGlobalId') || this.model.get('payingClientGlobalId') == this.invisibleClientId) {
                new YesNoModal({
                    question: 'Please select a client for all sale lines',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }
            PermModel.navigateToPermissionsModal(e, function () {
                this.refundCheckoutModal = new RefundCheckoutModalView({
                    refundValue: this.model.get('balance'),
                    paymentTypes: this.paymentTypes,
                    payingClient: this.clients.get(this.model.get('payingClientGlobalId')).toJSON()
                });
                this.$('.smart-receipt-wrapper').addClass('show-checkout-payments');
                this.listenTo(this.refundCheckoutModal, 'refundCheckoutEvent', function (r) {
                    this.refundCheckoutModal = undefined;

                    switch (r.type) {
                        case 'changeDue':
                            this.changeCheckoutPayments();
                            this.navigateToChangeModal(r.paymentObject);
                            break;
                        case 'refundCheckout':
                            this.finalizeSale(r.paymentObject);
                            break;
                        case 'refundSaleLineCheckout':
                            r.newSaleLine.invisibleLine = true;
                            this.collection.add(r.newSaleLine);
                            this.reCalcSale(this.finalizeSale.bind(this));
                            break;
                    }
                });
            }.bind(this));
        },

        navigateToNoSales: function (e) {
            if ($(e.target).hasClass('inactive')) return false;
            PermModel.navigateToPermissionsModal(e, function () {
                this.yesNoModal = new YesNoModal({
                    question: 'Are you sure you want to cancel this transaction?',
                    positiveText: 'Yes',
                    positiveCallback: function () {
                        this.yesNoModal.closeView();
                        this.yesNoModal = undefined;
                        if (this.model.get('saleTempId')) deleteHoldSale(JSON.parse(localStorage.getItem('workstation'))['drawerId'], this.model.get('saleTempId'));
                        this.refundedItems = [];
                        this.holdPayingClientId = '';
                        if (this.onRefundModal) {
                            this.onRefundModal.updateRefundedItems([]);
                        }
                        this.clearSales();
                        this.scroll.scrollTo(0, 0);
                    }.bind(this),
                    negativeText: 'No',
                    negativeCallback: function () {
                        this.closeView();
                    }
                }).render();
            }.bind(this));
        },

        navigateToCheckoutLineEdit: function (e) {
            $(e.target).closest('.checkout-line').removeClass('tapped');
            this.$('.sub-menu-types ul').hide();
            var o = {
                originalModel: this.checkouts.get($(e.target).data('id')).toJSON(),
                initialAmount: $(e.target).closest('.checkout-line').data('amount')
            };
            if (!this.onPhone) {
                this.navigateToTabletRight(true);
                this.$('.sub-menu-nav-item.selected').removeClass('selected');
            }
            this.navigateToCheckoutModal(o);
        },

        navigateToCheckoutModal: function (o) {
            var options = {
                balance: this.model.get('balance'),
                checkouts: this.checkouts.toJSON(),
                paymentTypes: this.checkoutPaymentTypes,
                checkoutsLiveReference: this.checkouts,
                clients: this.clients.toJSON(),
                payingClientGlobalId: this.model.get('payingClientGlobalId')
            };
            if (o) {
                options = _.extend(options, o);
            }
            this.checkoutModal = new CheckoutModalView(options).render();
            this.listenTo(this.checkoutModal, 'checkoutEvent', this.checkoutLineEdited);
        },

        beforeNavigateToChangeModal: function (e) {
            e.preventDefault();
            e.stopPropagation();
            if(this.queriedFinalize){
                new YesNoModal({
                    question: 'Finalize request still processing!',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            }
            if(this.model.get('balance') > 0) {
                new YesNoModal({
                    question: 'Please enter enough of a payment amount to satisfy the balance due',
                    positiveText: 'OK',
                    noCallback: true,
                    negativeText: ''
                }).render();
                return false;
            } else {
                var gCardsAmount = this.checkouts.getGiftCardsAmount();
                if (gCardsAmount && gCardsAmount > this.model.get('totalPayments')) {
                    new YesNoModal({
                        question: 'The amount of gift card payments cannot exceed the total amount of the ticket',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                }
                if (this.model.get('balance') < -100) {
                    var questionPart = "Payments = " + $.global.format(this.checkouts.getAmount(), 'c');
                    questionPart += ". <br/>Change will be " + $.global.format(this.model.get('changeDue'), 'c');
                    questionPart += ". <br/>Is this correct?";
                    this.yesNoModal = new YesNoModal({
                        question: 'Payment Verification<br/><br/>'+questionPart,
                        positiveText: 'YES',
                        positiveCallback: function () {
                            this.yesNoModal.closeView();
                            this.beforeNavigateToSignature();
                        }.bind(this),
                        negativeText: 'NO',
                        negativeCallback: function () {
                            this.closeView();
                        },
                        height: 290
                    }).render();
                    return false;
                }
            }
            this.beforeNavigateToSignature();
        },

        beforeNavigateToSignature: function() {
            if(this.checkouts.checkSignature() && window.SETTINGS.signatureCaptureEnabled) {
                this.onSignatureModal = new SignatureView({
                    total: this.model.get('totalPayments')
                }).render();
                this.listenToOnce(this.onSignatureModal, 'changeAsTipWithSignature', function(response){
                    this.checkouts.setSignature(response);
            this.navigateToChangeModal();
                });
            } else {
                this.navigateToChangeModal();
            }
        },

        navigateToChangeModal: function (payment) {
            var changeTotal = 0;
            _.each(this.tips,function(elem){
                changeTotal += elem.amount;
            });

            this.changeAsATipModal = new ChangeAsTipModal(_.extend(this.model.toJSON(), {
                changeTotal: this.model.get('changeDue') + changeTotal
            })).render();
            this.listenToOnce(this.changeAsATipModal, 'finalize', function (response) {
                this.changeAsATipModal = undefined;
                switch (response) {
                    case 'showTips':
                        this.navigateToAdditionalTips();
                        break;
                    case 'finalize':
                        //if(this.model.get('balance') < 0) {
                        //    this.model.set('changeDue', 0);
                        //}
                        if(payment) {
                            this.finalizeSale(payment);
                        } else {
                            this.finalizeSale();
                        }
                        break;
                }
            }.bind(this));
        },

        navigateToTips: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                this.$('#register-tab-right').removeClass('active');
                this.hideCheckoutLines();
                var tipsOptions = this.prepareForTips('general');
                this.addTipModal = new AddTipModal(_.extend(tipsOptions, {tipModalData: this.tipModalData})).render();
                this.listenToOnce(this.addTipModal, 'submitTips', function (response) {
                    this.submitTipsCallback(response, 'general');
                });
            }.bind(this));
        },

        navigateToAdditionalTips: function () {
            var tipsOptions = this.prepareForTips('additional');
            this.addTipModal = new AddTipModal(_.extend(tipsOptions, {changeDue: this.model.get('changeDue'), initialChange: $('#initial-change').text()})).render();
            this.listenToOnce(this.addTipModal, 'submitTips', function (response) {
                this.submitTipsCallback(response, 'additional');
            });
        },

        navigateToAdjustments: function() {
            this.$('.sub-menu-types ul').hide();

            this.adjustmentModal = new AdjustmentsModalView({
                collection: new Backbone.Collection(this.discounts.toJSON())
            }).render();
            this.listenTo(this.adjustmentModal, 'adjustmentEvent', function(response){
                if (response.type != 'doneLeave') {
                    this.adjustmentModal = undefined;
                }
                switch (response.type) {
                    case 'delDiscount':
                        _.each(this.collection.toJSON(), function (line) {
							var hasDiscount = false;
                            _.each(line.discounts, function(discount){
                                if( discount.discountId == response.discount.id){
									hasDiscount = true;
									return false;
                                }
                            }.bind(this));
							if(hasDiscount)
								this.saleLineDiscounted(response, line.id);
                        }.bind(this));
                        break;
                    case 'showManual':
                        this.navigateToManualAdjustment(response['discountId']);
                        break;
                    case 'showPredefined':
                        this.navigateToPredefinedAdjustments();
                        break;
                    case 'cancel':
                        break;
                    case 'doneLeave':
                        if (!this.onLandscape) {
                            this.hideTabletLandscapeRight();
                        }
                        break;
                }
            }.bind(this));
        },

        navigateToManualAdjustment: function(discountId){
            var options = {
                model : {},
                discounts: {},
                multipleDiscount: true,
                saleLines: this.collection.toJSON()
            };
            if(discountId){
                options.discountForEdit = _.findWhere(this.discounts.models, {id: discountId});
            }

            this.manualDiscounts = new DiscountSaleModal(options).render();
            this.listenToOnce(this.manualDiscounts, 'applyEvent', function(response){
                if(!response.type){
                    _.each(response.saleLinesForAdding, function(id){
                        this.saleLineDiscounted({type: 'addDiscount', discount: response.discount}, id);
                    }.bind(this));
                    _.each(response.saleLinesForDeleting, function(id){
                        this.saleLineDiscounted({type: 'delDiscount', discount: response.discount, discountId: response.discountId}, id);
                    }.bind(this));
                    _.each(response.saleLinesForEditing, function(id){
                        this.saleLineDiscounted({type: 'editDiscount', discount: response.discount}, id);
                    }.bind(this));
                } else {
                    _.each(response.saleLines, function(id){
                        this.saleLineDiscounted(response, id);
                    }.bind(this))
                }
                this.navigateToAdjustments();
                this.manualDiscounts = undefined;
            });
        },

        navigateToPredefinedAdjustments: function () {

            this.predefinedDiscountsModal = new PredefinedDiscountModalView({
                collection: new Backbone.Collection(this.collection.toJSON()),
                couponsRequestData: this.collection
            }).render();
            this.listenToOnce(this.predefinedDiscountsModal, 'closeEvent', function(response){

				var d1 = $.Deferred();
                if (response.type == 'addDiscount') {
					var alwaysBarrier = response.saleLines.length;
                    _.each(response.saleLines, function(id){
                        this.saleLineDiscounted(response, id).always(function(){
							--alwaysBarrier;
							if(alwaysBarrier <= 0)
								d1.resolve();
						}.bind(this));
                    }.bind(this));
                }
				else
					d1.resolve();

				$.when(d1).done(function(){
                    this.predefinedDiscountsModal && this.predefinedDiscountsModal.closeView();
					this.predefinedDiscountsModal = undefined;
					this.navigateToAdjustments();
				}.bind(this));
            });
        },

        navigateToAppointmentNotes: function(notes, index) {
            var appointObject = notes[index],
                appointNote = appointObject['note'];
            if (appointNote.trim().length == 0) return false;
            if (_.indexOf(appointObject["usages"], 'Register') == -1) {
                if (index < notes.length -1) {
                    this.navigateToAppointmentNotes(notes, index+1);
                } else {
                    return false;
                }
            }
            appointNote = 'Appointment Note<br/><br/>' + _.escape(JSON.stringify(appointNote).replace(/^"(.*)"$/g, '$1')).replace(/(\\n|\\r\\n|\\r)/gm, '<br/>');

            this.yesNoModal = new YesNoModal({
                question: appointNote,
                positiveText: 'OK',
                escapeHtml: false,
                wide: true,
                positiveCallback: function () {
                    this.yesNoModal.closeView();
                    if (index < notes.length - 1) {
                        this.navigateToAppointmentNotes(notes, index+1);
                    }
                }.bind(this),
                negativeText: ''
            }).render();
        },

        navigateToRefundModal: function () {
            this.onRefundModal = new OnRefundModalView({
                refundedItems: this.refundedItems,
                client: this.clients.getLastSelectedClient(this.lastSelected.get('clientId')),
                invisibleClientId: this.invisibleClientId
            }).render();
            this.listenTo(this.onRefundModal, 'refundListChanging', function(response){
                this.refundedItems = response;
            });
            this.listenTo(this.onRefundModal, 'refundEvent', this.onRefundUpdate);
        },

        navigateToOnHoldModal: function () {
            this.onHoldModal = new OnHoldModalView({
                restored: (this.model.get('saleTempId') ? true : false) || (this.collection.toJSON().length > 0)
            }).render();
            this.listenTo(this.onHoldModal, 'holdEvent', this.onHoldUpdate);
        },

        navigateToTransactionsModal: function () {
            this.transactionsModal = new TransactionsModal().render();
            this.listenTo(this.transactionsModal, 'transactionsEvent', function (response) {
                if (!response.doneLeave) this.transactionsModal = undefined;
                if (!this.onPhone) this.hideTabletLandscapeRight();
                switch (response.type) {
                    case 'returnToMenu':
                        this.navigateToPhoneReceiptSubMenu();
                        break;
                }
            }.bind(this));
        },

        // callbacks
        saleLineEdited: function (response) {
            var originalModel = this.collection.get(this.editSaleModal.model.get('id')),
                item = originalModel ? originalModel.get('item') : undefined,
                updateEditSaleModalHandler = function () {
                    this.updateEditSaleModal()
                }.bind(this);
            switch (response.type) {
                case 'remove':
                    this.collection.remove(this.collection.get(response.id));
                    break;
                case 'employee':
                    if (originalModel === undefined)
                        break;

                    originalModel.set('employee', response.employee);

                    originalModel.set('item', response.item);
                    this.saleLineChangeItem(originalModel);

                    this.getSaleLineAlerts(originalModel);
                    break;
                case 'appointment':
                    if (originalModel === undefined)
                        break;

                    if (!response.appointment) {
                        response.appointment = {
                            id: '',
                            shortcut: '',
                            displayName: '',
                            description: ''
                        }
                    }
                    originalModel.set('appointType', response.appointment);
                    break;
                case 'service':
                    if (originalModel === undefined)
                        break;

                    originalModel.set('item', response.item);
                    this.saleLineChangeItem(originalModel);

                    this.getSaleLineAlerts(originalModel);
                    break;
                case 'quantity':
                    if (originalModel === undefined)
                        break;

                    originalModel.set('quantity', response.quantity);
                    break;
                case 'discount':
                    if (originalModel === undefined)
                         break;

 					var discounts = originalModel.get('discounts');
 					var elem = this.discounts.where({discountEffectEnum: 1676, isManual: true });
 					elem = _.filter(elem, function(el){ return _.some(discounts, function(elm){ return elm.discountId == el.id; }); });
 					if(!elem.length){
 						this.discounts.add(response.discount);

 						// find order to set
 						discounts = _.sortBy(discounts, 'order');
 						var i=0;
 						for(; i < discounts.length - 1; i++)
 							if(discounts[i+1].order - discounts[i].order > 1)
 								break;

 						discounts.splice(i + 1, 0, { discountId: response.discount.id, order: discounts.length !=0 ? (discounts[i].order + 1) : 1 });
 					}
 					else {
 						elem = elem[0];
 						response.discount.id = discounts[_.findIndex(discounts, function(disc) { return disc.discountId == elem.id; })].discountId;
 						this.discounts.add(response.discount, {merge: true});
 					}

					originalModel.set('discounts', discounts);
                    this.reCalcSale(updateEditSaleModalHandler);

                    break;
                case 'tryToRemoveDiscount':
                    if (originalModel === undefined) {
                        break;
                    }
                    var discounts = _.reject(originalModel.get('discounts'), function (d) {
                        return this.discounts.findWhere({id: d.discountId}).get('discountEffectEnum') == 1676;
                    }.bind(this));
                    originalModel.set('discounts', discounts);
                    this.reCalcSale(updateEditSaleModalHandler);
                    break;
                case 'purchase-for':
                    if (originalModel === undefined)
                        break;

                    item['client'] = response.client;

                    originalModel.set('item', item);
                    this.saleLineChangeItem(originalModel);

                    break;
                case 'card-price':
                case 'card-value':
                    if (originalModel === undefined)
                        break;

                    item[response.name] = response.amount;
                    originalModel.set('item', item);
                    this.saleLineChangeItem(originalModel);

                    this.reCalcSale(updateEditSaleModalHandler);
                    break;
                case 'calc':
                    this.editSaleModal = undefined;
                    this.reCalcSale();
                    if (response.someParam && response.someParam.type == 'showDiscounts') {
                        this.navigateToSaleDiscount(null, response.someParam.id);
                    }
                    break;
            }
        },

        updateEditSaleModal: function () {
            if (this.editSaleModal) {
                this.editSaleModal.model.set(
                    this.collection.get(this.editSaleModal.model.get('id')).toJSON()
                );
            }
        },

        saleLineDiscounted: function (response, saleLineId) {
            var originalModel = saleLineId ? this.collection.get(saleLineId) : this.collection.get(this.discountSaleModal.model.get('id')),
                discounts = originalModel.get('discounts'),
                updateDiscountHandler = saleLineId ? '' : function () {
                    this.updateDiscountModal()
                }.bind(this);
            switch (response.type) {
                case 'addDiscount':
                    if (response.discount.discountEffectEnum == 1676 && response.discount.isManual) {
						var existSpecific = _.findIndex(discounts, function(elem) { return elem.discountEffectEnum == 1676 && elem.isManual; });
						if(existSpecific==-1)
							this.discounts.add(response.discount);
						else{
							response.discount.id = discounts[existSpecific].discountId;
							this.discounts.add(response.discount, {merge: true});
							this.reCalcSale(updateDiscountHandler);
							return;
						}
                    }
					else {
						this.discounts.add(response.discount);
						if(_.findIndex(discounts, function(elem) { elem.discountId == response.discount.id; }) != -1){
							new YesNoModal({
								question: 'That discount is already added and unavailable now!',
								positiveText: 'Ok',
								negativeText: '',
								noCallback: true
							}).render();
							return;
						}
					}

					discounts.push({ discountId: response.discount.id, order: discounts.length + 1 });
					_.each(discounts, function (d, i) { d.order = i + 1; });

					originalModel.set('discounts', discounts);
					return this.reCalcSale(updateDiscountHandler);

                    break;
                case 'editDiscount':
                    this.discounts.add(response.discount, {merge: true});
                    this.reCalcSale(updateDiscountHandler);
                    break;
                case 'delDiscount':
					discounts = _.without(discounts, _.findWhere(discounts, {discountId: response.discountId}));
					_.each(discounts, function (d, i) {
						d.order = i + 1;
					});
					originalModel.set('discounts', discounts);

					var jqxHr = this.reCalcSale(updateDiscountHandler);
					if(response.deleteDoneCallBack != null)
						jqxHr.always(response.deleteDoneCallBack);
                    break;
                case 'calc':
                    this.reCalcSale();
                    this.discountModelId = undefined;
                    if (!this.onPhone) {
                        this.hideTabletLandscapeRight();
                    }
                    break;
                case 'makeUndefined':
                    this.discountSaleModal = undefined;
                    break;
            }
        },

        updateDiscountModal: function () {
            if (this.discountSaleModal) {
                this.discountSaleModal.model.set(_.extend(
                    this.collection.get(this.discountModelId).toJSON(), {
                        allDiscounts: this.discounts.toJSON()
                    }
                ));

				this.discountSaleModal.UpdateCompleted = true;
            }
        },

        addClientToReceipt: function (client, fromAppointment) {
            var clientPresent = this.clients.get(client.id);
            if (this.clients.length === 0) {
                this.model.set('payingClientGlobalId', client.id);
            }
            if (this.clients.length == 1 && this.clients.get(this.invisibleClientId)) {
                if (fromAppointment) {
                    if (this.collection.length == 0) {
                        this.clientRemoved(this.clients.get(this.invisibleClientId));
                        this.clients.remove(this.invisibleClientId);
                    }
                    this.clients.add(client);
                    this.model.set('payingClientGlobalId', client.id);
                    if (!clientPresent) {
                        this.checkIfAppointments(client.id, fromAppointment);
                    } else {
                        if (client.checkForAppointments) {
                            this.checkIfAppointments(client.id, true);
                        }
                    }
                } else {
                    this.changeClientInReceipt({
                        oldId: this.invisibleClientId,
                        newClient: client
                    }, true);
                }
            } else {
                this.clients.add(client);
                if (!clientPresent) {
                    this.checkIfAppointments(client.id, fromAppointment);
                } else {
                    if (client.checkForAppointments) {
                        this.checkIfAppointments(client.id, true);
                    }
                }
            }
            this.changeLastSelectedClient(client.id);
        },

        addAppointmentDetails: function(response, clientId) {
            _.each(response, function (line) {
                this.collection.add(line);
            }.bind(this));

            getAppointmentNotes(clientId).done(function (r) {
                if ((r.status === undefined) && (r.length > 0)) {
                    r = r[0];
                    if (r.length > 0) this.navigateToAppointmentNotes(r, 0);
                }
            }.bind(this));
        },

        removeClientFromReceipt: function (clientId) {
            this.stopListening(this.model, 'change:payingClientGlobalId', this.changePayingClient);
			this.clientRemoved(this.clients.get(clientId));
            this.clients.remove(this.clients.get(clientId));
            this.collection.remove(_.map(this.collection.models, function (m) {
                if (m.get('client')['id'] === clientId) {
                    return m;
                }
            }));
            var clients = this.clients.models;
            if (clients.length == 0) {
                if(!this.collection.length) {
                    this.clearSales();
                }
                //this.stopHoldRequest = true;
                this.holdPayingClientId = undefined;
                this.model.set('payingClientGlobalId', this.invisibleClientId);
            } else {
                var m = clients[0],
                    payingClientId = this.model.get('payingClientGlobalId');
                if (payingClientId != m.get('id')) {
                    this.model.set('payingClientGlobalId', m.get('id'));
                }
            }
            this.listenTo(this.model, 'change:payingClientGlobalId', this.changePayingClient);
        },

        changeClientInReceipt: function (client, setPayingClient) {
            this.clientRemoved(this.clients.get(client.oldId), client.oldId === this.invisibleClientId);
            this.clients.remove(this.clients.get(client.oldId));
            this.clients.add(client.newClient);
            if (this.model.get('payingClientGlobalId') === client.oldId || setPayingClient) {
                this.model.set('payingClientGlobalId', client.newClient.id);
            }
            var mId = '';
            _.each(this.collection.models, function (m) {
                mId = m.get('id');
                if (mId.substr(mId, mId.indexOf('_')) == client.oldId) {
                    m.set('id', client.newClient.id + '_' + new Date().getTime());
                }

                m.set('client', client.newClient);
                if (client.oldId !== client.newClient.id) {
                    m.trigger('change:client', m);
                }
            });
            this.checkIfAppointments(client.newClient.id, false);
        },

        submitTipsCallback: function (response, type) {
            //
            if (type == 'additional' && response.allTips === undefined) {
                this.navigateToChangeModal();
                return;
            }
            this.addTipModal = undefined;
            var totalAmount = 0;
            this.tips = _.map(response.allTips, function(obj, k){
                totalAmount += obj.amount;
                return {
                    amount: obj.amount,
                    employee: obj.employee,
                    id: obj.id,
                    lineNumber: k+1
                }
            });
            /*
            if (totalAmount) {
                this.$('.button.no-sale').removeClass('inactive');
            }*/

            this.customTips = response.customTips;

            if (type == 'additional') {
                this.model.set('changeDue', response.newChangeDue);
                this.navigateToChangeModal();
            } else if (type == 'general') {
                this.tipModalData = {
                    tipAmount: response.tipModalData.tipAmount,
                    tipType: response.tipModalData.tipType
                };
            }

            this.reCalcSale();
        },

        checkoutLineEdited: function (response) {
            switch (response.type) {
                case 'edit':
                    this.checkouts.updateModel(response.checkout);
                    if (!this.onPhone) {
                        this.closeCheckoutModal();
                        this.$('.sub-menu-nav-item:first').trigger('tap', true);
                    }
                    break;
                case 'add':
                    this.checkouts.beforeAdd(response.checkout);
                    if (!response.doneLeave) this.checkoutModal = undefined;
                    break;
                case 'back':
                    this.checkoutModal = undefined;
                    this.navigateThroughPaymentsSubMenu();
                    break;
            }
            this.changeCheckoutPayments();
            this.hideTabletLandscapeRight(response.type == 'cancel');
            this.$('.sub-menu-types ul').show();
        },

        closeSalesCallback: function (r) {
            if (r != 'doneLeave') {
                this.salesModal = undefined;
            }
            if (this.onPhone) {
                switch (r) {
                    case 'returnToMenu':
                        this.navigateToPhoneReceiptSubMenu();
                        break;
                    case 'returnToSubMenu':
                        this.navigateToPhoneSalesSubMenu();
                        break;
                }
            } else {
                if (r != 'close') {
                    this.hideTabletLandscapeRight();
                }
            }
        },

        onRotate: function () {
            var $html = $('html');
            this.onLandscape = $html.hasClass('landscape');
            if (!this.onPhone) {
                if (this.onLandscape) {
                    this.plusClientId = undefined;
                    this.navigateToTabletRight();
                    this.$('#sub-menu-types-scroll').css('width', '');
                } else {
                    if (!this.$('#register-tab-right').hasClass('active')) {
                        this.hideTabletLandscapeRight();
                    }
                    var length = this.$('.smart-receipt-wrapper').hasClass('show-checkout') ? this.paymentTypesMenu.length : this.saleTypesMenu.length;
                    var width = 130 * length;
                    this.$('#sub-menu-types-scroll').width(width);
                }
                this.updatePaymentInfoHeight();
            }
            this.scroll && this.scroll._execEvent('beforeScrollStart');
        },

        onHoldUpdate: function (r) {
            if (!r.doneLeave) {
                this.onHoldModal = undefined;
            }
            if (!this.onPhone) {
                this.hideTabletLandscapeRight();
            }
            switch (r.type) {
                case 'extract':
                    var m = r.holdItem;
                    this.holdTransactionExtract(m);
                    break;
                case 'returnToMenu':
                    this.navigateToPhoneReceiptSubMenu();
                    break;
            }
        },

        onRefundUpdate: function(response) {
            if (!response.doneLeave) {
                this.onRefundModal = undefined;
            }
            if (!this.onPhone) {
                this.hideTabletLandscapeRight();
            }
            switch (response.type) {
                case 'extract':
                    var modelToAdd = response.refundItem;
                    if(modelToAdd['saleLine']['item']['type'] === Consts.SaleLineTypes.GIFTCARD) {
                        modelToAdd.saleLine.item.price = modelToAdd['refundSaleLineInfo']['currentBalance'];
                    }
                    // change client to lastSelected
                    var lastSelectedClient = this.clients.getLastSelectedClient(this.lastSelected.get('clientId'));
                    if (lastSelectedClient.id == this.invisibleClientId) {
                         this.changeClientInReceipt({
                             oldId: this.invisibleClientId,
                             newClient: modelToAdd.saleLine.client
                         });
                    } else {
                        modelToAdd.saleLine.client = lastSelectedClient;
                    }
                    // add saleLine 
                    this.collection.add(modelToAdd.saleLine);
                    break;
                case 'returnToMenu':
                    this.navigateToPhoneReceiptSubMenu();
                    break;
            }
        },

        printModalCallback: function (response) {
            this.onPrintModal = undefined;
            //this.prepareVisitNotes();
            this.clearSales();
            this.refresh();
        },

        // helpers
        getSaleLineAlerts: function (line) {
            var type = line.get('item').type;
            switch (type) {
                case Consts.SaleLineTypes.SERVICE:
                    if(!line.get('employee').id) return false;
                    getServiceAlerts(line.get('item').id, line.get('employee').id).done(function (data) {
                        line.set('alerts', data);
                    });
                    break;
                case Consts.SaleLineTypes.PRODUCT:
                    getProductAlerts(line.get('item').id).done(function (data) {
                        line.set('alerts', data);
                    });
                    break;
            }
        },

        prepareForTips: function (tipType) {
            var employeesList = _.map(this.model.get('saleLines'), function(line){
                if (line.item.type === Consts.SaleLineTypes.SERVICE && !line.isRefund) {
                    var model = line.employee;
                    model.totalLinePrice = line.totalLinePrice;
                    return model;
                }
            });
            var groupedEmployees = _.groupBy(_.reject(employeesList, function (emp) {
                return emp === undefined;
            }), 'id');

            return {
                employeeList: groupedEmployees,
                customTips: this.customTips.slice(0),
                allTips: this.tips.slice(0),
                tipType: tipType
            };
        },

        detectIfCanNotCheckout: function () {
            var canNot = this.model.get('totalPayments') < 0 || (this.model.get('totalPayments') == 0 && this.collection.length == 0);
            var refund = this.model.get('totalPayments') < 0;
            if (canNot) {
                if (refund) {
                    this.$('.smart-receipt-wrapper').addClass('show-refund-button');
                } else {
                    this.$('.smart-receipt-wrapper').removeClass('show-refund-button');
                this.$('.button.checkout').addClass('inactive');
                }
            } else {
                this.$('.smart-receipt-wrapper').removeClass('show-refund-button');
                this.$('.button.checkout').removeClass('inactive');
            }
            return {
                canNot: canNot,
                refund: refund
            };
        },

        reCalcSale: function (callback, async) {
			async = async || true;
            var model = this.model.beforeCalc();

            model['saleLines'] = this.collection.beforeCalc();
			model['discounts'] = this.discounts.beforeCalc(_.pluck(this.collection.toJSON(), 'discounts'));
            model['tips'] = this.tips;
            var data = {
                sale: model
            };

            this.calculationInProgress = true;
            return saleCalculationRequest(JSON.stringify(data), async).done(function (r) {
                if (r.status === undefined) {
                    this.afterReCalc(r);
                }
                if (callback) {
                    callback();
                }
            }.bind(this)).always(function() {
                this.calculationInProgress = false;
            }.bind(this));
        },

        afterReCalc: function (r) {
            var clients = _.map(this.collection.toJSON(), function (m) {
                return m.client.id ? m.client.id : this.invisibleClientId;
            }.bind(this));
            if (clients.length > 0) {
                r.payingClientGlobalId = clients[0];
            }
            this.model.set(r);
            this.collection.afterCalc(r);
            this.detectIfCanNotCheckout();
        },

        finalizeSale: function (payment) {
			this.queriedFinalize = true;
            var model = this.model.beforeCalc(),
                payments = this.checkouts.beforeCalc();
            
            model['saleLines'] = this.collection.beforeCalc();
            model['discounts'] = this.discounts.beforeCalc(_.pluck(this.collection.toJSON(), 'discounts'));
            if(payment) {
                payments = [payment];
            } else {
                if (payments.length == 0 && model.changeDue != 0) {
                    payments = [{
                        amount: -model.changeDue,
                        lineNumber: 1,
                        isRefund: true,
                        paymentType: _.findWhere(this.paymentTypes, {shortcut: "CASH"})
                    }];
                }
            }
            model['payments'] = payments;
            model['tips'] = this.tips;

            var data = {
                sale: model
            };
            data.sale.totalPayments = data.sale.totalPayments.toFixed(2);

            saleFinalizeRequest(JSON.stringify(data), JSON.parse(localStorage.getItem('workstation'))['drawerId'])
                .done(function (r) {
                    if (r.status === undefined) {

                        if (!r.id) {
                            new YesNoModal({
                                question: "This on hold transaction is already finalized",
                                positiveText: 'Ok',
                                negativeText: '',
                                noCallback: true
                            }).render();
                            this.clearSales();
                            return false;
                        }

                        var options = {
                            saleId: r.id,
                            clientEmail: this.clients.get(this.model.get('payingClientGlobalId')).get('primaryEmailAddress')
                        };
                        switch (window.SETTINGS.receiptOption) {
                            case 'Prompt':
                                options.action = 'prompt';
                                break;
                            case 'Email':
                                options.action = 'email';
                                break;
                            case 'Print':
                                this.printModalCallback();
                                return false;
                                break;
                        }
                        this.onPrintModal = new PrintView(options).render();
                        this.listenToOnce(this.onPrintModal, 'printModalEvent', this.printModalCallback);
                }
            }.bind(this))
			    .always(function(){
                    this.queriedFinalize = false;
                }.bind(this));
        },

        clearSales: function () {
            this.$('.button.no-sale, .button.hold').addClass('inactive');
            this.model.reset();
            this.collection.reset();
            this.discounts.reset();
            this.checkouts.reset();
            this.clients.reset();
            this.plusClientId = undefined;
            this.tips = [];
            this.customTips = [];
            this.lastSelected.clear();
            this.tipModalData.tipAmount = 0;
            this.tipModalData.tipType = 'percentage';
            this.changeCheckoutPayments();
            this.refresh();
        },

        changeCheckoutPayments: function () {
            var difference = +(parseFloat(this.model.get('totalPayments') - this.checkouts.getAmount()).toFixed(2)),
                title = difference >= 0 ? 'Balance ' : 'Change Due ';
            difference < 0 ? this.model.set('changeDue', -difference) : this.model.set('changeDue', 0);
            this.model.set('balance', difference);
            if (!this.onPhone) {
                this.checkoutModal && this.checkoutModal.checkoutBalanceCaption(difference);
            }
            this.$('.checkout-balance-title').empty().html(title);
            this.$('.checkout-balance-amount').empty().html($.global.format(Math.abs(difference), 'c'));
            this.changeFinalizeSaleState();
        },

        changeFinalizeSaleState: function () {
            if (this.model.get('balance').toFixed(2) <= 0) {
                this.$('.smart-receipt-wrapper').addClass('can-finalize');
            } else {
                this.$('.smart-receipt-wrapper').removeClass('can-finalize');
            }
        },

        renderTabletLandscapeRight: function () {
            if (this.$('.sub-menu-nav-item').length == 0) {
                this.renderTabletSubMenu('sales');
            }
        },

        renderReceiptSubMenu: function (type, noTrigger) {
            var menu = type == 'sales' ? this.receiptSubMenu : [{
                url: 'payments',
                title: 'Payment',
                isElevatable: 'true'
            }];
            this.$('.receipt-sub-menu').html(this.receiptSubMenuTemplate({
                menu: menu
            }));
            setTimeout(function () {
                if (!noTrigger) {
                    this.$('.sub-menu-item:first').trigger('touchend', true);
                }
            }.bind(this), 0);
        },

        renderTabletSubMenu: function (type) {
            var menu = type == 'sales' ? this.saleTypesMenu : this.paymentTypesMenu;
            if (this.tabletSubMenuType != type) {
                this.$('#sub-menu-types-scroll ul').html(this.subMenuTypesTemplate({
                    menu: menu
                }));
                this.tabletSubMenuType = type;
            }

            if (this.tabletSubMenuScroll) {
                this.tabletSubMenuScroll.refresh();
            } else {
                this.tabletSubMenuScroll = new IScroll('#sub-menu-types-scroll-wrap', {
                    tap: true,
                    disableMouse: true,
                    bounce: false,
                    scrollX: true,
                    scrollY: true
                });
                this.tabletSubMenuScroll.on('beforeScrollStart', function () {
                    this.refresh();
                });
            }
            this.onRotate();
            setTimeout(function () {
                var $subMenu = this.$('.sub-menu-nav-item'),
                    $selected = $subMenu.filter('.selected'),
                    $toTap = $selected.length ? $selected.removeClass('selected') : $subMenu.filter(':first');
                $subMenu.parent().show();
                $toTap.trigger('tap', true);
            }.bind(this), 0);
        },

        hideTabletLandscapeRight: function (removeActiveClass) {
            var $tabRight = this.$('#register-tab-right');
            if (!this.onPhone && !this.onLandscape) {
                $tabRight.addClass('hidden').removeClass('active');
            }
            if (removeActiveClass && $tabRight.hasClass('active')) $tabRight.removeClass('active');
        },

        closeUnneededModals: function () {
            if (!this.onPhone) {
                this.closeCheckoutModal();
                if (this.salesModal) {
                    this.salesModal.closeView();
                    this.salesModal = undefined;
                }
                if (this.adjustmentModal) {
                    this.adjustmentModal.closeView();
                    this.adjustmentModal = undefined;
                }
                if (this.manualDiscounts) {
                    this.manualDiscounts.closeView();
                    this.manualDiscounts = undefined;
                }
                if (this.predefinedDiscountsModal) {
                    this.predefinedDiscountsModal.closeView();
                    this.predefinedDiscountsModal = undefined;
                }
                if (this.onRefundModal) {
                    this.onRefundModal.closeView();
                    this.onRefundModal = undefined;
                }
                if (this.onHoldModal) {
                    if (this.onHoldModal.holdDeletedModal) {
                        this.onHoldModal.holdDeletedModal.customClose();
                    }
                    this.onHoldModal.closeView();
                    this.onHoldModal = undefined;
                }
                if (this.transactionsModal) {
                    this.transactionsModal.closeView();
                    this.transactionsModal = undefined;
                }
            }
        },

        closeCheckoutModal: function () {
            if (this.checkoutModal) {
                this.trigger('checkoutEvent', {type: 'cancel'});
                this.checkoutModal.closeView();
                this.checkoutModal = undefined;
            }
        },

        hideCheckoutLines: function (showInRight) {
            var $wrapper = this.$('.smart-receipt-wrapper').removeClass('show-checkout-payments');
            if ($wrapper.hasClass('show-checkout')) {
                this.$('.smart-receipt-wrapper').removeClass('show-checkout');
                this.scroll._execEvent('beforeScrollStart');
                if (!this.onPhone) {
                    this.renderReceiptSubMenu('sales', showInRight);
                    this.closeCheckoutModal();
                    this.$('.sub-menu-nav-item.selected').removeClass('selected');
                    this.hideTabletLandscapeRight(true);
                }
            }
        },

        updatePaymentInfoHeight: function () {
            if (!this.onPhone) {
                var height = 66 + this.$('.checkout-line').length * 35;

                if (!this.onLandscape) {
                    height = height > 115 ? height : 115;
                    height = height < 245 ? height : 245;
                } else {
                    height = 115;
                }
                $('.checkout-payments-info').css('height', height);
                this.checkoutScroll && this.checkoutScroll.refresh();
            }
        },

        addInvisibleClient: function () {
            this.clients.add({
                alertCount: 0,
                alerts: null,
                balance: 0,
                clientNoteCount: 0,
                clientNoteSummary: "",
                displayName: "",
                enableClientAccountCharging: true,
                firstName: "",
                id: this.invisibleClientId,
                imageId: "",
                lastName: "",
                primaryEmailAddress: "",
                primaryPhoneNumber: ""
            });
        },

        checkIfAppointments: function (clientId, fromAppointment) {
            getAppointmentsDetails(clientId).done(function (r) {
                if (r.status === undefined) {

                    if (fromAppointment) {
                        this.addAppointmentDetails(r, clientId);
                        this.checkIfHold(clientId);
                    } else {
                        if (r.length > 0) {
                            this.yesNoModal = new YesNoModal({
                                question: 'This client has appointments today. Would you like to import?',
                                positiveText: 'Yes',
                                positiveCallback: function () {
                                    this.yesNoModal.closeView();
                                    this.checkIfHold(clientId);
                                    this.addAppointmentDetails(r, clientId);
                                }.bind(this),
                                negativeText: 'No',
                                negativeCallback: function () {
                                    this.yesNoModal.closeView();
                                    this.checkIfHold(clientId);
                                }.bind(this)
                            }).render();
                        } else {
                            this.checkIfHold(clientId);
                        }
                    }

                }
            }.bind(this));
        },

        holdTransactionExtract: function (m) {
            this.stopListening(this.model, 'change:payingClientGlobalId', this.changePayingClient);
            this.clearSales();
            this.holdPayingClientId = m.payingClientGlobalId;
            this.model.set(m);
            this.listenTo(this.model, 'change:payingClientGlobalId', this.changePayingClient);
            this.detectIfCanNotCheckout();
                    
            // clients
            this.clients.add(m.clients);
            this.clientRemoved(this.clients.get(this.invisibleClientId));
            this.clients.remove(this.invisibleClientId);

            // discounts
            this.discounts.add(m.discounts);

            // saleLines
            _.each(m.saleLines, function (l, index) {
                this.collection.add(l, [index === m.saleLines.length - 1]);
                var line = this.collection.get(l.id);
                if (l.item.type != Consts.SaleLineTypes.GIFTCARD && l.item.type != Consts.SaleLineTypes.ACCOUNT) {
                    this.saleLineChangeDiscounts(line);
                    this.getSaleLineAlerts(line);
                }
            }.bind(this));
                    
            // payments
            _.each(m.payments, function (p) {
                p.paymentType = _.findWhere(this.checkoutPaymentTypes, { id: p.paymentType.id })
            }.bind(this));
            this.checkouts.add(m.payments);

            // tips
            this.tips = m.tips;
            this.customTips = _.map(m.customTips, function (tip) {
                return _.extend(tip, {
                    elementId: +new Date,
                    serviceTotal: 0,
                    templateType: "newEmployee"
                });
            });

            this.changeCheckoutPayments();
            this.$('.smart-receipt-wrapper').addClass('show-checkout-payments');

            //this.reCalcSale(function () {
            //    this.changeCheckoutPayments();
            //    this.$('.smart-receipt-wrapper').addClass('show-checkout-payments');
            //    // 
            //    //if (this.model.get('totalPayments') < 0) { // refund
            //    //    this.$('.smart-receipt-wrapper').addClass('show-checkout-payments');
            //    //} else { // normal checkout
            //    //    this.$('.button.checkout').trigger('touchend');
            //    //}
            //}.bind(this));
        },

        beforeGetFirstHoldTransaction: function () {
            if (this.collection.toJSON().length > 0 || (this.model.get('saleTempId') && this.collection.length)) {
                this.yesNoModal = new YesNoModal({
                    question: 'Warning: Current Transaction Will Be Overwritten When This Transaction is Returned. Would You Like to Continue?',
                    positiveText: 'Yes',
                    positiveCallback: function () {
                        this.yesNoModal.closeView();
                        this.getFirstHoldTransaction();
                    }.bind(this),
                    negativeText: 'No',
                    negativeCallback: function () {
                        this.closeView();
                    }
                }).render();
            } else {
                this.getFirstHoldTransaction();
            }
        },

        getFirstHoldTransaction: function () {
            var holdItem = this.clientHoldTransactions.toJSON()[0];
            this.holdPayingClientId = holdItem.payingClient.id;
            this.clientHoldTransactions.prepareForExtract(holdItem.saleTempId);
        },

        checkRefundTaxesVisibility: function () {
            if (this.collection.checkOnRefundItems()) { 
                this.$('.refund-tax').show();
            } else {
                this.$('.refund-tax').hide();
            }
        },

        changeLastSelectedClient: function (clientId, removed) {
            this.lastSelected.set('clientId', clientId);
            /*if (this.onRefundModal) {
                this.onRefundModal.model.set('client', this.clients.getLastSelectedClient(this.lastSelected.clientId, removed));
            }*/
        },

        prepareVisitNotes: function() {
            if (window.SETTINGS['registerDisplayVisitNotes']) {
                var clientsIds = this.clients.getNonWalkInClientsIds(); // get all clients except walkInMale and walkInFemale
                var clients = [];
                if (!clientsIds) return false;
                var servicesForClients = this.collection.getServicesForClients(clientsIds); // get all services grouped by clients
                if (_.isEmpty(servicesForClients)) return false;

                // get all visitNoteTypeItemId
                var visitNotesByClientShow = {};
                for (var clientId in servicesForClients) {
                    if (servicesForClients.hasOwnProperty(clientId)) {
                        servicesForClients[clientId] = _.compact(_.uniq(_.pluck(_.pluck(servicesForClients[clientId], 'item'), 'visitNoteTypeItemId')));

                        // make calls for each client to get deep info
                        getClientData(clientId).done(function (clientInfo) {
                            clients.push(clientInfo);
                            var vTypes = _.compact(_.uniq(_.pluck(_.pluck(clientInfo.visitNotes, 'noteType'), 'id')));

                            // verify visitNotesTypes in ClientInfo and visitNoteTypeItemId in services
                            vTypes = _.filter(vTypes, function (type) {
                                return _.indexOf(servicesForClients[clientInfo.id], type) !== -1;
                            });
                            visitNotesByClientShow[clientInfo.id] = vTypes.length == 0 ? [] : vTypes;
                            if (_.keys(visitNotesByClientShow).length == _.keys(servicesForClients).length) {
                                this.createNotesQueue(visitNotesByClientShow, clients);
                            }
                        }.bind(this));
                    }
                }
            }
        },

        createNotesQueue: function (visitNotesByClientShow, clients) {
            // add every unique visit note, that has minimum 1 service in receipt, in queue of popUp calls
            var queue = [];
            _.each(visitNotesByClientShow, function(noteList, clientId) {
                _.each(noteList, function(noteId) {
                    queue.push({
                        clientId: clientId,
                        noteId: noteId
                    });
                });
            });
            this.showVisitNotes(queue, clients);
        },

        showVisitNotes: function(queue, clients) {
            var clientModal;
            if(queue.length) {
                setTimeout(function() {
                    clientModal = new ClientModal(_.extend({
                        showVisitNotes: true,
                        filterVisitNotesBy: queue[queue.length-1].noteId
                    }, _.findWhere(clients, {id: queue[queue.length-1].clientId})));
                    queue.pop();
                    this.listenToOnce(clientModal, 'notesQueue', this.showVisitNotes.bind(this, queue, clients));
                }.bind(this), 0);
            }
        }

    });
});
