define([
    'jquery',
    'searchPlugin',
    'data/consts',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/tip-modal.html',
    'text!../../../templates/smart-receipt/employee-tip-template.html',
    'bridge'
], function ($, searchPlugin, Consts, ModalView, YesNoModal, AddTipModalTemplate, EmployeeTipTemplate) {
    'use strict';

    return  ModalView.extend({
        el: '#tip-sale-modal .content',
        template: _.template(AddTipModalTemplate),
        scrollWrap: '#tip-sale-modal-scroll-wrap',
        scrollbars: false,
        employeeTemplate: _.template(EmployeeTipTemplate),
        searchEmployeeTemplate: _.template('<div class="option" data-id="<%= id %>"><%= lastName %>, <%= firstName %></div>'),
        employeeFilterList: undefined,
        customTips: [],
        employeeList: [],
        _initialChange: undefined,
        events: {
            'tap .set-tip-amount': 'setTipAmount',
            'tap .tip-type': 'setTipType',
            'tap .clear-tip-amount': 'clearTipAmount',
            'tap #round-amount': 'roundAmount',
            'tap #apply-tip': 'applyTip',
            'tap #add-employee': 'addEmployee',
            'tap .employee-list-body .option': 'selectEmployee',
            'tap .remove-employee': 'removeEmployee',
            'tap #apply-spread': 'applySpread',
            'blur .additional-tip-value': 'recalculateChange'
        },
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                { id: 'close-tip', className: 'red cancel', value: 'Cancel', containerClass: 'media-element fr' },
                { id: 'submit-tip', className: 'green', value: 'Submit', containerClass: 'media-content' }
            ],
            tabletButtons: [
                { id: 'submit-tip', className: 'green', size: '50', value: 'Submit' },
                { id: 'close-tip', className: 'red', size: '50', value: 'Cancel' }
            ],
            touchCallBacks: {
                'submit-tip': 'submitTip',
                'close-tip': 'closeTip'
            }
        },

        initialize: function (options) {
            if(options.initialChange !== undefined)
                this._initialChange = parseFloat(options.initialChange).toFixed(2);

            this.modalButtonsBottom.phone = $('html').hasClass('mobile');

            this.employeeList = _.map(options.employeeList, function (g, key) {
                var existingEmpTip = _.filter(options.allTips, function (tip) { return tip.employee.id == key; })[0],
                    existingCustomTip = _.filter(options.customTips, function (tip) { return tip.employee.id == key; })[0],
                    toReturn = {
                        employee: {
                            code: g[0].code,
                            lastName: g[0].lastName,
                            firstName: g[0].firstName,
                            imageId: g[0].imageId,
                            id: g[0].id
                        },
                        id: key,
                        serviceTotal: _.reduce(g, function (m, x) {
                            return m + x.totalLinePrice;
                        }, 0),
                        templateType: 'existedEmployee',
                        amount: 0,
                        tipType: options.tipType
                    };
                
                if (existingCustomTip) {
                    if (toReturn.serviceTotal >= 0) {
                        toReturn.amount += existingCustomTip.amount;
                        options.customTips = _.without(options.customTips, _.findWhere(options.customTips, { id: key }));
                    } 
                }
                
                if (existingEmpTip) {
                    toReturn.amount = existingEmpTip.amount;
                }
                return toReturn;
            });
            this.employeeList = _.filter(this.employeeList, function (emp) {
                return emp.serviceTotal >= 0;
            });

            this.storedData = options.tipModalData;

            this.model = new Backbone.Model({
                tipType: options.tipType,
                changeDue: options.changeDue
            });

            this.customTips = options.customTips;
            this.allTips = this.employeeList.concat(this.customTips);
            this.employeeFilterList = _.pluck(this.employeeList, 'id');
            this.total = _.reduce(_.pluck(this.employeeList, 'serviceTotal'), function (memo, num){
                return memo + num;
            }, 0);
            this.listenTo(this.model, 'change:changeDue', function () {
                this.$('.change-due').html($.global.format(this.model.get('changeDue'), 'c'));
            });
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        render: function () {
            var template = '';
            ModalView.prototype.render.call(this);

            this.$('.change-due').html($.global.format(this.model.get('changeDue'), 'c'));

            this.scroll.on('beforeScrollStart', function (e) {
                this.refresh();
            });

            _.each(this.allTips, function (model) {
                model.tipType = this.model.get('tipType');
                template += this.employeeTemplate(model);
                
            }.bind(this));
            this.$('.employee-list-body').html(template);

            //init plugins
            _.each(this.$('.employee'), function (el) {
                $(el).searchPlugin({
                    filterFunction: filterEmployees,
                    template: this.searchEmployeeTemplate,
                    parentScroll: this.scroll,
                    noCloseButton: true,
                    onlyProperValues: true
                });
                $(el).searchPlugin('updateRemoveList', this.employeeFilterList);
            }.bind(this));

            _.each(this.$('.tip-value'), function (el) {
                $(el).inputFormat({
                    places: 10,
                    allowMinus: false
                });
            }.bind(this));

            _.each(this.$('.additional-tip-value'), function (el) {
                $(el).inputFormat({
                    places: 10,
                    allowMinus: false
                });
            }.bind(this));
            if ( this.model.get('tipType') == 'general'){
                this.$('#tip-amount').inputFormat({
                    type: this.storedData.tipType == 'percentage' ? 'percentage' : 'currency',
                    allowMinus: false
                });

                $('#tip-amount').inputFormat('setValue', this.storedData.tipAmount);
                $('.tip-type').removeClass('active-type');
                $('.' + this.storedData.tipType).addClass('active-type');
            }

            $(':focus').blur();

            return this;
        },

        setTipAmount: function (e) {
            this.$('#tip-amount').inputFormat('setValue', $(e.target).data('amount'));
        },

        clearTipAmount: function (e) {
            this.$('#tip-amount').inputFormat('setValue', 0);
            this.$('.tip-value').inputFormat('setValue', 0);
        },

        setTipType: function (e) {
            var $target = $(e.target);
            if ($target.hasClass('active-type')) {
                return false;
            } else {
                $('.tip-type').removeClass('active-type');
                var type = 'currency';

                if ($target.hasClass('percentage')) {
                    type = 'percentage';
                }
                this.$('#tip-amount').inputFormat('changeType', type);
                $target.addClass('active-type');
            }
        },

        roundAmount: function () {
            _.each($('.tip-value'), function (tip) {
                $(tip).inputFormat('setValue', Math.round($(tip).inputFormat('getValue')));
            });
        },

        applyTip: function () {
            var $tipValue = this.$('#tip-amount'),
                tipAmount = $tipValue.inputFormat('getValue'),
                tipType = $tipValue.inputFormat('getType'),
                $tipInput, serviceTotal, newValue;

            _.each(this.$('.employee-list-body .cols'), function (col) {
                $tipInput = $(col).find('.tip-value');
                var serviceTotal = $(col).find('.service-total').html();

                var fB = serviceTotal.indexOf("(");
                var sB = serviceTotal.indexOf(")");

                if (fB != -1 && sB != -1 && sB > fB)
                    serviceTotal = "-" + serviceTotal.replace("(", "").replace(")","");

                serviceTotal = $.global.parseFloat(serviceTotal);
                switch (tipType) {
                    case 'currency':
                        newValue = (serviceTotal * tipAmount) / this.total;
                        break;
                    case 'percentage':
                        newValue = (tipAmount / 100) * serviceTotal;
                        break;
                }
                if ($tipInput.hasClass('existedEmployee')) {
                    $tipInput.inputFormat('setValue', newValue.toFixed(2));
                }
            }.bind(this));
        },

        applySpread: function () {
            var onePercent = this.total / 100,
                changeDue = parseFloat(this.model.get('changeDue').toFixed(2)),
                totalAdditionalTip = 0;

            var totalNotExistedEmployeeChange = 0;
            _.each($('.employee-list-body .cols .additional-tip-value').not(".existedEmployee"), function (col) {
                totalNotExistedEmployeeChange += $(col).inputFormat('getValue');
            });

            _.each($('.employee-list-body .cols'), function (col) {
                if ($(col).find('.additional-tip-value').hasClass('existedEmployee')) {
                    var serviceTotal = +$(col).find('.service-total').data('service-total'),
                        amountPercentage = parseFloat(serviceTotal / onePercent) || 0,
                        additionalTip = parseFloat(((changeDue / 100) * amountPercentage).toFixed(2));
                    totalAdditionalTip += additionalTip;
                    $(col).find('.additional-tip-value').inputFormat('setValue', additionalTip);
                }
            });
            this.$('.change-due').html($.global.format(changeDue - totalAdditionalTip - totalNotExistedEmployeeChange, 'c'));
        },

		calculateChangeLines: function(){
			var resultDue = 0;
			_.each($('.employee-list-body .cols').filter(function (index, elem) { return $(elem).attr('data-employee-id') !== ''; }), function(elem){
				var e = $(elem);
				resultDue += this.plusDue(e, '.existing') + this.plusDue(e, '.additional-tip-value');
			}.bind(this));

			return resultDue.toFixed(2);
		},

		plusDue: function(parent, selector){
			var existingTip = parent.find(selector),
                existingTipHTML = '';

			if(existingTip !== undefined && existingTip !== '') {
				if(existingTip.inputFormat('getValue') !== 0) {
                    return existingTip.inputFormat('getValue');
                }
                existingTipHTML = existingTip.html();

                if (!existingTipHTML) {
                    existingTipHTML = "0";
                }
                if ($.global.parseFloat(existingTipHTML) !== 0) {
                    return $.global.parseFloat(existingTipHTML);
                }
			}

			return 0;
		},

        addEmployee: function (e) {
			if(resolveEmployeeEditable()){
				new YesNoModal({
				    question: 'Please choose an employee for the previous tip line.',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                return false;
			}

            $('.employee-list-body').append(this.employeeTemplate({
                templateType: 'newEmployee',
                serviceTotal: 0,
                amount: 0,
                elementId: +new Date,
                employee: {
                    lastName: '',
                    firstName: ''
                },
                id: '',
                tipType: this.model.get('tipType')
            }));

            this.$('.employee:last').searchPlugin({
                filterFunction: filterEmployees,
                template: this.searchEmployeeTemplate,
                parentScroll: this.scroll,
                noCloseButton: true,
                onlyProperValues: true
            });
            this.$('.employee:last').searchPlugin('updateRemoveList', this.employeeFilterList).focus();
            this.$('.tip-value:last').inputFormat({
                type: 'currency',
                places: 10,
                allowMinus: false
            });
            this.$('.additional-tip-value:last').inputFormat({
                type: 'currency',
                places: 10,
                allowMinus: false
            });
        },

        selectEmployee: function (e) {
            var selectedEmployee = _.findWhere(this.$('.employee:last').searchPlugin('getList'), {id: $(e.target).attr('data-id')});

            if ($(e.target).closest('.cols').attr('data-employee-id')) {
                this.employeeFilterList = _.without(this.employeeFilterList, $(e.target).closest('.cols').attr('data-employee-id'));
                this.customTips = _.without(this.customTips, _.findWhere(this.customTips, {id: $(e.target).closest('.cols').attr('data-employee-id')}));
            }

            selectedEmployee.amount = 0;
            this.customTips.push(selectedEmployee);
            this.employeeFilterList.push(selectedEmployee.id);
            _.each($('.employee'), function (el) {
                $(el).searchPlugin('updateRemoveList', this.employeeFilterList);
            }.bind(this));
            $(e.target).closest('.cols').attr('data-employee-id', selectedEmployee.id);
        },

        removeEmployee: function (e) {
			var df = $(e.target).closest('.cols').attr('data-employee-id');

            this.employeeFilterList = _.without(this.employeeFilterList, df);
            this.customTips = _.without(this.customTips, _.findWhere(this.customTips, {id: df}));

			$(e.target).closest('.cols').remove();
			this.recalculateChange(undefined);
        },

        recalculateChange: function (event) {
            this.$('.change-due').html($.global.format(this._initialChange - this.calculateChangeLines(), 'c'));
        },

		availableEmployees: function(){
			return $('.additional-tip-value').filter(function () { return $(this).closest('.cols').attr('data-employee-id') !== ''; });
		},

        submitTip: function (e) {
			$('.additional-tip-value').blur();

            var r = {};
            if (this.model.get('tipType') == 'general') {
                _.each($('.tip-value'), function (el) {
					var searchSource = $(el).hasClass('newEmployee') ? this.customTips: this.employeeList;
					var employeeId = $(el).closest('.cols').attr('data-employee-id');
					if(employeeId !== '') _.findWhere(searchSource, {id: employeeId}).amount = $(el).inputFormat('getValue');
                }.bind(this));
            } else {
				var totalDue = this._initialChange - this.calculateChangeLines();

                if(totalDue < 0){
                    new YesNoModal({
                        question: 'The total tip amount cannot exceed the change due',
                        positiveText: 'OK',
                        noCallback: true,
                        negativeText: ''
                    }).render();
                    return false;
                } else
					r.newChangeDue = totalDue;

                _.each(this.availableEmployees(), function (el) {
                    var employee;
                    if ($(el).hasClass('newEmployee'))
						employee = _.findWhere(this.customTips, {id: $(el).closest('.cols').attr('data-employee-id')});
                    else
						employee = _.findWhere(this.employeeList, {id: $(el).closest('.cols').attr('data-employee-id')});

					employee.amount = $(el).inputFormat('getValue') + employee.amount;
                }.bind(this));
            }

            this.customTips = _.map(this.customTips, function (obj, k) {
                if (!obj.templateType) {
                    return {
                        employee: {
                            code: obj.code,
                            lastName: obj.lastName,
                            firstName: obj.firstName,
                            imageId: obj.imageId,
							id: obj.id
                        },
                        amount: obj.amount,
                        id: obj.id,
                        serviceTotal: 0,
                        templateType: 'newEmployee',
                        elementId: +new Date
                    };
                } else {
                    return obj;
                }
            });
            r.customTips = this.customTips;
            r.allTips = this.employeeList.concat(this.customTips);
            r.tipModalData = {
                tipAmount: this.$('#tip-amount').inputFormat('getValue'),
                tipType: this.$('.percentage').hasClass('active-type') ? 'percentage' : 'explicit'
            };
            this.trigger('submitTips', r);
            this.closeView();
        },

        closeTip: function () {
            if (this.model.get('tipType') == 'additional') {
                this.trigger('submitTips', {});
            }
            this.closeView();
        }

    });

	function resolveEmployeeEditable() {
			return _.some($('.employee-list .employee-list-body .cols .employee'), function (elem) {
				var el = $(elem);
				return el.val() === '' || el.data('id') === '';
			});
		}
});
