/**
 * Created by petermomot on 7/8/15.
 */
define([
    'jquery',
    'moment',
    'data/consts',
    'data/permissions',
    'data/transactions',
    'views/yes-no-modal',
    'views/smart-receipt/transactions-modal',
    'views/smart-receipt/transactions-list-modal',
    'text!../../../templates/smart-receipt/transactions-modal.html',
    'text!../../../templates/smart-receipt/transactions-line.html',
    'bridge'
], function ($, moment, Consts, PermModel, transactionsCollection,
             YesNoModal, TransactionBaseModal, TransactionsFilteredModal,
             TransactionsModalTemplate, TransactionsItemTemplate) {
    'use strict';

    return TransactionBaseModal.extend({
        transactionsFilteredModal: undefined,
        modalButtonsBottom: {
            phone: false,
            phoneButtons: [
                {
                    id: 'transactions-cancel',
                    className: 'red cancel',
                    size: '20',
                    value: ''
                },
                {
                    id: 'transactions-filter',
                    className: 'violet-light',
                    size: '80',
                    value: 'filter'
                }
            ],
            tabletButtons: [
                {
                    id: 'transactions-cancel',
                    className: 'red',
                    size: '100',
                    value: 'cancel'
                }
            ],
            touchCallBacks: {
                'transactions-cancel': 'tapCancel',
                'transactions-filter': 'showFilteredTransactions'
            }
        },

        initialize: function (options) {
            var o = _.extend({}, options, {
                phone: $('html').hasClass('mobile'),
                PermModel: PermModel
            });
            if (o.phone) {
                this.breadCrumbs = ['Register', 'Transactions'];
                this.modalButtonsBottom.phone = true;
                this.el = '#sales-transactions-modal .content';
                this.setElement(this.el);
            }
            this.model = new Backbone.Model(o);
            TransactionBaseModal.prototype.initialize.call(this, options);
            if (!this.model.get('phone')) this.addListeners();
        },

        onRender: function () {
            this.$('#transactions-invoice').inputFormat({
                type: 'any',
                clearButton: true,
                allowEmpty: true,
                allowMinus: false,
                places: 14
            });
            this.$('#transactions-drawer').searchPlugin({
                filterFunction: getDrawers,
                parentScroll: this.scroll
            });
            this.$('#transactions-from-date, #transactions-to-date').val(this.prepareTodayForInput());

            if (!this.model.get('phone')) this.showFilteredTransactions();
        },

        customClose: function (response) {
            this.transactionsFilteredModal = undefined;
            if (this.model.get('phone')) {
                if (response) this.trigger('transactionsEvent', response);
                else this.trigger('transactionsEvent', { type: 'cancel' });
                this.closeView();
                this.resetAll();
            } else {
                this.trigger('transactionsEvent', {
                    type: 'cancel',
                    doneLeave: true
                });
            }
        },

        // modal buttons bottom event callback
        showFilteredTransactions: function () {
            this.transactionsCollection.reset();
            this.collectedFilterInfo.pageNumber = 0;
            this.collectFilterParams();
            this.getTransactionsAndRender();
            if (!this.model.get('phone')) return false;
            $(':focus').blur();
            this.transactionsFilteredModal = new TransactionsFilteredModal({
                criteria: this.$('#transactions-status').val(),
                collectedFilterInfo: this.collectedFilterInfo
            }).render();
            this.listenTo(this.transactionsFilteredModal, 'transactionFilteredEvent', function (r) {
                this.transactionsFilteredModal = undefined;
                if (r.type == 'returnToMenu') {
                    this.customClose({type: 'returnToMenu'});
                }
            }.bind(this));
        },

        collectFilterParams: function () {
            var tempEndDate = this.$('#transactions-to-date').val();
            this.collectedFilterInfo = {
                transactionNumber: this.$('#transactions-invoice').val(),
                startDate: this.$('#transactions-from-date').val(),
                endDate: tempEndDate ? tempEndDate + ' 23:59:59' : '',
                drawerId: this.$('#transactions-drawer').data('id'),
                saleState: Consts.SaleStateEnum[this.$('#transactions-status').val()],
                pageNumber: 0,
                pageSize: this.pageSize
            };
        },

        transactionAdded: function (item) {
            if (this.model.get('phone')) return false;
            TransactionBaseModal.prototype.transactionAdded.call(this, item);
        },

        prepareTodayForInput: function () {
            var now = new Date();
            var day = ("0" + now.getDate()).slice(-2);
            var month = ("0" + (now.getMonth() + 1)).slice(-2);
            return now.getFullYear() + "-" + (month) + "-" + (day);
        },

        transactionVoidCallback: function () {
            this.showFilteredTransactions();
        }
    });
});