﻿/**
 * Created by petermomot on 7/6/15.
 */
define([
    'jquery',
    'moment',
    'data/consts',
    'data/permissions',
    'data/transactions',
    'views/modal',
    'views/yes-no-modal',
    'text!../../../templates/smart-receipt/transactions-modal.html',
    'text!../../../templates/smart-receipt/transactions-line.html',
    'bridge'
], function ($, moment, Consts, PermModel, transactionsCollection,
             ModalView, YesNoModal,
             TransactionsModalTemplate, TransactionsItemTemplate) {
    'use strict';

    return ModalView.extend({
        el: '#tablet-sales-transactions-modal .content',
        template: _.template(TransactionsModalTemplate),
        transactionItemTemplate: _.template(TransactionsItemTemplate),
        scrollbars: 'custom',
        yesNoModal: undefined,
        transactionsCollection: transactionsCollection,
        pageSize: 20,
        requestInProcess: false,
        collectedFilterInfo: {},
        modalButtonsBottom: true,

        initialize: function (options) {
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        initializeIScroll: function () {
            var that = this,
                wrapper;

            this.$el.wrapInner('<div class="scroll-outside"><div class="scroll-wrap"><div></div></div></div>');
            wrapper = this.$('.scroll-wrap').get(0);
            this.scroll = new IScroll(wrapper, {
                tap: true,
                disableMouse: true,
                probeType: 2,
                scrollbars: this.scrollbars
            });
            this.scroll.on('beforeScrollStart', function () {
                this.refresh();
            });
            this.scroll.on('scroll', function () {
                if (that.collectedFilterInfo.pageNumber != 'last') {
                    this._triggerLoadNext = this.y < 0 && (this.y - this.maxScrollY) < -30;
                    this._scrollToY = this.y;
                    $(this.wrapper).toggleClass('active-loading', this.y < 0 && (this.y - this.maxScrollY) < 0);
                }
            });
            this.scroll.on('beforeScrollEnd', function () {
                $(this.wrapper).removeClass('active-loading loading');
                if (this._triggerLoadNext) {
                    this.enabled = false;
                    $(this.wrapper).addClass('loading');
                    // small delay to show visual effect
                    setTimeout(function () {
                        this._triggerLoadNext = false;
                        that.getTransactionsAndRender();
                    }.bind(this), 100);
                }
            });
        },

        addListeners: function () {
            this.listenTo(this.transactionsCollection, 'add', this.transactionAdded);
            this.listenTo(this.transactionsCollection, 'reset', this.transactionsCollectionReset);
            this.listenTo(this.transactionsCollection, 'emptyEvent', this.transactionsCollectionEmpty);
            this.listenTo(this.transactionsCollection, 'notEmptyEvent', this.transactionsCollectionNotEmpty);
        },

        events: {
            'tap .transaction-void': 'transactionVoid'
        },

        // modal buttons bottom event callback
        tapCancel: function () {
            this.customClose();
        },

        getTransactionsAndRender: function () {
            if(!this.requestInProcess) {
                this.requestInProcess = true;
                getTransactions(this.collectedFilterInfo).done(function (r) {
                    if (r.status === undefined) {
                        if (r.length == 0) {
                            if (this.transactionsCollection.length === 0) {
                                this.transactionsCollection.trigger('emptyEvent');
                            }
                            this.collectedFilterInfo.pageNumber = 'last';
                        } else {
                            this.transactionsCollection.trigger('notEmptyEvent');
                            this.transactionsCollection && this.transactionsCollection.add(r);
                            this.collectedFilterInfo.pageNumber += 1;
                        }
                    }
                }.bind(this)).always(function(){
                    this.requestInProcess = false;
                    if (this.scroll) {
                        this.scroll.enabled = true;
                        $(this.scroll.wrapper).removeClass('active-loading loading');
                    }
                }.bind(this));
            }
        },

        transactionAdded: function (item) {
            if (this.$('.transaction-line[data-id="' + item.get('saleId') + '"]').length) return false; // prevent duplicates in list
            this.$el && this.$('#transactions-items-wrap').append(this.transactionItemTemplate(_.extend(item.toJSON(), {
                PermModel: PermModel,
                moment: moment,
                phone: this.model.get('phone'),
                Consts: Consts
            })));
            this.scroll && this.scroll.refresh();
        },

        transactionsCollectionReset: function () {
            this.$el && this.$('#transactions-items-wrap').html('<div class="empty">Loading transactions</div>');
            if (this.scroll) {
                this.scroll.refresh();
                this.scroll.scrollTo(0, 0);
            }
        },

        transactionsCollectionEmpty: function () {
            this.transactionsCollection.collectionEmpty = true;
            this.$el && this.$('#transactions-items-wrap').html('<div class="empty">There are no transactions for this criteria</div>');
        },

        transactionsCollectionNotEmpty: function () {
            this.transactionsCollection.collectionEmpty = false;
            this.$el && this.$('.empty').remove();
        },

        resetAll: function () {
            this.transactionsCollection.reset();
            this.transactionsCollection = undefined;
            this.yesNoModal = undefined;
        },

        transactionVoid: function (e) {
            PermModel.navigateToPermissionsModal(e, function () {
                var transaction = this.transactionsCollection.findWhere({ 'saleId': $(e.target).data('id') }).toJSON();
                if (transaction.canBeVoided) {
                    this.yesNoModal = new YesNoModal({
                        question: 'Are you sure you want to void this sale?',
                        positiveText: 'Yes',
                        positiveCallback: function () {
                            this.yesNoModal.closeView();
                            this.transactionVoidRequest(transaction.saleId);
                        }.bind(this),
                        negativeText: 'No',
                        negativeCallback: function () {
                            this.closeView();
                        }
                    }).render();
                } else if (transaction.currentSaleStateEnum == Consts.SaleStateEnum.Void) {
                    this.yesNoModal = new YesNoModal({
                        question: 'This sale has already been voided',
                        positiveText: 'Ok',
                        negativeText: '',
                        noCallback: true
                    }).render();
                } else {
                    this.yesNoModal = new YesNoModal({
                        question: "This sale can't be voided",
                        positiveText: 'Ok',
                        negativeText: '',
                        noCallback: true
                    }).render();
                }
            }.bind(this));
        },

        transactionVoidRequest: function (saleId) {
            voidTransaction(JSON.stringify({ saleId: saleId })).done(function (r) {
                if (r.status === undefined) {
                    this.transactionVoidCallback();
                }
            }.bind(this));
        }

    });
});