/**
 * Created by petermomot on 2/16/15.
 */
define([
    'jquery',
    'searchPlugin',
    'views/modal',
    'text!../../templates/workstation-modal.html',
    'text!../../templates/workstation-option.html',
    'bridge'
], function ($, searchPlugin, ModalView, WorkstationTemplate, SearchWorkstationsTemplate) {
    'use strict';

    var Workstations = Backbone.Model.extend({
        defaults: {
            description: "",
            displayName: "",
            domain: "",
            drawerDisplayName: "",
            drawerId: "",
            id: "",
            shortcut: "",
            subdomain: "",
            userId: ""
        }
    });

    return ModalView.extend({
        el: '#workstation-modal .content',
        className: 'workstations form',
        template: _.template(WorkstationTemplate),
        scrollbars: false,
        model: new Workstations(),

        initialize: function () {
            if (JSON.parse(localStorage.getItem('workstation'))) {
                this.model.set(JSON.parse(localStorage.getItem('workstation')));
            }
            this.events = _.extend({}, ModalView.prototype.events, this.events);
        },

        events: {
            'touchend #button-ok': 'workstationSave',
            'touchend #button-cancel': 'closeView'
        },

        render: function () {
            ModalView.prototype.render.call(this);

            this.$('#workstation-select-input').searchPlugin({
                filterFunction: getWorkstations,
                parentScroll: this.scroll,
                fullScreen: true,
                template: _.template(SearchWorkstationsTemplate),
                bottomButtonsFixed: true,
                selectedCallBack: function(obj) {
                    if (obj)
                        ModalView.prototype.removeError($('#workstation-select-input'));
                }.bind(this)
            });

            return this;
        },

        workstationSave: function (e) {
            e.stopPropagation();
            e.stopImmediatePropagation();
            e.preventDefault();
            if(window.IS_ONLINE !== undefined && !window.IS_ONLINE) {
                return false;
            }
            var $input = this.$('#workstation-select-input'), workstation;
            if ($input.data('id')) {
                $input.parent().removeClass('error');
                workstation = $input.searchPlugin('getSelectedObject');

                if (!workstation) {
                    if(this.model.get('id') !== $input.data('id')) {
                        $input.parent().addClass('error');
                        //nativeAlert('error', 'Please select workstation.');
                        return false;
                    } else {
                        workstation = this.model.toJSON();
                    }
                }
            } else {
                $input.parent().addClass('error');
                //nativeAlert('error', 'Please select workstation.');
                return false;
            }

            workstation['userId'] = sessionStorage.getItem('userId');
            workstation['subdomain'] = localStorage.getItem('subdomain');
            workstation['domain'] = localStorage.getItem('domain');
            localStorage.setItem('workstation', JSON.stringify(workstation));
            Backbone.history.navigate('register', {trigger: true, replace: true});
        }
    });
});